@@ -87,11 +87,13 @@
 
 - (IBAction)aColor_FG:(id)sender
 {
+	UT_UNUSED(sender);
 	[self setColor:AP_TOOLBAR_ID_COLOR_FORE];
 }
 
 - (IBAction)aColor_BG:(id)sender
 {
+	UT_UNUSED(sender);
 	[self setColor:AP_TOOLBAR_ID_COLOR_BACK];
 }
 
@@ -380,7 +382,7 @@ UT_sint32 EV_CocoaToolbar::destroy(void)
  * This method rebuilds the toolbar and places it in the position it previously
  * occupied.
  */
-void EV_CocoaToolbar::rebuildToolbar(UT_sint32 oldpos)
+void EV_CocoaToolbar::rebuildToolbar(UT_sint32 /*oldpos*/)
 {
 	UT_ASSERT (UT_NOT_IMPLEMENTED);
 #if 0
@@ -407,6 +409,7 @@ void EV_CocoaToolbar::rebuildToolbar(UT_sint32 oldpos)
 
 - (void)drawRect:(NSRect)aRect
 {
+	UT_UNUSED(aRect);
 	[[NSColor lightGrayColor] set];
 
 	[NSBezierPath strokeRect:[self frame]];
@@ -749,9 +752,9 @@ bool EV_CocoaToolbar::refreshToolbar(AV_View * pView, AV_ChangeMask mask)
 	UT_ASSERT(pToolbarActionSet);
 	
 	UT_uint32 nrLabelItemsInLayout = m_pToolbarLayout->getLayoutItemCount();
-	for (UT_uint32 k=0; (k < nrLabelItemsInLayout); k++)
+	for (UT_uint32 j=0; (j < nrLabelItemsInLayout); j++)
 	{
-		EV_Toolbar_LayoutItem * pLayoutItem = m_pToolbarLayout->getLayoutItem(k);
+		EV_Toolbar_LayoutItem * pLayoutItem = m_pToolbarLayout->getLayoutItem(j);
 		UT_ASSERT(pLayoutItem);
 
 		XAP_Toolbar_Id tlbrid = pLayoutItem->getToolbarId();
