@@ -165,8 +165,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		return;
 	}
 
-	if (path = [bundle pathForResource:@"Blue16x15" ofType:@"png"]) {
-		if (m_imageBlue16x15 = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Blue16x15" ofType:@"png"];
+	if (path) {
+		m_imageBlue16x15 = [[NSImage alloc] initWithContentsOfFile:path];
+		if (m_imageBlue16x15) {
 			m_colorBlue16x15 = [NSColor colorWithPatternImage:m_imageBlue16x15];
 			[m_colorBlue16x15 retain];
 		}
@@ -175,8 +177,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_colorBlue16x15 = [NSColor blueColor];
 		[m_colorBlue16x15 retain];
 	}
-	if (path = [bundle pathForResource:@"Blue11x16" ofType:@"png"]) {
-		if (m_imageBlue11x16 = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Blue11x16" ofType:@"png"];
+	if (path) {
+		m_imageBlue11x16 = [[NSImage alloc] initWithContentsOfFile:path];
+		if (m_imageBlue11x16) {
 			m_colorBlue11x16 = [NSColor colorWithPatternImage:m_imageBlue11x16];
 			[m_colorBlue11x16 retain];
 		}
@@ -185,8 +189,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_colorBlue11x16 = [NSColor blueColor];
 		[m_colorBlue11x16 retain];
 	}
-	if (path = [bundle pathForResource:@"Grey16x15" ofType:@"png"]) {
-		if (m_imageGrey16x15 = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Grey16x15" ofType:@"png"];
+	if (path) {
+		m_imageGrey16x15 = [[NSImage alloc] initWithContentsOfFile:path];
+		if (m_imageGrey16x15) {
 			m_colorGrey16x15 = [NSColor colorWithPatternImage:m_imageGrey16x15];
 			[m_colorGrey16x15 retain];
 		}
@@ -195,8 +201,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_colorGrey16x15 = [NSColor grayColor];
 		[m_colorGrey16x15 retain];
 	}
-	if (path = [bundle pathForResource:@"Grey11x16" ofType:@"png"]) {
-		if (m_imageGrey11x16 = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Grey11x16" ofType:@"png"];
+	if (path) {
+		m_imageGrey11x16 = [[NSImage alloc] initWithContentsOfFile:path];
+		if (m_imageGrey11x16) {
 			m_colorGrey11x16 = [NSColor colorWithPatternImage:m_imageGrey11x16];
 			[m_colorGrey11x16 retain];
 		}
@@ -207,8 +215,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 	}
 
 	// Cursors
-	if (path = [bundle pathForResource:@"Cursor_E" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_E" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_E = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(7,7)];
 			[image release];
 		}
@@ -217,8 +227,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_E = [NSCursor arrowCursor];
 		[m_Cursor_E retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_N" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_N" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_N = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(7,8)];
 			[image release];
 		}
@@ -227,8 +239,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_N = [NSCursor arrowCursor];
 		[m_Cursor_N retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_NE" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_NE" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_NE = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(8,7)];
 			[image release];
 		}
@@ -237,8 +251,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_NE = [NSCursor arrowCursor];
 		[m_Cursor_NE retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_NW" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_NW" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_NW = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(7,7)];
 			[image release];
 		}
@@ -247,8 +263,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_NW = [NSCursor arrowCursor];
 		[m_Cursor_NW retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_S" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_S" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_S = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(7,7)];
 			[image release];
 		}
@@ -257,8 +275,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_S = [NSCursor arrowCursor];
 		[m_Cursor_S retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_SE" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_SE" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_SE = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(8,8)];
 			[image release];
 		}
@@ -267,8 +287,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_SE = [NSCursor arrowCursor];
 		[m_Cursor_SE retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_SW" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_SW" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_SW = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(7,8)];
 			[image release];
 		}
@@ -277,8 +299,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_SW = [NSCursor arrowCursor];
 		[m_Cursor_SW retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_W" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_W" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_W = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(8,7)];
 			[image release];
 		}
@@ -288,8 +312,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		[m_Cursor_W retain];
 	}
 
-	if (path = [bundle pathForResource:@"Cursor_Wait" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_Wait" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_Wait = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(8,7)];
 			[image release];
 		}
@@ -298,8 +324,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_Wait = [NSCursor arrowCursor];
 		[m_Cursor_Wait retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_LeftArrow" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_LeftArrow" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_LeftArrow = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(7,8)];
 			[image release];
 		}
@@ -308,8 +336,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_LeftArrow = [NSCursor arrowCursor];
 		[m_Cursor_LeftArrow retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_RightArrow" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_RightArrow" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_RightArrow = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(8,7)];
 			[image release];
 		}
@@ -318,8 +348,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_RightArrow = [NSCursor arrowCursor];
 		[m_Cursor_RightArrow retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_Compass" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_Compass" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_Compass = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(7,7)];
 			[image release];
 		}
@@ -328,8 +360,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_Compass = [NSCursor arrowCursor];
 		[m_Cursor_Compass retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_Exchange" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_Exchange" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_Exchange = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(7,7)];
 			[image release];
 		}
@@ -338,8 +372,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_Exchange = [NSCursor arrowCursor];
 		[m_Cursor_Exchange retain];
 	}
-	if (path = [bundle pathForResource:@"leftright_cursor" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"leftright_cursor" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_LeftRight = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(8,8)];
 			[image release];
 		}
@@ -348,8 +384,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_LeftRight = [NSCursor arrowCursor];
 		[m_Cursor_LeftRight retain];
 	}
-	if (path = [bundle pathForResource:@"updown_cursor" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"updown_cursor" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_UpDown = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(8,8)];
 			[image release];
 		}
@@ -358,8 +396,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_UpDown = [NSCursor arrowCursor];
 		[m_Cursor_UpDown retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_Crosshair" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_Crosshair" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_Crosshair = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(7,7)];
 			[image release];
 		}
@@ -368,8 +408,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_Crosshair = [NSCursor arrowCursor];
 		[m_Cursor_Crosshair retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_HandPointer" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_HandPointer" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_HandPointer = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(6,0)];
 			[image release];
 		}
@@ -378,8 +420,10 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_Cursor_HandPointer = [NSCursor arrowCursor];
 		[m_Cursor_HandPointer retain];
 	}
-	if (path = [bundle pathForResource:@"Cursor_DownArrow" ofType:@"png"]) {
-		if (image = [[NSImage alloc] initWithContentsOfFile:path]) {
+	path = [bundle pathForResource:@"Cursor_DownArrow" ofType:@"png"];
+	if (path) {
+		image = [[NSImage alloc] initWithContentsOfFile:path];
+		if (image) {
 			m_Cursor_DownArrow = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(6,0)];
 			[image release];
 		}
