@@ -109,6 +109,7 @@ void GR_Win32Graphics::_constructorCommonCode(HDC hdc)
 	m_bStartPage = false;
 	m_pFont = NULL;
 	m_pFontGUI = NULL;
+	m_bIsPreview = false;
 
 	m_cs = GR_Graphics::GR_COLORSPACE_COLOR;
 	m_cursor = GR_CURSOR_INVALID;
@@ -206,6 +207,7 @@ GR_Win32Graphics::GR_Win32Graphics(HDC hdc, const DOCINFO * pDocInfo, HGLOBAL hD
  	m_bPrint = true;
 	m_pDocInfo = pDocInfo;
 	m_hDevMode = hDevMode;
+	m_bIsPreview = (hDevMode == NULL);
 }
 
 GR_Win32Graphics::~GR_Win32Graphics()
@@ -216,7 +218,7 @@ GR_Win32Graphics::~GR_Win32Graphics()
 		
 	/* Release saved bitmaps */
 	HBITMAP hBit;
-	for (UT_uint32 i = 0; i < m_vSaveRectBuf.size (); i++)
+	for (UT_sint32 i = 0; i < m_vSaveRectBuf.size (); i++)
 	{
 		hBit = (HBITMAP)m_vSaveRectBuf.getNthItem (i);
 		DeleteObject(hBit);			
@@ -287,8 +289,8 @@ GR_Font* GR_Win32Graphics::getGUIFont(void)
 extern "C"
 int CALLBACK
 win32Internal_fontEnumProcedure(ENUMLOGFONT* pLogFont,
-								NEWTEXTMETRICEX* pTextMetric,
-								int Font_type,
+								NEWTEXTMETRICEX* /*pTextMetric*/,
+								int /*Font_type*/,
 								LPARAM lParam)
 {
 	LOGFONT *lf = (LOGFONT*)lParam;
@@ -298,17 +300,19 @@ win32Internal_fontEnumProcedure(ENUMLOGFONT* pLogFont,
 
 GR_Font* GR_Win32Graphics::_findFont(const char* pszFontFamily,
 									 const char* pszFontStyle,
-									 const char* pszFontVariant,
+									 const char* /*pszFontVariant*/,
 									 const char* pszFontWeight,
-									 const char* pszFontStretch,
+									 const char* /*pszFontStretch*/,
 									 const char* pszFontSize,
-									 const char* pszLang)
+									 const char* /*pszLang*/)
 {	
 	#ifdef GR_GRAPHICS_DEBUG
 	UT_DEBUGMSG(("GR_Win32Graphics::findFont %s %s %s\n", pszFontFamily, pszFontStyle, pszFontSize));	
 	#endif
 	
-	LOGFONT lf = { 0 };
+	LOGFONT lf;
+	memset(&lf, 0, sizeof(lf));
+
 	HDC hPrintDC = m_printHDC ? m_printHDC : m_hdc;
 
 	/*
@@ -360,7 +364,9 @@ GR_Font* GR_Win32Graphics::_findFont(const char* pszFontFamily,
 	}
 
 	// Get character set value from the font itself
-	LOGFONT enumlf = { 0 };
+	LOGFONT enumlf;
+	memset(&enumlf, 0, sizeof(enumlf));
+
 	enumlf.lfCharSet = DEFAULT_CHARSET;
 	strcpy(enumlf.lfFaceName, lf.lfFaceName);
 	EnumFontFamiliesEx(m_hdc, &enumlf,
@@ -372,7 +378,7 @@ GR_Font* GR_Win32Graphics::_findFont(const char* pszFontFamily,
 	return _newFont(lf, fPointSize, m_hdc, hPrintDC);
 }
 
-void GR_Win32Graphics::drawGlyph(UT_uint32 Char, UT_sint32 xoff, UT_sint32 yoff)
+void GR_Win32Graphics::drawGlyph(UT_uint32 /*Char*/, UT_sint32 /*xoff*/, UT_sint32 /*yoff*/)
 {
 	UT_ASSERT_HARMLESS(UT_TODO);
 }
@@ -708,7 +714,7 @@ void GR_Win32Graphics::getCoverage(UT_NumberVector& coverage)
 	//UT_ASSERT(UT_TODO);
 }
 
-UT_sint32 GR_Win32Graphics::measureUnRemappedChar(const UT_UCSChar c, UT_uint32 * height)
+UT_sint32 GR_Win32Graphics::measureUnRemappedChar(const UT_UCSChar c, UT_uint32 * /*height*/)
 {
 	#ifdef GR_GRAPHICS_DEBUG
 	UT_DEBUGMSG(("GR_Win32Graphics::measureUnRemappedChar\n"));	
@@ -939,12 +945,20 @@ bool GR_Win32Graphics::startPrint(void)
 {
 	UT_ASSERT(m_bPrint);
 	UT_ASSERT(!m_bStartPrint);
-	m_bStartPrint = ( StartDoc(m_hdc,m_pDocInfo) > 0 );
+
+	if (!m_bIsPreview)
+	  {
+	    m_bStartPrint = ( StartDoc(m_hdc,m_pDocInfo) > 0 );
+	  }
+	else
+	  {
+	    m_bStartPrint = true;
+	  }
 
 	return m_bStartPrint;
 }
 
-bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
+bool GR_Win32Graphics::startPage(const char * /*szPageLabel*/, UT_uint32 /*pageNumber*/,
 									bool bPortrait, UT_uint32 iWidth, UT_uint32 iHeight)
 {
 	// need these in 10th of milimiters
@@ -964,8 +978,8 @@ bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 		pDevMode->dmFields = DM_ORIENTATION | DM_PAPERLENGTH | DM_PAPERWIDTH;
 		pDevMode->dmOrientation = (bPortrait) ? DMORIENT_PORTRAIT : DMORIENT_LANDSCAPE;
 		pDevMode->dmPaperSize = 0;
-		pDevMode->dmPaperLength = iHeight;
-		pDevMode->dmPaperWidth = iWidth;
+		pDevMode->dmPaperLength = (bPortrait) ? iHeight : iWidth;
+		pDevMode->dmPaperWidth = (bPortrait) ? iWidth : iHeight;
 		
 		GlobalUnlock(m_hDevMode);
 
@@ -980,18 +994,22 @@ bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 
 	const int iRet = StartPage(m_hdc);
 
-	m_bStartPage = iRet > 0;
+	if (!m_bIsPreview) {
+	  m_bStartPage = iRet > 0;
 
-	if (iRet > 0)
-	{
-		// PHYSICALOFFSETX returns offset of printable area from the left edge
-		// of the physical printer paper. The value returned is in device units.
-		// Since the current mapping mode is MM_TEXT, this code _should_ work.
-		const POINT ptNew = {
-			-GetDeviceCaps(m_hdc, PHYSICALOFFSETX),
-			-GetDeviceCaps(m_hdc, PHYSICALOFFSETY)
-		};
-		SetViewportOrgEx(m_hdc, ptNew.x, ptNew.y, 0);
+	  if (m_bStartPage) {
+	      // PHYSICALOFFSETX returns offset of printable area from the left edge
+	      // of the physical printer paper. The value returned is in device units.
+	      // Since the current mapping mode is MM_TEXT, this code _should_ work.
+	      const POINT ptNew = {
+		-GetDeviceCaps(m_hdc, PHYSICALOFFSETX),
+		-GetDeviceCaps(m_hdc, PHYSICALOFFSETY)
+	      };
+	      SetViewportOrgEx(m_hdc, ptNew.x, ptNew.y, 0);
+	    }
+	}
+	else {
+	  m_bStartPage = true;
 	}
 
 	return m_bStartPage;
@@ -999,10 +1017,14 @@ bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 
 bool GR_Win32Graphics::endPrint(void)
 {
-	if (m_bStartPage)
+  if (m_bStartPage) {
 		EndPage(m_hdc);
+  }
 
-	return (EndDoc(m_hdc) > 0);
+  if (!m_bIsPreview)
+    return (EndDoc(m_hdc) > 0);
+  
+  return true;
 }
 
 /**
@@ -1231,7 +1253,7 @@ HWND GR_Win32Graphics::getHwnd(void) const
 	return m_hwnd;
 }
 
-void GR_Win32Graphics::setColorSpace(GR_Graphics::ColorSpace c)
+void GR_Win32Graphics::setColorSpace(GR_Graphics::ColorSpace /*c*/)
 {
 	// TODO:  maybe?
 	UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
@@ -1428,8 +1450,11 @@ void GR_Font::s_getGenericFontProperties(const char * szFontName,
 
 	// we borrow some code from GR_Win32Graphics::findFont()
 
-	LOGFONT lf = { 0 };
-	TEXTMETRIC tm = { 0 };
+	LOGFONT lf;
+	memset(&lf, 0, sizeof(lf));
+
+	TEXTMETRIC tm;
+	memset(&tm, 0, sizeof(tm));
 
 	// TODO i'm not sure why we special case these, but the other
 	// TODO code did, so i'm going to here.
@@ -1566,10 +1591,12 @@ GR_Win32Font::GR_Win32Font(LOGFONT & lf, double fPoints, HDC hdc, HDC printHDC)
 
 			_updateFontYMetrics(hdc, printHDC);
 
-			UT_String_sprintf(m_hashKey, "%s-%d-%d-%d-%d-%d",
+			UT_String_sprintf(m_hashKey, "%s-%d-%d-%d-%d-%d-%d-%d",
 							  lpFaceName,
 							  m_tm.tmHeight, m_tm.tmWeight, m_tm.tmItalic, m_tm.tmUnderlined,
-							  m_tm.tmStruckOut);
+							  m_tm.tmStruckOut,
+							  GetDeviceCaps(hdc, LOGPIXELSX),
+							  GetDeviceCaps(hdc, LOGPIXELSY));
 
 			// now we measure the default character
 			// 
@@ -1630,7 +1657,7 @@ GR_Win32Font::~GR_Win32Font()
 		bIsDC = dwObjType == OBJ_DC || dwObjType == OBJ_MEMDC;
 	}
 
-	for (UT_uint32 i = 0; i < m_allocFonts.getItemCount(); ++i)
+	for (UT_sint32 i = 0; i < m_allocFonts.getItemCount(); ++i)
 	  {
 		  allocFont *p = (allocFont *)m_allocFonts.getNthItem(i);
 
@@ -1829,7 +1856,9 @@ bool GR_Win32Font::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG)
 		// (whether the 9x version of the function is fully functional I do not know, but
 		// it does glyph index lookup).
 		UINT iIndx;
-		GCP_RESULTSW gcpResult = {0};
+		GCP_RESULTSW gcpResult;
+		memset(&gcpResult, 0, sizeof(gcpResult));
+
 		gcpResult.lStructSize = sizeof(GCP_RESULTS);
 		
 		// w32api changed lpGlyphs from UINT * to LPWSTR to match MS PSDK in w32api v2.4
@@ -1938,7 +1967,7 @@ HFONT GR_Win32Font::getDisplayFont(GR_Graphics * pG)
 	return getFontFromCache(pixels, false, zoom);
 }
 
-UT_sint32 GR_Win32Font::measureUnRemappedChar(UT_UCSChar c, UT_uint32 * height)
+UT_sint32 GR_Win32Font::measureUnRemappedChar(UT_UCSChar c, UT_uint32 * /*height*/)
 {
 #ifndef ABI_GRAPHICS_PLUGIN_NO_WIDTHS
 	// first of all, handle 0-width spaces ...
@@ -2002,7 +2031,7 @@ void GR_Win32Graphics::polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints)
 	delete[] points;
 }
 
-bool GR_Win32Graphics::_setTransform(const GR_Transform & tr)
+bool GR_Win32Graphics::_setTransform(const GR_Transform & /*tr*/)
 {
 #if 0
 	if(UT_IsWinNT())
@@ -2208,7 +2237,8 @@ BITMAPINFO * GR_Win32Graphics::ConvertDDBToDIB(HBITMAP bitmap, HPALETTE hPal, DW
 	
 	// Realloc the buffer so that it can hold all the bits	
 	dwLen += bi.biSizeImage;
-	if (handle = g_try_realloc(hDIB, dwLen))		
+	handle = g_try_realloc(hDIB, dwLen);
+	if (handle)
 		hDIB = handle;	
 	else{
 		g_free(hDIB);		
@@ -2469,3 +2499,63 @@ bool GR_Win32Graphics::fixDevMode(HGLOBAL hDevMode)
 	return true;
 }
 
+DOCINFO *GR_Win32Graphics::getDocInfo()
+{
+  DOCINFO *di;
+  
+  di = (DOCINFO*) UT_calloc(1, sizeof(DOCINFO));
+  memset( di, 0, sizeof(DOCINFO) );
+  di->cbSize = sizeof(DOCINFO);
+  di->lpszDocName = (LPCTSTR)TEXT("AbiWord Print Preview");
+  di->lpszOutput = (LPTSTR) NULL;
+  di->lpszDatatype = (LPTSTR) NULL;
+  di->fwType = 0;
+  
+  return di;
+}
+
+/*
+ *  Creates a HDC to the best printer to be used as a reference for CreateEnhMetaFile
+ */
+HDC GR_Win32Graphics::createbestmetafilehdc()
+{
+  DWORD neededsize;
+  DWORD noprinters;
+  LPPRINTER_INFO_5 printerinfo = NULL;
+  int bestres = 0;
+  HDC besthdc = 0;
+  
+  EnumPrinters(PRINTER_ENUM_CONNECTIONS|PRINTER_ENUM_LOCAL, NULL, 5, NULL, 0,
+	       &neededsize, &noprinters);
+  printerinfo = (LPPRINTER_INFO_5) malloc(neededsize);
+  
+  if (EnumPrinters(PRINTER_ENUM_CONNECTIONS|PRINTER_ENUM_LOCAL, NULL, 5,
+		   (LPBYTE)printerinfo, neededsize, &neededsize, &noprinters)) {
+    // init best resolution for hdc=0, which is screen resolution:    
+    HDC curhdc = GetDC(NULL);
+    if (curhdc) {
+      bestres = GetDeviceCaps(curhdc, LOGPIXELSX) + GetDeviceCaps(curhdc,
+								  LOGPIXELSY);
+      bestres = ReleaseDC(NULL, curhdc); 
+    }
+      
+    for (UT_uint32 i = 0; i < noprinters; i++) {
+      curhdc = CreateDC("WINSPOOL", printerinfo[i].pPrinterName, NULL, NULL);
+      if (curhdc) {
+	int curres = GetDeviceCaps(curhdc, LOGPIXELSX) + GetDeviceCaps(curhdc,
+								       LOGPIXELSY);
+	if (curres > bestres) {
+	  if (besthdc)
+	    DeleteDC(besthdc);
+	  bestres = curres;
+	  besthdc = curhdc;
+	} else {
+	  DeleteDC(curhdc);
+	}
+      }
+    }
+  }
+  
+  free(printerinfo);
+  return besthdc;
+}
