@@ -19,6 +19,10 @@
  * 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <ctype.h>
 #include <math.h>
 #include <string.h>
@@ -99,7 +103,7 @@ UT_sint32 GR_Font::getCharWidthFromCache (UT_UCSChar c) const
 #endif
 }
 
-bool GR_Font::doesGlyphExist(UT_UCS4Char g)
+bool GR_Font::doesGlyphExist(UT_UCS4Char g) const
 {
 	if(getCharWidthFromCache(g) == GR_CW_ABSENT)
 	{
@@ -123,6 +127,95 @@ GR_CharWidths* GR_Font::newFontWidths(void) const
 #endif
 }
 
+AllCarets::AllCarets(GR_Graphics * pG,
+					 GR_Caret ** pCaret,
+					 UT_GenericVector<GR_Caret *>* vecCarets  ):
+	m_pG(pG),
+	m_pLocalCaret(pCaret),
+	m_vecCarets(vecCarets)
+{
+}
+GR_Caret *  AllCarets::getBaseCaret(void)
+{
+	return *m_pLocalCaret;
+}
+
+void	    AllCarets::enable(void)
+{
+	if(*m_pLocalCaret)
+		(*m_pLocalCaret)->enable();
+	for(UT_sint32 i =0; i< m_vecCarets->getItemCount();i++)
+	{
+		m_vecCarets->getNthItem(i)->enable();
+	}
+			
+}
+void		AllCarets::disable(bool bNoMulti)
+{
+	if((*m_pLocalCaret))
+		(*m_pLocalCaret)->disable(bNoMulti);
+	for(UT_sint32 i =0; i< m_vecCarets->getItemCount();i++)
+	{
+		m_vecCarets->getNthItem(i)->disable(bNoMulti);
+	}
+}
+
+void		AllCarets::setBlink(bool bBlink)
+{
+	if((*m_pLocalCaret))
+		(*m_pLocalCaret)->setBlink(bBlink);
+	for(UT_sint32 i =0; i< m_vecCarets->getItemCount();i++)
+	{
+		m_vecCarets->getNthItem(i)->setBlink(bBlink);;
+	}
+}
+
+void        AllCarets::setWindowSize(UT_uint32 width, UT_uint32 height)
+{
+	if((*m_pLocalCaret))
+		(*m_pLocalCaret)->setWindowSize(width, height);
+	for(UT_sint32 i =0; i< m_vecCarets->getItemCount();i++)
+	{
+		m_vecCarets->getNthItem(i)->setWindowSize(width, height);
+	}
+}
+
+void		AllCarets::setCoords(UT_sint32 x, UT_sint32 y, UT_uint32 h,
+						  UT_sint32 x2, UT_sint32 y2, UT_uint32 h2, 
+						  bool bPointDirection, 
+						  const UT_RGBColor * pClr)
+{
+	if((*m_pLocalCaret))
+		(*m_pLocalCaret)->setCoords(x, y, h, x2, y2, h2, bPointDirection, pClr);
+	for(UT_sint32 i =0; i< m_vecCarets->getItemCount();i++)
+	{
+		m_vecCarets->getNthItem(i)->setCoords(x, y, h, x2, y2, h2, bPointDirection, pClr);
+	}
+}
+
+void		AllCarets::setInsertMode (bool mode)
+{
+	if((*m_pLocalCaret))
+		(*m_pLocalCaret)->setInsertMode(mode);
+	for(UT_sint32 i =0; i< m_vecCarets->getItemCount();i++)
+	{
+		m_vecCarets->getNthItem(i)->setInsertMode(mode);
+	}
+}
+
+void		AllCarets::forceDraw(void)
+{
+	if((*m_pLocalCaret))
+		(*m_pLocalCaret)->forceDraw();
+	for(UT_sint32 i =0; i< m_vecCarets->getItemCount();i++)
+	{
+		m_vecCarets->getNthItem(i)->forceDraw();
+	}
+}
+
+
+
+
 GR_Graphics::GR_Graphics()
 	: m_iZoomPercentage(100),
 	  m_iFontAllocNo(0),
@@ -138,7 +231,8 @@ GR_Graphics::GR_Graphics()
 	  m_iPrevYOffset(0),
 	  m_iPrevXOffset(0),
 	  m_hashFontCache(19),
-	  m_paintCount(0)
+	  m_paintCount(0),
+	  m_AllCarets(this,&m_pCaret,&m_vecCarets)
 {
 }
 
@@ -183,13 +277,13 @@ GR_Font* GR_Graphics::findFont(const char* pszFontFamily,
 
 GR_Graphics::~GR_Graphics()
 {
-        xxx_UT_DEBUGMSG(("Deleting graphics class %x \n",this));
+	xxx_UT_DEBUGMSG(("Deleting graphics class %x \n",this));
 	DELETEP(m_pCaret);
 	UT_sint32 i = 0;
-	for(i=0; i< static_cast<UT_sint32>(m_vecCarets.getItemCount());i++)
+	for(i=0; i< m_vecCarets.getItemCount();i++)
 	{
-	    GR_Caret * pCaret = m_vecCarets.getNthItem(i);
-	    DELETEP(pCaret);
+		GR_Caret * pCaret = m_vecCarets.getNthItem(i);
+		DELETEP(pCaret);
 	}
 }
 
@@ -199,33 +293,51 @@ void GR_Graphics::_destroyFonts ()
 	m_hashFontCache.clear ();
 }
 
-GR_Caret * GR_Graphics::getNthCaret(UT_sint32 i)
+GR_Caret * GR_Graphics::getNthCaret(UT_sint32 i) const
 {
-        if(i>= static_cast<UT_sint32>(m_vecCarets.getItemCount()))
-	     return NULL;
-        return m_vecCarets.getNthItem(i);
+	if (i>= m_vecCarets.getItemCount())
+		return NULL;
+	return m_vecCarets.getNthItem(i);
 }
 
-GR_Caret * GR_Graphics::getCaret(UT_UTF8String & sDocUUID)
+GR_Caret * GR_Graphics::getCaret(const std::string& sID) const
 {
-        UT_sint32 i= 0;
+	UT_sint32 i= 0;
 	for(i=0; i<m_vecCarets.getItemCount();i++)
 	{
-	    if(m_vecCarets.getNthItem(i)->getUUID() == sDocUUID)
-	    {
-		return m_vecCarets.getNthItem(i);
-	    }
+		if(m_vecCarets.getNthItem(i)->getID() == sID)
+		{
+			return m_vecCarets.getNthItem(i);
+		}
 	}
 	return NULL;
 }
 
-GR_Caret * GR_Graphics::createCaret(UT_UTF8String & sDocUUID)
+AllCarets * GR_Graphics::allCarets(void)
 {
-        GR_Caret * pCaret = new GR_Caret(this,sDocUUID);
-        m_vecCarets.addItem(pCaret);
+	return &m_AllCarets;
+}
+
+GR_Caret * GR_Graphics::createCaret(const std::string& sID)
+{
+	GR_Caret * pCaret = new GR_Caret(this,sID);
+	m_vecCarets.addItem(pCaret);
 	return pCaret;
 }
 
+void GR_Graphics::removeCaret(const std::string& sID)
+{
+	for(UT_sint32 i = 0; i < m_vecCarets.getItemCount(); i++)
+	{
+		GR_Caret* pC = m_vecCarets.getNthItem(i);
+		if (pC->getID() == sID)
+		{			
+			DELETEP(pC);
+			m_vecCarets.deleteNthItem(i);
+		}
+	}
+}
+
 /*!
  * WARNING! WARNING! WARNING!
  * Only gr_UnixGraphics should call this!
@@ -302,7 +414,7 @@ UT_sint32 GR_Graphics::_tduR(UT_sint32 layoutUnits) const
 
 UT_sint32 GR_Graphics::tlu(UT_sint32 deviceUnits) const
 {
-	return static_cast<UT_sint32>((static_cast<double>(deviceUnits) * static_cast<double>(getResolution()) * 100.) / (static_cast<double>(getDeviceResolution()) * static_cast<double>(getZoomPercentage())));;
+	return static_cast<UT_sint32>((static_cast<double>(deviceUnits) * static_cast<double>(getResolution()) * 100.) / (static_cast<double>(getDeviceResolution()) * static_cast<double>(getZoomPercentage())));
 }
 
 double GR_Graphics::tduD(double layoutUnits) const
@@ -326,35 +438,41 @@ double	GR_Graphics::ftluD(double fontUnits) const
 	return (fontUnits * static_cast<double>(getResolution()) / static_cast<double>(getDeviceResolution()));
 }
 
-void GR_Graphics::setLineProperties ( double    inWidthPixels, 
-				      JoinStyle inJoinStyle,
-				      CapStyle  inCapStyle,
-				      LineStyle inLineStyle )
+void GR_Graphics::setLineProperties ( double    /*inWidthPixels*/, 
+									  JoinStyle /*inJoinStyle*/,
+									  CapStyle  /*inCapStyle*/,
+									  LineStyle /*inLineStyle*/ )
 {
   UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 }
 
-UT_uint32 GR_Graphics::getMaxCharacterWidth(const UT_UCSChar*s, UT_uint32 Length)
+void GR_Graphics::getMaxCharacterDimension(const UT_UCSChar*s, UT_uint32 length, UT_uint32 &width, UT_uint32 &height)
 {
-	UT_GrowBufElement *pWidths = new UT_GrowBufElement[Length];
+	UT_GrowBufElement *pWidths = new UT_GrowBufElement[length];
 
-	measureString(s, 0, Length, pWidths);
 
-	UT_sint32 MaxWidth = 0;
+	UT_uint32 maxHeight = 0;
+	measureString(s, 0, length, pWidths, &maxHeight);
 
-	for(UT_uint32 i = 0; i < Length; i++)
+	UT_sint32 maxWidth = 0;
+
+	for(UT_uint32 i = 0; i < length; i++)
 	{
-		if(pWidths[i] > MaxWidth)
-			MaxWidth = pWidths[i];
+		if(pWidths[i] > maxWidth)
+			maxWidth = pWidths[i];
 	}
 
 	DELETEPV(pWidths);
 
-	return MaxWidth;
+	width = maxWidth;
+	if (maxHeight > 0) { 
+		height = maxHeight;
+	}
 }
 
 UT_uint32 GR_Graphics::measureString(const UT_UCSChar* s, int iOffset,
-										 int num,  UT_GrowBufElement* pWidths)
+										 int num,  UT_GrowBufElement* pWidths, 
+									 UT_uint32 * /*height*/)
 {
 	// Generic base class version defined in terms of measureUnRemappedChar().
 	// Platform versions can roll their own if it makes a performance difference.
@@ -540,7 +658,7 @@ void GR_Graphics::polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints)
  * handled.
  * Returns true if the exposed redraw method is running.
  */
-const bool  GR_Graphics::isSpawnedRedraw(void) const
+bool GR_Graphics::isSpawnedRedraw(void) const
 {
 	return m_bSpawnedRedraw;
 }
@@ -548,7 +666,7 @@ const bool  GR_Graphics::isSpawnedRedraw(void) const
 /*!
  * sets/clears the redraw running variable
  */
-void  GR_Graphics::setSpawnedRedraw( bool exposeState)
+void GR_Graphics::setSpawnedRedraw( bool exposeState)
 {
 	m_bSpawnedRedraw = exposeState;
 }
@@ -557,7 +675,7 @@ void  GR_Graphics::setSpawnedRedraw( bool exposeState)
  * Informs if there are unprocessed expose information present
  * Returns true is so.
  */
-const bool GR_Graphics::isExposePending(void) const
+bool GR_Graphics::isExposePending(void) const
 {
 	return m_bExposePending;
 }
@@ -588,7 +706,7 @@ void GR_Graphics::setRecentRect(UT_Rect * pRect)
  * Informs if a process is accessing the global merged expose area.
  * Returns true is so.
  */
-const bool GR_Graphics::isExposedAreaAccessed(void) const
+bool GR_Graphics::isExposedAreaAccessed(void) const
 {
 	return m_bIsExposedAreaAccessed;
 }
@@ -905,12 +1023,12 @@ bool GR_Graphics::itemize(UT_TextIterator & text, GR_Itemization & I)
 					
 					while(text.getStatus() == UTIter_OK)
 					{
-						UT_UCS4Char c = text.getChar();
+						UT_UCS4Char c2 = text.getChar();
 						UT_return_val_if_fail(text.getStatus() == UTIter_OK, false);
 
 						++text;
 						
-						iNextType = UT_bidiGetCharType(c);
+						iNextType = UT_bidiGetCharType(c2);
 						xxx_UT_DEBUGMSG(("GR_Graphics::itemize: iNextType 0x%04x\n", iNextType));
 						
 						if(iNextType == iPrevType)
@@ -1068,7 +1186,7 @@ bool GR_Graphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& pri)
 
 	GR_XPRenderInfo * pRI = (GR_XPRenderInfo *)pri;
 
-	GR_Font *pFont = const_cast<GR_Font*>(si.m_pFont);
+	const GR_Font *pFont = si.m_pFont;
 	
 	// make sure that the buffers are of sufficient size ...
 	if(si.m_iLength > pRI->m_iBufferSize) //buffer too small, reallocate
@@ -1092,12 +1210,25 @@ bool GR_Graphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& pri)
 
 	UT_UCS4Char glyph, current;
 	UT_UCS4Char * dst_ptr = pRI->m_pChars;
-	
+	bool previousWasSpace = si.m_previousWasSpace;
+
 	for(UT_sint32 i = 0; i < si.m_iLength; ++i, ++si.m_Text)
 	{
 		UT_return_val_if_fail(si.m_Text.getStatus() == UTIter_OK, false);
 		current = si.m_Text.getChar();
 
+		if (si.m_TextTransform == GR_ShapingInfo::LOWERCASE)
+			current = g_unichar_tolower(current);
+		else if (si.m_TextTransform == GR_ShapingInfo::UPPERCASE)
+			current = g_unichar_toupper(current);
+		else if (si.m_TextTransform == GR_ShapingInfo::CAPITALIZE) {
+				if (previousWasSpace) {
+					current = g_unichar_toupper(current);
+				}
+		} // else si.m_TextTransform == GR_ShapingInfo::NONE
+
+		previousWasSpace = g_unichar_isspace(current);
+
 		if(si.m_iVisDir == UT_BIDI_RTL)
 			glyph = s_getMirrorChar(current);
 		else
@@ -1476,7 +1607,8 @@ void GR_Graphics::justify(GR_RenderInfo & ri)
 	}
 }
 
-UT_uint32 GR_Graphics::XYToPosition(const GR_RenderInfo & ri, UT_sint32 x, UT_sint32 y) const
+UT_uint32 GR_Graphics::XYToPosition(const GR_RenderInfo & ri, UT_sint32 /*x*/, 
+									UT_sint32 /*y*/) const
 {
 	UT_return_val_if_fail(ri.getType() == GRRI_XP, 0);
 	GR_XPRenderInfo & RI = (GR_XPRenderInfo &) ri;
@@ -1487,9 +1619,9 @@ UT_uint32 GR_Graphics::XYToPosition(const GR_RenderInfo & ri, UT_sint32 x, UT_si
 }
 
 void GR_Graphics::positionToXY(const GR_RenderInfo & ri,
-						 	  UT_sint32& x, UT_sint32& y,
-							  UT_sint32& x2, UT_sint32& y2,
-							  UT_sint32& height, bool& bDirection) const
+							   UT_sint32& /*x*/, UT_sint32& /*y*/,
+							   UT_sint32& /*x2*/, UT_sint32& /*y2*/,
+							   UT_sint32& /*height*/, bool& /*bDirection*/) const
 {
 	UT_return_if_fail(ri.getType() == GRRI_XP);
 	GR_XPRenderInfo & RI = (GR_XPRenderInfo &) ri;
@@ -1498,12 +1630,12 @@ void GR_Graphics::positionToXY(const GR_RenderInfo & ri,
 	UT_return_if_fail(UT_NOT_IMPLEMENTED);
 }
 
-UT_uint32 GR_Graphics::adjustCaretPosition(GR_RenderInfo & ri, bool bForward)
+UT_uint32 GR_Graphics::adjustCaretPosition(GR_RenderInfo & ri, bool /*bForward*/)
 {
 	return ri.m_iOffset;
 }
 
-void GR_Graphics::adjustDeletePosition(GR_RenderInfo & ri)
+void GR_Graphics::adjustDeletePosition(GR_RenderInfo & )
 {
 	return;
 }
@@ -1682,3 +1814,31 @@ bool GR_GraphicsFactory::isRegistered(UT_uint32 iClassId) const
 
 	return true;
 }
+
+#if defined(TOOLKIT_GTK)
+#include "gr_UnixNullGraphics.h"
+#elif defined(TOOLKIT_WIN)
+#include "gr_Win32Graphics.h"
+#elif defined(TOOLKIT_COCOA)
+#warning implement offscreen
+#endif
+
+/**
+ * Creates an offscreen graphics context. Only used for measuring font metrics and whatnot, not actually for drawing
+ */
+/* static */
+GR_Graphics* GR_Graphics::newNullGraphics()
+{
+	// todo: support other platforms when possible
+	
+#if defined(TOOLKIT_GTK)
+	GR_UnixNullGraphicsAllocInfo ai;
+	return XAP_App::getApp()->newGraphics(GRID_UNIX_NULL, (GR_AllocInfo&)ai);
+#elif defined(TOOLKIT_WIN)
+	GR_Win32AllocInfo ai (GR_Win32Graphics::createbestmetafilehdc(), GR_Win32Graphics::getDocInfo(), NULL);
+	return XAP_App::getApp()->newGraphics(GRID_WIN32, (GR_AllocInfo&)ai);
+#elif defined(TOOLKIT_COCOA)
+#endif
+	
+	return NULL;
+}
