@@ -188,9 +188,7 @@ enum GR_GraphicsId
 
 	/* IDs for built-in classes: DO NOT CHANGE THE ASSIGNED VALUES !!!*/
 	/* (these classes cannot be unregistered) */
-	GRID_BEOS            =  0x101, // deprecated
 	GRID_COCOA           =  0x102,
-	GRID_QNX             =  0x103,
 	GRID_WIN32           =  0x104,
 	GRID_UNIX            =  0x105,
 	GRID_UNIX_PS         =  0x106,
@@ -208,7 +206,6 @@ enum GR_GraphicsId
 	GRID_UNIX_PANGO_PRINT = 0x203,
 	GRID_UNIX_PANGO_PIXMAP = 0x204,
 
-
 	GRID_LAST_EXTENSION = 0x0000ffff,
 
 	/* id's for plugins will be auto-generatoed from between here */
@@ -268,7 +265,7 @@ enum GR_Capability
 	Platform implementation needs to override getType() so that
 	graphicsAllocator() can do type-checking.
 */
-class GR_AllocInfo
+class ABI_EXPORT GR_AllocInfo
 {
   public:
 	virtual ~GR_AllocInfo() {}
@@ -288,7 +285,7 @@ typedef const char *  (*GR_Descriptor)(void);
    use. The factory provides access to all graphics classes known to
    the application.
 */
-class GR_GraphicsFactory
+class ABI_EXPORT GR_GraphicsFactory
 {
 
   public:
@@ -346,19 +343,46 @@ enum GRShapingResult
 #define GR_OC_LEFT_FLUSHED 0x40000000 // flip bit 31
 #define GR_OC_MAX_WIDTH    0x3fffffff
 
+class ABI_EXPORT AllCarets
+{
+	friend class GR_Graphics;
+ public:
+	AllCarets(GR_Graphics * pG,
+			  GR_Caret ** pCaret,
+			  UT_GenericVector<GR_Caret *>* vecCarets  );
+	virtual ~AllCarets(){}
+	GR_Caret *  getBaseCaret(void);
+	void	    enable(void);
+	void		disable(bool bNoMulti = false);
+	void		setBlink(bool bBlink);
+	void        setWindowSize(UT_uint32 width, UT_uint32 height);
+	void		setCoords(UT_sint32 x, UT_sint32 y, UT_uint32 h,
+						  UT_sint32 x2 = 0, UT_sint32 y2 = 0, UT_uint32 h2 = 0, 
+						  bool bPointDirection = false, 
+						  const UT_RGBColor * pClr = NULL);
+	void		setInsertMode (bool mode);
+	void		forceDraw(void);
+
+ private:
+	GR_Graphics * m_pG;
+	GR_Caret **    m_pLocalCaret;
+	UT_GenericVector<GR_Caret *>* m_vecCarets;
+};
+
+
 class ABI_EXPORT GR_Graphics
 {
 	friend class GR_Painter;
 	friend class GR_Caret;
-
+	friend class AllCarets;
  public:
 	virtual ~GR_Graphics();
 
 	// the static method allows us to retrive the the class id for
 	// purposes of registration; we also need the virtual to identify
 	// the class from a generic GR_Graphics pointer
-	static UT_uint32 s_getClassId() {UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED); return GRID_UNKNOWN;}
-	virtual UT_uint32 getClassId() {return s_getClassId();}
+//	static UT_uint32 s_getClassId() {UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED); return GRID_UNKNOWN;}
+	virtual UT_uint32 getClassId() = 0;
 	
 	virtual GR_Capability getCapability() {UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED); return GRCAP_UNKNOWN;}
 #if 0
@@ -367,6 +391,8 @@ class ABI_EXPORT GR_Graphics
 	static const char *    graphicsDescriptor(void){UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED); return "???";}
 	static GR_Graphics *   graphicsAllocator(GR_AllocInfo&){UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED); return NULL;}
 #endif
+
+	AllCarets *   allCarets(void);
 	
 	UT_sint32	tdu(UT_sint32 layoutUnits) const;
 	UT_sint32	tlu(UT_sint32 deviceUnits) const;
@@ -385,7 +411,7 @@ class ABI_EXPORT GR_Graphics
 	virtual UT_uint32 getFontDescent() = 0;
 	virtual UT_uint32 getFontHeight() = 0;
 	void              invalidateCache(void);
-	virtual bool      canQuickPrint(void) { return false;}
+	virtual bool      canQuickPrint(void) const { return false;}
 	virtual UT_uint32 measureString(const UT_UCSChar*s,
 									int iOffset,
 									int num,
@@ -398,8 +424,7 @@ class ABI_EXPORT GR_Graphics
 	virtual UT_uint32 getFontAscent(const GR_Font *)  = 0;
 	virtual UT_uint32 getFontDescent(const GR_Font *) = 0;
 	virtual UT_uint32 getFontHeight(const GR_Font *)  = 0;
-	virtual double    getResolutionRatio(void)
-	{return 1.0;}
+	virtual double    getResolutionRatio(void) const {return 1.0;}
 
 	void         getMaxCharacterDimension (const UT_UCSChar*s, UT_uint32 Length, UT_uint32 &width, UT_uint32 &height);
 
@@ -429,12 +454,12 @@ class ABI_EXPORT GR_Graphics
 									   const char* pszLang);
 #else
 	static const char* findNearestFont(const char* pszFontFamily,
-									   const char* pszFontStyle,
-									   const char* pszFontVariant,
-									   const char* pszFontWeight,
-									   const char* pszFontStretch,
-									   const char* pszFontSize,
-									   const char* pszLang)
+									   const char* /*pszFontStyle*/,
+									   const char* /*pszFontVariant*/,
+									   const char* /*pszFontWeight*/,
+									   const char* /*pszFontStretch*/,
+									   const char* /*pszFontSize*/,
+									   const char* /*pszLang*/)
 		{return pszFontFamily;}
 #endif
 	
@@ -566,13 +591,14 @@ class ABI_EXPORT GR_Graphics
 #define COUNT_3D_COLORS 5
 
 	virtual void      setColor3D(GR_Color3D c) = 0;
-	virtual bool      getColor3D(GR_Color3D name, UT_RGBColor &color) { return false; }
+	virtual bool      getColor3D(GR_Color3D /*name*/, UT_RGBColor & /*color*/) 
+	{ return false; }
 
 	//
 	// Methods to deal with background repainting as used in the Unix FE. These
 	// make redraws really fast and fix bug 119
 	//
-	const bool        isSpawnedRedraw(void) const;
+	bool              isSpawnedRedraw(void) const;
 	void              setSpawnedRedraw(bool exposeState);
 
 	void              setPendingRect(UT_sint32 x,
@@ -583,9 +609,9 @@ class ABI_EXPORT GR_Graphics
 	void              unionPendingRect( UT_Rect * pRect);
 	void              setRecentRect( UT_Rect * pRect);
 	const UT_Rect *   getPendingRect(void) const;
-	const bool        isExposePending(void) const;
+	bool              isExposePending(void) const;
 	void              setExposePending(bool bExposePending);
-	const bool        isExposedAreaAccessed(void) const;
+	bool              isExposedAreaAccessed(void) const;
 	void              setExposedAreaAccessed(bool bAccessedState);
 	void              setDontRedraw( bool bDontRedraw);
 	bool              isDontRedraw(void);
@@ -611,11 +637,11 @@ class ABI_EXPORT GR_Graphics
 			m_pCaret = new GR_Caret(this);
 		}
 
-	GR_Caret *        getCaret() { return m_pCaret; }
-	GR_Caret *        createCaret(UT_UTF8String & sDocUUID);
-	GR_Caret *        getCaret(UT_UTF8String & sDocUUID);
-	GR_Caret *        getNthCaret(UT_sint32 i);
-	
+	GR_Caret *        createCaret(const std::string& sID);
+	GR_Caret *        getCaret(const std::string& sID) const;
+	GR_Caret *        getNthCaret(UT_sint32 i) const;
+	void              removeCaret(const std::string& sID);
+
 	virtual void	  saveRectangle(UT_Rect & r, UT_uint32 iIndx) = 0;
 	virtual void	  restoreRectangle(UT_uint32 iIndx) = 0;
 	virtual UT_uint32 getDeviceResolution(void) const = 0;
@@ -668,7 +694,7 @@ class ABI_EXPORT GR_Graphics
 
 	// indicates if special caret positioning has to be done for the run of text; this allows us
 	// to speed things up when this is not needed
-	virtual bool needsSpecialCaretPositioning(GR_RenderInfo & ri) VIRTUAL_SFX {return false;}
+	virtual bool needsSpecialCaretPositioning(GR_RenderInfo & /*ri*/) VIRTUAL_SFX {return false;}
 	
 	// adjusts caret position if given script restricts where caret can be placed
 	// the caller has to set initial position within the run in ri.m_iOffset, overall length of
@@ -712,10 +738,13 @@ class ABI_EXPORT GR_Graphics
 	virtual const UT_VersionInfo & getVersion() const {UT_ASSERT_HARMLESS( UT_NOT_IMPLEMENTED ); return s_Version;}
 	UT_uint32         getPaintCount(void) const
 		{ return  m_paintCount;}
+
+	static GR_Graphics* newNullGraphics();
 	
  protected:
 
 	GR_Graphics();
+	GR_Caret *        getCaret() { return m_pCaret; }
 
 	// todo: make these pure virtual
 	virtual void _beginPaint () {}
@@ -776,9 +805,8 @@ class ABI_EXPORT GR_Graphics
 											 int* pCharWidths = NULL);
 	
 	virtual GR_Image *	  genImageFromRectangle(const UT_Rect & r) = 0;
-
  private:
-	virtual bool _setTransform(const GR_Transform & tr)
+	virtual bool _setTransform(const GR_Transform & /*tr*/)
 		{
 			UT_ASSERT_HARMLESS( UT_NOT_IMPLEMENTED );
 			return false;
@@ -839,6 +867,7 @@ class ABI_EXPORT GR_Graphics
 	static UT_uint32        s_iInstanceCount;
 	static UT_UCS4Char      s_cDefaultGlyph;
 	UT_GenericVector<GR_Caret *>  m_vecCarets;
+	AllCarets               m_AllCarets;
 };
 
 #endif /* GR_GRAPHICS_H */
