@@ -2,6 +2,8 @@
 /*
  * go-file.c :
  *
+ * Copyright (C) 2009 Hubert Figuiere <hub@figuiere.net>. 
+ *     Whose contributions are under GPLv2+
  * Copyright (C) 2004 Morten Welinder (terra@gnome.org)
  * Copyright (C) 2003, Red Hat, Inc.
  *
@@ -29,7 +31,6 @@
 #include <goffice/goffice-config.h>
 #include <glib/gi18n-lib.h>
 #endif
-
 #include "ut_go_file.h"
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-input.h>
@@ -45,18 +46,26 @@
 #ifdef WITH_GSF_INPUT_HTTP
 #include <gsf/gsf-input-http.h>
 #endif
+#include <stdio.h>
 
-#ifdef WITH_GNOMEVFS
-#define GOFFICE_WITH_GNOME
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include <gsf-gnome/gsf-input-gnomevfs.h>
-#include <gsf-gnome/gsf-output-gnomevfs.h>
+#if HAVE_GSF_GIO
+#  include <gsf/gsf-input-gio.h>
+#  include <gsf/gsf-output-gio.h>
+#elif defined(WITH_GNOMEVFS)
+#  define GOFFICE_WITH_GNOME
+#  include <libgnomevfs/gnome-vfs-mime-utils.h>
+#  include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#  include <gsf-gnome/gsf-input-gnomevfs.h>
+#  include <gsf-gnome/gsf-output-gnomevfs.h>
 #endif
 
-#ifdef WITH_GNOMEUI
-#include <libgnome/gnome-url.h>
+#ifdef HAVE_GTK214
+// needed for gtk_show_uri()
+#include <gdk/gdk.h>
+#include <gtk/gtk.h>
+#elif defined(WITH_GNOMEVFS)
+// needed for gnome_vfs_url_show()
+#include <libgnomevfs/gnome-vfs-utils.h>
 #endif
 
 #if defined G_OS_WIN32
@@ -68,9 +77,18 @@
 
 #include <string.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <time.h>
 
+#ifdef WIN32
+  #ifndef S_ISDIR
+  #define S_ISDIR(m) (((m) & _S_IFMT ) == _S_IFDIR)
+  #endif
+#else
+#include <unistd.h>
+#endif
+
+#include "ut_types.h"
+
 #ifndef _
 #define _(X) X
 #endif
@@ -335,7 +353,7 @@ UT_go_path_is_uri (const char * path)
 char *
 UT_go_filename_from_uri (const char *uri)
 {
-#ifdef GOFFICE_WITH_GNOME
+#if defined(GOFFICE_WITH_GNOME)
 	return gnome_vfs_get_local_path_from_uri (uri);
 #else
 	return g_filename_from_uri (uri, NULL, NULL);
@@ -354,7 +372,7 @@ UT_go_filename_to_uri (const char *filename)
 
 	simp = UT_go_filename_simplify (filename, UT_GO_DOTDOT_TEST, TRUE);
 
-#ifdef GOFFICE_WITH_GNOME
+#if defined(GOFFICE_WITH_GNOME)
 	uri = gnome_vfs_get_uri_from_local_path (simp);
 #else
 	uri = g_filename_to_uri (simp, NULL, NULL);
@@ -535,7 +553,7 @@ UT_go_url_simplify (const char *uri)
 
 	if (g_ascii_strncasecmp (uri, "file:///", 8) == 0) {
 		char *filename = UT_go_filename_from_uri (uri);
-		char *simp = filename ? UT_go_filename_to_uri (filename) : NULL;
+		simp = filename ? UT_go_filename_to_uri (filename) : NULL;
 		g_free (filename);
 		return simp;
 	}
@@ -575,7 +593,8 @@ is_uri_relative (const char *uri)
 			      || ('-' == *current)
 			      || ('+' == *current)
 			      || ('.' == *current)) ;
-             current++);
+             current++) {
+	}
 
         return  !(':' == *current);
 }
@@ -626,7 +645,8 @@ remove_internal_relative_components (char *uri_current)
 						} else if (segment_prev - uri_current >= 2) {
 							segment_prev -= 2;
 							for ( ; segment_prev > uri_current && segment_prev[0] != '/' 
-							      ; segment_prev-- );
+							      ; segment_prev-- ) {
+							}
 							if (segment_prev[0] == '/') {
 								segment_prev++;
 							}
@@ -792,7 +812,7 @@ UT_go_url_resolve_relative (const char *ref_uri, const char *rel_uri)
 {
 	char *simp, *uri;
 
-#ifdef GOFFICE_WITH_GNOME
+#if defined(GOFFICE_WITH_GNOME)
 	uri = gnome_vfs_uri_make_full_from_relative (ref_uri, rel_uri);
 #else
 	g_return_val_if_fail (ref_uri != NULL, NULL);
@@ -916,7 +936,18 @@ UT_go_shell_arg_to_uri (const char *arg)
 		return res;
 	}
 
-#ifdef GOFFICE_WITH_GNOME
+#if HAVE_GSF_GIO
+	{
+		GFile *f = g_file_new_for_commandline_arg (arg);
+		char *uri = g_file_get_uri (f);
+		g_object_unref (G_OBJECT (f));
+		if (uri) {
+			char *uri2 = UT_go_url_simplify(uri);
+			g_free (uri);
+			return uri2;
+		}
+	}
+#elif defined(GOFFICE_WITH_GNOME)
 	{
 		/*
 		 * oink://     --> NULL
@@ -952,7 +983,11 @@ UT_go_basename_from_uri (const char *uri)
 {
 	char *res;
 
-#ifdef GOFFICE_WITH_GNOME
+#if HAVE_GSF_GIO
+	GFile *f = g_file_new_for_uri (uri);
+	char *basename = g_file_get_basename (f);
+	g_object_unref (G_OBJECT (f));
+#elif defined(GOFFICE_WITH_GNOME)
 	char *raw_uri = gnome_vfs_unescape_string (uri, G_DIR_SEPARATOR_S);
 	char *basename = raw_uri ? g_path_get_basename (raw_uri) : NULL;
 	g_free (raw_uri);
@@ -985,7 +1020,7 @@ UT_go_dirname_from_uri (const char *uri, gboolean brief)
 {
 	char *dirname_utf8, *dirname;
 
-#ifdef GOFFICE_WITH_GNOME
+#if defined(GOFFICE_WITH_GNOME)
 	char *raw_uri = gnome_vfs_unescape_string (uri, G_DIR_SEPARATOR_S);
 	dirname = raw_uri ? g_path_get_dirname (raw_uri) : NULL;
 	g_free (raw_uri);
@@ -1012,7 +1047,13 @@ UT_go_dirname_from_uri (const char *uri, gboolean brief)
 gboolean 
 UT_go_directory_create (char const *uri, int mode, GError **error)
 {
-#ifdef GOFFICE_WITH_GNOME
+#if HAVE_GSF_GIO
+	GFile *f = g_file_new_for_uri (uri);
+	gboolean res = g_file_make_directory (f, NULL, error);
+	g_object_unref (G_OBJECT (f));
+	UT_UNUSED(mode);
+	return res;
+#elif defined(GOFFICE_WITH_GNOME)
 	GnomeVFSResult vfs_result;
 
 	vfs_result = gnome_vfs_make_directory (uri, mode);
@@ -1026,6 +1067,9 @@ UT_go_directory_create (char const *uri, int mode, GError **error)
 	return TRUE;
 #else
 	char *filename;
+	if(error) {
+		*error = NULL;
+	}
 
 	filename = UT_go_filename_from_uri (uri);
 	if (filename) {
@@ -1118,7 +1162,9 @@ UT_go_file_open_impl (char const *uri, GError **err)
 		return result;
 	}
 
-#ifdef GOFFICE_WITH_GNOME
+#if HAVE_GSF_GIO
+	return gsf_input_gio_new_for_uri (uri, err);
+#elif defined(GOFFICE_WITH_GNOME)
 	return gsf_input_gnomevfs_new (uri, err);
 #elif defined(WITH_GSF_INPUT_HTTP)
 	{
@@ -1165,6 +1211,19 @@ UT_go_file_open (char const *uri, GError **err)
 	return NULL;
 }
 
+static GsfOutput *
+gsf_output_proxy_create (GsfOutput *wrapped, char const *uri, GError **err)
+{
+	if (!wrapped) {
+		g_set_error (err, gsf_output_error_id (), 0,
+			     "Unable to write to %s", uri);
+		return NULL;
+	}
+	
+	/* guarantee that file descriptors will be seekable */
+	return gsf_output_proxy_new (wrapped);
+}
+
 static GsfOutput *
 UT_go_file_create_impl (char const *uri, GError **err)
 {
@@ -1188,17 +1247,13 @@ UT_go_file_create_impl (char const *uri, GError **err)
 		FILE *fil = fd2 != -1 ? fdopen (fd2, "wb") : NULL;
 		GsfOutput *result = fil ? gsf_output_stdio_new_FILE (uri, fil, FALSE) : NULL;
 
-		if (!result) {
-			g_set_error (err, gsf_output_error_id (), 0,
-				     "Unable to write to %s", uri);
-			return NULL;
-		}
-
 		/* guarantee that file descriptors will be seekable */
-		return gsf_output_proxy_new (result);
+		return gsf_output_proxy_create(result, uri, err);
 	}
 
-#ifdef GOFFICE_WITH_GNOME
+#if HAVE_GSF_GIO
+	return gsf_output_proxy_create(gsf_output_gio_new_for_uri (uri, err), uri, err);
+#elif defined(GOFFICE_WITH_GNOME)
 	return gsf_output_gnomevfs_new (uri, err);
 #else
 	g_set_error (err, gsf_output_error_id (), 0,
@@ -1222,7 +1277,7 @@ UT_go_file_create (char const *uri, GError **err)
 }
 
 gboolean
-UT_go_file_remove (char const *uri, GError **err)
+UT_go_file_remove (char const *uri, GError ** err)
 {
 	char *filename;
 
@@ -1236,7 +1291,18 @@ UT_go_file_remove (char const *uri, GError **err)
 	}
 
 
-#ifdef GOFFICE_WITH_GNOME
+#if HAVE_GSF_GIO
+
+	{
+		GFile *f = g_file_new_for_uri (uri);
+		gboolean res = g_file_delete (f, NULL, err);
+		g_object_unref (G_OBJECT (f));
+
+		return res;
+	}
+
+#elif defined(GOFFICE_WITH_GNOME)
+	UT_UNUSED(err);
 	return (gnome_vfs_unlink (uri) == GNOME_VFS_OK);
 #else
 	g_set_error (err, gsf_output_error_id (), 0,
@@ -1298,7 +1364,12 @@ UT_go_file_split_urls (const char *data)
 gboolean
 UT_go_file_exists (char const *uri)
 {
-#if defined (GOFFICE_WITH_GNOME)
+#if HAVE_GSF_GIO
+	GFile *f = g_file_new_for_uri (uri);
+	gboolean res = g_file_query_exists (f, NULL);
+	g_object_unref (G_OBJECT (f));
+	return res;
+#elif defined (GOFFICE_WITH_GNOME)
 	GnomeVFSURI *vfs_uri = gnome_vfs_uri_new (uri);
 	if (vfs_uri) {
 		gboolean exists = gnome_vfs_uri_exists (vfs_uri);
@@ -1490,7 +1561,7 @@ UT_go_file_get_date (char const *uri, UT_GOFileDateType type)
 {
 	time_t tm = -1;
 
-#ifdef GOFFICE_WITH_GNOME
+#if defined(GOFFICE_WITH_GNOME)
 	GnomeVFSFileInfo *file_info;
 
 	GnomeVFSResult result;
@@ -1558,7 +1629,7 @@ UT_go_file_get_date_changed (char const *uri)
 
 /* ------------------------------------------------------------------------- */
 
-#ifndef WITH_GNOMEUI
+#ifndef WITH_GNOMEVFS
 static char *
 check_program (char const *prog)
 {
@@ -1582,8 +1653,11 @@ UT_go_url_show (gchar const *url)
 	return NULL;
 #else
 	GError *err = NULL;
-#ifdef WITH_GNOMEUI
-	gnome_url_show (url, &err);
+#ifdef HAVE_GTK214
+	gtk_show_uri (NULL, url, GDK_CURRENT_TIME, &err);
+	return err;
+#elif defined(WITH_GNOMEVFS)
+	gnome_vfs_url_show (url);
 	return err;
 #else
 	gchar *browser = NULL;
@@ -1698,7 +1772,20 @@ UT_go_url_check_extension (gchar const *uri,
 gchar *
 UT_go_get_mime_type (gchar const *uri)
 {
-#ifdef GOFFICE_WITH_GNOME
+#if HAVE_GSF_GIO
+	gboolean content_type_uncertain = FALSE;
+	char *content_type = g_content_type_guess (uri, NULL, 0, &content_type_uncertain);
+	if (content_type) {
+		char *mime_type = g_content_type_get_mime_type (content_type);
+		g_free (content_type);
+
+		if (mime_type)
+			return mime_type;
+	}
+
+	return g_strdup ("application/octet-stream");
+
+#elif defined(GOFFICE_WITH_GNOME)
 	return gnome_vfs_get_mime_type (uri);
 #elif 0 /* defined(G_OS_WIN32) */
 	LPWSTR wuri, mime_type;
@@ -1720,6 +1807,7 @@ UT_go_get_mime_type (gchar const *uri)
 	 */
 	return g_strdup ("text/plain");
 #else
+	UT_UNUSED(uri);
 	return g_strdup ("application/octet-stream");
 #endif
 }
@@ -1727,7 +1815,7 @@ UT_go_get_mime_type (gchar const *uri)
 gchar
 *UT_go_get_mime_type_for_data	(gconstpointer data, int data_size)
 {
-#ifdef GOFFICE_WITH_GNOME
+#if defined(GOFFICE_WITH_GNOME)
 	return g_strdup (gnome_vfs_get_mime_type_for_data (data, data_size));
 #elif 0 /* defined G_OS_WIN32 */
 	LPWSTR mime_type;
@@ -1744,6 +1832,8 @@ gchar
 	 */
 	return g_strdup ("text/plain");
 #else
+	UT_UNUSED(data);
+	UT_UNUSED(data_size);
 	return g_strdup ("application/octet-stream");
 #endif
 }
@@ -1751,7 +1841,7 @@ gchar
 gchar const
 *UT_go_mime_type_get_description	(gchar const *mime_type)
 {
-#ifdef GOFFICE_WITH_GNOME
+#if defined(GOFFICE_WITH_GNOME)
 	return gnome_vfs_mime_get_description (mime_type);
 #else
 	return mime_type;
@@ -1796,6 +1886,7 @@ UT_go_shell_argv_to_glib_encoding (gint argc, gchar const **argv)
 
 	return (gchar const **) args;
 #else
+	UT_UNUSED(argc);
 	return argv;
 #endif
 }
