@@ -17,17 +17,14 @@
  * 02111-1307, USA.
  */
 
-#include <stdio.h>
-
-#include <stdlib.h>
-#include <string.h>
-
-#include <algorithm>
-
-#ifdef __QNXNTO__
-#include <strings.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
 #endif
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <algorithm>
 #include <math.h>
 #include <ctype.h>
 
@@ -1614,7 +1611,7 @@ const char* std_size_string(float f)
   return string;
 }
 
-#ifndef WIN32
+#ifndef TOOLKIT_WIN
 
 UT_BidiCharType UT_bidiGetCharType(UT_UCS4Char c)
 {
