@@ -632,13 +632,6 @@ UT_uint32 hashcode(const char *p)
 //
 ////////////////////////////////////////////////////////////////////////
 
-static const char * s_UTF8_GenericBaseID = "UT_UTF8String";
-
-const char * UT_UTF8String::GenericBaseID () const
-{
-	return s_UTF8_GenericBaseID;
-}
-
 UT_UTF8String::UT_UTF8String () :
 	pimpl(new UT_UTF8Stringbuf)
 {
@@ -665,8 +658,8 @@ UT_UTF8String::UT_UTF8String (const char *str, const char *encoding)
 }
 
 
-UT_UTF8String::UT_UTF8String (const UT_UTF8String & rhs) :
-	pimpl(new UT_UTF8Stringbuf(*rhs.pimpl))
+UT_UTF8String::UT_UTF8String (const UT_UTF8String & rhs)
+	: pimpl(new UT_UTF8Stringbuf(*rhs.pimpl))
 {
 	// 
 }
@@ -775,8 +768,9 @@ UT_UTF8String &	UT_UTF8String::operator+=(const UT_UCS4Char            rhs)
 
 UT_UTF8String &	UT_UTF8String::operator+=(const char * rhs)
 {
-	UT_return_val_if_fail(rhs && *rhs, *this);
-	pimpl->append (rhs);
+	UT_return_val_if_fail(rhs, *this);
+	if(*rhs)
+		pimpl->append (rhs);
 	return *this;
 }
 
@@ -1284,20 +1278,34 @@ UT_UCS4String::UT_UCS4String(const UT_UCS4String& rhs)
 {
 }
 
+void UT_UCS4String::_loadUtf8(const char * _utf8_str, size_t bytelength)
+{
+	UT_UCS4Char ucs4;
+	do {
+		ucs4 = UT_Unicode::UTF8_to_UCS4 (_utf8_str, bytelength);
+		if (ucs4) {
+			pimpl->append (&ucs4, 1);
+		}
+	} while(ucs4 != 0);
+}
+
+
 /* construct from a string in UTF-8 format
  */
-UT_UCS4String::UT_UCS4String(const char * utf8_str, size_t bytelength /* 0 == zero-terminate */)
+UT_UCS4String::UT_UCS4String(const char * _utf8_str, size_t bytelength /* 0 == zero-terminate */)
 	:	pimpl(new UT_StringImpl<UT_UCS4Char>)
 {
 	if (bytelength == 0) {
-		if (utf8_str == 0 || *utf8_str == '\0') return;
-		bytelength = strlen (utf8_str);
-	}
-	while (true) {
-		UT_UCS4Char ucs4 = UT_Unicode::UTF8_to_UCS4 (utf8_str, bytelength);
-		if (ucs4 == 0) break; // end-of-string
-		pimpl->append (&ucs4, 1);
+		if (_utf8_str == 0 || *_utf8_str == '\0') return;
+		bytelength = strlen (_utf8_str);
 	}
+	_loadUtf8(_utf8_str, bytelength);
+}
+
+UT_UCS4String::UT_UCS4String(const std::string & str /* zero-terminated utf-8 encoded */)
+	:	pimpl(new UT_StringImpl<UT_UCS4Char>)
+{
+	_loadUtf8(str.c_str(), str.size());
 }
 
 /* construct from a string in UTF-8 format
@@ -1305,18 +1313,18 @@ UT_UCS4String::UT_UCS4String(const char * utf8_str, size_t bytelength /* 0 == ze
  * if (strip_whitespace != true) replace CR-LF & CR by LF
  * non-breaking spaces (&nbsp; UCS_NBSP 0x0a) are not white space
  */
-UT_UCS4String::UT_UCS4String(const char * utf8_str, size_t bytelength /* 0 == zero-terminate */, bool strip_whitespace)
+UT_UCS4String::UT_UCS4String(const char * _utf8_str, size_t bytelength /* 0 == zero-terminate */, bool strip_whitespace)
 	:	pimpl(new UT_StringImpl<UT_UCS4Char>)
 {
 	if (bytelength == 0) {
-		if (utf8_str == 0 || *utf8_str == '\0') return;
-		bytelength = strlen (utf8_str);
+		if (_utf8_str == 0 || *_utf8_str == '\0') return;
+		bytelength = strlen (_utf8_str);
 	}
-	UT_UCS4Char ucs4a = UT_Unicode::UTF8_to_UCS4 (utf8_str, bytelength);
+	UT_UCS4Char ucs4a = UT_Unicode::UTF8_to_UCS4 (_utf8_str, bytelength);
 	while (true) {
 		if (ucs4a == 0) 
 			break; // end-of-string
-		UT_UCS4Char ucs4b = UT_Unicode::UTF8_to_UCS4 (utf8_str, bytelength);
+		UT_UCS4Char ucs4b = UT_Unicode::UTF8_to_UCS4 (_utf8_str, bytelength);
 		if ((UCS_NBSP != ucs4a) && UT_UCS4_isspace (ucs4a)) {
 			if (strip_whitespace) {
 				if (!UT_UCS4_isspace (ucs4b)) {
