@@ -31,8 +31,10 @@
 #include <stdarg.h>
 
 #if defined(__MINGW32__)
-#undef snprintf
-#define _GLIBCXX_USE_C99_DYNAMIC 1
+#  undef snprintf
+#  if __GNUC__ <= 3
+#    define _GLIBCXX_USE_C99_DYNAMIC 1
+#  endif
 #endif
 
 #include <string>
@@ -54,13 +56,6 @@ class UT_String;
 class UT_UTF8String;
 class UT_UCS4String;
 
-class ABI_EXPORT UT_GenericBase
-{
-public:
-	virtual const char *	GenericBaseID () const = 0;
-
-	virtual ~UT_GenericBase () { }
-};
 
 // yes, this is screaming for a template
 
@@ -138,9 +133,10 @@ ABI_EXPORT size_t UT_String_findRCh(const UT_String &st, char ch);
  * Fill \inStr with the results of evaulating the printf formatted string 
  * \inFormat and return the reference to \inStr
  */
-ABI_EXPORT UT_String& UT_String_sprintf(UT_String & inStr, const char * inFormat, ...);
+ABI_EXPORT UT_String& UT_String_sprintf(UT_String & inStr, const char * inFormat, ...) ABI_PRINTF_FORMAT(2,3);
 ABI_EXPORT UT_String& UT_String_vprintf (UT_String & inStr, const char *format,
-				   va_list      args1);
+                                         va_list      args1) 
+    ABI_PRINTF_FORMAT(2,0);
 ABI_EXPORT UT_String& UT_String_vprintf (UT_String & inStr, const UT_String & format,
 					 va_list      args1);
 
@@ -148,8 +144,10 @@ ABI_EXPORT UT_String& UT_String_vprintf (UT_String & inStr, const UT_String & fo
  * Returns a new UT_String object with the results of evaluating the printf
  * formatted string \inFormat
  */
-ABI_EXPORT UT_String UT_String_sprintf(const char * inFormat, ...);
-ABI_EXPORT UT_String UT_String_vprintf(const char * inFormat, va_list args1);
+ABI_EXPORT UT_String UT_String_sprintf(const char * inFormat, ...)
+    ABI_PRINTF_FORMAT(1,2);
+ABI_EXPORT UT_String UT_String_vprintf(const char * inFormat, va_list args1)
+    ABI_PRINTF_FORMAT(1,0);
 ABI_EXPORT UT_String UT_String_vprintf(const UT_String & inFormat, va_list args1);
 
 /***************************************************************************/
@@ -174,11 +172,9 @@ ABI_EXPORT void UT_String_addPropertyString(UT_String & sPropertyString, const U
 //	UT_UTF8String, a simple wrapper for zero terminated 'UTF-8' strings.
 //
 
-class ABI_EXPORT UT_UTF8String : public UT_GenericBase
+class ABI_EXPORT UT_UTF8String
 {
 public:
-	virtual const char *	GenericBaseID () const;
-
 	UT_UTF8String ();
 	UT_UTF8String (const char * sz, size_t n = 0 /* 0 == null-termination */);
 	UT_UTF8String (const char *sz, const char *encoding);
@@ -301,6 +297,8 @@ ABI_EXPORT void UT_UTF8String_setProperty(UT_UTF8String & sPropertyString, const
 
 ABI_EXPORT void UT_UTF8String_addPropertyString(UT_UTF8String & sPropertyString, const UT_UTF8String & sNewProp);
 
+ABI_EXPORT void UT_UTF8String_replaceString(UT_UTF8String & sString, const UT_UTF8String & sOldValue,const UT_UTF8String & sNewValue );
+
 ////////////////////////////////////////////////////////////////////////
 //
 //  UCS-4 string
@@ -329,6 +327,7 @@ public:
 	/* construct from a string in UTF-8 format
 	 */
 	UT_UCS4String(const char * utf8_str, size_t bytelength = 0 /* 0 == zero-terminate */);
+	UT_UCS4String(const std::string & str /* zero-terminated utf-8 encoded */);
 
 	/* construct from a string in UTF-8 format
 	 * if (strip_whitespace == true) replace all white space sequences with a single UCS_SPACE
@@ -369,6 +368,7 @@ public:
 	const char * utf8_str ();
 
 private:
+	void _loadUtf8(const char * utf8_str, size_t bytelength); // implementation detail for the UTF-8 constructor
 	class UT_StringImpl<UT_UCS4Char>* pimpl;
 };
 
