@@ -2,7 +2,7 @@
 
 /* AbiSource Application Framework
  * Copyright (C) 1998 AbiSource, Inc.
- * Copyright (C) 2003 Hubert Figuiere
+ * Copyright (C) 2003, 2009 Hubert Figuiere
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,7 +27,6 @@
 #include "ut_debugmsg.h"
 #include "ut_string.h"
 #include "ut_misc.h"
-#include "ut_hash.h"
 #include "ut_units.h"
 #include "xap_CocoaDialog_Utilities.h"
 #include "xap_CocoaDlg_FontChooser.h"
@@ -226,7 +225,7 @@ void XAP_CocoaDialog_FontChooser::bgColorChanged(void)
 	_colorChanged([m_dlg bgColor], "bgcolor", buf_color);
 }
 
-void XAP_CocoaDialog_FontChooser::runModal(XAP_Frame * pFrame)
+void XAP_CocoaDialog_FontChooser::runModal(XAP_Frame * /*pFrame*/)
 {
 	UT_ASSERT(m_pApp);
 
@@ -245,15 +244,16 @@ void XAP_CocoaDialog_FontChooser::runModal(XAP_Frame * pFrame)
 
 
 	// Set the defaults in the list boxes according to dialog data
-	[m_dlg selectFont:(char*)getVal("font-family")];
-	[m_dlg selectStyle:(char*)getVal("font-style") withWeight:(char*)getVal("font-weight")];
-	[m_dlg selectSize:(char*)getVal("font-size")];
+	[m_dlg selectFont:(char*)getVal("font-family").c_str()];
+	[m_dlg selectStyle:(char*)getVal("font-style").c_str() withWeight:(char*)getVal("font-weight").c_str()];
+	[m_dlg selectSize:(char*)getVal("font-size").c_str()];
 
 	// Set color in the color selector
-	if (getVal("color"))
+	const std::string sColor = getVal("color");
+	if (!sColor.empty())
 	{
 		UT_RGBColor c;
-		UT_parseColor(getVal("color"), c);
+		UT_parseColor(sColor.c_str(), c);
 
 		NSColor *color = GR_CocoaGraphics::_utRGBColorToNSColor(c);
 		[m_dlg setTextColor:color];
@@ -263,11 +263,11 @@ void XAP_CocoaDialog_FontChooser::runModal(XAP_Frame * pFrame)
 	}
 	
 	// Set color in the color selector
-	const gchar * pszBGCol = getVal("bgcolor");
-	if (pszBGCol && strcmp(pszBGCol,"transparent") != 0)
+	const std::string sBGCol = getVal("bgcolor");
+	if (!sBGCol.empty() && strcmp(sBGCol.c_str(),"transparent") != 0)
 	{
 		UT_RGBColor c;
-		UT_parseColor(getVal("bgcolor"), c);
+		UT_parseColor(sBGCol.c_str(), c);
 
 		NSColor *color = GR_CocoaGraphics::_utRGBColorToNSColor(c);
 		[m_dlg setBgColor:color];
@@ -298,17 +298,17 @@ void XAP_CocoaDialog_FontChooser::runModal(XAP_Frame * pFrame)
 	m_dlg = nil;
 
 	UT_DEBUGMSG(("FontChooserEnd: Family[%s%s] Size[%s%s] Weight[%s%s] Style[%s%s] Color[%s%s] Underline[%d%s] StrikeOut[%d%s]\n",
-				 ((getVal("font-family")) ? getVal("font-family") : ""),	((m_bChangedFontFamily) ? "(chg)" : ""),
-				 ((getVal("font-size")) ? getVal("font-size") : ""),		((m_bChangedFontSize) ? "(chg)" : ""),
-				 ((getVal("font-weight")) ? getVal("font-weight") : ""),	((m_bChangedFontWeight) ? "(chg)" : ""),
-				 ((getVal("font-style")) ? getVal("font-style") : ""),		((m_bChangedFontStyle) ? "(chg)" : ""),
-				 ((getVal("color")) ? getVal("color") : "" ),				((m_bChangedColor) ? "(chg)" : ""),
-				 (m_bUnderline),							((m_bChangedUnderline) ? "(chg)" : ""),
-				 (m_bStrikeout),							((m_bChangedStrikeOut) ? "(chg)" : "")));
+				 getVal("font-family").c_str(),	((m_bChangedFontFamily) ? "(chg)" : ""),
+				 getVal("font-size").c_str(),		((m_bChangedFontSize) ? "(chg)" : ""),
+				 getVal("font-weight").c_str(),	((m_bChangedFontWeight) ? "(chg)" : ""),
+				 getVal("font-style").c_str(),		((m_bChangedFontStyle) ? "(chg)" : ""),
+				 getVal("color").c_str(),				((m_bChangedColor) ? "(chg)" : ""),
+				 m_bUnderline,							((m_bChangedUnderline) ? "(chg)" : ""),
+				 m_bStrikeout,							((m_bChangedStrikeOut) ? "(chg)" : "")));
 
 }
 
-bool XAP_CocoaDialog_FontChooser::getEntryString(char ** string)
+bool XAP_CocoaDialog_FontChooser::getEntryString(const char ** string)
 {
 	UT_ASSERT(string);
 
@@ -328,7 +328,7 @@ void XAP_CocoaDialog_FontChooser::updatePreview(void)
 	}
 	
 	// if a font has been set since this dialog was launched, draw things with it
-	char * entryString;
+	const char * entryString;
 
 	if (!getEntryString(&entryString)) {
 		return;
@@ -374,8 +374,7 @@ void XAP_CocoaDialog_FontChooser::_deleteGC(void)
 
 - (id)initFromNib
 {
-	self = [super initWithWindowNibName:@"xap_CocoaDlg_FontChooser"];
-	return self;
+	return [super initWithWindowNibName:@"xap_CocoaDlg_FontChooser"];
 }
 
 -(void)discardXAP
@@ -431,13 +430,13 @@ void XAP_CocoaDialog_FontChooser::_deleteGC(void)
 		m_stylesDataSource = [[XAP_StringListDataSource alloc] init];
 		UT_UTF8String label;
 		pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleRegular, label);
-		[m_stylesDataSource addUT_UTF8String:label];
+		[m_stylesDataSource addCString:label.utf8_str()];
 		pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleItalic, label);
-		[m_stylesDataSource addUT_UTF8String:label];
+		[m_stylesDataSource addCString:label.utf8_str()];
 		pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleBold, label);
-		[m_stylesDataSource addUT_UTF8String:label];
+		[m_stylesDataSource addCString:label.utf8_str()];
 		pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleBoldItalic, label);
-		[m_stylesDataSource addUT_UTF8String:label];
+		[m_stylesDataSource addCString:label.utf8_str()];
 		[_styleList setDataSource:m_stylesDataSource];
 		[_styleList setDelegate:self];
 	
@@ -470,6 +469,7 @@ void XAP_CocoaDialog_FontChooser::_deleteGC(void)
 
 -(IBAction)okAction:(id)sender
 {
+	UT_UNUSED(sender);
 	[[NSColorPanel sharedColorPanel] orderOut:self];
 
 	static_cast<XAP_CocoaDialog_FontChooser *>(_xap)->_okAction();
@@ -477,6 +477,7 @@ void XAP_CocoaDialog_FontChooser::_deleteGC(void)
 
 -(IBAction)cancelAction:(id)sender
 {
+	UT_UNUSED(sender);
 	[[NSColorPanel sharedColorPanel] orderOut:self];
 
 	static_cast<XAP_CocoaDialog_FontChooser *>(_xap)->_cancelAction();
