@@ -80,8 +80,8 @@ XAP_Win32Dialog_FileOpenSaveAs::~XAP_Win32Dialog_FileOpenSaveAs(void)
 static bool SuffixInList(const char *haystack, const char *needle)
 {
 	int l = strlen(needle);
-	const char *s;
-	if (s = strstr(haystack, needle))
+	const char *s = strstr(haystack, needle);
+	if (s)
 		if (s[l] == 0 || s[l] == ';')
 			return true;
 	return false;
@@ -298,7 +298,7 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 
 		const char * szURI = g_filename_from_uri(m_szInitialPathname, NULL, NULL);
 		if(!szURI)
-			szURI = "";
+			szURI = static_cast<const char *> (UT_calloc (1, sizeof (char)));
 
 		strcpy(szDir,AP_Win32App::s_fromUTF8ToWinLocale(szURI).c_str());
 		char * pLastSlash = strrchr(szDir, '/');
@@ -497,7 +497,7 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 
  Static routine
  */
-UINT CALLBACK XAP_Win32Dialog_FileOpenSaveAs::s_hookSaveAsProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
+UINT CALLBACK XAP_Win32Dialog_FileOpenSaveAs::s_hookSaveAsProc(HWND hDlg, UINT msg, WPARAM /*wParam*/, LPARAM lParam)
 {
 	XAP_Win32Dialog_FileOpenSaveAs* pThis;
 	static char buff[MAX_DLG_INS_PICT_STRING];
@@ -641,7 +641,6 @@ UINT CALLBACK XAP_Win32Dialog_FileOpenSaveAs::s_hookInsertPicProc(HWND hDlg, UIN
 UINT XAP_Win32Dialog_FileOpenSaveAs::_previewPicture(HWND hDlg)
 {
 	HWND hFOSADlg	= GetParent(hDlg);
-	HWND hFrame 	= GetParent(hFOSADlg);
 	HWND hThumbnail = GetDlgItem(hDlg,XAP_RID_DIALOG_INSERT_PICTURE_IMAGE_PREVIEW);
 
 	const XAP_StringSet*  pSS	= XAP_App::getApp()->getStringSet();
@@ -782,8 +781,6 @@ UINT XAP_Win32Dialog_FileOpenSaveAs::_previewPicture(HWND hDlg)
 UINT XAP_Win32Dialog_FileOpenSaveAs::_initPreviewDlg(HWND hDlg)
 {
 	HWND hFOSADlg	= GetParent(hDlg);
-	HWND hFrame 	= GetParent(hFOSADlg);
-	HWND hThumbnail = GetDlgItem(hDlg,XAP_RID_DIALOG_INSERT_PICTURE_IMAGE_PREVIEW);
 
 	const XAP_StringSet*  pSS		  = XAP_App::getApp()->getStringSet();
 	UT_return_val_if_fail(pSS, false);
