@@ -24,6 +24,7 @@
 #ifndef AD_DOCUMENT_H
 #define AD_DOCUMENT_H
 
+#include <string>
 // TODO should the filename be UT_UCSChar rather than char ?
 
 /* pre-emptive dismissal; ut_types.h is needed by just about everything,
@@ -44,7 +45,7 @@ class XAP_Frame;
 class AV_View;
 
 // a helper class for history tracking
-class AD_VersionData
+class ABI_EXPORT AD_VersionData
 {
   public:
 
@@ -94,7 +95,7 @@ enum AD_HISTORY_STATE
 
 
 // a helper class for keeping track of revisions in the document
-class AD_Revision
+class ABI_EXPORT AD_Revision
 {
   public:
 	AD_Revision(UT_uint32 iId, UT_UCS4Char * pDesc, time_t start, UT_uint32 iVer = 0)
@@ -137,6 +138,8 @@ public:
 	XAP_ResourceManager &	resourceManager () const { return *m_pResourceManager; }
 
 	const char *			getFilename(void) const;
+	const std::string     & getPrintFilename(void) const;
+	void                    setPrintFilename(const std::string & sFile);
 	void                    clearFilename(void) {_setFilename(NULL); forceDirty();}
 	// TODO - this should be returning IEFileType, 
 	// but that's AP stuff, so it's not here
@@ -171,16 +174,16 @@ public:
 	/**
 	 * Returns the # of seconds since the last save of this file 
 	 */
-	UT_uint32       getTimeSinceSave () const { return (time(NULL) - m_lastSavedTime); }
+	time_t          getTimeSinceSave () const { return (time(NULL) - m_lastSavedTime); }
 	time_t          getLastSavedTime() const {return m_lastSavedTime;}
 	void            setLastSavedTime(time_t t) {m_lastSavedTime = t;}
 	virtual UT_uint32 getLastSavedAsType() const = 0;
 
-	UT_uint32       getTimeSinceOpen () const { return (time(NULL) - m_lastOpenedTime); }
+	time_t          getTimeSinceOpen () const { return (time(NULL) - m_lastOpenedTime); }
 	time_t          getLastOpenedTime() const {return m_lastOpenedTime;}
 	void            setLastOpenedTime(time_t t) {m_lastOpenedTime = t;}
 
-	UT_uint32       getEditTime()const {return (m_iEditTime + (time(NULL) - m_lastOpenedTime));}
+	time_t          getEditTime()const {return (m_iEditTime + (time(NULL) - m_lastOpenedTime));}
 	void            setEditTime(UT_uint32 t) {m_iEditTime = t;}
 
 	void            setDocVersion(UT_uint32 i){m_iVersion = i;}
@@ -189,22 +192,26 @@ public:
 	void			setEncodingName(const char * szEncodingName);
 	const char *	getEncodingName() const;
 
-	bool			isPieceTableChanging(void);
+	bool			isPieceTableChanging(void) const;
 
 	virtual void setMetaDataProp (const UT_String & key, const UT_UTF8String & value) = 0;
 	virtual bool getMetaDataProp (const UT_String & key, UT_UTF8String & outProp) const = 0;
 
+	// RIVERA TODO not working and may not be needed
+	virtual void setAnnotationProp (const UT_String & key, const UT_UTF8String & value) = 0;
+	virtual bool getAnnotationProp (const UT_String & key, UT_UTF8String & outProp) const = 0;
+	
 	// history tracking
 	void            addRecordToHistory(const AD_VersionData & v);
 	void            purgeHistory();
-	UT_uint32       getHistoryCount()const {return m_vHistory.getItemCount();}
-	UT_uint32       getHistoryNthId(UT_uint32 i)const;
-	time_t          getHistoryNthTime(UT_uint32 i)const;
-	time_t          getHistoryNthTimeStarted(UT_uint32 i)const;
-	UT_uint32       getHistoryNthEditTime(UT_uint32 i)const;
-	const UT_UUID&  getHistoryNthUID(UT_uint32 i)const;
-	bool            getHistoryNthAutoRevisioned(UT_uint32 i)const;
-	UT_uint32       getHistoryNthTopXID(UT_uint32 i)const;
+	UT_sint32       getHistoryCount()const {return m_vHistory.getItemCount();}
+	UT_uint32       getHistoryNthId(UT_sint32 i)const;
+	time_t          getHistoryNthTime(UT_sint32 i)const;
+	time_t          getHistoryNthTimeStarted(UT_sint32 i)const;
+	time_t          getHistoryNthEditTime(UT_sint32 i)const;
+	const UT_UUID&  getHistoryNthUID(UT_sint32 i)const;
+	bool            getHistoryNthAutoRevisioned(UT_sint32 i)const;
+	UT_uint32       getHistoryNthTopXID(UT_sint32 i)const;
 
 	AD_HISTORY_STATE       verifyHistoryState(UT_uint32 &iVersion) const;
 	const AD_VersionData * findHistoryRecord(UT_uint32 iVersion) const;
@@ -232,11 +239,13 @@ public:
 	UT_uint64       getNewUUID64() const;
 	
 	bool            addRevision(UT_uint32 iId, UT_UCS4Char * pDesc,
-								time_t tStart, UT_uint32 iVersion);
+								time_t tStart, UT_uint32 iVersion, bool bGenCR=true);
 	
 	bool            addRevision(UT_uint32 iId, const UT_UCS4Char * pDesc, UT_uint32 iLen,
-								time_t tStart, UT_uint32 iVersion);
-	
+								time_t tStart, UT_uint32 iVersion, bool bGenCR=true);
+	bool            addRevision(AD_Revision * pRev, bool bGenCR=true);
+	virtual bool    createAndSendDocPropCR( const gchar ** pAtts, const gchar ** pProps) = 0;
+
 	const UT_GenericVector<AD_Revision*> &         getRevisions() {return m_vRevisions;}
 	UT_uint32           getHighestRevisionId() const;
 	const AD_Revision*  getHighestRevision() const;
@@ -287,8 +296,6 @@ public:
 
 	void            _purgeRevisionTable();
 	void            _adjustHistoryOnSave();
-	UT_UUID *       _getDocUUID()const {return m_pUUID;};
-	char *			_getFilename(void) const;
 	void			_setFilename(char * name) {FREEP(m_szFilename); m_szFilename = name;}
 	void            _setForceDirty(bool b) {m_bForcedDirty = b;}
 	void            _setPieceTableChanging(bool b) {m_bPieceTableChanging = b;}
@@ -311,7 +318,7 @@ private:
 	bool			m_bPieceTableChanging;
 	time_t          m_lastSavedTime;
 	time_t          m_lastOpenedTime;
-	UT_uint32       m_iEditTime;     // previous edit time (up till open)
+	time_t          m_iEditTime;     // previous edit time (up till open)
 	UT_uint32       m_iVersion;
 
 	// these are for tracking versioning
@@ -332,6 +339,9 @@ private:
 	UT_UUID *       m_pMyUUID;
 	bool            m_bDoNotAdjustHistory;
 	bool            m_bAfterFirstSave;
+ 	UT_UTF8String   m_sMyUUIDString;
+ 	UT_UTF8String   m_sOrigUUIDString;
+	std::string     m_sPrintFilename;
 };
 
 
