@@ -23,6 +23,8 @@
 #ifndef XAP_Frame_H
 #define XAP_Frame_H
 
+#include <string>
+
 /* pre-emptive dismissal; ut_types.h is needed by just about everything,
  * so even if it's commented out in-file that's still a lot of work for
  * the preprocessor to do...
@@ -126,7 +128,7 @@ public:
 	void                        setView(AV_View * pView) {m_pView = pView;}
 	void                        setDoc(AD_Document * pDoc) {m_pDoc = pDoc;}
 	const char *				getFilename() const;
-	const char *				getTitle(int len) const;
+	const UT_UTF8String &		getTitle() const;
 	const char *				getNonDecoratedTitle() const;
 
 	XAP_FrameMode getFrameMode(); 
@@ -160,12 +162,12 @@ public:
 	virtual void                toggleLeftRuler(bool /*bRulerOn*/) = 0;
 	virtual void				toggleBar(UT_uint32 /* iBarNb */, bool /* bBarOn */) { }
 	virtual void				toggleStatusBar(bool /* bStatusBarOn */) { }
-	virtual bool				getBarVisibility(UT_uint32 iBarNb) { return true; }
+	virtual bool				getBarVisibility(UT_uint32 /*iBarNb*/) { return true; }
 
    	EV_Mouse *					getMouse() { return m_pFrameImpl->m_pMouse; }
 	EV_Keyboard *				getKeyboard() { return m_pFrameImpl->m_pKeyboard; }
 
-	EV_Toolbar *                getToolbar(UT_uint32 ibar);
+	EV_Toolbar *                getToolbar(UT_sint32 ibar);
 	UT_sint32                   findToolbarNr(EV_Toolbar * pTB);
 	virtual void                rebuildMenus(void) { m_pFrameImpl->_rebuildMenus();}
 	bool                        repopulateCombos();
@@ -199,6 +201,10 @@ public:
 													   XAP_Dialog_MessageBox::tButtons buttons,
 													   XAP_Dialog_MessageBox::tAnswer default_answer);
 													   
+	XAP_Dialog_MessageBox::tAnswer		showMessageBox(const std::string & sz,
+													   XAP_Dialog_MessageBox::tButtons buttons,
+													   XAP_Dialog_MessageBox::tAnswer default_answer);
+
 	XAP_Dialog_MessageBox::tAnswer		showMessageBox(const char * sz,
 													   XAP_Dialog_MessageBox::tButtons buttons,
 													   XAP_Dialog_MessageBox::tAnswer default_answer);
@@ -213,11 +219,11 @@ public:
 	UT_Error	    backup(const char* stExt = 0, UT_sint32 iEFT = -1);
 	UT_String       makeBackupName (const char * szExt = 0);
 
-	const bool                  isStatusBarShown(void) const { return m_bShowStatusbar;}
-	const bool                  isMenuBarShown(void) const { return m_bShowMenubar;}
-	virtual void                setStatusBarShown(bool bShowStatusbar) {}
-	virtual void                setMenuBarShown(bool bShowMenubar) {}
-	UT_uint32                   getTimeSinceSave() const;
+	bool                        isStatusBarShown(void) const { return m_bShowStatusbar;}
+	bool                        isMenuBarShown(void) const { return m_bShowMenubar;}
+	virtual void                setStatusBarShown(bool /*bShowStatusbar*/) {}
+	virtual void                setMenuBarShown(bool /*bShowMenubar*/) {}
+	time_t                      getTimeSinceSave() const;
 	bool                        isFrameLocked(void) const 
 	                            {return m_bIsFrameLocked;}
 	void                        setFrameLocked(bool bLock) 
@@ -228,7 +234,10 @@ public:
 	UT_RGBColor getColorSelForeground () const;
 
 	void dragText();
-
+	virtual UT_sint32 getDocumentAreaXoff(void)
+	{ return 0;}
+	virtual UT_sint32 getDocumentAreaYoff(void)
+	{ return 0;}
 protected:
 	friend class XAP_FrameImpl;
 
