@@ -33,6 +33,7 @@
 #include "ut_string_class.h"
 #include "ut_go_file.h"
 #include "xap_Prefs.h"
+#include "xap_App.h"
 
 struct xmlToIdMapping {
 	const char *m_name;
@@ -146,6 +147,13 @@ bool XAP_PrefsScheme::setValueBool(const gchar * szKey, bool bValue)
 	return setValue(szKey, reinterpret_cast<const gchar*>((bValue) ? "1" : "0"));
 }
 
+bool XAP_PrefsScheme::setValueInt(const gchar * szKey, const int nValue)
+{
+	gchar szValue[32];
+	sprintf(szValue, "%d", nValue);
+	return setValue(szKey, szValue);
+}
+
 bool XAP_PrefsScheme::getValue(const gchar * szKey, const gchar ** pszValue) const
 {
 	gchar *pEntry = m_hash.pick(szKey);
@@ -167,6 +175,19 @@ bool XAP_PrefsScheme::getValue(const UT_String &stKey, UT_String &stValue) const
 	return true;
 }
 
+bool XAP_PrefsScheme::getValueInt(const gchar * szKey, int& nValue) const
+{
+	const gchar * szValue = NULL;
+	if (!getValue(szKey,&szValue))
+		return false;				// bogus keyword ??
+
+	if (!szValue || !*szValue)
+		return false;				// no value for known keyword ??
+		
+	nValue = atoi(szValue);
+	return true;
+}
+
 bool XAP_PrefsScheme::getValueBool(const gchar * szKey, bool * pbValue) const
 {
 	*pbValue = false;				// assume something
@@ -205,7 +226,7 @@ bool XAP_PrefsScheme::getNthValue(UT_uint32 k, const gchar ** pszKey, const gcha
 	if (!m_bValidSortedKeys) {
 		UT_GenericVector<const UT_String*> * vecD = m_hash.keys();
 		UT_GenericVector<const char*> vecKeys(vecD->getItemCount(), 4, true);
-		UT_uint32 i=0;
+		UT_sint32 i=0;
 		m_sortedKeys.clear();
 		for(i=0; i< vecD->getItemCount(); i++)
 		{
@@ -264,12 +285,12 @@ void XAP_Prefs::setUseEnvLocale(bool bUse)
 
 /*****************************************************************/
 
-UT_uint32 XAP_Prefs::getMaxRecent(void) const
+UT_sint32 XAP_Prefs::getMaxRecent(void) const
 {
 	return m_iMaxRecent;
 }
 
-void XAP_Prefs::setMaxRecent(UT_uint32 k)
+void XAP_Prefs::setMaxRecent(UT_sint32 k)
 {
 	UT_ASSERT_HARMLESS(k<=XAP_PREF_LIMIT_MaxRecent);
 
@@ -279,12 +300,12 @@ void XAP_Prefs::setMaxRecent(UT_uint32 k)
 	m_iMaxRecent = k;
 }
 
-UT_uint32 XAP_Prefs::getRecentCount(void) const
+UT_sint32 XAP_Prefs::getRecentCount(void) const
 {
 	return m_vecRecent.getItemCount();
 }
 
-const char * XAP_Prefs::getRecent(UT_uint32 k) const
+const char * XAP_Prefs::getRecent(UT_sint32 k) const
 {
 	// NB: k is one-based
 	UT_return_val_if_fail(k <= m_iMaxRecent, NULL);
@@ -313,7 +334,7 @@ void XAP_Prefs::addRecent(const char * szRecent)
 		return;
 	}
 	// was it already here? 
-	for (UT_uint32 i=0; i<m_vecRecent.getItemCount(); i++)
+	for (UT_sint32 i=0; i<m_vecRecent.getItemCount(); i++)
 	{
 		sz = m_vecRecent.getNthItem(i);
 		if ((sz==szRecent) || !strcmp(sz, szRecent))
@@ -335,7 +356,7 @@ void XAP_Prefs::addRecent(const char * szRecent)
 	_pruneRecent();
 }
 
-void XAP_Prefs::removeRecent(UT_uint32 k)
+void XAP_Prefs::removeRecent(UT_sint32 k)
 {
 	UT_return_if_fail(k>0);
 	UT_return_if_fail(k<=getRecentCount());
@@ -349,12 +370,12 @@ void XAP_Prefs::removeRecent(UT_uint32 k)
 void XAP_Prefs::_pruneRecent(void)
 {
 	UT_sint32 i;
-	UT_uint32 count = getRecentCount();
+	UT_sint32 count = getRecentCount();
 
 	if (m_iMaxRecent == 0)
 	{
 		// nuke the whole thing
-		for (i = static_cast<signed>(count); i > 0 ; i--)
+		for (i = count; i > 0 ; i--)
 		{
 			char * sz = m_vecRecent.getNthItem(i-1);
 			FREEP(sz);
@@ -365,7 +386,7 @@ void XAP_Prefs::_pruneRecent(void)
 	else if (count > m_iMaxRecent)
 	{
 		// prune entries past m_iMaxRecent
-		for (i = static_cast<signed>(count); i > static_cast<signed>(m_iMaxRecent); i--)
+		for (i = count; i > m_iMaxRecent; i--)
 			removeRecent(i);
 	}
 }
@@ -466,10 +487,9 @@ void XAP_Prefs::log(const char * where, const char * what, XAPPrefsLog_Level lev
 
 /*****************************************************************/
 
-XAP_Prefs::XAP_Prefs(XAP_App * pApp) 
+XAP_Prefs::XAP_Prefs() 
 	: m_ahashChanges( 20 )
 {
-	m_pApp = pApp;
 	m_bAutoSavePrefs = (atoi(XAP_PREF_DEFAULT_AutoSavePrefs) ? true : false);
 	m_bUseEnvLocale = (atoi(XAP_PREF_DEFAULT_UseEnvLocale) ? true : false);
 	m_currentScheme = NULL;
@@ -697,6 +717,28 @@ bool XAP_Prefs::getPrefsValueBool(const gchar * szKey, bool * pbValue, bool bAll
 	return false;
 }
 
+bool XAP_Prefs::getPrefsValueInt(const gchar * szKey, int& nValue, bool bAllowBuiltin) const
+{
+	// a convenient routine to get a name/value pair from the current scheme
+
+	UT_return_val_if_fail(m_currentScheme,false);
+
+	if (m_currentScheme->getValueInt(szKey,nValue))
+		return true;
+	if (bAllowBuiltin && m_builtinScheme->getValueInt(szKey,nValue))
+		return true;
+	// It is legal for there to be arbitrary preference tags that start with 
+	// "Debug", and Abi apps won't choke.  The idea is that developers can use
+	// these to selectively trigger development-time behaviors.
+	if (g_ascii_strncasecmp(szKey, DEBUG_PREFIX, sizeof(DEBUG_PREFIX) - 1) == 0)
+	{
+		nValue = -1;
+		return true;
+	}
+
+	return false;
+}
+
 
 static int n_compare (const char *name, const xmlToIdMapping *id)
 {
@@ -815,7 +857,7 @@ void XAP_Prefs::startElement(const gchar *name, const gchar **atts)
 				// TODO you'll lose your MRU list if you alternate between
 				// TODO the two types of executables.
 				
-				const char * szThisApp = m_pApp->getApplicationName();
+				const char * szThisApp = XAP_App::getApp()->getApplicationName();
 				UT_DEBUGMSG(("Found preferences for application [%s] (this is [%s]).\n",
 							a[1],szThisApp));
 				if (strcmp(static_cast<const char*>(a[1]),szThisApp) != 0)
@@ -1109,8 +1151,6 @@ void XAP_Prefs::startElement(const gchar *name, const gchar **atts)
 		}
 	}
 	// successful parse of tag...
-
-
 IgnoreThisScheme:
 	DELETEP(pNewScheme);
 	return;								// success
@@ -1283,7 +1323,7 @@ bool XAP_Prefs::savePrefsFile(void)
 	UT_ASSERT(m_builtinScheme);
 	
 	fprintf(fp,"\n<AbiPreferences app=\"%s\" ver=\"%s\">\n",
-			m_pApp->getApplicationName(),
+			XAP_App::getApp()->getApplicationName(),
 			"1.0");
 	{
 		fprintf(fp,("\n"
@@ -1620,7 +1660,7 @@ void XAP_Prefs::addListener	  ( PrefsListener pFunc, void *data )
 // otherwise, will delete all calls to pFunc
 void XAP_Prefs::removeListener ( PrefsListener pFunc, void *data )
 {
-	UT_uint32 index;
+	UT_sint32 index;
 	tPrefsListenersPair *pPair;
 
 	for ( index = 0; index < m_vecPrefsListeners.getItemCount(); index++ )
@@ -1674,7 +1714,7 @@ void XAP_Prefs::endBlockChange()
 
 void XAP_Prefs::_sendPrefsSignal( UT_StringPtrMap *hash  )
 {
-	UT_uint32	index;
+	UT_sint32	index;
 	for ( index = 0; index < m_vecPrefsListeners.getItemCount(); index++ )
 	{
 		tPrefsListenersPair *p = m_vecPrefsListeners.getNthItem( index );
@@ -1683,7 +1723,7 @@ void XAP_Prefs::_sendPrefsSignal( UT_StringPtrMap *hash  )
 		if(!p || !p->m_pFunc)
 			continue;
 	
-		(p->m_pFunc)(m_pApp, this, hash, p->m_pData);
+		(p->m_pFunc)(this, hash, p->m_pData);
 	}
 }
 
