@@ -40,6 +40,7 @@ dcl(MSG_NoUndo, "This operation cannot be undone. Are you sure you want to proce
 
 /* Default name for new, untitled document */
 dcl(UntitledDocument,			"Untitled%d")
+dcl(ReadOnly,					"Read-Only")
 
 /*  Styles */
 dcl(STYLE_NUMBER_LIST,		"Numbered List")
@@ -110,8 +111,8 @@ dcl(DLG_Unit_pica,				"pica")
 dcl(DLG_UnixMB_Yes, 			"_Yes")
 dcl(DLG_UnixMB_No,				"_No")
 
-dcl(DLG_QNXMB_Yes,				"Yes")
-dcl(DLG_QNXMB_No,				"No")
+dcl(DLG_MB_Yes,					"Yes")
+dcl(DLG_MB_No,					"No")
 
 /* More Windows dialog */
 dcl(DLG_MW_MoreWindows, 		"View Document")
@@ -148,6 +149,11 @@ dcl(DLG_UFS_ScriptLabel,		"Script:")
 dcl(DLG_UFS_SampleFrameLabel,	"Sample")
 dcl(DLG_UFS_SuperScript, 		"Superscript")
 dcl(DLG_UFS_SubScript, 			"Subscript")
+dcl(DLG_UFS_Effects_None, "(None)")
+dcl(DLG_UFS_Effects_UpperCase, "Uppercase")
+dcl(DLG_UFS_Effects_LowerCase, "Lowercase")
+dcl(DLG_UFS_Effects_TitleCase, "Title Case")
+dcl(DLG_UFS_Effects_SmallCaps, "Small Capitals")
 
 /* Unix FileOpenSaveAs dialog */
 dcl(DLG_FOSA_OpenTitle, 				"Open File")
@@ -175,6 +181,7 @@ dcl(DLG_InvalidPathname,				"Invalid pathname.")
 dcl(DLG_NoSaveFile_DirNotExist, 		"A directory in the given pathname does not exist.")
 dcl(DLG_NoSaveFile_DirNotWriteable, 	"The directory '%s' is write-protected.")
 dcl(DLG_OverwriteFile,					"File already exists.  Overwrite file '%s'?")
+dcl(DLG_FOSA_ExtensionDoesNotMatch,		"The given file extension does not match the chosen file type. Do you want to use this name anyway?")
 
 /* Password dialog */
 dcl(DLG_Password_Title,					"Enter Password")
@@ -270,9 +277,16 @@ dcl(DLG_CLIPART_Error, "Clip Art could not be loaded")
 
 /* About Dialog */
 dcl(DLG_ABOUT_Title,					"About %s")
+dcl(DLG_ABOUT_Description,              "%s is an Open Source application licensed under the GNU GPL.\nYou are free to redistribute this application.")
+dcl(DLG_ABOUT_Version,                  "Version: %s")
+dcl(DLG_ABOUT_Build,                    "Build options: %s")
+dcl(DLG_ABOUT_URL,                      "For more information: http://www.abisource.com/")
 
 /* image size dialog */
 dcl(DLG_Image_Title, "Image Properties")
+dcl(DLG_Image_DescTabLabel, "&Description")
+dcl(DLG_Image_WrapTabLabel, "&Wrapping")
+dcl(DLG_Image_PlacementTabLabel, "&Placement")
 dcl(DLG_Image_Width, "Width:")
 dcl(DLG_Image_Height, "Height:")
 dcl(DLG_Image_Aspect, "Preserve aspect ratio")
@@ -329,8 +343,12 @@ dcl(LANG_AR_SA, 							"Arabic (Saudi Arabia)")
 dcl(LANG_HY_AM, 							"Armenian")
 dcl(LANG_AS_IN, 							"Assamese")
 dcl(LANG_AST_ES, 							"Asturian (Spain)")
+dcl(LANG_AYM_BO, 							"Aymara (La Paz)")
+dcl(LANG_AYC_BO, 							"Aymara (Oruro)")
+dcl(LANG_AYR, 								"Central Aymara")
 dcl(LANG_EU_ES, 							"Basque")
 dcl(LANG_BE_BY, 							"Belarusian")
+dcl(LANG_BE_LATIN,							"Belarusian, Latin")
 dcl(LANG_BN_IN,                             "Bengali")
 dcl(LANG_BR_FR,								"Breton")		// Jordi 19/10/2002
 dcl(LANG_BG_BG, 							"Bulgarian")
@@ -395,11 +413,14 @@ dcl(LANG_NE_NP, 							"Nepali (Nepal)")
 dcl(LANG_NN_NO, 							"Norwegian Nynorsk")
 dcl(LANG_OC_FR,								"Occitan")		// Jordi 19/10/2002
 dcl(LANG_PL_PL, 							"Polish")
+dcl(LANG_PS,								"Pashto")
 dcl(LANG_PT_BR, 							"Portuguese (Brazil)")
 dcl(LANG_PT_PT, 							"Portuguese (Portugal)")
 dcl(LANG_PA_IN,                             "Punjabi (Gurmukhi)")
 dcl(LANG_PA_PK,                             "Punjabi (Shahmukhi)")
 dcl(LANG_QU_BO,                             "Quechua")
+dcl(LANG_QUH_BO,                            "Quechua (3 vowels)")
+dcl(LANG_QUL_BO,                            "Quechua (5 vowels)")
 dcl(LANG_RO_RO, 							"Romanian")
 dcl(LANG_RU_RU, 							"Russian (Russia)")
 dcl(LANG_SC_IT,								"Sardinian")	// Jordi 19/10/2002
@@ -422,6 +443,7 @@ dcl(LANG_UR, 								"Urdu")
 dcl(LANG_UZ_UZ, 							"Uzbek")
 dcl(LANG_VI_VN, 							"Vietnamese")
 dcl(LANG_CY_GB, 							"Welsh")
+dcl(LANG_WO_SN, 							"Wolof (Senegal)")
 dcl(LANG_YI,								"Yiddish")
 
 
