@@ -18,6 +18,7 @@
  */
 
 #include "fb_ColumnBreaker.h"
+#include "fl_DocLayout.h"
 #include "fl_SectionLayout.h"
 #include "fp_ContainerObject.h"
 #include "fp_TableContainer.h"
@@ -73,6 +74,56 @@ void fb_ColumnBreaker::setStartPage(fp_Page * pPage)
 	}
 	return;
 }
+
+/*!
+ * Returns true if Annotations are to be display
+ */
+bool fb_ColumnBreaker::_displayAnnotations(void)
+{
+  return m_pDocSec->getDocLayout()->displayAnnotations();
+}
+
+/*!
+ * Returns NULL if no rebreak is required. 
+ Otherwise returns a pointer to the page requiring the rebreak.
+*/
+fp_Page * fb_ColumnBreaker::needsRebreak(fl_DocSectionLayout * pSL)
+{
+    fl_ContainerLayout * pCL = pSL->getLastLayout();
+    fl_BlockLayout * pBL = NULL;
+    if(pCL && (pCL->getContainerType() == FL_CONTAINER_BLOCK))
+    {
+        pBL = static_cast<fl_BlockLayout *>(pCL);
+    }
+    else if(pCL)
+    {
+        pBL = pCL->getPrevBlockInDocument();
+    }
+    else
+    {
+        UT_ASSERT_HARMLESS(pCL);
+	return _getLastValidPage(pSL);
+    }
+    if(pBL)
+    {
+        fp_Line * pLine = static_cast<fp_Line *>(pBL->getLastContainer());
+	if(pLine == NULL)
+	{
+	     return _getLastValidPage(pSL);
+	}
+	fp_Page * pPage = pLine->getPage();
+	if(pPage == NULL)
+	{
+	     return _getLastValidPage(pSL);
+	}
+	else if(pLine->getY() > pPage->getHeight())
+	{
+	     return pPage;
+	}
+    }
+    return NULL;
+}
+
 	
 /*!
   Layout sections on pages
@@ -82,8 +133,51 @@ void fb_ColumnBreaker::setStartPage(fp_Page * pPage)
   blocks, and lines are laid out on the pages. Doing so it refers to
   the various layout configurations such as orphan/widow controls and
   break Runs embedded in the text.
+  At the eof the break we check to see that that all the text has been layed
+ out.
+ If it hasn't we rebreak from invalid page forward.
+ FIXME we should try to detect the error in the code rather than rebreak.
 */
+
 UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
+{
+  fp_Page * pStartPage = m_pStartPage;
+  pSL->setNeedsSectionBreak(false,m_pStartPage);
+  UT_sint32 iVal = _breakSection(pSL, pStartPage);
+  UT_sint32 icnt = 0;
+  pStartPage = needsRebreak(pSL);
+  while(pStartPage && (icnt < 10))
+  {
+      iVal = _breakSection(pSL,pStartPage);
+      pStartPage = needsRebreak(pSL);
+      icnt++;
+  }
+  FL_DocLayout * pDL = pSL->getDocLayout();
+  pDL->deleteEmptyColumnsAndPages();
+  return iVal;
+}
+
+fp_Page * fb_ColumnBreaker::_getLastValidPage(fl_DocSectionLayout * pSL)
+{
+  fp_Page * pPage = NULL;
+  fp_Page * pFoundPage = NULL;
+  UT_sint32 i = 0;
+  FL_DocLayout * pDL = pSL->getDocLayout();
+  for(i=0; i<pDL->countPages();i++)
+  {
+      pPage = pDL->getNthPage(i);
+      if(pPage->getOwningSection() == pSL)
+      {
+	  pFoundPage = pPage;
+      }
+      else if(pFoundPage != NULL)
+	break;
+  }
+  return pFoundPage;
+}
+
+
+UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * pStartPage)
 {
 	m_bReBreak = false;
 	m_pDocSec = pSL;
@@ -93,7 +187,6 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 	fp_Column* pPrevColumn = NULL;
 	xxx_UT_DEBUGMSG(("Doing ColumnBreak for section %x at page %x \n",pSL,m_pStartPage));
 //	UT_ASSERT(pSL->needsSectionBreak());
-	pSL->setNeedsSectionBreak(false,m_pStartPage);
 	pFirstLayout = pSL->getFirstLayout();
 	if (!pFirstLayout)
 	{
@@ -114,10 +207,10 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 	}
 	pCurColumn = static_cast<fp_Column*>(pSL->getFirstContainer());
 	xxx_UT_DEBUGMSG(("fb_ColumnBreaker: For DocSec %x first column %x \n",pSL,pCurColumn));
-	if(m_pStartPage)
+	if(pStartPage)
 	{
-		xxx_UT_DEBUGMSG(("Layout from page %x \n",m_pStartPage));
-		pCurColumn = m_pStartPage->getNthColumnLeader(0);
+		xxx_UT_DEBUGMSG(("Layout from page %x \n",pStartPage));
+		pCurColumn = pStartPage->getNthColumnLeader(0);
 		pOuterContainer = pCurColumn->getFirstContainer();
 		if(pOuterContainer && pOuterContainer->getContainerType() == FP_CONTAINER_LINE)
 		{
@@ -153,6 +246,8 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 //
 // check and update wrapped text around positioned objects
 //
+			        UT_sint32 iPage = m_pDocSec->getDocLayout()->findPage(pPrevPage);
+				m_pDocSec->getDocLayout()->setFramePageNumbers(iPage);
 				fp_Container * pNextContainer = pPrevPage->updatePageForWrapping(pCurColumn);
 				xxx_UT_DEBUGMSG(("Returned container updatePage %x \n",pNextContainer));
 				if(pNextContainer != NULL)
@@ -212,7 +307,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 		while (pCurContainer)
 		{
 			xxx_UT_DEBUGMSG(("curContainer %x type %d \n",pCurContainer,pCurContainer->getContainerType()));
-			if(pCurContainer->getContainerType() == FP_CONTAINER_FOOTNOTE)
+			if(pCurContainer->getContainerType() == FP_CONTAINER_FOOTNOTE || ((pCurContainer->getContainerType() == FP_CONTAINER_ANNOTATION)))
 			{
 //
 // skip this! We've already taken it's height into account.
@@ -280,7 +375,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 					pCurContainer = _getNext(pCurContainer);
 					continue;
 				}
-				// Excellent.  If we have a footnote, we can start deducting
+				// Excellent.  If we have a footnote or annotation, we can start deducting
 				// from the working height if the footnote container is not on
 				// this page.
 				if (pCurLine->containsFootnoteReference())
@@ -288,12 +383,13 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 					// Ok.  Now, deduct the proper amount from iMaxColHeight.
 
 					// OK get a vector of the footnote containers in this line.
+				 
 					UT_GenericVector<fp_FootnoteContainer*> vecFootnotes;
 					pCurLine->getFootnoteContainers(&vecFootnotes);
 					fp_Page *pCurPage = pCurColumn->getPage();
 					// Now loop through all these and add them to the height.
 					UT_sint32 i =0;
-					for(i=0; i< static_cast<UT_sint32>(vecFootnotes.getItemCount());i++)
+					for(i=0; i< vecFootnotes.getItemCount();i++)
 					{
 						fp_FootnoteContainer * pFC = vecFootnotes.getNthItem(i);
 						if(pFC && ((pFC->getPage() == NULL) || (pFC->getPage() != pCurPage)))
@@ -304,6 +400,29 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 					UT_DEBUGMSG(("got footnote section height %d\n", iFootnoteHeight));
 					iWorkingColHeight += iFootnoteHeight;
 				}
+				if(pCurLine->containsAnnotations() && _displayAnnotations())
+				  {
+					// Ok.  Now, deduct the proper amount from iMaxColHeight.
+
+					// OK get a vector of the Annotation containers in this line.
+				 
+					UT_GenericVector<fp_AnnotationContainer*> vecAnnotations;
+					pCurLine->getAnnotationContainers(&vecAnnotations);
+					fp_Page *pCurPage = pCurColumn->getPage();
+					// Now loop through all these and add them to the height.
+					UT_sint32 i =0;
+					for(i=0; i< vecAnnotations.getItemCount();i++)
+					{
+						fp_AnnotationContainer * pAC = vecAnnotations.getNthItem(i);
+						if(pAC && ((pAC->getPage() == NULL) || (pAC->getPage() != pCurPage)))
+						{
+							iFootnoteHeight += pAC->getHeight();
+						}				
+					}	
+					UT_DEBUGMSG(("got Total section height %d\n", iFootnoteHeight));
+					iWorkingColHeight += iFootnoteHeight;
+
+				  }
 			}
 
 			if ((iWorkingColHeight + iTotalContainerSpace) > iMaxColHeight)
@@ -352,7 +471,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 						UT_ASSERT(pCurContainer->getContainerType() == FP_CONTAINER_TOC);
 						fp_TOCContainer * pTOCOffend = static_cast<fp_TOCContainer *>(pOffendingContainer);
 						pLastContainerToKeep = pTOCOffend->getPrevContainerInSection();
-					    UT_DEBUGMSG(("Can't break TOC. pCurContainer %x pTabOffend %x pLastContainerToKeep %x \n",pCurContainer,pTOCOffend,pLastContainerToKeep));
+					    UT_DEBUGMSG(("Can't break TOC. pCurContainer %p pTabOffend %p pLastContainerToKeep %p \n",pCurContainer,pTOCOffend,pLastContainerToKeep));
 						break;
 					}
 				}
@@ -625,7 +744,8 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 		bEquivColumnBreak = bEquivColumnBreak && ( iMaxColHeight < (iWorkingColHeight + iTotalContainerSpace));
 		if (pLastContainerToKeep)
 		{
-			while(pLastContainerToKeep && (pLastContainerToKeep->getContainerType() == FP_CONTAINER_FOOTNOTE ))
+			while(pLastContainerToKeep && (pLastContainerToKeep->getContainerType() == FP_CONTAINER_FOOTNOTE )
+			      && (pLastContainerToKeep->getContainerType() == FP_CONTAINER_ANNOTATION))
 			{
 				pLastContainerToKeep = pLastContainerToKeep->getPrevContainerInSection();
 			}
@@ -646,11 +766,13 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 		xxx_UT_DEBUGMSG(("Doing column fill now pCurContainer %x pFirstContainer %x \n",pCurContainer,pFirstContainerToKeep));
 		pCurContainer = pFirstContainerToKeep;
 		fp_TableContainer * pTab = static_cast<fp_TableContainer *>(pFirstContainerToKeep);
+		UT_UNUSED(pTab); // only for debug.
 		UT_sint32 conPos = 0;
 		while (pCurContainer)
 		{
 			xxx_UT_DEBUGMSG(("Container %x is in Column %x Type %d numCons %d \n",pCurContainer,pCurColumn,pCurContainer->getContainerType(),pCurColumn->countCons()));
-			if(pCurContainer->getContainerType() == FP_CONTAINER_FOOTNOTE)
+			if(pCurContainer->getContainerType() == FP_CONTAINER_FOOTNOTE  || 
+			   (pCurContainer->getContainerType() == FP_CONTAINER_ANNOTATION) )
 			{
 //
 // Skip this. It doesn't go in this column at all.
@@ -710,7 +832,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 				}
 			}
 //
-// Now make sure footnotes are on the same page as the reference.
+// Now make sure footnotes and annotations are on the same page as the reference.
 //
 			if(pCurContainer->getContainerType() == FP_CONTAINER_LINE)
 			{
@@ -727,7 +849,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 					fp_Page * pCurPage = pCurColumn->getPage();
 					UT_ASSERT(pCurPage);
 					UT_sint32 i =0;
-					for(i=0; i< static_cast<UT_sint32>(vecFootnotes.getItemCount());i++)
+					for(i=0; i< vecFootnotes.getItemCount();i++)
 					{
 						fp_FootnoteContainer * pFC = vecFootnotes.getNthItem(i);
 						if(pFC != NULL)
@@ -748,6 +870,40 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 							}
 						}
 					}
+
+				}
+				if(pCurLine->containsAnnotations() && _displayAnnotations())
+				{
+					// OK get a vector of the footnote containers in this line.
+					UT_GenericVector<fp_AnnotationContainer*> vecAnnotations;
+					pCurLine->getAnnotationContainers(&vecAnnotations);
+				
+					// Now loop through all these and check they're on this
+					// page. If not add them.
+					fp_Page * pCurPage = pCurColumn->getPage();
+					UT_ASSERT(pCurPage);
+					UT_sint32 i =0;
+					for(i=0; i< vecAnnotations.getItemCount();i++)
+					{
+						fp_AnnotationContainer * pAC = vecAnnotations.getNthItem(i);
+						if(pAC != NULL)
+						{
+							fp_Page * myPage = pAC->getPage();
+							xxx_UT_DEBUGMSG(("Annotation %x is on Page %x \n",pAC,myPage));
+							if(myPage != pCurPage)
+							{
+								if(myPage == NULL)
+								{
+									pCurPage->insertAnnotationContainer(pAC);
+								}
+								else
+								{
+									myPage->removeAnnotationContainer(pAC);
+									pCurPage->insertAnnotationContainer(pAC);
+								}
+							}
+						}
+					}
 				}
 			}
 //
@@ -773,7 +929,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 						fp_Page * pCurPage = pCurColumn->getPage();
 						UT_ASSERT(pCurPage);
 						UT_sint32 i =0;
-						for(i=0; i< static_cast<UT_sint32>(vecFootnotes.getItemCount());i++)
+						for(i=0; i< vecFootnotes.getItemCount();i++)
 						{
 							xxx_UT_DEBUGMSG(("Found reference %d in broken table %x \n",i,pCurTable));
 							fp_FootnoteContainer * pFC = vecFootnotes.getNthItem(i);
@@ -794,6 +950,38 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 							}
 						}
 					}
+					if(pCurTable->containsAnnotations())
+					{
+						// OK get a vector of the footnote containers in this line.
+						UT_GenericVector<fp_AnnotationContainer*> vecAnnotations;
+						pCurTable->getAnnotationContainers(&vecAnnotations);
+						
+					// Now loop through all these and check they're on this
+					// page. If not add them.
+						fp_Page * pCurPage = pCurColumn->getPage();
+						UT_ASSERT(pCurPage);
+						UT_sint32 i =0;
+						for(i=0; i< vecAnnotations.getItemCount();i++)
+						{
+							xxx_UT_DEBUGMSG(("Found reference %d in broken table %x \n",i,pCurTable));
+							fp_AnnotationContainer * pAC = vecAnnotations.getNthItem(i);
+							fp_Page * myPage = pAC->getPage();
+							xxx_UT_DEBUGMSG(("Annotation %x is on Page %x \n",pAC,myPage));
+							if(myPage != pCurPage)
+							{
+								xxx_UT_DEBUGMSG((" Moving anchor from %x to %x \n",myPage,pCurPage));
+								if(myPage == NULL)
+								{
+									pCurPage->insertAnnotationContainer(pAC);
+								}
+								else
+								{
+									myPage->removeAnnotationContainer(pAC);
+									pCurPage->insertAnnotationContainer(pAC);
+								}
+							}
+						}
+					}
 				}
 			}
 //
@@ -819,8 +1007,8 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 				if((pLastContainerToKeep!=NULL) && (_getNext(pCurContainer) == NULL))
 				{
 					UT_DEBUGMSG(("Non null LastContainerToKeep yet next container is NULL!!!!!!!!!!!! \n"));
-					UT_DEBUGMSG((" CurContainer %x type %d \n",pCurContainer,pCurContainer->getContainerType()));
-					UT_DEBUGMSG((" FirstContainer to keep %x Last container to keep %x \n",pTab,pLastContainerToKeep));
+					UT_DEBUGMSG((" CurContainer %p type %d \n",pCurContainer,pCurContainer->getContainerType()));
+					UT_DEBUGMSG((" FirstContainer to keep %p Last container to keep %p \n",pTab,pLastContainerToKeep));
 					UT_DEBUGMSG(("Try to recover.... \n"));
 					//	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 					pLastContainerToKeep = NULL;
@@ -838,7 +1026,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 				UT_DEBUGMSG(("4111 bug \n"));
 			}
 			UT_ASSERT(pLastContainerToKeep->getColumn()==pCurColumn);
-			fp_Page* pPrevPage = pCurColumn->getPage();
+			fp_Page* pPrevPage2 = pCurColumn->getPage();
 			fp_Column* pNextColumn = pCurColumn;
 
 			do
@@ -866,7 +1054,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 				             == FP_CONTAINER_LINE &&
 				   static_cast<fp_Line *>(pLastContainerToKeep)
 				        ->containsForcedPageBreak() &&
-				   (pNextColumn->getPage() == pPrevPage));
+				   (pNextColumn->getPage() == pPrevPage2));
 			if(pCurColumn != NULL && (pCurColumn == pNextColumn))
 			{
 				pCurColumn->layout();
@@ -995,6 +1183,8 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 		{
 			UT_ASSERT(pOuterContainer == NULL);
 			fp_Page * pPage = pPrevColumn->getPage();
+			UT_sint32 iPage = m_pDocSec->getDocLayout()->findPage(pPage);
+			m_pDocSec->getDocLayout()->setFramePageNumbers(iPage);
 			fp_Container * pNextContainer = pPage->updatePageForWrapping(pCurColumn);
 			xxx_UT_DEBUGMSG(("Returned container updatePage %x \n",pNextContainer));
 			if(pNextContainer == NULL)
@@ -1215,7 +1405,7 @@ bool fb_ColumnBreaker::_breakTOC(fp_Container*& pOffendingContainer,
 {
 	bool bDoTOCBreak;
 
-    UT_DEBUGMSG(("breakTOC:!!!!!!!!!!!! Offending TOC is %x \n",pOffendingContainer));
+    UT_DEBUGMSG(("breakTOC:!!!!!!!!!!!! Offending TOC is %p \n",pOffendingContainer));
 	fp_TOCContainer * pTOC = static_cast<fp_TOCContainer *>(pOffendingContainer);
 	if(!pTOC->isThisBroken())
 	{
@@ -1305,7 +1495,8 @@ bool fb_ColumnBreaker::_breakTOC(fp_Container*& pOffendingContainer,
 		    xxx_UT_DEBUGMSG(("SEVIOR: Created broken table %x \n",pOffendingContainer));
 			fp_TOCContainer * pNewTOC = static_cast<fp_TOCContainer *>(pOffendingContainer);
 			UT_ASSERT(pBroke->getHeight() > 0);
-			UT_ASSERT(pNewTOC->getHeight() > 0);
+			UT_UNUSED(pNewTOC);
+			UT_ASSERT(pNewTOC && pNewTOC->getHeight() > 0);
 			pLastContainerToKeep = static_cast<fp_Container *>(pTOC);
 			xxx_UT_DEBUGMSG(("SEVIOR: Set lasttokeep 1 %x \n",pLastContainerToKeep));
 		}
