@@ -32,7 +32,8 @@
 #include "pd_Document.h"
 #include "pt_PieceTable.h"
 #include "pf_Frag.h"
-
+#include "xap_App.h"
+#include "xap_Frame.h"
 #include "ut_string.h"
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
@@ -154,9 +155,11 @@ void fl_AutoNum::fixHierarchy(void)
 	if(m_pItems.getItemCount() >0)
 	{
 		PL_StruxDocHandle sdh = static_cast<PL_StruxDocHandle>(m_pItems.getNthItem(0));
+		XAP_Frame * pFrame = XAP_App::getApp()->getLastFocussedFrame();
+		FV_View * pView = static_cast<FV_View *>(pFrame->getCurrentView());
 		bool bFound = m_pDoc->getAttributeFromSDH(sdh,
-												  m_pView ? m_pView->isShowRevisions() : true,
-												  m_pView ? m_pView->getRevisionLevel(): PD_MAX_REVISION,
+												  pView ? pView->isShowRevisions() : true,
+												  pView ? pView->getRevisionLevel(): PD_MAX_REVISION,
 												  PT_PARENTID_ATTRIBUTE_NAME,&pszParentID);
 		if(bFound)
 		{
@@ -307,7 +310,7 @@ void    fl_AutoNum::findAndSetParentItem(void)
 			fl_AutoNum * pParent = m_pDoc->getNthList(iList);
 			UT_uint32 i=0;
 			PL_StruxDocHandle pParentItem = pParent->getNthBlock(i);
-			PT_DocPosition posParent=0;
+			posParent=0;
 			if(pParentItem != NULL)
 			{
 				posParent = m_pDoc->getStruxPosition(pParentItem);
@@ -324,7 +327,7 @@ void    fl_AutoNum::findAndSetParentItem(void)
 			if( i > 0)
 			{
 				i--;
-				pParentItem = pParent->getNthBlock(i);;
+				pParentItem = pParent->getNthBlock(i);
 				posParent = m_pDoc->getStruxPosition(pParentItem);
 				if( posParent > posClosest)
 				{
@@ -418,7 +421,7 @@ void    fl_AutoNum::_getLabelstr( UT_UCSChar labelStr[], UT_uint32 * insPoint,
 	leftDelim[i] = '\0';
 	i += 2;
 	rTmp = i;
-	while (i < G_N_ELEMENTS(p) && p[i] || p[i] != '\0')
+	while ((i < G_N_ELEMENTS(p)) && p[i])
 	{
 		// FIXME check the bounds to not overflow rightDelim
 		// Update: the arrays are now the same length to prevent overflows
@@ -676,7 +679,7 @@ UT_uint32 fl_AutoNum::getStartValue32() const
 	return m_iStartValue;
 }
 
-void fl_AutoNum::insertFirstItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pLast, UT_uint32 depth, bool bDoFix)
+void fl_AutoNum::insertFirstItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pLast, UT_uint32 /*depth*/, bool bDoFix)
 {
 	UT_sint32 i = -1;
 	if(m_pItems.getItemCount() > 0)
@@ -695,6 +698,8 @@ void fl_AutoNum::insertFirstItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pLas
 		m_pParentItem = pLast;
 		m_bDirty = true;
 	}
+	if(m_pDoc->areListUpdatesAllowed() == false)
+		return;
 	if ( getAutoNumFromSdh(pItem) == this)
 		_updateItems(0,NULL);
 }
@@ -846,7 +851,7 @@ UT_uint32 fl_AutoNum::getNumLabels() const
 	return m_pItems.getItemCount();
 }
 
-UT_sint32 fl_AutoNum::getPositionInList(PL_StruxDocHandle pItem, UT_uint32 depth) const
+UT_sint32 fl_AutoNum::getPositionInList(PL_StruxDocHandle pItem, UT_uint32 /*depth*/) const
 {
 	UT_ASSERT(m_pItems.getItemCount() > 0);
 
@@ -960,7 +965,7 @@ PL_StruxDocHandle fl_AutoNum::getLastItemInHeiracy(void)
        return pLastItem;
 }
 
-const bool fl_AutoNum::isItem(PL_StruxDocHandle pItem) const
+bool fl_AutoNum::isItem(PL_StruxDocHandle pItem) const
 {
 	if (m_pItems.findItem(const_cast<void *>(pItem)) == -1)
 		return false;
@@ -968,7 +973,7 @@ const bool fl_AutoNum::isItem(PL_StruxDocHandle pItem) const
 		return true;
 }
 
-const bool fl_AutoNum::isEmpty()
+bool fl_AutoNum::isEmpty()
 {
 	if (m_pItems.getItemCount() > 0)
 		return false;
@@ -1025,7 +1030,7 @@ bool fl_AutoNum::isLastOnLevel(PL_StruxDocHandle pItem)
 	UT_sint32 itemloc = m_pItems.findItem(const_cast<void *>(pItem));
 	if (itemloc == -1)
 		return false;
-	if(itemloc == static_cast<UT_sint32>(m_pItems.getItemCount() - 1))
+	if(itemloc == m_pItems.getItemCount() - 1)
 		return true;
 	else
 		return false;
@@ -1090,7 +1095,7 @@ void fl_AutoNum::_setParent(fl_AutoNum * pParent)
 
 		sprintf(szParent,"%d",m_iParentID);
 		m_bDirty = true;
-		UT_uint32 i = 0;
+		UT_sint32 i = 0;
 		for(i=0; i < m_pItems.getItemCount() ; i++)
 		{
 			PL_StruxDocHandle sdh = static_cast<PL_StruxDocHandle>(m_pItems.getNthItem(i));
@@ -1121,7 +1126,7 @@ void fl_AutoNum::update(UT_uint32 start)
 	}
 }
 
-void fl_AutoNum::_updateItems(UT_uint32 start, PL_StruxDocHandle notMe)
+void fl_AutoNum::_updateItems(UT_sint32 start, PL_StruxDocHandle notMe)
 {
 	//	UT_DEBUGMSG(("Entering _updateItems\n"));
 	UT_sint32 j;
@@ -1134,7 +1139,7 @@ void fl_AutoNum::_updateItems(UT_uint32 start, PL_StruxDocHandle notMe)
 		//	}
 		UT_sint32 numlists = m_pDoc->getListsCount();
 		m_bUpdatingItems = true;
-		for (UT_uint32 i = start; i < m_pItems.getItemCount(); i++)
+		for (UT_sint32 i = start; i < m_pItems.getItemCount(); i++)
 		{
 			//       	 	UT_DEBUGMSG(("Entering _updateItems for loop\n"));
 			PL_StruxDocHandle pTmp = static_cast<PL_StruxDocHandle>(m_pItems.getNthItem(i));
@@ -1187,9 +1192,9 @@ bool fl_AutoNum::isContainedByList(PL_StruxDocHandle pItem)
 }
 
 
-PL_StruxDocHandle fl_AutoNum::getNthBlock( UT_uint32 list_num)
+PL_StruxDocHandle fl_AutoNum::getNthBlock( UT_sint32 list_num)
 {
-	if(list_num <0 || list_num >= m_pItems.getItemCount())
+	if(list_num >= m_pItems.getItemCount())
 		return static_cast<PL_StruxDocHandle>(NULL);
 	else
 		return static_cast<PL_StruxDocHandle>(m_pItems.getNthItem(list_num));
