@@ -23,6 +23,10 @@
 #ifndef FL_BLOCKLAYOUT_H
 #define FL_BLOCKLAYOUT_H
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #ifdef FMT_TEST
 #include <stdio.h>
 #endif
@@ -99,7 +103,7 @@ typedef enum {
 
 class SpellChecker;
 class fl_TabStop;
-void buildTabStops(GR_Graphics * pG, const char* pszTabStops, UT_GenericVector<fl_TabStop*> &m_vecTabs);
+void buildTabStops(const char* pszTabStops, UT_GenericVector<fl_TabStop*> &m_vecTabs);
 
 class ABI_EXPORT fl_BlockLayout : public fl_ContainerLayout
 {
@@ -110,7 +114,7 @@ class ABI_EXPORT fl_BlockLayout : public fl_ContainerLayout
 	friend class fl_TOCLayout;
 	friend class fb_LineBreaker;
 
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 	// TODO: shack - code should be moved from toggleAuto to a function in
 	// here - to handle the squiggles
 	friend void FL_DocLayout::_toggleAutoSpell(bool bSpell);
@@ -143,7 +147,7 @@ public:
 	virtual bool		recalculateFields(UT_uint32 iUpdateCount);
 
 	virtual void		redrawUpdate();
-	virtual void        updateLayout(bool bDoAll) {}
+	virtual void        updateLayout(bool /*bDoAll*/) {}
 	virtual fp_Container * getNewContainer(fp_Container * pCon = NULL);
 	FV_View *		getView(void) const { 
 		UT_return_val_if_fail( m_pLayout, NULL ); 
@@ -154,47 +158,48 @@ public:
 	const PP_PropertyType * getPropertyType(const gchar * szName, 
 											tProperty_type Type, bool bExpandStyles = true) const;
 	void setAlignment(UT_uint32 iAlignCmd);
-	UT_sint32       getLength(void);
-	bool            isEmbeddedType(void);
-	bool            isNotTOCable(void);
-	bool            isLastRunInBlock(fp_Run * pRun);
+	UT_sint32       getLength(void) const;
+	bool            isEmbeddedType(void) const;
+	bool            isNotTOCable(void) const;
+	bool            isLastRunInBlock(fp_Run * pRun) const;
 	void            updateOffsets(PT_DocPosition posEmbedded, 
 								  UT_uint32 iEmebbedSize, UT_sint32 iSuggestedDiff);
 	void            updateEnclosingBlockIfNeeded(void);
-	fl_BlockLayout * getEnclosingBlock(void);
-	UT_sint32       getEmbeddedOffset(UT_sint32 startOffset, fl_ContainerLayout *& pEmbedCL);
+	fl_BlockLayout * getEnclosingBlock(void) const;
+	UT_sint32       getEmbeddedOffset(UT_sint32 startOffset, fl_ContainerLayout *& pEmbedCL) const;
 	void            shuffleEmbeddedIfNeeded(fl_BlockLayout * pBlock, UT_uint32 blockOffset);
 
-	bool            getXYOffsetToLine(UT_sint32 & xoff, UT_sint32 & yoff, fp_Line * pLine);
+	bool            getXYOffsetToLine(UT_sint32 & xoff, UT_sint32 & yoff, fp_Line * pLine) const;
 	bool            setFramesOnPage(fp_Line * pLastLine);
-	UT_sint32       getMinWrapWidth(void);
+	UT_sint32       getMinWrapWidth(void) const;
 	UT_sint32       getHeightOfBlock(void);
-	fp_Line *       findLineWithFootnotePID(UT_uint32 pid);
-	UT_sint32 getMaxNonBreakableRun(void);
-	fp_Line* findPrevLineInDocument(fp_Line*);
-	fp_Line* findNextLineInDocument(fp_Line*);
-	virtual void     appendTextToBuf(UT_GrowBuf & buf);
-	void             appendUTF8String(UT_UTF8String & sText);
+	fp_Line *       findLineWithFootnotePID(UT_uint32 pid) const;
+	UT_sint32 getMaxNonBreakableRun(void) const;
+	fp_Line* findPrevLineInDocument(fp_Line*) const;
+	fp_Line* findNextLineInDocument(fp_Line*) const;
+	virtual void     appendTextToBuf(UT_GrowBuf & buf) const;
+	void             appendUTF8String(UT_UTF8String & sText) const;
 	virtual fp_Run* getFirstRun(void) const { return m_pFirstRun; }
 	inline void setFirstRun(fp_Run* pRun) { m_pFirstRun = pRun; }
-
+	void        clearPrint(void) const;
 	inline bool isListItem(void) const { return m_bListItem; }
-	bool isFirstInList(void);
+	bool isFirstInList(void) const;
 //	inline fl_AutoNum * getAutoNum(void) const { return m_pAutoNum; }
-	void	getListAttributesVector(UT_GenericVector<const gchar*> * va);
-	void  getListPropertyVector(UT_GenericVector<const gchar*> * vp);
+	void	getListAttributesVector(UT_GenericVector<const gchar*> * va) const;
+	void  getListPropertyVector(UT_GenericVector<const gchar*> * vp) const;
 
-	char *	getFormatFromListType(FL_ListType iListType);
+	void  refreshRunProperties(void) const;
+	char *	getFormatFromListType(FL_ListType iListType) const;
 	void remItemFromList(void);
 	virtual void listUpdate(void);
 	void resumeList( fl_BlockLayout * prevList);
 	void prependList( fl_BlockLayout * nextList);
-	FL_ListType decodeListType(char * listformat);
-	FL_ListType getListType(void);
-	gchar* getListStyleString( FL_ListType iListType);
-	FL_ListType getListTypeFromStyle( const gchar * style);
-	fl_BlockLayout * getNextList(UT_uint32 id);
-	bool isListLabelInBlock(void);
+	FL_ListType decodeListType(char * listformat) const;
+	FL_ListType getListType(void) const;
+	gchar* getListStyleString( FL_ListType iListType) const;
+	FL_ListType getListTypeFromStyle( const gchar * style) const;
+	fl_BlockLayout * getNextList(UT_uint32 id) const;
+	bool isListLabelInBlock(void) const;
 	void StartList( const gchar * style, PL_StruxDocHandle prevSDH = NULL);
 
 	void StartList( FL_ListType lType, UT_uint32 start,
@@ -204,20 +209,20 @@ public:
 
 	void StopListInBlock(void);
 	void deleteListLabel(void);
-	UT_UCSChar * getListLabel(void);
+	UT_UCSChar * getListLabel(void) const;
 	void transferListFlags(void);
-	UT_uint32 getLevel(void);
+	UT_uint32 getLevel(void) const;
 	void setStarting( bool bValue);
 	void setStopping( bool bValue);
-	fl_BlockLayout * getPreviousList(UT_uint32 id);
-	fl_BlockLayout * getPreviousList(void);
-	fl_BlockLayout * getPreviousListOfSameMargin(void);
-	inline fl_BlockLayout * getParentItem(void);
+	fl_BlockLayout * getPreviousList(UT_uint32 id) const;
+	fl_BlockLayout * getPreviousList(void) const;
+	fl_BlockLayout * getPreviousListOfSameMargin(void) const;
+	fl_BlockLayout * getParentItem(void) const;
 
-#ifndef WITHOUT_SPELL
-	void findSpellSquigglesForRun(fp_Run* pRun);
-	void drawGrammarSquiggles(void);
-	void findGrammarSquigglesForRun(fp_Run* pRun);
+#ifdef ENABLE_SPELL
+	void findSpellSquigglesForRun(fp_Run* pRun) const;
+	void drawGrammarSquiggles(void) const;
+	void findGrammarSquigglesForRun(fp_Run* pRun) const;
 #endif
 	
 	UT_uint32 canSlurp(fp_Line* pLine) const;
@@ -225,16 +230,16 @@ public:
 	PT_DocPosition getPosition(bool bActualBlockPos=false) const;
 	fp_Run* findPointCoords(PT_DocPosition position, bool bEOL, 
 							UT_sint32& x, UT_sint32& y, UT_sint32& x2, 
-							UT_sint32& y2, UT_sint32& height, bool& bDirection);
+							UT_sint32& y2, UT_sint32& height, bool& bDirection) const;
 
 	fp_Run* findRunAtOffset(UT_uint32 offset) const;
 	
 	bool	getBlockBuf(UT_GrowBuf * pgb) const;
 
-	void clearScreen(GR_Graphics*);
+	void clearScreen(GR_Graphics*) const;
 
-	void                getStyle(UT_UTF8String & sStyle);
-	inline UT_sint32	getTextIndent(void) const { return m_iTextIndent; }
+	void                getStyle(UT_UTF8String & sStyle) const;
+	UT_sint32	        getTextIndent(void) const;
 	inline UT_sint32	getLeftMargin(void) const { return m_iLeftMargin; }
 	inline UT_sint32	getRightMargin(void) const { return m_iRightMargin; }
 	inline UT_sint32	getTopMargin(void) const { return m_iTopMargin; }
@@ -257,30 +262,30 @@ public:
 	inline UT_BidiCharType getDominantDirection(void) const { return m_iDomDirection; }
 	void setDominantDirection(UT_BidiCharType iDirection);
 
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 	inline fl_SpellSquiggles* getSpellSquiggles(void) const { return m_pSpellSquiggles; }
 	inline fl_GrammarSquiggles* getGrammarSquiggles(void) const { return  m_pGrammarSquiggles; }
 #endif
 
-	bool isHdrFtr(void);
+	bool isHdrFtr(void) const;
 	void setHdrFtr(void) { m_bIsHdrFtr = true;}
 	void clearHdrFtr(void) { m_bIsHdrFtr = false;}
 
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 	bool checkSpelling(void);
 #endif
 	void debugFlashing(void);
 	bool	findNextTabStop(UT_sint32 iStartX, UT_sint32 iMaxX,
 							UT_sint32& iPosition, eTabType& iType,
-							eTabLeader &iLeader );
+							eTabLeader &iLeader ) const;
 	bool	findPrevTabStop(UT_sint32 iStartX, UT_sint32 iMaxX,
 							UT_sint32& iPosition, eTabType& iType,
-							eTabLeader &iLeader );
+							eTabLeader &iLeader ) const;
 	bool    hasUpdatableField(void) { return m_bHasUpdatableField;}
 	void    setUpdatableField(bool bValue) { m_bHasUpdatableField = bValue;}
 	inline UT_sint32 getDefaultTabInterval(void) const { return m_iDefaultTabInterval; }
 	inline UT_sint32 getTabsCount(void) const { 
-		return static_cast<UT_sint32>(m_vecTabs.getItemCount()); 
+		return m_vecTabs.getItemCount(); 
 	}
 
 	bool doclistener_populateSpan(const PX_ChangeRecord_Span * pcrs, PT_BlockOffset blockOffset, UT_uint32 len);
@@ -338,7 +343,7 @@ public:
 	virtual void			collapse(void);
 	virtual bool			isCollapsed(void) const
 		{return m_bIsCollapsed;}
-	void					coalesceRuns(void);
+	void					coalesceRuns(void) const;
 	virtual void			setNeedsReformat(fl_ContainerLayout * pCL, UT_uint32 offset = 0);
 	inline bool 		    needsReformat(void) const
 		{ return (m_iNeedsReformat >= 0); }
@@ -346,12 +351,12 @@ public:
 	virtual bool 		    needsRedraw(void) const
 		{ return m_bNeedsRedraw; }
 	virtual void			markAllRunsDirty(void);
-	UT_sint32               findLineInBlock(fp_Line * pLine);
+	UT_sint32               findLineInBlock(fp_Line * pLine) const;
 	
-	bool                    isWordDelimiter(UT_UCS4Char c, UT_UCS4Char next, UT_UCS4Char prev, UT_uint32 iBlockPos);
-	bool                    isSentenceSeparator(UT_UCS4Char c, UT_uint32 iBlockPos);
-#ifndef WITHOUT_SPELL
-	bool					checkWord(fl_PartOfBlock* pPOB);
+	bool                    isWordDelimiter(UT_UCS4Char c, UT_UCS4Char next, UT_UCS4Char prev, UT_uint32 iBlockPos) const;
+	bool                    isSentenceSeparator(UT_UCS4Char c, UT_uint32 iBlockPos) const;
+#ifdef ENABLE_SPELL
+	bool					checkWord(fl_PartOfBlock* pPOB) const;
 	void					recheckIgnoredWords();
 #endif
 	void                    setStyleInTOC(bool b)
@@ -359,9 +364,9 @@ public:
 	void                    forceSectionBreak(void);
 	bool                    isContainedByTOC(void) const
 	    { return m_bIsTOC;}
-	FootnoteType            getTOCNumType(void);
-	eTabLeader              getTOCTabLeader(UT_sint32 iOff);
-	UT_sint32               getTOCTabPosition(UT_sint32 iOff);
+	FootnoteType            getTOCNumType(void) const;
+	eTabLeader              getTOCTabLeader(UT_sint32 iOff) const;
+	UT_sint32               getTOCTabPosition(UT_sint32 iOff) const;
 	void                    setAccumHeight(UT_sint32 i)
 	{ m_iAccumulatedHeight =i;}
 	UT_sint32               getAccumHeight(void) const
@@ -382,9 +387,9 @@ public:
 												PT_DocPosition & begPos,
 												PT_DocPosition & endPos,
 												UT_UTF8String & sWord,
-												bool bIgnoreSpace);
+												bool bIgnoreSpace) const;
 
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 	/** put in queue for spellchecking after prev. If prev == NULL is put at the head */
 	void enqueueToSpellCheckAfter(fl_BlockLayout *prev);
 	/** remove from the spellchecking queue */
@@ -415,26 +420,26 @@ private:
 	
 protected:
 
-	void					_recalcPendingWord(UT_uint32 iOffset, UT_sint32 chg);
+	void					_recalcPendingWord(UT_uint32 iOffset, UT_sint32 chg) const;
 	bool					_doCheckWord(fl_PartOfBlock* pPOB,
 										 const UT_UCSChar* pBlockText,
 										 UT_sint32 iLength,
 										 bool bAddSquiggle = true,
-										 bool bClearScreen = true);
+										 bool bClearScreen = true) const;
 
-#ifndef WITHOUT_SPELL
-	bool					_spellCheckWord(const UT_UCSChar * word, UT_uint32 len, UT_uint32 blockPos);
-	SpellChecker * _getSpellChecker (UT_uint32 blockPos);
+#ifdef ENABLE_SPELL
+	bool					_spellCheckWord(const UT_UCSChar * word, UT_uint32 len, UT_uint32 blockPos) const;
+	SpellChecker * _getSpellChecker (UT_uint32 blockPos) const;
 #endif
 	
 	bool					_truncateLayout(fp_Run* pTruncRun);
 
 #ifndef NDEBUG
-	void					_assertRunListIntegrityImpl(void);
+	void					_assertRunListIntegrityImpl(void) const;
 #endif
-	inline void 			_assertRunListIntegrity(void);
+	void             			_assertRunListIntegrity(void) const;
 
-	void					_mergeRuns(fp_Run* pFirstRunToMerge, fp_Run* pLastRunToMerge);
+	void					_mergeRuns(fp_Run* pFirstRunToMerge, fp_Run* pLastRunToMerge) const;
 
 	bool					_doInsertRun(fp_Run* pNewRun);
 	bool					_delete(PT_BlockOffset blockOffset, UT_uint32 len);
@@ -444,6 +449,7 @@ protected:
 	bool					_doInsertFieldEndRun(PT_BlockOffset blockOffset);
 	bool					_doInsertBookmarkRun(PT_BlockOffset blockOffset);
 	bool					_doInsertHyperlinkRun(PT_BlockOffset blockOffset);
+	bool					_doInsertAnnotationRun(PT_BlockOffset blockOffset);
 	bool					_doInsertMathRun(PT_BlockOffset blockOffset,PT_AttrPropIndex indexAP,PL_ObjectHandle oh);
 	bool					_doInsertEmbedRun(PT_BlockOffset blockOffset,PT_AttrPropIndex indexAP,PL_ObjectHandle oh);
 //	bool					_deleteBookmarkRun(PT_BlockOffset blockOffset);
@@ -468,7 +474,7 @@ protected:
 
 	bool					_checkMultiWord(UT_sint32 iStart,
 											UT_sint32 eor,
-											bool bToggleIP);
+											bool bToggleIP) const;
 
 	UT_uint32				_getLastChar();
 	void					_stuffAllRunsOnALine(void);
@@ -476,7 +482,7 @@ protected:
 	void					_purgeEndOfParagraphRun(void);
 	void					_breakLineAfterRun(fp_Run* /*pRun*/);
 
-	static void 			_prefsListener(XAP_App * /*pApp*/, XAP_Prefs *pPrefs, UT_StringPtrMap * /*phChanges*/, void * data);
+	static void 			_prefsListener(XAP_Prefs *pPrefs, UT_StringPtrMap * /*phChanges*/, void * data);
 
 	void					_createListLabel(void);
 	void					_deleteListLabel(void);
@@ -514,7 +520,7 @@ protected:
 	bool                    m_bStartList;
 	bool                    m_bStopList;
     bool                    m_bListLabelCreated;
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 	fl_SpellSquiggles *     m_pSpellSquiggles;
 	fl_GrammarSquiggles *   m_pGrammarSquiggles;
 	fl_BlockLayout          *m_nextToSpell;
@@ -566,7 +572,7 @@ public:
 	bool             isInvisible(void)
 	{ return m_bIsInvisible;}
 	void             setGrammarMessage(UT_UTF8String & sMsg);
-	void             getGrammarMessage(UT_UTF8String & sMsg);
+	void             getGrammarMessage(UT_UTF8String & sMsg) const;
 	
 private:
 	UT_sint32	m_iOffset;
@@ -609,14 +615,14 @@ protected:
 	UT_uint32		iOffset;
 };
 
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 class ABI_EXPORT fl_BlockSpellIterator
 {
 	friend class fl_BlockLayout;
 
 	UT_GrowBuf*     m_pgb;
 
-	fl_BlockLayout* m_pBL;
+	const fl_BlockLayout* m_pBL;
 
 	UT_sint32       m_iWordOffset;
 	UT_sint32       m_iWordLength;
@@ -635,7 +641,7 @@ class ABI_EXPORT fl_BlockSpellIterator
     bool            _ignoreLastWordCharacter(const UT_UCSChar c) const;
 
 public:
-	fl_BlockSpellIterator(fl_BlockLayout* pBL, UT_sint32 iPos = 0);
+	fl_BlockSpellIterator(const fl_BlockLayout* pBL, UT_sint32 iPos = 0);
 	~fl_BlockSpellIterator();
 
 	bool            nextWordForSpellChecking(const UT_UCSChar*& pWord,
@@ -645,13 +651,13 @@ public:
 	void              updateBlock(void);
 	void              updateSentenceBoundaries(void);
 
-	UT_sint32         getBlockLength(void);
+	UT_sint32         getBlockLength(void) const;
 
 	void              revertToPreviousWord(void);
 
-    const UT_UCSChar* getCurrentWord(UT_sint32& iLength);
-    const UT_UCSChar* getPreWord(UT_sint32& iLength);
-    const UT_UCSChar* getPostWord(UT_sint32& iLength);
+    const UT_UCSChar* getCurrentWord(UT_sint32& iLength) const;
+    const UT_UCSChar* getPreWord(UT_sint32& iLength) const;
+    const UT_UCSChar* getPostWord(UT_sint32& iLength) const;
 };
 #endif
 
