@@ -227,6 +227,8 @@ const char * fl_ContainerLayout::getContainerString(void)
 		return "FL_CONTAINER_FRAME";
 	case FL_CONTAINER_TOC:
 		return "FL_CONTAINER_TOC";
+	case FL_CONTAINER_ANNOTATION:
+		return "FL_CONTAINER_ANNOTATION";
 	default:
 		return "NOT_IMPLEMENTED";
 	}
@@ -381,15 +383,15 @@ void fl_ContainerLayout::lookupFoldedLevel(void)
  * This method appends all the text in the current layout to the supplied 
  * GrowBuf.
  */
-void fl_ContainerLayout::appendTextToBuf(UT_GrowBuf & buf)
+void fl_ContainerLayout::appendTextToBuf(UT_GrowBuf & buf) const
 {
 	if(getContainerType() == FL_CONTAINER_BLOCK)
 	{
-		fl_BlockLayout * pBL = static_cast<fl_BlockLayout *>(this);
+		const fl_BlockLayout * pBL = static_cast<const fl_BlockLayout *>(this);
 		pBL->appendTextToBuf(buf);
 		return;
 	}
-	fl_ContainerLayout * pCL = getFirstLayout();
+	const fl_ContainerLayout * pCL = getFirstLayout();
 	while(pCL)
 	{
 		pCL->appendTextToBuf(buf);
@@ -430,7 +432,7 @@ fl_ContainerLayout * fl_ContainerLayout::getPrev(void) const
 	return m_pPrev;
 }
 
-fl_DocSectionLayout * fl_ContainerLayout::getDocSectionLayout(void)
+fl_DocSectionLayout * fl_ContainerLayout::getDocSectionLayout(void) const
 {
 	fl_ContainerLayout * pCL = myContainingLayout();
 	while(pCL!= NULL && ((pCL->getContainerType() != FL_CONTAINER_DOCSECTION) && (pCL->getContainerType() != FL_CONTAINER_HDRFTR)))
@@ -490,7 +492,7 @@ FL_DocLayout* fl_ContainerLayout::getDocLayout(void) const
 	{
 		pMyContainer = pMyContainer->myContainingLayout();
 	}
-	return const_cast<fl_DocSectionLayout *>(static_cast<const fl_DocSectionLayout *>(pMyContainer))->getDocLayout();
+	return static_cast<const fl_DocSectionLayout *>(pMyContainer)->getDocLayout();
 }
 
 void fl_ContainerLayout::setContainingLayout(fl_ContainerLayout * pL)
@@ -613,6 +615,14 @@ fl_BlockLayout* fl_ContainerLayout::getNextBlockInDocument(void) const
 				goto next_is_null;
 			}
 		}
+		else if(pNext->getContainerType() == FL_CONTAINER_ANNOTATION)
+		{
+			pNext = pNext->getNext();
+			if(pNext == NULL)
+			{
+				goto next_is_null;
+			}
+		}
 		else if(pNext->getContainerType() == FL_CONTAINER_ENDNOTE)
 		{
 			pNext = pNext->getNext();
@@ -697,6 +707,10 @@ fl_BlockLayout* fl_ContainerLayout::getPrevBlockInDocument(void) const
 		{
 			pPrev = pPrev->getLastLayout();
 		}
+		else if(pPrev->getContainerType() == FL_CONTAINER_ANNOTATION)
+		{
+			pPrev = pPrev->getLastLayout();
+		}
 		else if(pPrev->getContainerType() == FL_CONTAINER_TOC)
 		{
 			pPrev = pPrev->getLastLayout();
@@ -774,6 +788,16 @@ fl_ContainerLayout * fl_ContainerLayout::insert(PL_StruxDocHandle sdh, fl_Contai
 		{
 			pL = static_cast<fl_ContainerLayout *>(new fl_BlockLayout(sdh,pPrev, static_cast<fl_SectionLayout *>(pPrev->myContainingLayout()), indexAP));
 		}
+		else if ((pPrev!= NULL) && (pPrev->getContainerType() == FL_CONTAINER_ANNOTATION))
+		{
+			pL = static_cast<fl_ContainerLayout *>(new fl_BlockLayout(sdh,pPrev, static_cast<fl_SectionLayout *>(this), indexAP));
+			fp_Container * pFirstC = pL->getFirstContainer();
+			//
+			// This sets indent for a annotation label.
+			//
+			if(pFirstC)
+			  pFirstC->recalcMaxWidth(true);
+		}
 		else
 		{
 			pL = static_cast<fl_ContainerLayout *>(new fl_BlockLayout(sdh, static_cast<fl_BlockLayout *>(pPrev), static_cast<fl_SectionLayout *>(this), indexAP));
@@ -858,6 +882,16 @@ fl_ContainerLayout * fl_ContainerLayout::insert(PL_StruxDocHandle sdh, fl_Contai
 			pPrev->_insertIntoList(pL);
 		break;
 	}
+	case FL_CONTAINER_ANNOTATION:
+	{
+		fl_DocSectionLayout * pDSL = getDocSectionLayout();
+		pL = static_cast<fl_ContainerLayout *>(new fl_AnnotationLayout(getDocLayout(), 
+					  pDSL, 
+					  sdh, indexAP, this));
+		if (pPrev)
+			pPrev->_insertIntoList(pL);
+		break;
+	}
 	case FL_CONTAINER_TOC:
 	{
 		fl_DocSectionLayout * pDSL = getDocSectionLayout();
@@ -1188,7 +1222,7 @@ bool fl_ContainerLayout::isOnScreen() const
 
 void fl_ContainerLayout::addFrame(fl_FrameLayout * pFrame)
 {
-	UT_DEBUGMSG(("Adding frame %x to list in container %x \n",pFrame,this));
+	UT_DEBUGMSG(("Adding frame %p to list in container %p \n",pFrame,this));
 	UT_sint32 i = m_vecFrames.findItem(pFrame);
 	if(i>= 0)
 	{
@@ -1201,7 +1235,7 @@ void fl_ContainerLayout::addFrame(fl_FrameLayout * pFrame)
 
 UT_sint32 fl_ContainerLayout::getNumFrames(void) const
 {
-	return static_cast<UT_sint32>(m_vecFrames.getItemCount());
+	return m_vecFrames.getItemCount();
 }
 
 fl_FrameLayout * fl_ContainerLayout::getNthFrameLayout(UT_sint32 i) const
@@ -1227,7 +1261,7 @@ fp_FrameContainer * fl_ContainerLayout::getNthFrameContainer(UT_sint32 i) const
 
 void fl_ContainerLayout:: removeFrame(fl_FrameLayout * pFrame)
 {
-	UT_DEBUGMSG(("Remove Frame %x from this container %x \n",pFrame,this));
+	UT_DEBUGMSG(("Remove Frame %p from this container %p \n",pFrame,this));
 	UT_sint32 i = m_vecFrames.findItem(pFrame);
 	if(i >= 0)
 	{
