@@ -53,6 +53,7 @@
 #include "gr_Graphics.h"
 #include "pd_Document.h"
 #include "pp_AttrProp.h"
+#include "xap_App.h"
 #include "xap_Frame.h"
 #include "xap_Prefs.h"
 #include "fv_View.h"
@@ -118,7 +119,7 @@ bool fl_DocListener::populate(PL_StruxFmtHandle sfh,
 		if(pL->getType() != PTX_Block)
 		{
 			m_pDoc->miniDump(pL->getStruxDocHandle(),8);
-			UT_DEBUGMSG(("Illegal strux is %x \n",pL->getStruxDocHandle()));
+			UT_DEBUGMSG(("Illegal strux is %p \n",pL->getStruxDocHandle()));
 		}			
 		UT_ASSERT(pL->getType() == PTX_Block);
 
@@ -126,7 +127,7 @@ bool fl_DocListener::populate(PL_StruxFmtHandle sfh,
 		if(pCL->getPrev()!= NULL && pCL->getPrev()->getLastContainer()==NULL)
 		{
 			UT_DEBUGMSG(("In DocListner no LastLine in Previous Block Fixing this now \n"));
-			UT_DEBUGMSG(("getPrev = %d this = %d \n",pCL->getPrev(),pCL));
+			UT_DEBUGMSG(("getPrev = %p this = %p \n",pCL->getPrev(),pCL));
 			if( pCL->getSectionLayout()->getType() != FL_SECTION_HDRFTR)
 				pCL->getPrev()->format();
 		}
@@ -144,7 +145,7 @@ bool fl_DocListener::populate(PL_StruxFmtHandle sfh,
 		if(pCL->getLastContainer()==NULL)
 		{
 			UT_DEBUGMSG(("In  DocListner no LastLine in this block fixing this now \n"));
-			UT_DEBUGMSG(("getPrev = %d this = %d \n",pCL->getPrev(),pCL));
+			UT_DEBUGMSG(("getPrev = %p this = %p \n",pCL->getPrev(),pCL));
 			if(pCL->getSectionLayout()->getType() != FL_SECTION_HDRFTR && pCL->getPrev()!= NULL)
 				pCL->format();
 			//UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
@@ -210,7 +211,7 @@ bool fl_DocListener::populate(PL_StruxFmtHandle sfh,
 		if(pL->getType() != PTX_Block)
 		{
 			m_pDoc->miniDump(pL->getStruxDocHandle(),8);
-			UT_DEBUGMSG(("Illegal strux is %x \n",pL->getStruxDocHandle()));
+			UT_DEBUGMSG(("Illegal strux is %p \n",pL->getStruxDocHandle()));
 			UT_return_val_if_fail((pL->getType() == PTX_Block),false);
 		}			
 		UT_ASSERT(pL->getType() == PTX_Block);
@@ -401,21 +402,33 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 
 	case PTX_SectionFootnote:
 	case PTX_SectionEndnote:
+	case PTX_SectionAnnotation:
 	{
 		bool isFoot = (pcrx->getStruxType() == PTX_SectionFootnote);
 		UT_ASSERT(m_pCurrentSL);
 		m_bFootnoteInProgress = true;
-	    fl_SectionLayout * pSL = NULL;
+		fl_SectionLayout * pSL = NULL;
 		if(isFoot)
 		{
 			UT_DEBUGMSG(("fl_DocListener::populateStrux for 'SectionFootnote'\n"));
 			pSL = (fl_SectionLayout *) m_pCurrentSL->append(sdh, pcr->getIndexAP(),FL_CONTAINER_FOOTNOTE);
 		}
-		else
+		else if(pcrx->getStruxType() == PTX_SectionEndnote)
 		{
 			UT_DEBUGMSG(("fl_DocListener::populateStrux for 'SectionEndnote'\n"));
 			pSL = (fl_SectionLayout *) m_pCurrentSL->append(sdh, pcr->getIndexAP(),FL_CONTAINER_ENDNOTE);
 		}
+		else if(pcrx->getStruxType() == PTX_SectionAnnotation)
+		{
+			UT_DEBUGMSG(("fl_DocListener::populateStrux for 'SectionAnnotation'\n"));
+			pSL = (fl_SectionLayout *) m_pCurrentSL->append(sdh, pcr->getIndexAP(),FL_CONTAINER_ANNOTATION);
+		}
+		else
+		  {
+		    UT_DEBUGMSG(("Unknown strux type in footnotes \n"));
+		    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		  }
+
 		*psfh = (PL_StruxFmtHandle)pSL;
 		m_pCurrentSL = (fl_SectionLayout*)pSL;
 		break;
@@ -423,6 +436,7 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 
 	case PTX_EndFootnote:
 	case PTX_EndEndnote:
+	case PTX_EndAnnotation:
 	{
 //
 // CurrentSL is a Footnote. Return this and set the m_pCurrentSL to it's 
@@ -438,14 +452,19 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 			UT_DEBUGMSG(("fl_DocListener::populateStrux for 'EndFootnote'\n"));
 			UT_ASSERT(pCL->getContainerType() == FL_CONTAINER_FOOTNOTE);
 		}
-		else
+		else if(pcrx->getStruxType() == PTX_EndEndnote)
 		{
 			UT_DEBUGMSG(("fl_DocListener::populateStrux for 'EndEndnote'\n"));
 			UT_ASSERT(pCL->getContainerType() == FL_CONTAINER_ENDNOTE);
 		}
+		else if(pcrx->getStruxType() == PTX_EndAnnotation)
+		{
+			UT_DEBUGMSG(("fl_DocListener::populateStrux for 'EndAnnotation'\n"));
+			UT_ASSERT(pCL->getContainerType() == FL_CONTAINER_ANNOTATION);
+		}
 #endif
 		*psfh = (PL_StruxFmtHandle) pCL;
-		m_pCurrentSL = (fl_SectionLayout *) static_cast<fl_FootnoteLayout *>(m_pCurrentSL)->getDocSectionLayout();
+		m_pCurrentSL = (fl_SectionLayout *) static_cast<fl_EmbedLayout *>(m_pCurrentSL)->getDocSectionLayout();
 		fl_BlockLayout * pBL = NULL;
 		if(isFoot)
 		{
@@ -453,12 +472,18 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 			pFL->setFootnoteEndIn();
 			pBL = (fl_BlockLayout *) pFL->getFirstLayout();
 		}
-		else
+		else if(pcrx->getStruxType() == PTX_EndEndnote)
 		{
 			fl_EndnoteLayout * pEL = (fl_EndnoteLayout *) pCL;
 			pEL->setFootnoteEndIn();
 			pBL = (fl_BlockLayout *) pEL->getFirstLayout();
 		}
+		else if(pcrx->getStruxType() == PTX_EndAnnotation)
+		{
+			fl_AnnotationLayout * pAL = (fl_AnnotationLayout *) pCL;
+			pAL->setFootnoteEndIn();
+			pBL = (fl_BlockLayout *) pAL->getFirstLayout();
+		}
 		UT_ASSERT(pBL);
 		if(pBL)
 		{
@@ -542,7 +567,7 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 					m_pLayout->addHdrFtrSection(pSL);
 					pDocSL->setHdrFtr(hfType, pSL);
 					*psfh = (PL_StruxFmtHandle)pSL;
-					UT_DEBUGMSG(("Sevior: HeaderFooter created %x \n",pSL));
+					UT_DEBUGMSG(("Sevior: HeaderFooter created %p \n",pSL));
 					
 					m_pCurrentSL = pSL;
 				}
@@ -557,7 +582,7 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 	}
 	case PTX_Block:
 	{
-		UT_DEBUGMSG(("SEVIOR: Doing populate block sdh %x \n",sdh));
+		UT_DEBUGMSG(("SEVIOR: Doing populate block sdh %p \n",sdh));
 		if(m_pCurrentSL == NULL)
 		{
 			m_pDoc->miniDump(sdh,6);
@@ -568,7 +593,7 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 //
 		fl_ContainerLayout*	pCL = NULL;
 		fl_ContainerLayout * pCon = getTopContainerLayout();
-		if((pCon != NULL)  && (m_pCurrentSL->getContainerType() != FL_CONTAINER_FOOTNOTE) && (m_pCurrentSL->getContainerType() != FL_CONTAINER_ENDNOTE))
+		if((pCon != NULL)  && (m_pCurrentSL->getContainerType() != FL_CONTAINER_FOOTNOTE) && (m_pCurrentSL->getContainerType() != FL_CONTAINER_ENDNOTE) && (m_pCurrentSL->getContainerType() != FL_CONTAINER_ANNOTATION))
 		{
 			if(pCon->getContainerType() != FL_CONTAINER_CELL)
 			{
@@ -580,7 +605,7 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 				return false;
 			}
 			fl_CellLayout * pCell = (fl_CellLayout *) pCon;
-			UT_DEBUGMSG(("SEVIOR: Appending block to cell %x \n",pCell));
+			UT_DEBUGMSG(("SEVIOR: Appending block to cell %p \n",pCell));
 			// Append a new BlockLayout to this cell
 
 			pCL = pCell->append(sdh, pcr->getIndexAP(),FL_CONTAINER_BLOCK);
@@ -640,7 +665,7 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 			if(pCL->getSectionLayout()->getType() != FL_SECTION_HDRFTR && pCL->getPrev() != NULL)
 			{
 				UT_DEBUGMSG(("In DocListner no LastLine in block append. Fixing this now \n"));
-				UT_DEBUGMSG(("getPrev = %d this = %d \n",pCL->getPrev(),pCL));
+				UT_DEBUGMSG(("getPrev = %p this = %p \n",pCL->getPrev(),pCL));
 				pCL->format();
 			}
 		}
@@ -657,7 +682,7 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 		UT_DEBUGMSG(("!!!!Appending Table \n"));
 		if(m_pCurrentSL->getHdrFtrLayout())
 		{
-			UT_DEBUGMSG(("Appending Table into HdrFtr %x \n",m_pCurrentSL->getHdrFtrLayout()));
+			UT_DEBUGMSG(("Appending Table into HdrFtr %p \n",m_pCurrentSL->getHdrFtrLayout()));
 		}
 		if(pCon == NULL)
 		{
@@ -685,7 +710,7 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 	{
 		UT_ASSERT(m_pCurrentSL);
 		UT_ASSERT(m_pCurrentSL->getContainerType() == FL_CONTAINER_DOCSECTION);
-		fl_ContainerLayout*	pCL = NULL;
+		fl_ContainerLayout*	pCL2 = NULL;
 //
 // Look to see if we're in a table.
 //
@@ -709,14 +734,14 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 
 
 		UT_DEBUGMSG(("!!!!Appending Frame \n"));
-		pCL = m_pCurrentSL->append(sdh, pcr->getIndexAP(),FL_CONTAINER_FRAME);
-		if (!pCL)
+		pCL2 = m_pCurrentSL->append(sdh, pcr->getIndexAP(),FL_CONTAINER_FRAME);
+		if (!pCL2)
 		{
 			UT_DEBUGMSG(("no memory for TableLayout"));
 			return false;
 		}
-		m_pCurrentSL = static_cast<fl_SectionLayout *>(pCL);
-		*psfh = (PL_StruxFmtHandle)pCL;
+		m_pCurrentSL = static_cast<fl_SectionLayout *>(pCL2);
+		*psfh = (PL_StruxFmtHandle)pCL2;
 	}
 	break;
 	case PTX_EndFrame:
@@ -757,10 +782,10 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 		fl_TableLayout * pTable = (fl_TableLayout *) pCon;
 		if(pTable->getHdrFtrLayout())
 		{
-			UT_DEBUGMSG(("Appending a Cell to a Table in a HDrFtr %x \n",pTable->getHdrFtrLayout()));
+			UT_DEBUGMSG(("Appending a Cell to a Table in a HDrFtr %p \n",pTable->getHdrFtrLayout()));
 		}
 		fl_ContainerLayout*	pCL = pTable->append(sdh, pcr->getIndexAP(),FL_CONTAINER_CELL);
-		UT_DEBUGMSG(("SEVIOR: Appending Cell: layout is %x \n",pCL));
+		UT_DEBUGMSG(("SEVIOR: Appending Cell: layout is %p \n",pCL));
 		pTable->attachCell(pCL);
 		if (!pCL)
 		{
@@ -880,7 +905,7 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 {
 	UT_return_val_if_fail( sfh, false );
 	
-	//UT_DEBUGMSG(("fl_DocListener::change\n"));
+	UT_DEBUGMSG(("fl_DocListener::change\n"));
 	bool bResult = false;
 	AV_ChangeMask chgMask = AV_CHG_NONE;
 	
@@ -1096,6 +1121,14 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 			pFL->doclistener_deleteStrux(pcrx);
 			goto finish_up;
 		}
+		case PTX_SectionAnnotation:
+		{
+			fl_Layout * pL = (fl_Layout *)sfh;
+			UT_ASSERT(pL->getType() == PTX_SectionAnnotation);
+			fl_AnnotationLayout * pAL = (fl_AnnotationLayout *) pL;
+			pAL->doclistener_deleteStrux(pcrx);
+			goto finish_up;
+		}
 		case PTX_SectionEndnote:
 		{
 			fl_Layout * pL = (fl_Layout *)sfh;
@@ -1153,6 +1186,15 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 
 			goto finish_up;
 		}
+		case PTX_EndAnnotation:
+		{
+			fl_Layout * pL = (fl_Layout *)sfh;
+			UT_ASSERT(pL->getType() == PTX_SectionAnnotation);
+			fl_AnnotationLayout * pAL = static_cast<fl_AnnotationLayout *>(pL);
+			pAL->doclistener_deleteEndEmbed(pcrx);
+
+			goto finish_up;
+		}
 		case PTX_EndEndnote:
 		{
 			fl_Layout * pL = (fl_Layout *)sfh;
@@ -1173,7 +1215,7 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 	{
 		const PX_ChangeRecord_StruxChange * pcrxc = static_cast<const PX_ChangeRecord_StruxChange *> (pcr);
 
-		fl_Layout * pL = (fl_Layout *)sfh;
+		fl_Layout * pL2 = (fl_Layout *)sfh;
 
 		// TODO getOldIndexAP() is only intended for use by the document.
 		// TODO this assert is probably wrong. --- BUT EVERYTIME IT HAS
@@ -1181,20 +1223,26 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 		// UT_ASSERT(pL->getAttrPropIndex() == pcrxc->getOldIndexAP());
 		// UT_ASSERT(pL->getAttrPropIndex() != pcr->getIndexAP());
 
-		switch (pL->getType())
+		switch (pL2->getType())
 		{
 		case PTX_Section:
 		case PTX_SectionEndnote:
+		case PTX_SectionAnnotation:
 		case PTX_SectionFootnote:
 		{
-			fl_DocSectionLayout* pSL = static_cast<fl_DocSectionLayout*>(pL);
+			fl_DocSectionLayout* pSL = static_cast<fl_DocSectionLayout*>(pL2);
 			
 			PT_AttrPropIndex indexAP = pcr->getIndexAP();
 			const PP_AttrProp* pAP = NULL;
 			
 			bool bres = (m_pDoc->getAttrProp(indexAP, &pAP) && pAP);
 			UT_ASSERT(bres);
-			PL_StruxDocHandle sdh = pL->getStruxDocHandle();
+			if(!bres)
+			{
+				UT_WARNINGMSG(("getAttrProp() failed in %s:%d",
+							   __FILE__, __LINE__));
+			}
+			PL_StruxDocHandle sdh = pL2->getStruxDocHandle();
 	
 			const gchar* pszSectionType = NULL;
 			pAP->getAttribute("type", pszSectionType);
@@ -1267,7 +1315,7 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 			// Tomas, June 7, 2003
 			
 			chgMask = AV_CHG_FMTBLOCK | AV_CHG_FMTCHAR;
-			fl_SectionLayout * pCL = static_cast<fl_SectionLayout *>(pL);
+			fl_SectionLayout * pCL = static_cast<fl_SectionLayout *>(pL2);
 			fl_SectionLayout* pCLSL = pCL->getSectionLayout();
 			if(pCLSL->getType() == FL_SECTION_SHADOW)
 			{
@@ -1298,7 +1346,7 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 // Doesn't know if it's a header or a footer or the DocSection and hences pages
 // It associated with. Tell it now.
 //
-			fl_HdrFtrSectionLayout* pHFSL = static_cast<fl_HdrFtrSectionLayout*>(pL);
+			fl_HdrFtrSectionLayout* pHFSL = static_cast<fl_HdrFtrSectionLayout*>(pL2);
 			
 			PT_AttrPropIndex indexAP = pcr->getIndexAP();
 //
@@ -1310,6 +1358,11 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 			
 			bool bres = (m_pDoc->getAttrProp(indexAP, &pHFAP) && pHFAP);
 			UT_ASSERT(bres);
+			if(!bres)
+			{
+				UT_WARNINGMSG(("getAttrProp() failed in %s:%d",
+							   __FILE__, __LINE__));
+			}
 	
 			const gchar* pszHFSectionType = NULL;
 			pHFAP->getAttribute("type", pszHFSectionType);
@@ -1353,28 +1406,28 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 		}
         case PTX_SectionTable:
 		{
-			fl_TableLayout * pTL = (fl_TableLayout *) pL;
+			fl_TableLayout * pTL = (fl_TableLayout *) pL2;
 			UT_ASSERT(pTL->getContainerType() == FL_CONTAINER_TABLE);
  			bResult = pTL->doclistener_changeStrux(pcrxc);
 			goto finish_up;
 		}
 		case PTX_SectionCell:
 		{
-			fl_CellLayout * pCL = (fl_CellLayout *) pL;
+			fl_CellLayout * pCL = (fl_CellLayout *) pL2;
 			UT_ASSERT(pCL->getContainerType() == FL_CONTAINER_CELL);
 			bResult = pCL->doclistener_changeStrux(pcrxc);
 			goto finish_up;
 		}
 		case PTX_SectionFrame:
 		{
-			fl_FrameLayout * pFL = (fl_FrameLayout *) pL;
+			fl_FrameLayout * pFL = (fl_FrameLayout *) pL2;
 			UT_ASSERT(pFL->getContainerType() == FL_CONTAINER_FRAME);
 			bResult = pFL->doclistener_changeStrux(pcrxc);
 			goto finish_up;
 		}
 		case PTX_SectionTOC:
 		{
-			fl_TOCLayout * pTOCL = (fl_TOCLayout *) pL;
+			fl_TOCLayout * pTOCL = (fl_TOCLayout *) pL2;
 			UT_ASSERT(pTOCL->getContainerType() == FL_CONTAINER_TOC);
 			bResult = pTOCL->doclistener_changeStrux(pcrxc);
 			goto finish_up;
@@ -1593,6 +1646,44 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 	        bResult = true;
 		goto finish_up;
 	}	
+	case PX_ChangeRecord::PXT_ChangeDocProp:
+	{
+	        PT_AttrPropIndex iAP = pcr->getIndexAP();
+		const PP_AttrProp * pAP = NULL;
+		m_pLayout->getDocument()->getAttrProp(iAP, &pAP);
+		const gchar * szValue=NULL;
+		bool b= pAP->getAttribute( PT_DOCPROP_ATTRIBUTE_NAME,szValue);
+		UT_DEBUGMSG(("Doing DocProp change value %s \n",szValue));
+		if(!b)
+		{
+		    bResult = false;
+		    goto finish_up;
+		}
+		if(strcmp(szValue,"pagesize") == 0)
+		{
+		    bResult = m_pLayout->setDocViewPageSize(pAP);
+		}
+		else if(strcmp(szValue,"changeauthor") == 0)
+		{
+		  	m_pLayout->refreshRunProperties();
+			FV_View * pView = m_pLayout->getView();
+			pView->updateScreen(false ); 
+		}
+		else if(strcmp(szValue,"addauthor") == 0)
+		{
+		  	m_pLayout->refreshRunProperties();
+			FV_View * pView = m_pLayout->getView();
+			const gchar * szAuthorId = NULL;
+			pAP->getProperty("id",szAuthorId);
+			if(szAuthorId && *szAuthorId)
+			{
+			  UT_sint32 id = atoi(szAuthorId);
+			  pView->addCaret(0,id);
+			}
+			pView->updateScreen(false ); 
+		}
+		goto finish_up;
+	}	
 	default:
 	{
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
@@ -1696,10 +1787,18 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 	{
 		UT_DEBUGMSG(("Inserting SectionFootnote strux at pos %d \n",pcr->getPosition()));
 	}
+	else if(pcrx->getStruxType() == PTX_SectionAnnotation)
+	{
+		UT_DEBUGMSG(("Inserting SectionAnnotation strux at pos %d \n",pcr->getPosition()));
+	}
 	else if(pcrx->getStruxType() == PTX_EndFootnote)
 	{
 		UT_DEBUGMSG(("Inserting EndFootnote strux at pos %d \n",pcr->getPosition()));
 	}
+	else if(pcrx->getStruxType() == PTX_EndAnnotation)
+	{
+		UT_DEBUGMSG(("Inserting EndAnnotation strux at pos %d \n",pcr->getPosition()));
+	}
 	else if(pcrx->getStruxType() == PTX_SectionTOC)
 	{
 		UT_DEBUGMSG(("Inserting SectionTOC strux at pos %d \n",pcr->getPosition()));
@@ -1753,6 +1852,7 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 	case PTX_Section:		   // the immediately prior strux is a section.
 	case PTX_SectionHdrFtr:	   //  ... or a HdrFtr.
 	case PTX_SectionFootnote:  //  ... or a Footnote.
+	case PTX_SectionAnnotation:  //  ... or a Annotation.
 	case PTX_SectionEndnote:  //  ... or a Endnote.
     {
 		switch (pcrx->getStruxType())	// see what we are inserting.
@@ -1760,6 +1860,7 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 		case PTX_Section:				// we are inserting a section.
 		case PTX_SectionHdrFtr:			//  ... or a HdrFtr section.
 		case PTX_SectionFootnote:        //  ... or a Footnote section.
+		case PTX_SectionAnnotation:        //  ... or a Annotation section.
 		case PTX_SectionEndnote:        //  ... or a Endnote section.
 		case PTX_SectionTOC:  //  ... or a Table Of Contents.
 			// We are inserting a section immediately after a section
@@ -1787,7 +1888,7 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 		{
 			UT_DEBUGMSG(("Insert Table immediately after Section \n"));
 			fl_SectionLayout * pSL = static_cast<fl_SectionLayout *>(pL);
-			return pSL->bl_doclistener_insertTable(FL_SECTION_TABLE, pcrx,sdh,lid,pfnBindHandles);
+			return pSL->bl_doclistener_insertTable(FL_SECTION_TABLE, pcrx,sdh,lid,pfnBindHandles) != NULL;
 		}
 #endif
 		default:						// unknown strux.
@@ -1890,6 +1991,14 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			   bool bResult = pCLSL->bl_doclistener_insertSection(pCL, FL_SECTION_FOOTNOTE, pcrx,sdh,lid,pfnBindHandles);
 			   return bResult;
 		   }
+		case PTX_SectionAnnotation:
+		   {
+			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
+			   fl_SectionLayout* pCLSL = pCL->getSectionLayout();
+
+			   bool bResult = pCLSL->bl_doclistener_insertSection(pCL, FL_SECTION_ANNOTATION, pcrx,sdh,lid,pfnBindHandles);
+			   return bResult;
+		   }
 		case PTX_SectionEndnote:
 		   {
 		           UT_DEBUGMSG(("Doing valid endnote insertion into block \n"));
@@ -1955,6 +2064,16 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			   bool bResult = pCLSL->bl_doclistener_insertEndEmbed(pCL, pcrx,sdh,lid,pfnBindHandles);
 			   return bResult;
 		   }
+		case PTX_EndAnnotation:
+		   {
+			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout*>(pL);
+
+			   fl_AnnotationLayout* pCLSL = 
+				   static_cast<fl_AnnotationLayout *>(pCL->myContainingLayout());
+			   UT_ASSERT(pCLSL->getContainerType() == FL_CONTAINER_ANNOTATION);
+			   bool bResult = pCLSL->bl_doclistener_insertEndEmbed(pCL, pcrx,sdh,lid,pfnBindHandles);
+			   return bResult;
+		   }
 		case PTX_EndEndnote:
 		   {
 			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout*>(pL);
@@ -2291,7 +2410,7 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			   // The immediately prior strux is a Frame.  So, this
 			   // will become the first table of the Frame.
 		           fl_SectionLayout * pSL = static_cast<fl_SectionLayout *>(pL);
-			   return pSL->bl_doclistener_insertTable(FL_SECTION_TABLE, pcrx,sdh,lid,pfnBindHandles);
+			   return pSL->bl_doclistener_insertTable(FL_SECTION_TABLE, pcrx,sdh,lid,pfnBindHandles) != NULL;
 
 		  }
 		case PTX_EndCell:	  
@@ -2374,7 +2493,7 @@ bool fl_DocListener::signal(UT_uint32 iSignal)
 	case PD_SIGNAL_DOCDIRTY_CHANGED:
 		m_pLayout->notifyListeners(AV_CHG_DIRTY);
 		break;
-	case PD_SIGNAL_DOCSAVED:
+	case PD_SIGNAL_SAVEDOC:
 		break;
 	case PD_SIGNAL_DOCCLOSED:
 		break;
