@@ -101,7 +101,8 @@ fl_FrameLayout::fl_FrameLayout(FL_DocLayout* pLayout,
 	  m_iBoundingSpace(0),
 	  m_iFrameWrapMode(FL_FRAME_ABOVE_TEXT),
 	  m_bIsTightWrap(false),
-	  m_iPrefPage(-1)
+	  m_iPrefPage(-1),
+	  m_bRelocate(false)
 {
 	UT_ASSERT(m_pDocSL->getContainerType() == FL_CONTAINER_DOCSECTION);
 }
@@ -109,7 +110,7 @@ fl_FrameLayout::fl_FrameLayout(FL_DocLayout* pLayout,
 fl_FrameLayout::~fl_FrameLayout()
 {
 	// NB: be careful about the order of these
-	UT_DEBUGMSG(("Deleting Framelayout %x \n",this));
+	UT_DEBUGMSG(("Deleting Framelayout %p \n",this));
 	_purgeLayout();
 	fp_FrameContainer * pFC = static_cast<fp_FrameContainer *>(getFirstContainer());
 	while(pFC)
@@ -267,7 +268,7 @@ UT_uint32 fl_FrameLayout::getLength(void)
  * This code actually inserts a block AFTER the frame in the docsectionlayout
  * Code copied from tablelayout
  */
-bool fl_FrameLayout::insertBlockAfter(fl_ContainerLayout* pLBlock,
+bool fl_FrameLayout::insertBlockAfter(fl_ContainerLayout* /*pLBlock*/,
 											  const PX_ChangeRecord_Strux * pcrx,
 											  PL_StruxDocHandle sdh,
 											  PL_ListenerId lid,
@@ -350,7 +351,7 @@ bool fl_FrameLayout::bl_doclistener_insertEndFrame(fl_ContainerLayout*,
 /*!
  * This signals an incomplete frame section.
  */
-bool fl_FrameLayout::doclistener_deleteEndFrame( const PX_ChangeRecord_Strux * pcrx)
+bool fl_FrameLayout::doclistener_deleteEndFrame( const PX_ChangeRecord_Strux * /*pcrx*/)
 {
 	m_bHasEndFrame = false;
 	return true;
@@ -379,7 +380,7 @@ bool fl_FrameLayout::doclistener_changeStrux(const PX_ChangeRecord_StruxChange *
 	UT_GenericVector<fl_ContainerLayout *> AllLayouts;
 	AllLayouts.clear();
 	fp_Page * pPage = NULL;
-	UT_uint32 i = 0;
+	UT_sint32 i = 0;
 	if(pFrameC)
 	{
 	    pPage = pFrameC->getPage();
@@ -435,7 +436,7 @@ void fl_FrameLayout::markAllRunsDirty(void)
 	}
 }
 
-void fl_FrameLayout::updateLayout(bool bDoAll)
+void fl_FrameLayout::updateLayout(bool /*bDoAll*/)
 {
 	// ingnore frames in normal view mode
 	FV_View * pView = getDocLayout()->getView();
@@ -477,6 +478,7 @@ void fl_FrameLayout::redrawUpdate(void)
 
 bool fl_FrameLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 {
+	UT_UNUSED(pcrx);
 	UT_ASSERT(pcrx->getType()==PX_ChangeRecord::PXT_DeleteStrux);
 #if 0
 	fp_FrameContainer * pFrameC = getFirstContainer();
@@ -488,7 +490,7 @@ bool fl_FrameLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 	fp_FrameContainer * pFrameC = static_cast<fp_FrameContainer *>(getFirstContainer());
 	UT_GenericVector<fl_BlockLayout *> vecBlocks;
 	pFrameC->getBlocksAroundFrame(vecBlocks);
-	UT_uint32 i = 0;
+	UT_sint32 i = 0;
 	for(i=0; i< vecBlocks.getItemCount();i++)
 	{
 	  fl_BlockLayout * pBL = vecBlocks.getNthItem(i);
@@ -504,7 +506,7 @@ bool fl_FrameLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 //
 	fl_ContainerLayout * pCL = getPrev();
 	myContainingLayout()->remove(this);
-	UT_DEBUGMSG(("Unlinking frame Layout %x \n",this));
+	UT_DEBUGMSG(("Unlinking frame Layout %p \n",this));
 //
 // Remove from the list of frames in the previous block
 //
@@ -519,7 +521,7 @@ bool fl_FrameLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 	}
 	fl_BlockLayout * pBL = static_cast<fl_BlockLayout *>(pCL);
 	bool bFound = false;
-	for(i=0; i<static_cast<UT_uint32>(pBL->getNumFrames()) && !bFound;i++)
+	for(i=0; i<pBL->getNumFrames() && !bFound;i++)
 	{
 	  fl_FrameLayout * pF = pBL->getNthFrameLayout(i);
 	  if(pF == this)
@@ -587,16 +589,6 @@ void fl_FrameLayout::_createFrameContainer(void)
 	fp_FrameContainer * pFrameContainer = new fp_FrameContainer(static_cast<fl_SectionLayout *>(this));
 	setFirstContainer(pFrameContainer);
 	setLastContainer(pFrameContainer);
-	fl_ContainerLayout * pCL = myContainingLayout();
-	while(pCL!= NULL && pCL->getContainerType() != FL_CONTAINER_DOCSECTION)
-	{
-		pCL = pCL->myContainingLayout();
-	}
-	fl_DocSectionLayout * pDSL = static_cast<fl_DocSectionLayout *>(pCL);
-	UT_ASSERT(pDSL != NULL);
-
-	fp_Container * pCon = pCL->getLastContainer();
-	UT_ASSERT(pCon);
 	pFrameContainer->setWidth(m_iWidth);
 	pFrameContainer->setHeight(m_iHeight);
 	// Now do Frame image
@@ -633,7 +625,7 @@ fp_Container* fl_FrameLayout::getNewContainer(fp_Container *)
 	return static_cast<fp_Container *>(getLastContainer());
 }
 
-void fl_FrameLayout::_insertFrameContainer(fp_Container * pNewFC)
+void fl_FrameLayout::_insertFrameContainer(fp_Container * /*pNewFC*/)
 {
 
 // This is all done fl_BlockLayout::setFramesOnPage
@@ -680,24 +672,24 @@ void fl_FrameLayout::format(void)
 	{
 		getNewContainer();
 	}
-	fl_ContainerLayout*	pBL = getFirstLayout();
+	fl_ContainerLayout*	pBL2 = getFirstLayout();
 	bool bOnPage = false;
-	while (pBL)
+	while (pBL2)
 	{
-		pBL->format();
+		pBL2->format();
 		UT_sint32 count = 0;
-		while(pBL->getLastContainer() == NULL || pBL->getFirstContainer()==NULL)
+		while(pBL2->getLastContainer() == NULL || pBL2->getFirstContainer()==NULL)
 		{
 			UT_DEBUGMSG(("Error formatting a block try again \n"));
 			count = count + 1;
-			pBL->format();
+			pBL2->format();
 			if(count > 3)
 			{
 				UT_DEBUGMSG(("Give up trying to format. Hope for the best :-( \n"));
 				break;
 			}
 		}
-		pBL = pBL->getNext();
+		pBL2 = pBL2->getNext();
 	}
 	static_cast<fp_FrameContainer *>(getFirstContainer())->layout();
 	if(!m_bIsOnPage)
@@ -779,7 +771,7 @@ void fl_FrameLayout::format(void)
 	}
 }
 
-void fl_FrameLayout::setNeedsReformat(fl_ContainerLayout * pCL,UT_uint32 offset)
+void fl_FrameLayout::setNeedsReformat(fl_ContainerLayout * /*pCL*/,UT_uint32 /*offset*/)
 {
         m_bNeedsReformat = true;
         myContainingLayout()->setNeedsReformat(this);
@@ -818,6 +810,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 	const gchar * pszBoundingSpace = NULL;
 	const gchar * pszTightWrapped = NULL;
 	const gchar * pszPrefPage = NULL;
+
 // Frame Type
 
 	if(!pSectionAP || !pSectionAP->getProperty("frame-type",pszFrameType))
@@ -916,6 +909,21 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 	{
 		m_bIsTightWrap = false;
 	}
+	//
+	// Should be relocated before final placement
+	//
+	if(!pSectionAP || !pSectionAP->getProperty("relocate",pszTightWrapped))
+	{
+		m_bRelocate = false;
+	}
+	else if(strcmp(pszTightWrapped,"1") == 0)
+	{
+		m_bRelocate = true;
+	}
+	else
+	{
+		m_bRelocate = false;
+	}
 
 // Xpos
 
