@@ -59,6 +59,7 @@
 #include "ut_png.h"
 #include "fg_GraphicRaster.h"
 #include "fg_GraphicVector.h"
+#include "xap_App.h"
 
 static void s_border_properties (const char * border_color, const char * border_style, const char * border_width,
 								 const char * color, PP_PropertyMap::Line & line);
@@ -106,7 +107,7 @@ fl_TableLayout::fl_TableLayout(FL_DocLayout* pLayout, PL_StruxDocHandle sdh,
 	  m_bDoingDestructor(false)
 
 {
-	UT_DEBUGMSG(("Created Table Layout %x \n",this));
+	UT_DEBUGMSG(("Created Table Layout %p \n",this));
 	UT_ASSERT(pLayout);
 	m_vecColProps.clear();
 	m_vecRowProps.clear();
@@ -120,10 +121,8 @@ fl_TableLayout::~fl_TableLayout()
 	m_bDoingDestructor = true;
 	_purgeLayout();
 	fp_TableContainer * pTC = static_cast<fp_TableContainer *>(getFirstContainer());
-	if (pTC)
-	{
-		delete pTC;
-	}
+	DELETEP(pTC);
+
 	setFirstContainer(NULL);
 	setLastContainer(NULL);
 	UT_VECTOR_PURGEALL(fl_ColProps *, m_vecColProps);
@@ -225,7 +224,7 @@ void fl_TableLayout::setTableContainerProperties(fp_TableContainer * pTab)
           list, otherwise just append it to the end.
   \return The newly created Table container
 */
-fp_Container* fl_TableLayout::getNewContainer(fp_Container * pPrevTab)
+fp_Container* fl_TableLayout::getNewContainer(fp_Container * /*pPrevTab*/)
 {
 	UT_ASSERT(getFirstContainer() == NULL);
 	createTableContainer();
@@ -380,12 +379,12 @@ void fl_TableLayout::insertTableContainer( fp_TableContainer * pNewTab)
 		{
 			i = pUpCon->findCon(pPrevCon);
 			xxx_UT_DEBUGMSG(("SEVIOR!!!!!!!!!! New Table %x inserted into %x \n",pNewTab,pUpCon));
-			if(i >= 0 && (i+1) < static_cast<UT_sint32>(pUpCon->countCons()))
+			if(i >= 0 && (i+1) < pUpCon->countCons())
 			{
 				pUpCon->insertConAt(pNewTab,i+1);
 				pNewTab->setContainer(pUpCon);
 			}
-			else if( i >=0 &&  (i+ 1) == static_cast<UT_sint32>(pUpCon->countCons()))
+			else if( i >=0 &&  (i+ 1) == pUpCon->countCons())
 			{
 				pUpCon->addCon(pNewTab);
 				pNewTab->setContainer(pUpCon);
@@ -715,7 +714,7 @@ void fl_TableLayout::markAllRunsDirty(void)
 	}
 }
 
-void fl_TableLayout::updateLayout(bool bDoAll)
+void fl_TableLayout::updateLayout(bool /*bDoAll*/)
 {			
 	xxx_UT_DEBUGMSG(("updateTableLayout  in TableLayout\n"));
 	if(getDocument()->isDontImmediateLayout())
@@ -866,7 +865,7 @@ void fl_TableLayout::updateTable(void)
 }
 
 
-bool fl_TableLayout::bl_doclistener_insertBlock(fl_ContainerLayout* pLBlock,
+bool fl_TableLayout::bl_doclistener_insertBlock(fl_ContainerLayout* /*pLBlock*/,
 											  const PX_ChangeRecord_Strux * pcrx,
 											  PL_StruxDocHandle sdh,
 											  PL_ListenerId lid,
@@ -922,6 +921,7 @@ bool fl_TableLayout::bl_doclistener_insertTable( const PX_ChangeRecord_Strux * p
 																	   PL_ListenerId lid,
 																	   PL_StruxFmtHandle sfhNew))
 {
+	UT_UNUSED(iType);
 	UT_ASSERT(iType == FL_SECTION_TABLE);
 	UT_ASSERT(pcrx->getType()==PX_ChangeRecord::PXT_InsertStrux);
 	UT_ASSERT(pcrx->getStruxType()==PTX_SectionTable);
@@ -1100,7 +1100,7 @@ void fl_TableLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 	{
 		if(atoi(pszHomogeneous) == 1)
 		{
-			m_bIsHomogeneous = true;;
+			m_bIsHomogeneous = true;
 		}
 	}
 	else
@@ -1272,7 +1272,8 @@ void fl_TableLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 		GR_Graphics * pG = getDocLayout()->getGraphics();
 		UT_return_if_fail( pView && pG );
 		
-		if((pView->getViewMode() == VIEW_NORMAL) || (pView->getViewMode() == VIEW_WEB) && m_iLeftColPos < 0 &&
+		if(((pView->getViewMode() == VIEW_NORMAL) || (pView->getViewMode() == VIEW_WEB)) && 
+		   m_iLeftColPos < 0 &&
 		   !pG->queryProperties(GR_Graphics::DGP_PAPER))
 		{
 			m_iLeftColPos = 0;
@@ -1414,7 +1415,7 @@ void fl_TableLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 				UT_String sSub = sProps.substr(i,(j-i));
 				i = j + 1;
 				bool bNew = false;
-				if(iProp >= static_cast<UT_sint32>(m_vecRowProps.getItemCount()))
+				if(iProp >= m_vecRowProps.getItemCount())
 				{
 					bNew = true;
 					pRowP = new fl_RowProps;
@@ -1441,7 +1442,7 @@ void fl_TableLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
  	}
 	else
 	{
-		UT_uint32 i = 0;
+		UT_sint32 i = 0;
 		for(i=0; i< m_vecRowProps.getItemCount(); i++)
 		{
 			fl_RowProps * pRowP = m_vecRowProps.getNthItem(i);
@@ -1592,7 +1593,8 @@ void fl_TableLayout::_lookupMarginProperties(const PP_AttrProp* pSectionAP)
 		GR_Graphics * pG = getDocLayout()->getGraphics();
 		UT_return_if_fail( pView && pG );
 
-		if((pView->getViewMode() == VIEW_NORMAL) || (pView->getViewMode() == VIEW_WEB) && m_iLeftColPos < 0 &&
+		if(((pView->getViewMode() == VIEW_NORMAL) || (pView->getViewMode() == VIEW_WEB)) && 
+		   m_iLeftColPos < 0 &&
 		   !pG->queryProperties(GR_Graphics::DGP_PAPER))
 		{
 			m_iLeftColPos = 0;
@@ -2113,6 +2115,7 @@ fp_Container* fl_CellLayout::getNewContainer(fp_Container * pPrev)
 //
 // One cell container per cell layout
 //
+	UT_UNUSED(pPrev);
 	UT_ASSERT(pPrev == NULL);
 	UT_ASSERT((getFirstContainer() == NULL) && (getLastContainer()==NULL));
 	createCellContainer();
@@ -2227,7 +2230,7 @@ bool fl_CellLayout::isLayedOut(void) const
 	return true;
 }
 
-void fl_CellLayout::updateLayout(bool bDoAll)
+void fl_CellLayout::updateLayout(bool /*bDoAll*/)
 {
 	fl_ContainerLayout*	pBL = getFirstLayout();
 	bool bNeedsFormat = false;
@@ -2592,7 +2595,7 @@ void fl_CellLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 		{
 			UT_sint32 i = 0;
 			UT_sint32 cellW = 0; 
-			for(i=getLeftAttach(); i<getRightAttach() && i<static_cast<UT_sint32>(pVecCols->getItemCount());i++)
+			for(i=getLeftAttach(); i<getRightAttach() && i<pVecCols->getItemCount();i++)
 			{
 				fl_ColProps* pCol = pVecCols->getNthItem(i);
 				cellW += pCol->m_iColWidth;
@@ -2607,7 +2610,7 @@ void fl_CellLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 		{
 			UT_sint32 i = 0;
 			UT_sint32 cellH = 0; 
-			for(i=getTopAttach(); i<getBottomAttach() && i<static_cast<UT_sint32>(pVecRows->getItemCount());i++)
+			for(i=getTopAttach(); i<getBottomAttach() && i<pVecRows->getItemCount();i++)
 			{
 				fl_RowProps* pRow = pVecRows->getNthItem(i);
 				cellH += pRow->m_iRowHeight;
@@ -2778,29 +2781,28 @@ static void s_background_properties (const char * pszBgStyle, const char * pszBg
 									 PP_PropertyMap::Background & background)
 {
 	if (pszBgStyle)
+	{
+		if (strcmp (pszBgStyle, "0") == 0)
 		{
-			if (strcmp (pszBgStyle, "0") == 0)
-				{
-					background.m_t_background = PP_PropertyMap::background_none;
-				}
-			else if (strcmp (pszBgStyle, "1") == 0)
-				{
-					if (pszBgColor)
-						{
-							background.m_t_background = PP_PropertyMap::background_type (pszBgColor);
-							if (background.m_t_background == PP_PropertyMap::background_solid)
-								UT_parseColor (pszBgColor, background.m_color);
-						}
-
-				}
+			background.m_t_background = PP_PropertyMap::background_none;
 		}
+		else if (strcmp (pszBgStyle, "1") == 0)
+		{
+			if (pszBgColor)
+			{
+				background.m_t_background = PP_PropertyMap::background_type (pszBgColor);
+				if (background.m_t_background == PP_PropertyMap::background_solid)
+					UT_parseColor (pszBgColor, background.m_color);
+			}
+		}
+	}
 
 	if (pszBackgroundColor)
-		{
-			background.m_t_background = PP_PropertyMap::background_type (pszBackgroundColor);
-			if (background.m_t_background == PP_PropertyMap::background_solid)
-				UT_parseColor (pszBackgroundColor, background.m_color);
-		}
+	{
+		background.m_t_background = PP_PropertyMap::background_type (pszBackgroundColor);
+		if (background.m_t_background == PP_PropertyMap::background_solid)
+			UT_parseColor (pszBackgroundColor, background.m_color);
+	}
 }
 
 static void s_border_properties (const char * border_color, const char * border_style, const char * border_width,
@@ -2816,19 +2818,19 @@ static void s_border_properties (const char * border_color, const char * border_
 
 	PP_PropertyMap::TypeColor t_border_color = PP_PropertyMap::color_type (border_color);
 	if (t_border_color)
-		{
-			line.m_t_color = t_border_color;
-			if (t_border_color == PP_PropertyMap::color_color)
-				UT_parseColor (border_color, line.m_color);
-		}
+	{
+		line.m_t_color = t_border_color;
+		if (t_border_color == PP_PropertyMap::color_color)
+			UT_parseColor (border_color, line.m_color);
+	}
 	else if (color)
-		{
-			PP_PropertyMap::TypeColor t_color = PP_PropertyMap::color_type (color);
+	{
+		PP_PropertyMap::TypeColor t_color = PP_PropertyMap::color_type (color);
 
-			line.m_t_color = t_color;
-			if (t_color == PP_PropertyMap::color_color)
-				UT_parseColor (color, line.m_color);
-		}
+		line.m_t_color = t_color;
+		if (t_color == PP_PropertyMap::color_color)
+			UT_parseColor (color, line.m_color);
+	}
 
 	line.m_t_linestyle = PP_PropertyMap::linestyle_type (border_style);
 	if (!line.m_t_linestyle)
@@ -2836,26 +2838,28 @@ static void s_border_properties (const char * border_color, const char * border_
 
 	line.m_t_thickness = PP_PropertyMap::thickness_type (border_width);
 	if (line.m_t_thickness == PP_PropertyMap::thickness_length)
+	{
+		if (UT_determineDimension (border_width, (UT_Dimension)-1) == DIM_PX)
 		{
-			if (UT_determineDimension (border_width, (UT_Dimension)-1) == DIM_PX)
-				{
-					double thickness = UT_LAYOUT_RESOLUTION * UT_convertDimensionless (border_width);
-					line.m_thickness = static_cast<UT_sint32>(thickness / UT_PAPER_UNITS_PER_INCH);
-				}
-			else
-				line.m_thickness = UT_convertToLogicalUnits (border_width);
-
-			if (!line.m_thickness)
-				{
-					double thickness = UT_LAYOUT_RESOLUTION;
-					line.m_thickness = static_cast<UT_sint32>(thickness / UT_PAPER_UNITS_PER_INCH);
-				}
+			double thickness = UT_LAYOUT_RESOLUTION * UT_convertDimensionless (border_width);
+			line.m_thickness = static_cast<UT_sint32>(thickness / UT_PAPER_UNITS_PER_INCH);
 		}
-	else // ??
+		else
+			line.m_thickness = UT_convertToLogicalUnits (border_width);
+
+		if (!line.m_thickness)
 		{
+			// default to 0.72pt
 			double thickness = UT_LAYOUT_RESOLUTION;
 			line.m_thickness = static_cast<UT_sint32>(thickness / UT_PAPER_UNITS_PER_INCH);
 		}
+	}
+	else // ??
+	{
+		// default to 0.72pt
+		double thickness = UT_LAYOUT_RESOLUTION;
+		line.m_thickness = static_cast<UT_sint32>(thickness / UT_PAPER_UNITS_PER_INCH);
+	}
 }
 
 /*!
