@@ -46,11 +46,12 @@
  */
 fp_VerticalContainer::fp_VerticalContainer(FP_ContainerType iType, fl_SectionLayout* pSectionLayout) :
 	fp_Container(iType, pSectionLayout),
+	m_iRedrawHeight(-1),
 	m_iWidth(0),
 	m_iHeight(0),
 	m_iMaxHeight(0),
 	m_iX(0),
-	m_iY(INITIAL_Y_POS),
+	m_iY(INITIAL_OFFSET),
 	m_bIntentionallyEmpty(0),
 	m_imaxContainerHeight(0)
 {
@@ -81,8 +82,11 @@ void fp_VerticalContainer::setWidth(UT_sint32 iWidth)
 		return;
 	}
 	m_iWidth = iWidth;
-	getSectionLayout()->setImageWidth(iWidth);
-	getFillType()->setWidth(getGraphics(),iWidth);
+	if(getContainerType() != FP_CONTAINER_COLUMN)
+	{
+	    getSectionLayout()->setImageWidth(iWidth);
+	    getFillType()->setWidth(getGraphics(),iWidth);
+	}
 
 	// TODO we really need to force a re-line-break operation on every block herein
 
@@ -133,12 +137,14 @@ void fp_VerticalContainer::setHeight(UT_sint32 iHeight)
 		}
 	}
 	m_iHeight = iHeight;
-	if(getContainerType() == FP_CONTAINER_CELL)
+	if(getContainerType() != FP_CONTAINER_COLUMN)
 	{
-	  getSectionLayout()->setImageHeight(getMaxHeight()); // was iHeight
+	  if(getContainerType() == FP_CONTAINER_CELL)
+	  {
+	      getSectionLayout()->setImageHeight(getMaxHeight()); // was iHeight
+	  }
+	  getFillType()->setHeight(getGraphics(),iHeight);
 	}
-	getFillType()->setHeight(getGraphics(),iHeight);
-
 }
 
 /*!
@@ -184,7 +190,6 @@ UT_sint32 fp_VerticalContainer::getY(void) const
 		{
 			pDSL =  static_cast<fl_DocSectionLayout *>(pSL->getDocSectionLayout());
 		}
-//		if((pSL->getContainerType() == FL_CONTAINER_DOCSECTION) || (pSL->getContainerType() == FL_CONTAINER_FOOTNOTE))
 		if(pSL->getContainerType() == FL_CONTAINER_DOCSECTION)
 		{
 			return m_iY - pDSL->getTopMargin();
@@ -566,6 +571,15 @@ void fp_VerticalContainer::getOffsets(fp_ContainerObject* pContainer, UT_sint32&
 		       yoff -= pDSL->getTopMargin();
 		}
 	}
+	if(pCon && getPage() && (pCon->getContainerType() == FP_CONTAINER_ANNOTATION) && 
+	   getPage()->getDocLayout()->displayAnnotations())
+	{
+	        if(getPage() && getView() && (getView()->getViewMode() != VIEW_PRINT))
+		{
+		       fl_DocSectionLayout * pDSL = getPage()->getOwningSection();
+		       yoff -= pDSL->getTopMargin();
+		}
+	}
 }
 
 
@@ -653,7 +667,8 @@ void fp_VerticalContainer::getScreenOffsets(fp_ContainerObject* pContainer,
 	fp_Container * pCon = static_cast<fp_Container *>(this);
 	bool bCell = false;
 	bool bTable = false;
-	UT_sint32 xcell,ycell = 0;
+	UT_sint32 xcell = 0;
+	UT_sint32 ycell = 0;
 	if((getContainerType() == FP_CONTAINER_TABLE) && (pContainer->getContainerType() == FP_CONTAINER_CELL))
 	{
 	        pCon =  static_cast<fp_Container *>(pContainer);
@@ -789,6 +804,19 @@ void fp_VerticalContainer::getScreenOffsets(fp_ContainerObject* pContainer,
 		       yoff -= pDSL->getTopMargin();
 		}
 	}
+	else if(pCon->getContainerType() == FP_CONTAINER_ANNOTATION)
+	{
+		fp_AnnotationContainer * pAC = static_cast<fp_AnnotationContainer *>(pCon);
+		pAC->getPage()->getScreenOffsets(pAC, col_x, col_y);
+
+		xoff += col_x;
+		yoff += col_y;
+	        if(pAC->getPage() && getView() && (getView()->getViewMode() != VIEW_PRINT))
+		{
+		       fl_DocSectionLayout * pDSL = getPage()->getOwningSection();
+		       yoff -= pDSL->getTopMargin();
+		}
+	}
 	else if(pCon->getContainerType() == FP_CONTAINER_FRAME)
 	{
 		fp_FrameContainer * pFC = static_cast<fp_FrameContainer *>(pCon);
@@ -837,6 +865,7 @@ void fp_VerticalContainer::removeContainer(fp_Container* pContainer,bool bClear)
 bool fp_VerticalContainer::insertContainer(fp_Container* pNewContainer)
 {
 	UT_return_val_if_fail(pNewContainer,false);
+	UT_ASSERT(pNewContainer->getContainerType() != FP_CONTAINER_ANNOTATION);
 	pNewContainer->clearScreen();
 	xxx_UT_DEBUGMSG(("Insert  Container after CS %x in column %x \n",pNewContainer,this));
 	insertConAt(pNewContainer, 0);
@@ -862,6 +891,7 @@ UT_sint32	fp_VerticalContainer::getColumnGap(void) const
 bool fp_VerticalContainer::addContainer(fp_Container* pNewContainer)
 {
 	UT_return_val_if_fail(pNewContainer,false);
+	UT_ASSERT(pNewContainer->getContainerType() != FP_CONTAINER_ANNOTATION);
 	if(pNewContainer->getContainer() != NULL)
 	{
 		pNewContainer->clearScreen();
@@ -885,7 +915,8 @@ bool fp_VerticalContainer::addContainer(fp_Container* pNewContainer)
 bool fp_VerticalContainer::insertContainerAfter(fp_Container*	pNewContainer, fp_Container*	pAfterContainer)
 {
 	UT_ASSERT(pAfterContainer);
-	UT_ASSERT(pNewContainer);
+	UT_return_val_if_fail(pNewContainer, false);
+	UT_ASSERT(pNewContainer->getContainerType() != FP_CONTAINER_ANNOTATION);
 
 	UT_sint32 count = countCons();
 	UT_sint32 ndx = findCon(pAfterContainer);
@@ -1004,7 +1035,7 @@ bool fp_VerticalContainer::validate(void)
 	UT_sint32 curBot = 0;
 	UT_sint32 oldTop = -1;
 	UT_sint32 oldBot = -1;
-	UT_uint32 i =0;
+	UT_sint32 i =0;
 	bool bValid = true;
 	for(i=0; i<countCons();i++)
 	{
@@ -1029,7 +1060,7 @@ bool fp_VerticalContainer::validate(void)
 				continue;
 			}
 		}
-		curBot = curTop + iH;                    ;
+		curBot = curTop + iH;
 		UT_ASSERT(oldBot <= curTop);
 		if(oldBot > curTop)
 		{
@@ -1073,15 +1104,24 @@ void fp_VerticalContainer::draw(dg_DrawArgs* pDA)
 	bool bStartedDrawing = false;
 	dg_DrawArgs da = *pDA;
 	UT_uint32 count = countCons();
+	UT_sint32 iCurrHeight = 0;
 	xxx_UT_DEBUGMSG(("number of container %d \n",count));
 	for (UT_uint32 i = 0; i < count; i++)
 	{
 		fp_ContainerObject* pContainer = static_cast<fp_ContainerObject*>(getNthCon(i));
+		if(pContainer->getY() == INITIAL_OFFSET)
+		  continue ; // container is not yet placed 
 		bool bInTable = false;
 		bool bInTOC = false;
 
 		da.xoff = pDA->xoff + pContainer->getX();
 		da.yoff = pDA->yoff + pContainer->getY();
+		iCurrHeight = pContainer->getY() + pContainer->getHeight();
+		xxx_UT_DEBUGMSG(("iCurrHeight %d | m_iRedrawHeight %d\n", iCurrHeight, m_iRedrawHeight));
+		if((m_iRedrawHeight > 0) && (iCurrHeight >  m_iRedrawHeight))
+		{
+		    da.bDirtyRunsOnly = false;
+		}
 		xxx_UT_DEBUGMSG(("Draw container %x yoff %d\n",pContainer,da.yoff));
 		xxx_UT_DEBUGMSG(("Draw container %x xoff %d\n",pContainer,da.xoff));
 #if 0
@@ -1136,7 +1176,8 @@ void fp_VerticalContainer::draw(dg_DrawArgs* pDA)
 			break;
 		}
 	}
-    _drawBoundaries(pDA);
+	m_iRedrawHeight = -1;
+	_drawBoundaries(pDA);
 }
 
 /*!
@@ -1214,6 +1255,20 @@ void fp_VerticalContainer::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosit
 			pContainer = pContainerUpper;
 		}
 	}
+	if(getContainerType() == FP_CONTAINER_CELL && (i == 0))
+	{
+	    fp_CellContainer *pCellCon = static_cast<fp_CellContainer *>(this);
+	    if((x < getX()) && (pCellCon->getLeftAttach() == 0) )
+	    {
+		fl_CellLayout * pCL = static_cast<fl_CellLayout *>(getSectionLayout());
+		pos = pCL->getPosition(true)+1;
+		bBOL = true;
+		bEOL = false;
+		//		if((pCellCon->getTopAttach() == 0))
+		//  pos--;
+		return;
+	    } 
+	}
 	if(pContainer->getContainerType() == FP_CONTAINER_TABLE)
 	{
 		xxx_UT_DEBUGMSG(("SEVIOR: Looking in a table \n"));
@@ -1245,10 +1300,10 @@ void fp_VerticalContainer::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosit
 //
 // Deal with wrapped lines where more than one line can have the same Y
 //
-		fp_Line * pLine = static_cast<fp_Line *>(pContainer);
-		if(pLine->isWrapped())
+		fp_Line * pLine2 = static_cast<fp_Line *>(pContainer);
+		if(pLine2->isWrapped())
 		{
-			fp_Line * pNext = static_cast<fp_Line *>(pLine->getNext());
+			fp_Line * pNext = static_cast<fp_Line *>(pLine2->getNext());
 			if(pNext && pNext->isSameYAsPrevious())
 			{
 				fp_ContainerObject *pBest = pContainer;
@@ -1282,12 +1337,12 @@ void fp_VerticalContainer::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosit
 											pos, bBOL, bEOL,isTOC);
 			}
 		}
-		else if(!pLine->canContainPoint())
+		else if(!pLine2->canContainPoint())
 		{
 			// lines that cannot contain point are those that are located in blocks that
 			// cannot contain point (hidden, collapsed, etc. So we need to find the block
 			// that can contain point
-			fl_BlockLayout * pBlock = pLine->getBlock();
+			fl_BlockLayout * pBlock = pLine2->getBlock();
 			UT_return_if_fail( pBlock );
 
 			pBlock = pBlock->getNextBlockInDocument();
@@ -1300,7 +1355,7 @@ void fp_VerticalContainer::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosit
 			if(!pBlock)
 			{
 				// look the other way (reusing pNext, even though it will be previous)
-				pBlock = pLine->getBlock();
+				pBlock = pLine2->getBlock();
 
 				pBlock = pBlock->getPrevBlockInDocument();
 
@@ -1376,7 +1431,7 @@ void fp_VerticalContainer::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosit
 						return;
 					}
 
-					UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );;
+					UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
 				}
 				else
 				{
@@ -1460,7 +1515,7 @@ UT_uint32 fp_VerticalContainer::distanceFromPoint(UT_sint32 x, UT_sint32 y)
  Before the postition of the container is changed, its content is
  first cleared from the screen.
  */
-void fp_VerticalContainer::setX(UT_sint32 iX, bool bDontClearIfNeeded)
+void fp_VerticalContainer::setX(UT_sint32 iX, bool /*bDontClearIfNeeded*/)
 {
 	if (iX == m_iX)
 	{
@@ -1538,12 +1593,12 @@ void fp_VerticalContainer::bumpContainers(fp_ContainerObject* pLastContainerToKe
 
 	UT_ASSERT(ndx >= 0);
 	UT_sint32 i;
-	fp_TOCContainer *pTOC = NULL;
+	fp_TOCContainer *pTOC2 = NULL;
 	fp_VerticalContainer* pNextContainer = static_cast<fp_VerticalContainer*>(getNext());
 	UT_return_if_fail(pNextContainer);
 	if (pNextContainer->isEmpty())
 	{
-		for (i=ndx; i< static_cast<UT_sint32>(countCons()); i++)
+		for (i=ndx; i< countCons(); i++)
 		{
 			fp_Container* pContainer = static_cast<fp_Container*>(getNthCon(i));
 			pContainer->clearScreen();
@@ -1576,7 +1631,7 @@ void fp_VerticalContainer::bumpContainers(fp_ContainerObject* pLastContainerToKe
 	else
 	{
 		bool bTOC = false;
-		for (i=static_cast<UT_sint32>(countCons()) - 1; i >= ndx; i--)
+		for (i=countCons() - 1; i >= ndx; i--)
 		{
 			bTOC = false;
 			fp_Container* pContainer = static_cast<fp_Container*>(getNthCon(i));
@@ -1597,16 +1652,32 @@ void fp_VerticalContainer::bumpContainers(fp_ContainerObject* pLastContainerToKe
 			}
 			if(pContainer->getContainerType() == FP_CONTAINER_TOC)
 			{
-				pTOC = static_cast<fp_TOCContainer *>(pContainer);
-				xxx_UT_DEBUGMSG(("Found TOC %x index %d prev %x to bump to filled Col \n",pTOC,i,pTOC->getPrevContainerInSection()));
-				if(!pTOC->isThisBroken())
+				pTOC2 = static_cast<fp_TOCContainer *>(pContainer);
+				xxx_UT_DEBUGMSG(("Found TOC %x index %d prev %x to bump to filled Col \n",pTOC2,i,pTOC2->getPrevContainerInSection()));
+				if(!pTOC2->isThisBroken())
 				{
-					pTOC->deleteBrokenTOCs(true);
+					pTOC2->deleteBrokenTOCs(true);
 				}
 				bTOC = true;
 			}
 #endif
+			fp_Line * pLine = NULL;
+			UT_sint32 iOldMaxWidth = 0;
+			if(pContainer->getContainerType() == FP_CONTAINER_LINE)
+			{
+			    pLine =  static_cast<fp_Line *>(pContainer);
+			    iOldMaxWidth = pLine->getMaxWidth();
+			}
 			pNextContainer->insertContainer(pContainer);
+			//
+			// Max Line widths can change after a bump. If so
+			// we must reformat from the start of the line
+			//
+			if(pLine && (pLine->getMaxWidth() != iOldMaxWidth))
+			{
+			    UT_DEBUGMSG(("MaxWidthChanged from container bump \n"));
+			    pLine->setReformat();
+			}
 			if(bTOC)
 			{
 				//UT_sint32 iTOC = pNextContainer->findCon(pContainer);
@@ -1614,12 +1685,12 @@ void fp_VerticalContainer::bumpContainers(fp_ContainerObject* pLastContainerToKe
 			}
 		}
 	}
-	if(pTOC)
+	if(pTOC2)
 	{
 		//UT_sint32 iTOC = pNextContainer->findCon(pTOC);
 		xxx_UT_DEBUGMSG(("TOC Final location %d in next Container \n",iTOC));
 	}
-	for (i=static_cast<UT_sint32>(countCons()) - 1; i >= ndx; i--)
+	for (i=countCons() - 1; i >= ndx; i--)
 	{
 		deleteNthCon(i);
 	}
@@ -1657,7 +1728,7 @@ fp_Column::~fp_Column()
 void fp_Column::collapseEndnotes(void)
 {
 	UT_sint32 i = 0;
-	for(i=static_cast<UT_sint32>(countCons())-1; i>= 0; i--)
+	for(i=countCons()-1; i>= 0; i--)
 	{
 		fp_Container * pCon = static_cast<fp_Container *>(getNthCon(i));
 		if(pCon->getContainerType() == FP_CONTAINER_ENDNOTE)
@@ -1767,19 +1838,28 @@ void fp_Column::layout(void)
 {
 	clearWrappedLines();
 	_setMaxContainerHeight(0);
-	UT_sint32 iY = 0, iPrevY = 0;
+	UT_sint32 iY = 0, iPrevY2 = 0;
 	UT_sint32 iOldY  =-1;
 	UT_GenericVector<fl_BlockLayout *> vecBlocks;
 	fp_Line * pLastLine = NULL;
-	fp_Container *pContainer, *pPrevContainer = NULL;
+	fp_Container *pContainer = NULL;
+	fp_Container *pPrevContainer = NULL;
 	UT_sint32 i  = 0;
-	for (i=0; i < static_cast<UT_sint32>(countCons()) ; i++)
+	//
+	// RedrawHeight makes sure we redraw from whereever a line
+	// changes position.
+	//
+	m_iRedrawHeight = -1;
+	for (i=0; i < countCons(); i++)
 	{
 		pContainer = static_cast<fp_Container*>(getNthCon(i));
 
 		// ignore footnotes
 		if (pContainer->getContainerType() == FP_CONTAINER_FOOTNOTE)
 			continue;
+		// ignore annotations
+		if (pContainer->getContainerType() == FP_CONTAINER_ANNOTATION)
+			continue;
 
 		xxx_UT_DEBUGMSG(("Column Layout: Container %d Container %x Type %d \n",i,pContainer,pContainer->getContainerType()));
 //
@@ -1813,6 +1893,9 @@ void fp_Column::layout(void)
 		if(pContainer->getY() != iY)
 		{
 			pContainer->clearScreen();
+			if((m_iRedrawHeight == -1) && (pContainer->getY() > 0))
+			  m_iRedrawHeight = pContainer->getY();
+
 		}
 		xxx_UT_DEBUGMSG(("Layout: container %d setY %d \n",i,iY));
 //
@@ -1846,7 +1929,7 @@ void fp_Column::layout(void)
 		{
 			pLastLine = static_cast<fp_Line *>(pContainer);
 			iHeight = pLastLine->getHeight();
-			UT_sint32 count = static_cast<UT_sint32>(vecBlocks.getItemCount());
+			UT_sint32 count = vecBlocks.getItemCount();
 			if(count == 0)
 			{
 				vecBlocks.addItem(pLastLine->getBlock());
@@ -1883,27 +1966,45 @@ void fp_Column::layout(void)
 				fp_Line * pLine = static_cast<fp_Line *>(pPrevContainer);
 				while(pLine && pLine->isSameYAsPrevious())
 				{
-					pLine->setAssignedScreenHeight(iY - iPrevY);
+					pLine->setAssignedScreenHeight(iY - iPrevY2);
 					pLine = static_cast<fp_Line *>(pLine->getPrev());
 				}
 				if(pLine)
 				{
-					pLine->setAssignedScreenHeight(iY - iPrevY);
+					pLine->setAssignedScreenHeight(iY - iPrevY2);
 				}
 			}
 			else
 			{
-				xxx_UT_DEBUGMSG(("layout: Assigned screen height %x %d \n",pPrevContainer,iY-iPrevY));
-				pPrevContainer->setAssignedScreenHeight(iY - iPrevY);
+				xxx_UT_DEBUGMSG(("layout: Assigned screen height %x %d \n",pPrevContainer,iY-iPrevY2));
+				pPrevContainer->setAssignedScreenHeight(iY - iPrevY2);
 			}
 		}
-		iPrevY = iY;
+		iPrevY2 = iY;
 		iY += iContainerHeight;
 		iY += iContainerMarginAfter;
 		//iY +=  0.5;
 
 		pPrevContainer = pContainer;
 	}
+	// Correct height position of the last line
+	if (pPrevContainer)
+	{
+		UT_ASSERT((iY - iPrevY2 + getGraphics()->tlu(1)) > 0);
+		if(pPrevContainer->getContainerType() == FP_CONTAINER_LINE)
+		{
+			fp_Line * pLine = static_cast<fp_Line *>(pPrevContainer);
+			while(pLine && pLine->isSameYAsPrevious())
+			{
+				pLine->setAssignedScreenHeight(iY - iPrevY2);
+				pLine = static_cast<fp_Line *>(pLine->getPrev());
+			}
+			if(pLine)
+			{
+				pLine->setAssignedScreenHeight(iY - iPrevY2);
+			}
+		}
+	}
 //
 // Set the frames on the page
 //
@@ -1920,28 +2021,6 @@ void fp_Column::layout(void)
 			pBlock->setFramesOnPage(pLastLine);
 		}
 	}
-	// Correct height position of the last line
-	if (pPrevContainer)
-	{
-		UT_ASSERT((iY - iPrevY + getGraphics()->tlu(1)) > 0);
-		if(pPrevContainer->getContainerType() == FP_CONTAINER_LINE)
-		{
-			fp_Line * pLine = static_cast<fp_Line *>(pPrevContainer);
-			while(pLine && pLine->isSameYAsPrevious())
-			{
-				pLine->setAssignedScreenHeight(iY - iPrevY);
-				pLine = static_cast<fp_Line *>(pLine->getPrev());
-			}
-			if(pLine)
-			{
-				pLine->setAssignedScreenHeight(iY - iPrevY);
-			}
-		}
-		else
-		{
-			pPrevContainer->setAssignedScreenHeight(iY - iPrevY);
-		}
-	}
 //	validate();
 
 	if (getHeight() == iY)
@@ -2246,7 +2325,7 @@ void fp_ShadowContainer::clearHdrFtrBoundaries(void)
 {
 	if(!m_bHdrFtrBoxDrawn)
 		return;
-	UT_RGBColor * pClr = getPage()->getFillType()->getColor();
+	const UT_RGBColor * pClr = getPage()->getFillType()->getColor();
 	getGraphics()->setLineWidth(getGraphics()->tlu(1));
 	getGraphics()->setColor(*pClr);
 //
@@ -2348,7 +2427,7 @@ fl_HdrFtrSectionLayout* fp_HdrFtrContainer::getHdrFtrSectionLayout(void) const
  \retval xoff Container's X offset relative the screen actually -10000
  \retval yoff Container's Y offset relative the screen actually -10000
  */
-void fp_HdrFtrContainer::getScreenOffsets(fp_ContainerObject* pContainer,
+void fp_HdrFtrContainer::getScreenOffsets(fp_ContainerObject* /*pContainer*/,
 									UT_sint32& xoff, UT_sint32& yoff)
 {
 	xoff = -100000;
@@ -2368,7 +2447,7 @@ void fp_HdrFtrContainer::clearScreen(void)
  NOP for Draw's
  \param pDA Draw arguments
  */
-void fp_HdrFtrContainer::draw(dg_DrawArgs* pDA)
+void fp_HdrFtrContainer::draw(dg_DrawArgs* /*pDA*/)
 {
 
 }
