@@ -40,11 +40,6 @@
 #include "fv_View.h"
 #include "gr_Painter.h"
 
-#ifdef _MSC_VER
-// MSVC++ warns about using 'this' in initializer list.
-#pragma warning(disable: 4355)
-#endif
-
 /*!
   Create container
   \param iType Container type
@@ -85,6 +80,8 @@ const char * fp_ContainerObject::getContainerString(void)
 		return "FP_CONTAINER_ENDNOTE";
 	case FP_CONTAINER_FOOTNOTE:
 		return "FP_CONTAINER_FOOTNOTE";
+	case FP_CONTAINER_ANNOTATION:
+		return "FP_CONTAINER_ANNOTATION";
 	case FP_CONTAINER_COLUMN_POSITIONED:
 		return "FP_CONTAINER_COLUMN_POSITIONED";
 	case FP_CONTAINER_COLUMN_SHADOW:
@@ -117,6 +114,7 @@ bool fp_ContainerObject::isColumnType(void) const
 	  || (m_iConType == FP_CONTAINER_FRAME)
 	  || (m_iConType == FP_CONTAINER_COLUMN_POSITIONED)
 	  || (m_iConType == FP_CONTAINER_FOOTNOTE)
+	  || (m_iConType == FP_CONTAINER_ANNOTATION)
 	  ;
   return b;
 }
@@ -220,7 +218,7 @@ void fp_Container::clearBrokenContainers(void)
 	}
 	if (0 != getBrokenCount())
 	{
-		UT_uint32 i =0;
+		UT_sint32 i =0;
 
 		for(i=0;(i<countCons()) && (0 != getBrokenCount());i++)
 		{
@@ -315,6 +313,10 @@ fp_Page * fp_Container::getPage(void) const
 	{
 		return static_cast<fp_FootnoteContainer *>(pCon)->getPage();
 	}
+	if(pCon->getContainerType() == FP_CONTAINER_ANNOTATION)
+	{
+		return static_cast<fp_AnnotationContainer *>(pCon)->getPage();
+	}
 	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 	return NULL;
 }
@@ -367,13 +369,13 @@ void fp_Container::addCon(fp_ContainerObject * pCon)
 	pCon->ref();
 }
 
-fp_ContainerObject *  fp_Container:: getNthCon(UT_uint32 i) const
+fp_ContainerObject *  fp_Container:: getNthCon(UT_sint32 i) const
 { 
 	if(countCons() == 0) return NULL;
 	return m_vecContainers.getNthItem(i);
 }
 
-UT_uint32  fp_Container::countCons(void) const
+UT_sint32  fp_Container::countCons(void) const
 {
 	return m_vecContainers.getItemCount();
 }
@@ -396,7 +398,7 @@ bool fp_Container::getPageRelativeOffsets(UT_Rect &r) const
 	
 	UT_return_val_if_fail(pColumnC,false);
 	fl_DocSectionLayout * pDSL = NULL;
-	if(pColumnC->getContainerType() != FP_CONTAINER_FOOTNOTE)
+	if(pColumnC->getContainerType() != FP_CONTAINER_FOOTNOTE || pColumnC->getContainerType() != FP_CONTAINER_ANNOTATION)
 	{
 		if(pColumnC->getContainerType() == FP_CONTAINER_FRAME)
 		{
@@ -420,12 +422,18 @@ bool fp_Container::getPageRelativeOffsets(UT_Rect &r) const
 			}
 		}
 	}
-	else
+	else if (pColumnC->getContainerType() == FP_CONTAINER_FOOTNOTE)
 	{
 		fp_FootnoteContainer * pFC = static_cast<fp_FootnoteContainer *>(pColumnC);
 		fl_FootnoteLayout * pFL = static_cast<fl_FootnoteLayout *>(pFC->getSectionLayout());
 		pDSL = static_cast<fl_DocSectionLayout *>(pFL->myContainingLayout());
 	}
+	else if (pColumnC->getContainerType() == FP_CONTAINER_ANNOTATION)
+	{
+		fp_AnnotationContainer * pAC = static_cast<fp_AnnotationContainer *>(pColumnC);
+		fl_AnnotationLayout * pAL = static_cast<fl_AnnotationLayout *>(pAC->getSectionLayout());
+		pDSL = static_cast<fl_DocSectionLayout *>(pAL->myContainingLayout());
+	}
 	UT_return_val_if_fail(pDSL,false);
 	UT_ASSERT(pDSL->getContainerType() == FL_CONTAINER_DOCSECTION);
 	r.left   = pDSL->getLeftMargin();
@@ -717,7 +725,7 @@ void fg_FillType::_regenerateImage(GR_Graphics * pG)
 /*!
  * Return the most appropriate color.
  */
-UT_RGBColor * fg_FillType::getColor(void)
+const UT_RGBColor * fg_FillType::getColor(void) const
 {
 	if(m_bColorSet)
 	{
@@ -830,7 +838,7 @@ void fg_FillType::Fill(GR_Graphics * pG, UT_sint32 & srcX, UT_sint32 & srcY, UT_
 
 	if(m_pContainer && (m_pContainer->getContainerType() == FP_CONTAINER_CELL))
 	{
-		fp_CellContainer * pCell = static_cast<fp_CellContainer *>(m_pContainer);
+		const fp_CellContainer * pCell = static_cast<const fp_CellContainer *>(m_pContainer);
 		UT_sint32 xoff,yoff;
 		pCell->getLeftTopOffsets(xoff,yoff);
 		if(m_FillType == FG_FILL_IMAGE)
@@ -841,7 +849,7 @@ void fg_FillType::Fill(GR_Graphics * pG, UT_sint32 & srcX, UT_sint32 & srcY, UT_
 	}
 	if(m_pContainer && (m_pContainer->getContainerType() == FP_CONTAINER_FRAME))
 	{
-		fp_FrameContainer * pFrame = static_cast<fp_FrameContainer *>(m_pContainer);
+		const fp_FrameContainer * pFrame = static_cast<const fp_FrameContainer *>(m_pContainer);
 		UT_sint32 xoff = pFrame->getXPad();
 		UT_sint32 yoff = pFrame->getYPad();
 		if(m_FillType == FG_FILL_IMAGE)
