@@ -1,4 +1,3 @@
-
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
  * Copyright (C) 2002 Patrick Lam <plam@mit.edu>
@@ -73,7 +72,7 @@ fp_FrameContainer::fp_FrameContainer(fl_SectionLayout* pSectionLayout)
  */
 fp_FrameContainer::~fp_FrameContainer()
 {
-  UT_DEBUGMSG(("Delete FrameContainer %x \n",this));
+  UT_DEBUGMSG(("Delete FrameContainer %p \n",this));
 	m_pPage = NULL;
 }
 
@@ -88,7 +87,7 @@ void fp_FrameContainer::setPage(fp_Page * pPage)
 		UT_GenericVector<fl_ContainerLayout *> AllLayouts;
 		AllLayouts.clear();
 		m_pPage->getAllLayouts(AllLayouts);
-		UT_uint32 i = 0;
+		UT_sint32 i = 0;
 		for(i=0; i<AllLayouts.getItemCount(); i++)
 		{
 		      fl_ContainerLayout * pCL = AllLayouts.getNthItem(i);
@@ -109,6 +108,14 @@ void fp_FrameContainer::setPage(fp_Page * pPage)
 	}
 }
 
+/*!
+ * Returns true if the frame should be relocated before placing on a page
+ */
+bool fp_FrameContainer::isRelocate(void) const
+{
+     fl_FrameLayout * pFL = static_cast<fl_FrameLayout *>(getSectionLayout());
+     return pFL->isRelocate();
+}
 /*!
  * Returns true if the supplied screen rectangle overlaps with frame
  * container. This method takes account of transparening and tight wrapping.
@@ -296,7 +303,7 @@ void fp_FrameContainer::clearScreen(void)
 	getFillType()->getParent()->Fill(getGraphics(),srcX,srcY,xoff,yoff,getFullWidth()+leftThick+rightThick,getFullHeight()+topThick+botThick+getGraphics()->tlu(1) +1);
 	fp_Container * pCon = NULL;
 	UT_sint32 i = 0;
-	for(i=0; i< static_cast<UT_sint32>(countCons()); i++)
+	for(i=0; i< countCons(); i++)
 	{
 		pCon = static_cast<fp_Container *>(getNthCon(i));
 		pCon->clearScreen();
@@ -355,7 +362,7 @@ UT_sint32 fp_FrameContainer::getHeight(void) const
 }
 
 	
-void fp_FrameContainer::setContainer(fp_Container * pContainer)
+void fp_FrameContainer::setContainer(fp_Container * /*pContainer*/)
 {
 	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 }
@@ -379,7 +386,7 @@ void fp_FrameContainer::getBlocksAroundFrame(UT_GenericVector<fl_BlockLayout *>
   {
     return;
   }
-  UT_uint32 iColLeader = 0;
+  UT_sint32 iColLeader = 0;
   fp_Column * pCol = NULL;
   fl_BlockLayout * pCurBlock = NULL;
   fp_Line * pCurLine = NULL;
@@ -388,40 +395,40 @@ void fp_FrameContainer::getBlocksAroundFrame(UT_GenericVector<fl_BlockLayout *>
   {
       UT_sint32 iPage = getPreferedPageNo();
       if(iPage >0)
-	setPreferedPageNo(iPage-1);
+          setPreferedPageNo(iPage-1);
       return;
   }
   for(iColLeader = 0; iColLeader < pPage->countColumnLeaders(); iColLeader++)
   {
-    pCol = pPage->getNthColumnLeader(iColLeader);
-    while(pCol)
-    {
-      UT_uint32 i = 0;
-      UT_sint32 iYCol = pCol->getY(); // Vertical position relative to page.
-      for(i=0; i< pCol->countCons(); i++)
+      pCol = pPage->getNthColumnLeader(iColLeader);
+      while(pCol)
       {
-	pCurCon = static_cast<fp_Container *>(pCol->getNthCon(i));
-	if(pCurCon->getContainerType() == FP_CONTAINER_LINE)
-	{
-	  pCurLine = static_cast<fp_Line *>(pCurCon);
-	  UT_sint32 iYLine = iYCol + pCurLine->getY();
-	  xxx_UT_DEBUGMSG(("iYLine %d FullY %d FullHeight %d \n",iYLine,getFullY(),getFullHeight()));
-	  if((iYLine + pCurLine->getHeight() > getFullY()) && (iYLine < (getFullY() + getFullHeight())))
-	  {
-	    //
-	    // Line overlaps frame in Height. Add it's block to the vector.
-	    //
-	    if(pCurLine->getBlock() != pCurBlock)
-	    {
-	      pCurBlock = pCurLine->getBlock();
-	      vecBlocks.addItem(pCurBlock);
-	      xxx_UT_DEBUGMSG(("Add Block %x to vector \n",pCurBlock));
-	    }
-	  }
-	}
+          UT_sint32 i = 0;
+          UT_sint32 iYCol = pCol->getY(); // Vertical position relative to page.
+          for(i=0; i< pCol->countCons(); i++)
+          {
+              pCurCon = static_cast<fp_Container *>(pCol->getNthCon(i));
+              if(pCurCon->getContainerType() == FP_CONTAINER_LINE)
+              {
+                  pCurLine = static_cast<fp_Line *>(pCurCon);
+                  UT_sint32 iYLine = iYCol + pCurLine->getY();
+                  xxx_UT_DEBUGMSG(("iYLine %d FullY %d FullHeight %d \n",iYLine,getFullY(),getFullHeight()));
+                  if((iYLine + pCurLine->getHeight() > getFullY()) && (iYLine < (getFullY() + getFullHeight())))
+                  {
+                      //
+                      // Line overlaps frame in Height. Add it's block to the vector.
+                      //
+                      if(pCurLine->getBlock() != pCurBlock)
+                      {
+                          pCurBlock = pCurLine->getBlock();
+                          vecBlocks.addItem(pCurBlock);
+                          xxx_UT_DEBUGMSG(("Add Block %x to vector \n",pCurBlock));
+                      }
+                  }
+              }
+          }
+          pCol = pCol->getFollower();
       }
-      pCol = pCol->getFollower();
-    }
   }
   if(vecBlocks.getItemCount() == 0)
   {
@@ -430,15 +437,15 @@ void fp_FrameContainer::getBlocksAroundFrame(UT_GenericVector<fl_BlockLayout *>
       fl_BlockLayout * pB = NULL;
       if(pCon && pCon->getContainerType() == FP_CONTAINER_LINE)
       {
-	  pB = static_cast<fp_Line *>(pCon)->getBlock();
+          pB = static_cast<fp_Line *>(pCon)->getBlock();
       }
       else if(pCon)
       {
-	  fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pCon->getSectionLayout());
-	  pB = pCL->getNextBlockInDocument();
+          fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pCon->getSectionLayout());
+          pB = pCL->getNextBlockInDocument();
       }
       if(pB != NULL)
-	vecBlocks.addItem(pB);
+          vecBlocks.addItem(pB);
   }
 
 }
@@ -665,7 +672,7 @@ void fp_FrameContainer::draw(dg_DrawArgs* pDA)
 	if(pPrevRect == NULL)
 	{
 		pDA->pG->setClipRect(pRect);
-		xxx_UT_DEBUGMSG(("Clip bottom is %d \n",pRect->top + pRect->height));
+		UT_DEBUGMSG(("Clip bottom is %d \n",pRect->top + pRect->height));
 		bRemoveRectAfter = true;
 	}
 	else if(!pRect->intersectsRect(pPrevRect))
