@@ -73,6 +73,11 @@ fp_Page::fp_Page(FL_DocLayout* pLayout,
 	m_rDamageRect.top = 0;
 	m_rDamageRect.width = 0;
 	m_rDamageRect.height = 0;
+	m_vecFootnotes.clear();
+	m_vecAnnotations.clear();
+	m_vecAboveFrames.clear();
+	m_vecBelowFrames.clear();
+
 	xxx_UT_DEBUGMSG(("!!!!!!!!!!!!!!!!!!!!!!!!!!Created Page %x \n",this));
 }
 
@@ -94,7 +99,7 @@ fp_Page::~fp_Page()
 		 if(pHdrFtr != NULL && pHdrFtr->isPageHere(this))
 		 {
 		   pHdrFtr->deletePage(this);
-		   UT_DEBUGMSG(("Remove Page from Hdr %x in page destructor \n",pHdrFtr));
+		   UT_DEBUGMSG(("Remove Page from Hdr %p in page destructor \n",pHdrFtr));
 		 }
 	    }
 	    if(m_pFooter != NULL)
@@ -103,7 +108,7 @@ fp_Page::~fp_Page()
 		 if(pHdrFtr != NULL && pHdrFtr->isPageHere(this))
 		 {
 		   pHdrFtr->deletePage(this);
-		   UT_DEBUGMSG(("Remove Page from Ftr %x in page destructor \n",pHdrFtr));
+		   UT_DEBUGMSG(("Remove Page from Ftr %p in page destructor \n",pHdrFtr));
 		 }
 	    }
 	}
@@ -121,13 +126,76 @@ fg_FillType * fp_Page::getFillType(void)
 
 bool fp_Page::isEmpty(void) const
 {
-	if((m_vecColumnLeaders.getItemCount() == 0) && (m_vecFootnotes.getItemCount() == 0) && (m_vecAboveFrames.getItemCount() == 0) && (m_vecBelowFrames.getItemCount() == 0))
+	if((m_vecColumnLeaders.getItemCount() == 0) && (m_vecFootnotes.getItemCount() == 0) && (m_vecAnnotations.getItemCount() == 0) && (m_vecAboveFrames.getItemCount() == 0) && (m_vecBelowFrames.getItemCount() == 0))
 	{
 		return true;
 	}
 	return false;
 }
 
+/*!
+ * This method sets the page number in all thepages frames.
+ */
+
+void fp_Page::setPageNumberInFrames(void)
+{
+	UT_sint32 iPage = getDocLayout()->findPage(this);
+	UT_sint32 i = 0;
+	for(i=0; i< static_cast<UT_sint32>(countAboveFrameContainers()); i++)
+	{
+		fp_FrameContainer * pFrame = getNthAboveFrameContainer(i);
+		pFrame->setPreferedPageNo(iPage);
+	}
+	for(i=0; i< static_cast<UT_sint32>(countBelowFrameContainers()); i++)
+	{
+		fp_FrameContainer * pFrame = getNthBelowFrameContainer(i);
+		pFrame->setPreferedPageNo(iPage);
+	}
+}
+
+/*!
+ * Fill a vector with all the layouts referenced from this page.
+ */
+void fp_Page::getAllLayouts(UT_GenericVector<fl_ContainerLayout *> & AllLayouts){
+	fp_Column * pCol = NULL;
+	UT_sint32 i = 0;
+	fl_ContainerLayout * pPrevCL = NULL;
+	fl_ContainerLayout * pCurCL = NULL;
+	for(i= 0; i< m_vecColumnLeaders.getItemCount(); i++)
+	{
+		pCol = m_vecColumnLeaders.getNthItem(i);
+		while(pCol)
+		{
+			UT_sint32 j= 0;
+			fp_ContainerObject * pCon = NULL;
+			for(j = 0; j< pCol->countCons(); j++)
+			{
+				pCon = pCol->getNthCon(j);
+				if(pCon->getContainerType() == FP_CONTAINER_LINE)
+				{
+					pCurCL = static_cast<fl_ContainerLayout *>(static_cast<fp_Line *>(pCon)->getBlock());
+					if(pCurCL != pPrevCL)
+					{
+						pPrevCL = pCurCL;
+						AllLayouts.addItem(pPrevCL);
+					}
+				}
+				if(pCon->getContainerType() == FP_CONTAINER_TABLE)
+				{
+					pCurCL = static_cast<fl_ContainerLayout *>(static_cast<fp_TableContainer *>(pCon)->getSectionLayout());
+					if(pCurCL != pPrevCL)
+					{
+						pPrevCL = pCurCL;
+						AllLayouts.addItem(pPrevCL);
+					}
+				}
+
+			}
+			pCol = pCol->getFollower();
+		}
+	}
+}
+
 bool fp_Page::isOnScreen(void)
 {
 	if(!m_pView)
@@ -140,7 +208,9 @@ bool fp_Page::isOnScreen(void)
 	{
 		return false;
 	}
-	if(yoff > getHeight())
+	if(!m_pView)
+		return false;
+	if(yoff > m_pView->getWindowHeight())
 	{
 		return false;
 	}
@@ -184,25 +254,30 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 		return NULL;
 	}
 	m_iCountWrapPasses++;
-#if DEBUG
+#if 0
 	UT_sint32 iPage = getDocLayout()->findPage(this);
-	xxx_UT_DEBUGMSG(("Wrap passes = %d page %x page number %d \n",m_iCountWrapPasses,this,iPage ));
+	UT_DEBUGMSG(("Wrap passes = %d page %x page number %d \n",m_iCountWrapPasses,this,iPage ));
+	if(getPrev())
+		{
+			iPage = getDocLayout()->findPage(getPrev());
+			xxx_UT_DEBUGMSG(("Prev page %x Prev page number %d Number Frames %d \n",getPrev(),iPage,getPrev()->countAboveFrameContainers() ));
+		}
 #endif
 	if(m_iCountWrapPasses > 100)
-        {
-	  UT_DEBUGMSG(("Number of wrapped passed = %d \n",m_iCountWrapPasses));
+    {
+	  xxx_UT_DEBUGMSG(("Number of wrapped passed = %d \n",m_iCountWrapPasses));
 	}
 	UT_sint32 i= 0;
 	UT_sint32 nWrapped = 0;
-	fp_Container * pFirst = NULL;
+	fp_Container * pFirst2 = NULL;
 	fl_BlockLayout * pFirstBL = NULL;
 	for(i=0; i < static_cast<UT_sint32>(countColumnLeaders()); i++)
 	{
 		fp_Column * pCol = getNthColumnLeader(i);
 		if(i == 0)
 		{
-			pFirst = static_cast<fp_Container *>(pCol->getNthCon(0));
-			if(pFirst == NULL)
+			pFirst2 = static_cast<fp_Container *>(pCol->getNthCon(0));
+			if(pFirst2 == NULL)
 			{
 				return NULL;
 			}
@@ -210,7 +285,7 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 		while(pCol)
 		{
 			nWrapped += pCol->getNumWrapped();
-			pCol = static_cast<fp_Column *>(pCol->getNext());
+			pCol = static_cast<fp_Column *>(pCol->getFollower());
 		}
 	}
 	UT_sint32 nWrappedObjs = 0;
@@ -225,20 +300,40 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 	if((nWrapped == 0) && (nWrappedObjs == 0))
 	{
 		xxx_UT_DEBUGMSG(("page %x nWrapped %d nWrappedObjs %d does not need updating \n",this,nWrapped,nWrappedObjs));
-		return NULL;
+		fp_Page * pPrevPage = getPrev();
+		//
+		// In creating lines that correctly wrap frames on the previous page
+		// the next page may have inherited some partially broken lines. 
+		// The following code check for the this situation so it can be 
+		// fixed here.
+		//
+		if(pPrevPage && (pPrevPage->countAboveFrameContainers() > 0))
+		{
+				for(i=0; i< static_cast<UT_sint32>(pPrevPage->countAboveFrameContainers()); i++)
+				{
+						fp_FrameContainer * pFrame = pPrevPage->getNthAboveFrameContainer(i);
+						if(pFrame->isWrappingSet())
+						{
+								nWrappedObjs++;
+						}
+				}
+				
+		}
+		if(nWrappedObjs == 0)
+			return NULL;
 	}
 	bool bFormatAllWrapped = ((nWrapped > 0) && (nWrappedObjs == 0));
 	UT_GenericVector<_BL *> vecBL;
 	vecBL.clear();
 	for(i=0; i < static_cast<UT_sint32>(countColumnLeaders()); i++)
 	{
-		fp_Column * pCol = getNthColumnLeader(i);
-		while(pCol)
+		fp_Column * pCol2 = getNthColumnLeader(i);
+		while(pCol2)
 		{
 			UT_sint32 j = 0;
-			for(j=0; j < static_cast<UT_sint32>(pCol->countCons()); j++)
+			for(j=0; j < pCol2->countCons(); j++)
 			{
-				fp_Container * pCon = static_cast<fp_Container *>(pCol->getNthCon(j));
+				fp_Container * pCon = static_cast<fp_Container *>(pCol2->getNthCon(j));
 				xxx_UT_DEBUGMSG(("Look at con %x at j %d \n",pCon,j));
 				if(pCon->getContainerType() == FP_CONTAINER_LINE)
 				{
@@ -257,7 +352,7 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 // SOMETHING HAS GONE HORRIBALLY WRONG. Try to recover by collapsing the
 // content in this column and rebuilding
 //
-						UT_DEBUGMSG(("-ve width here!!!! %x left %d right %d \n",pLine,recLeft.width,recRight.width));
+						UT_DEBUGMSG(("-ve width here!!!! %p left %d right %d \n",pLine,recLeft.width,recRight.width));
 						UT_VECTOR_PURGEALL(_BL *, vecBL);
 						fl_BlockLayout * pBL = pLine->getBlock();
 						fl_BlockLayout * pFirst = pBL;
@@ -282,6 +377,7 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 						fp_Column * pFirstCol = static_cast<fp_Column *>(pFirst->getFirstContainer()->getColumn());
 						pBL = pFirst;
 						UT_GenericVector<fl_BlockLayout *> vecCollapse;
+						vecCollapse.addItem(pBL);
 						bLoop = true;
 						while(bLoop)
 						{
@@ -300,7 +396,7 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 							}
 						}
 						UT_sint32 k = 0;
-						for(k=0; k<static_cast<UT_sint32>(vecCollapse.getItemCount());k++)
+						for(k=0; k<vecCollapse.getItemCount();k++)
 						{
 							pBL = vecCollapse.getNthItem(k);
 							pBL->collapse();
@@ -367,7 +463,7 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 							fl_BlockLayout * pBL = pLine->getBlock();
 							bool bPrev = false;
 							UT_sint32 k = 0;
-							for(k=0; k<static_cast<UT_sint32>(vecBL.getItemCount()); k++)
+							for(k=0; k<vecBL.getItemCount(); k++)
 							{
 								_BL * ppBL = vecBL.getNthItem(k);
 								if(ppBL->m_pBL == pBL)
@@ -384,11 +480,11 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 							while(pLine && pLine->getBlock() == pBL)
 							{
 								k++;
-								if(k >= static_cast<UT_sint32>(pCol->countCons()))
+								if(k >= pCol2->countCons())
 								{
 									break;
 								}
-								pCon = static_cast<fp_Container *>(pCol->getNthCon(k));
+								pCon = static_cast<fp_Container *>(pCol2->getNthCon(k));
 								if(pCon->getContainerType() == FP_CONTAINER_LINE)
 								{
 									pLine = static_cast<fp_Line *>(pCon);
@@ -497,7 +593,7 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 							fl_BlockLayout * pBL = pLine->getBlock();
 							bool bPrev = false;
 							UT_sint32 k = 0;
-							for(k=0; k<static_cast<UT_sint32>(vecBL.getItemCount()); k++)
+							for(k=0; k<vecBL.getItemCount(); k++)
 							{
 								_BL * ppBL = vecBL.getNthItem(k);
 								if(ppBL->m_pBL == pBL)
@@ -514,11 +610,11 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 							while(pLine && pLine->getBlock() == pBL)
 							{
 								k++;
-								if(k >= static_cast<UT_sint32>(pCol->countCons()))
+								if(k >= pCol2->countCons())
 								{
 									break;
 								}
-								pCon = static_cast<fp_Container *>(pCol->getNthCon(k));
+								pCon = static_cast<fp_Container *>(pCol2->getNthCon(k));
 								if(pCon->getContainerType() == FP_CONTAINER_LINE)
 								{
 									pLine = static_cast<fp_Line *>(pCon);
@@ -535,7 +631,7 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 				if(j< 0)
 				  j = 0;
 			}
-			pCol = static_cast<fp_Column *>(pCol->getFollower());
+			pCol2 = static_cast<fp_Column *>(pCol2->getFollower());
 		}
 	}
 	if(vecBL.getItemCount() == 0)
@@ -544,7 +640,7 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 	}
 	_BL * pBLine = vecBL.getNthItem(0);
 	pFirstBL = pBLine->m_pBL;
-	for(i=0; i<static_cast<UT_sint32>(vecBL.getItemCount()); i++)
+	for(i=0; i<vecBL.getItemCount(); i++)
 	{
 		pBLine = vecBL.getNthItem(i);
 		xxx_UT_DEBUGMSG((" Doing line %x \n",pBLine->m_pL));
@@ -578,12 +674,15 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 	}
 	pNextCol = static_cast<fp_Column *>(pNewFirstCon->getColumn());
 	pNewFirstCon = static_cast<fp_Container *>(pNextCol->getNthCon(0));
-	if(pNewFirstCon->getContainerType() == FP_CONTAINER_LINE)
-	  {
-	    fp_Line * pFLine = static_cast<fp_Line *>(pNewFirstCon);
-	    UT_ASSERT(pFLine->getBlock() && (pFLine->getBlock()->findLineInBlock(pFLine) >= 0));
+	if(pNewFirstCon && pNewFirstCon->getContainerType() == FP_CONTAINER_LINE)
+	{
+#if DEBUG
+		fp_Line * pFLine = static_cast<fp_Line *>(pNewFirstCon);
+		UT_ASSERT(pFLine->getBlock() && 
+				  (pFLine->getBlock()->findLineInBlock(pFLine) >= 0));
 	    //	    UT_ASSERT(!pFLine-isEmpty());
-	  }
+#endif
+	}
 	return pNewFirstCon;
 }
 
@@ -654,7 +753,7 @@ fp_TableContainer * fp_Page::getContainingTable(PT_DocPosition pos)
 		pColumn = getNthColumnLeader(i);
 		while(pColumn)
 		{
-			for(j =0; j< static_cast<UT_sint32>(pColumn->countCons()) && !bFound;j++)
+			for(j =0; j< pColumn->countCons() && !bFound;j++)
 			{
 				fp_Container * pCon = static_cast<fp_Container*>(pColumn->getNthCon(j));
 				if(pCon->getContainerType() == FP_CONTAINER_TABLE)
@@ -694,13 +793,21 @@ UT_sint32 fp_Page::getAvailableHeight(void) const
 		fp_FootnoteContainer * pFC = getNthFootnoteContainer(i);
 		avail -= pFC->getHeight();
 	}
+	if(getDocLayout()->displayAnnotations())
+	{
+			for(i=0; i< static_cast<UT_sint32>(countAnnotationContainers()); i++)
+			{
+					fp_AnnotationContainer * pAC = getNthAnnotationContainer(i);
+					avail -= pAC->getHeight();
+			}
+	}
 	return avail;
 }
 
 /*!
  * This method returns the height available to the requested column. It 
  * subtracts the height given to previous columns on the page as well as the 
- * height given to footnotes.
+ * height given to footnotes and annotations.
 */  
 UT_sint32 fp_Page::getAvailableHeightForColumn(const fp_Column * pColumn) const
 {
@@ -733,13 +840,21 @@ UT_sint32 fp_Page::getAvailableHeightForColumn(const fp_Column * pColumn) const
 		}
 	}
 //
-// Now subtract the footnotes on this page too
+// Now subtract the footnotes and annotations on this page too
 //
 	for(i=0; i< static_cast<UT_sint32>(countFootnoteContainers()); i++)
 	{
 		fp_FootnoteContainer * pFC = getNthFootnoteContainer(i);
 		avail -= pFC->getHeight();
 	}
+	if(getDocLayout()->displayAnnotations())
+	{
+			for(i=0; i< static_cast<UT_sint32>(countAnnotationContainers()); i++)
+			{
+					fp_AnnotationContainer * pAC = getNthAnnotationContainer(i);
+					avail -= pAC->getHeight();
+			}
+	}
 	return avail;
 
 }
@@ -754,7 +869,7 @@ UT_sint32 fp_Page::getFilledHeight(fp_Container * prevContainer) const
 	UT_sint32 totalHeight = 0;
     UT_sint32 maxHeight = 0;
 	fp_Column * pColumn = NULL;
-	UT_uint32 i =0;
+	UT_sint32 i =0;
 	fp_Column * prevColumn = NULL;
 	bool bstop = false;
 	if(prevContainer)
@@ -921,7 +1036,7 @@ void fp_Page::_drawCropMarks(dg_DrawArgs* pDA)
     }
 }
 
-void fp_Page::draw(dg_DrawArgs* pDA, bool bAlwaysUseWhiteBackground)
+void fp_Page::draw(dg_DrawArgs* pDA, bool /*bAlwaysUseWhiteBackground*/)
 {
 //
 // Fill the Page with the page color
@@ -1057,6 +1172,27 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool bAlwaysUseWhiteBackground)
 		pFC->draw(&da);
 	}
 
+	// draw annotations
+	if(getDocLayout()->displayAnnotations())
+	{
+			count = m_vecAnnotations.getItemCount();
+			for (i=0; i<count; i++)
+			{
+					fp_AnnotationContainer* pAC = m_vecAnnotations.getNthItem(i);
+					dg_DrawArgs da = *pDA;
+					if(m_pView && (m_pView->getViewMode() != VIEW_PRINT) &&
+					   !pDA->pG->queryProperties(GR_Graphics::DGP_PAPER))
+					{
+							fp_Column* pFirstColumnLeader = getNthColumnLeader(0);
+							fl_DocSectionLayout* pFirstSectionLayout = (pFirstColumnLeader->getDocSectionLayout());
+							da.yoff -= pFirstSectionLayout->getTopMargin();
+					}
+					da.xoff += pAC->getX();
+					da.yoff += pAC->getY();
+					pAC->draw(&da);
+			}
+	}
+
 	// draw Above Frames
 	count = m_vecAboveFrames.getItemCount();
 	for (i=0; i<count; i++)
@@ -1155,7 +1291,7 @@ bool fp_Page::needsRedraw(void) const
 	return m_bNeedsRedraw;
 }
 
-UT_uint32 fp_Page::countColumnLeaders(void) const
+UT_sint32 fp_Page::countColumnLeaders(void) const
 {
 	return m_vecColumnLeaders.getItemCount();
 }
@@ -1200,7 +1336,7 @@ bool fp_Page::TopBotMarginChanged(void)
  */
 bool fp_Page::breakPage(void)
 {
-	UT_uint32 count = countColumnLeaders();
+	UT_sint32 count = countColumnLeaders();
 	if (count == 0)
 	{
 		return true;
@@ -1215,7 +1351,7 @@ bool fp_Page::breakPage(void)
 	UT_sint32 availHeight = getHeight() - iBottomMargin;
 		
 	// we need the height of the footnotes on this page, to deduct.
-	UT_uint32 i = 0;
+	UT_sint32 i = 0;
 	UT_uint32 iFootnoteHeight = 2*pFirstSectionLayout->getFootnoteLineThickness();
 	for (i = 0; i < countFootnoteContainers(); i++)
 	{
@@ -1223,6 +1359,18 @@ bool fp_Page::breakPage(void)
 	}
 	iY =+ iFootnoteHeight;
 
+		
+	// we need the height of the annotations on this page, to deduct.
+	if(getDocLayout()->displayAnnotations())
+	{
+		    UT_sint32 iAnnotationHeight = 0;
+			for (i = 0; i < countAnnotationContainers(); i++)
+			{
+					iAnnotationHeight += getNthAnnotationContainer(i)->getHeight();
+			}
+			iY =+ iAnnotationHeight;
+	}
+
 	for (i=0; i<count; i++)
 	{
 		fp_Column* pLeader = getNthColumnLeader(i);
@@ -1425,10 +1573,8 @@ void fp_Page::updateColumnX()
 	if (count == 0)
 		return;
 
-	fp_Column* pFirstColumnLeader = getNthColumnLeader(0);
 	fp_Column * pLastCol = NULL;
-	fl_DocSectionLayout* pFirstSectionLayout = (pFirstColumnLeader->getDocSectionLayout());
-	UT_ASSERT(m_pOwner == pFirstSectionLayout);
+	UT_ASSERT(m_pOwner == getNthColumnLeader(0)->getDocSectionLayout());
 
 
 	UT_sint32 iLeftMargin = 0;
@@ -1440,7 +1586,7 @@ void fp_Page::updateColumnX()
 		UT_ASSERT(pLeader->getContainerType() == FP_CONTAINER_COLUMN);
 		fl_DocSectionLayout* pSL = (pLeader->getDocSectionLayout());
 
-		if(m_pView->getViewMode() == VIEW_NORMAL &&
+		if((m_pView->getViewMode() == VIEW_NORMAL || m_pView->getViewMode() == VIEW_WEB) &&
 		   !m_pLayout->getGraphics()->queryProperties(GR_Graphics::DGP_PAPER))
 		{
 			iLeftMargin = m_pView->getNormalModeXOffset();
@@ -1496,12 +1642,14 @@ void fp_Page::_reformat(void)
 {
 	// this is naive, because columns can cause the footnotes
 	// to change pages.  But it'll do for now.
-	_reformatColumns(); _reformatFootnotes();
+	_reformatColumns(); 
+	_reformatFootnotes();
+	_reformatAnnotations();
 }
 
 void fp_Page::_reformatColumns(void)
 {
-	UT_uint32 count = countColumnLeaders();
+	UT_sint32 count = countColumnLeaders();
 	if (count == 0)
 		return;
 
@@ -1520,17 +1668,17 @@ void fp_Page::_reformatColumns(void)
 	UT_sint32 iY = iTopMargin;
 
 	// we need the height of the footnotes on this page, to deduct.
-	UT_uint32 i = 0;
+	UT_sint32 i = 0;
 	UT_uint32 iFootnoteHeight = 2*pFirstSectionLayout->getFootnoteLineThickness();
 	for (i = 0; i < countFootnoteContainers(); i++)
 	{
 		iFootnoteHeight += getNthFootnoteContainer(i)->getHeight();
 	}
-
+	UT_uint32 iAnnotationHeight = getAnnotationHeight();
 	for (i = 0; i < count; i++)
 	{
 #if 0
-		if (iY >= (static_cast<UT_sint32>(getHeight() - iBottomMargin - iFootnoteHeight)))
+		if (iY >= (static_cast<UT_sint32>(getHeight() - iBottomMargin - iFootnoteHeight -AnnotationHeight)))
 		{
 			xxx_UT_DEBUGMSG(("SEVIOR: Page incorrectly laid out iYlayoutuints= %d  \n",iY));
 //			m_pOwner->markForRebuild();
@@ -1549,7 +1697,7 @@ void fp_Page::_reformatColumns(void)
 		UT_ASSERT(pLeader->getContainerType() == FP_CONTAINER_COLUMN);
 		fl_DocSectionLayout* pSL = (pLeader->getDocSectionLayout());
 
-		if(m_pView->getViewMode() == VIEW_NORMAL &&
+		if((m_pView->getViewMode() == VIEW_NORMAL || m_pView->getViewMode() == VIEW_WEB) &&
 		   !m_pLayout->getGraphics()->queryProperties(GR_Graphics::DGP_PAPER))
 		{
 			iLeftMargin = m_pView->getNormalModeXOffset();
@@ -1589,7 +1737,7 @@ void fp_Page::_reformatColumns(void)
 			UT_ASSERT(pTmpCol->getContainerType() == FP_CONTAINER_COLUMN);
 			pTmpCol->setX(iX);
 			pTmpCol->setY(iY);
-			pTmpCol->setMaxHeight(getHeight() - iBottomMargin - iY - iFootnoteHeight);
+			pTmpCol->setMaxHeight(getHeight() - iBottomMargin - iY - iFootnoteHeight - iAnnotationHeight);
 			pTmpCol->setWidth(iColWidth);
 
 			if(pSL->getColumnOrder())
@@ -1682,16 +1830,16 @@ void fp_Page::_reformatColumns(void)
 void fp_Page::clearScreenFootnotes(void)
 {
 	UT_sint32 i =0;
-	for (i = 0; i < static_cast<UT_sint32>(countFootnoteContainers()); i++)
+	for (i = 0; i < countFootnoteContainers(); i++)
 	{
 		getNthFootnoteContainer(i)->clearScreen();
 	}
 }
 
-UT_sint32 fp_Page::getFootnoteHeight(void)
+UT_sint32 fp_Page::getFootnoteHeight(void) const
 {
 	UT_uint32 iFootnoteHeight = 0;
-	UT_uint32 i = 0;
+	UT_sint32 i = 0;
 	for (i = 0; i < countFootnoteContainers(); i++)
 	{
 		iFootnoteHeight += getNthFootnoteContainer(i)->getHeight();
@@ -1713,8 +1861,9 @@ void fp_Page::_reformatFootnotes(void)
 	UT_ASSERT(m_pOwner == pFirstSectionLayout);
 	UT_sint32 iBottomMargin = pFirstSectionLayout->getBottomMargin();
 	UT_uint32 pageHeight = getHeight() - iBottomMargin;
+	pageHeight -= getAnnotationHeight();
 	UT_uint32 iFootnoteHeight = 0;
-	UT_uint32 i = 0;
+	UT_sint32 i = 0;
 	for (i = 0; i < countFootnoteContainers(); i++)
 	{
 		iFootnoteHeight += getNthFootnoteContainer(i)->getHeight();
@@ -1727,7 +1876,7 @@ void fp_Page::_reformatFootnotes(void)
 		fp_FootnoteContainer * pFC = getNthFootnoteContainer(i);
 		fl_DocSectionLayout* pSL = (getNthColumnLeader(0)->getDocSectionLayout());
 
-		if(m_pView->getViewMode() == VIEW_NORMAL &&
+		if((m_pView->getViewMode() == VIEW_NORMAL || m_pView->getViewMode() == VIEW_WEB) &&
 		   !m_pLayout->getGraphics()->queryProperties(GR_Graphics::DGP_PAPER))
 			pFC->setX(m_pView->getTabToggleAreaWidth());
 		else
@@ -1738,6 +1887,75 @@ void fp_Page::_reformatFootnotes(void)
 	}
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// Annotations
+//////////////////////////////////////////////////////////////////////////////
+
+void fp_Page::clearScreenAnnotations(void)
+{
+	UT_sint32 i =0;
+	for (i = 0; i < countAnnotationContainers(); i++)
+	{
+		getNthAnnotationContainer(i)->clearScreen();
+	}
+}
+
+UT_sint32 fp_Page::getAnnotationHeight(void) const
+{
+	if(!getDocLayout()->displayAnnotations())
+	{
+			return 0;
+	}
+	UT_uint32 iAnnotationHeight = 0;
+	UT_sint32 i = 0;
+	for (i = 0; i < countAnnotationContainers(); i++)
+	{
+		iAnnotationHeight += getNthAnnotationContainer(i)->getHeight();
+	}
+	return iAnnotationHeight;
+}
+
+void fp_Page::_reformatAnnotations(void)
+{
+	if(m_vecColumnLeaders.getItemCount() == 0)
+	{
+//
+// Page is being deleted.
+//
+		return;
+	}
+	if(!getDocLayout()->displayAnnotations())
+		return;
+	fp_Column* pFirstColumnLeader = getNthColumnLeader(0);
+	fl_DocSectionLayout* pFirstSectionLayout = (pFirstColumnLeader->getDocSectionLayout());
+	UT_ASSERT(m_pOwner == pFirstSectionLayout);
+	UT_sint32 iBottomMargin = pFirstSectionLayout->getBottomMargin();
+	UT_uint32 pageHeight = getHeight() - iBottomMargin;
+	UT_uint32 iAnnotationHeight = 0;
+	UT_sint32 i = 0;
+	for (i = 0; i < countAnnotationContainers(); i++)
+	{
+		iAnnotationHeight += getNthAnnotationContainer(i)->getHeight();
+	}
+
+	pageHeight -= iAnnotationHeight;
+	xxx_UT_DEBUGMSG(("got page height %d, footnote height %d\n", pageHeight, iAnnotationHeight));
+	for (i = 0; i < countAnnotationContainers(); i++)
+	{
+		fp_AnnotationContainer * pAC = getNthAnnotationContainer(i);
+		fl_DocSectionLayout* pSL = (getNthColumnLeader(0)->getDocSectionLayout());
+
+		if((m_pView->getViewMode() == VIEW_NORMAL || m_pView->getViewMode() == VIEW_WEB) &&
+		   !m_pLayout->getGraphics()->queryProperties(GR_Graphics::DGP_PAPER))
+			pAC->setX(m_pView->getTabToggleAreaWidth());
+		else
+			pAC->setX(pSL->getLeftMargin());
+
+		pAC->setY(pageHeight);
+		pageHeight += getNthAnnotationContainer(i)->getHeight();
+	}
+}
+
 /*!
   Remove column leader from page
   \param pLeader Leader to remove
@@ -1795,7 +2013,7 @@ void fp_Page::removeColumnLeader(fp_Column* pLeader)
 // Change ownership of the page. First remove this page from the set owned by
 // the old docSectionLayout.
 //
-		UT_DEBUGMSG(("fp_Page: Remove page %x from DSL %x \n",this,m_pOwner)); 
+		UT_DEBUGMSG(("fp_Page: Remove page %p from DSL %p \n",this,m_pOwner)); 
 		m_pOwner->deleteOwnedPage(this,false);
 		fl_DocSectionLayout * pDSLNew = pFirstColumnLeader->getDocSectionLayout();
 //
@@ -1889,12 +2107,33 @@ void fp_Page::footnoteHeightChanged(void)
 	}
 }
 
+
+
+void fp_Page::annotationHeightChanged(void)
+{
+	clearScreenAnnotations();
+	m_pOwner->setNeedsSectionBreak(true,getPrev());
+	if(breakPage())
+	{
+		_reformat();
+	}
+	else
+	{
+		UT_DEBUGMSG(("SEVIOR: Mark for rebuild from annotationheight changed. \n"));
+		m_pOwner->markForRebuild();
+//
+// FIXME see if this code works instead
+//
+//		m_pOwner->setNeedsSectionBreak(true,getPrev());
+		UT_ASSERT(0);
+	}
+}
+
 void fp_Page::columnHeightChanged(fp_Column* pCol)
 {
-	fp_Column* pLeader = pCol->getLeader();
 	xxx_UT_DEBUGMSG(("SEVIOR: Column height changed \n"));
-	UT_sint32 ndx = m_vecColumnLeaders.findItem(pLeader);
-	UT_ASSERT(ndx >= 0);
+	UT_UNUSED(pCol);
+	UT_ASSERT(m_vecColumnLeaders.findItem(pCol->getLeader()) >= 0);
 	if(breakPage())
 	{
 		_reformat();
@@ -1987,7 +2226,7 @@ PT_DocPosition fp_Page::getFirstLastPos(bool bFirst) const
 	return pos;
 }
 
-void fp_Page::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, bool& bBOL, bool& bEOL,bool &isTOC, bool bUseHdrFtr, fl_HdrFtrShadow ** pShadow )
+void fp_Page::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, bool& bBOL, bool& bEOL,bool &isTOC, bool bUseHdrFtr, fl_HdrFtrShadow ** pShadow ) const
 {
 	fl_HdrFtrShadow * pShad = NULL;
 	if(pShadow == NULL)
@@ -2014,9 +2253,9 @@ void fp_Page::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, boo
  \return pos The Document position corresponding the text at location x,y
  \return pShadow A pointer to the shadow corresponding to this header/footer
  */
-void fp_Page::mapXYToPosition(bool bNotFrames,UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, bool& bBOL, bool& bEOL, bool &isTOC, bool bUseHdrFtr, fl_HdrFtrShadow ** pShadow )
+void fp_Page::mapXYToPosition(bool bNotFrames,UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, bool& bBOL, bool& bEOL, bool &isTOC, bool bUseHdrFtr, fl_HdrFtrShadow ** pShadow ) const
 {
-	int count = m_vecColumnLeaders.getItemCount();
+	UT_sint32 count = m_vecColumnLeaders.getItemCount();
 	UT_uint32 iMinDist = 0xffffffff;
 	fp_VerticalContainer * pMinDist = NULL;
 	fp_Column* pColumn = NULL;
@@ -2041,7 +2280,7 @@ void fp_Page::mapXYToPosition(bool bNotFrames,UT_sint32 x, UT_sint32 y, PT_DocPo
 		// loop from high z to low z
 		// Because we draw from old to new, the new appears on top
 		// and because the new appears on top, it should be treated as such
-		for (i=(static_cast<UT_sint32>(countAboveFrameContainers()-1)); i>=0; i--)
+		for (i = (countAboveFrameContainers()-1); i>=0; i--)
 		{
 			pFrameC = getNthAboveFrameContainer(i);
 			bool isImage = false;
@@ -2098,7 +2337,7 @@ void fp_Page::mapXYToPosition(bool bNotFrames,UT_sint32 x, UT_sint32 y, PT_DocPo
 				}
 			}
 		}
-		for (i=(static_cast<UT_sint32>(countBelowFrameContainers()-1)); i>=0; i--)
+		for (i = countBelowFrameContainers()-1; i>=0; i--)
 		{
 			pFrameC = getNthBelowFrameContainer(i);
 			bool isImage = false;
@@ -2167,9 +2406,9 @@ void fp_Page::mapXYToPosition(bool bNotFrames,UT_sint32 x, UT_sint32 y, PT_DocPo
 		if(m_pView && m_pView->getViewMode() == VIEW_PRINT)
 		{
 			fp_ShadowContainer * hf[2] = { m_pHeader, m_pFooter };
-			for (UT_uint32 i = 0; i < NrElements(hf); i++)
+			for (UT_uint32 j = 0; j < G_N_ELEMENTS(hf); j++)
 			{
-				fp_ShadowContainer * p = hf[i];
+				fp_ShadowContainer * p = hf[j];
 
 				if(p == NULL || !p->getFirstContainer())
 					continue;
@@ -2271,6 +2510,50 @@ void fp_Page::mapXYToPosition(bool bNotFrames,UT_sint32 x, UT_sint32 y, PT_DocPo
 			}
 		}
 	}
+
+//
+// Now look in annotations
+//
+	if(getDocLayout()->displayAnnotations())
+	{
+			fp_AnnotationContainer * pAC = NULL;
+			for (i=0; i<static_cast<UT_sint32>(countAnnotationContainers()); i++)
+			{
+					pAC = getNthAnnotationContainer(i);
+					if (pAC->getFirstContainer())
+					{
+							if ((x >= pAC->getX())
+								&& (x < (pAC->getX() + pAC->getWidth()))
+								&& (y >= pAC->getY())
+								&& (y < (pAC->getY() + pAC->getHeight()))
+								)
+						    {
+									pAC->mapXYToPosition(x - pAC->getX(), y - pAC->getY(), pos, bBOL, bEOL,isTOC);
+									return;
+							}
+							
+							iDist = pAC->distanceFromPoint(x, y);
+							if (iDist < iMinDist)
+							{
+									iMinDist = iDist;
+									pMinDist = static_cast<fp_VerticalContainer *>(pAC);
+							}
+							
+							if ( (y >= pAC->getY())
+								 && (y < (pAC->getY() + pAC->getHeight()))) 
+							{
+									if (iDist < iMinXDist)
+									{
+											iMinXDist = iDist;
+											pMinXDist = static_cast<fp_VerticalContainer *>(pAC);
+									}
+							}
+					}
+			}
+	}
+
+
+
 	if (pMinXDist)
 	{
 		pMinXDist->mapXYToPosition(x - pMinXDist->getX(), y - pMinXDist->getY(), pos, bBOL, bEOL,isTOC);
@@ -2407,11 +2690,11 @@ void fp_Page::frameHeightChanged(void)
 void fp_Page::clearScreenFrames(void)
 {
 	UT_sint32 i =0;
-	for (i = 0; i < static_cast<UT_sint32>(countAboveFrameContainers()); i++)
+	for (i = 0; i < countAboveFrameContainers(); i++)
 	{
 		getNthAboveFrameContainer(i)->clearScreen();
 	}
-	for (i = 0; i < static_cast<UT_sint32>(countBelowFrameContainers()); i++)
+	for (i = 0; i < countBelowFrameContainers(); i++)
 	{
 		getNthBelowFrameContainer(i)->clearScreen();
 	}
@@ -2460,6 +2743,18 @@ void fp_Page::markDirtyOverlappingRuns(fp_FrameContainer * pFrameC)
 		pFC->markDirtyOverlappingRuns(*pMyFrameRect);
 	}
 
+
+	// Now Annotations
+	if(getDocLayout()->displayAnnotations())
+	{
+			count = m_vecAnnotations.getItemCount();
+			for (i=0; i<count; i++)
+			{
+					fp_AnnotationContainer* pAC = m_vecAnnotations.getNthItem(i);
+					pAC->markDirtyOverlappingRuns(*pMyFrameRect);
+			}
+	}
+
 	// Now Frames
 
 	count = m_vecAboveFrames.getItemCount();
@@ -2486,18 +2781,18 @@ void fp_Page::markDirtyOverlappingRuns(fp_FrameContainer * pFrameC)
 }
 
 
-UT_uint32 fp_Page::countAboveFrameContainers(void) const
+UT_sint32 fp_Page::countAboveFrameContainers(void) const
 {
         return m_vecAboveFrames.getItemCount();
 }
 
 
-UT_uint32 fp_Page::countBelowFrameContainers(void) const
+UT_sint32 fp_Page::countBelowFrameContainers(void) const
 {
         return m_vecBelowFrames.getItemCount();
 }
 
-UT_sint32 fp_Page::findFrameContainer(fp_FrameContainer * pFC)
+UT_sint32 fp_Page::findFrameContainer(fp_FrameContainer * pFC) const
 {
         UT_sint32 i; 
         if(pFC->isAbove())
@@ -2539,20 +2834,20 @@ bool fp_Page::insertFrameContainer(fp_FrameContainer * pFC)
 	return true;
 }
 
-void fp_Page::removeFrameContainer(fp_FrameContainer * pFC)
+void fp_Page::removeFrameContainer(fp_FrameContainer * _pFC)
 {
 
-	markDirtyOverlappingRuns(pFC);
+	markDirtyOverlappingRuns(_pFC);
 	UT_sint32 ndx = 0;
 	bool isAbove = false;
-	if(pFC->isAbove())
+	if(_pFC->isAbove())
 	{
-	    ndx = m_vecAboveFrames.findItem(pFC);
+	    ndx = m_vecAboveFrames.findItem(_pFC);
 	    isAbove = true;
 	}
 	else
 	{
-	    ndx = m_vecBelowFrames.findItem(pFC);
+	    ndx = m_vecBelowFrames.findItem(_pFC);
 	}
 	if(ndx>=0)
 	{
@@ -2586,12 +2881,12 @@ void fp_Page::removeFrameContainer(fp_FrameContainer * pFC)
 
 // Footnote methods
 
-UT_uint32 fp_Page::countFootnoteContainers(void) const
+UT_sint32 fp_Page::countFootnoteContainers(void) const
 {
 	return m_vecFootnotes.getItemCount();
 }
 
-UT_sint32 fp_Page::findFootnoteContainer(fp_FootnoteContainer * pFC)
+UT_sint32 fp_Page::findFootnoteContainer(fp_FootnoteContainer * pFC) const
 {
 	UT_sint32 i = m_vecFootnotes.findItem(pFC);
 	return i;
@@ -2604,7 +2899,7 @@ fp_FootnoteContainer* fp_Page::getNthFootnoteContainer(UT_sint32 n) const
 
 bool fp_Page::insertFootnoteContainer(fp_FootnoteContainer * pFC)
 {
-	UT_uint32 i =0;
+	UT_sint32 i =0;
 	UT_sint32 j = findFootnoteContainer(pFC);
 	if(j >= 0)
 	{
@@ -2642,9 +2937,9 @@ bool fp_Page::insertFootnoteContainer(fp_FootnoteContainer * pFC)
 	return true;
 }
 
-void fp_Page::removeFootnoteContainer(fp_FootnoteContainer * pFC)
+void fp_Page::removeFootnoteContainer(fp_FootnoteContainer * _pFC)
 {
-	UT_sint32 ndx = m_vecFootnotes.findItem(pFC);
+	UT_sint32 ndx = m_vecFootnotes.findItem(_pFC);
 	if(ndx>=0)
 	{
 		m_vecFootnotes.deleteNthItem(ndx);
@@ -2660,3 +2955,103 @@ void fp_Page::removeFootnoteContainer(fp_FootnoteContainer * pFC)
 	}
 	return;
 }
+
+
+
+// Annotation methods
+
+UT_sint32 fp_Page::countAnnotationContainers(void) const
+{
+	return m_vecAnnotations.getItemCount();
+}
+
+UT_sint32 fp_Page::findAnnotationContainer(fp_AnnotationContainer * pAC) const
+{
+	UT_sint32 i = m_vecAnnotations.findItem(pAC);
+	return i;
+}
+
+fp_AnnotationContainer* fp_Page::getNthAnnotationContainer(UT_sint32 n) const 
+{
+	return m_vecAnnotations.getNthItem(n);
+} 
+
+UT_sint32 fp_Page::getAnnotationPos(UT_uint32 pid) const
+{
+	fp_AnnotationContainer * pACon = NULL;
+	UT_sint32 i = 0;
+	for(i = 0; i< countAnnotationContainers(); i++)
+	{
+			pACon = getNthAnnotationContainer(i);
+			if(!pACon)
+				return 0;
+			if(pid == pACon->getPID())
+				return i;
+	}
+	return 0;
+}
+
+bool fp_Page::insertAnnotationContainer(fp_AnnotationContainer * pAC)
+{
+	UT_sint32 i =0;
+	UT_sint32 j = findAnnotationContainer(pAC);
+	if(j >= 0)
+	{
+		return false;
+	}
+	UT_uint32 loc =0;
+	UT_sint32 fVal = pAC->getValue();
+	fp_AnnotationContainer * pFTemp = NULL;
+	for(i=0; i< m_vecAnnotations.getItemCount();i++)
+	{
+		pFTemp = m_vecAnnotations.getNthItem(i);
+		if(fVal < pFTemp->getValue())
+		{
+			loc = i;
+			break;
+		}
+	}
+	if(pFTemp == NULL)
+	{
+		m_vecAnnotations.addItem(pAC);
+	}
+	else if( i>= m_vecAnnotations.getItemCount())
+	{
+		m_vecAnnotations.addItem(pAC);
+	}
+	else
+	{
+		m_vecAnnotations.insertItemAt(pAC, loc);
+	}
+	if(pAC)
+	{
+		pAC->setPage(this);
+	}
+	if(getDocLayout()->displayAnnotations())
+	{
+		_reformat();
+	}
+	return true;
+}
+
+void fp_Page::removeAnnotationContainer(fp_AnnotationContainer * _pAC)
+{
+	UT_sint32 ndx = m_vecAnnotations.findItem(_pAC);
+	if(ndx>=0)
+	{
+		m_vecAnnotations.deleteNthItem(ndx);
+		if(getDocLayout()->displayAnnotations())
+		{
+				for(ndx=0; ndx < static_cast<UT_sint32>(countAnnotationContainers());ndx++)
+				{			
+						fp_AnnotationContainer * pAC = getNthAnnotationContainer(ndx);
+						fl_AnnotationLayout * pAL = static_cast<fl_AnnotationLayout *>(pAC->getSectionLayout());
+						pAC->clearScreen();
+						pAL->markAllRunsDirty();
+				}
+		}
+		_reformat();
+		return;
+	}
+	return;
+}
