@@ -33,11 +33,13 @@ class fp_Column;
 class fp_Container;
 class fp_ShadowContainer;
 class fp_FootnoteContainer;
+class fp_AnnotationContainer;
 class fp_FrameContainer;
 class fl_DocSectionLayout;
 class fl_HdrFtrSectionLayout;
 class fl_HdrFtrShadow;
 class fl_FootnoteLayout;
+class fl_AnnotationLayout;
 class fl_FrameLayout;
 class FV_View;
 class GR_Graphics;
@@ -45,7 +47,7 @@ class fp_TableContainer;
 class fl_TOCLayout;
 struct dg_DrawArgs;
 
-class _BL
+class ABI_EXPORT _BL
 	{
 	public:
 		_BL(fl_BlockLayout * pBL,fp_Line * pL) :
@@ -85,8 +87,8 @@ public:
 	inline fl_DocSectionLayout* getOwningSection(void) const { return m_pOwner; }
 
 	PT_DocPosition		getFirstLastPos(bool bFirst) const;
-	void				mapXYToPosition(bool bNotFrames,UT_sint32 xPos, UT_sint32 yPos, PT_DocPosition& pos, bool& bBOL, bool& bEOL,bool & isTOC, bool bUseHdrFtr = false, fl_HdrFtrShadow ** pShadow = NULL);
-	void				mapXYToPosition(UT_sint32 xPos, UT_sint32 yPos, PT_DocPosition& pos, bool& bBOL, bool& bEOL, bool & isTOC,bool bUseHdrFtr = false, fl_HdrFtrShadow ** pShadow = NULL);
+	void				mapXYToPosition(bool bNotFrames,UT_sint32 xPos, UT_sint32 yPos, PT_DocPosition& pos, bool& bBOL, bool& bEOL,bool & isTOC, bool bUseHdrFtr = false, fl_HdrFtrShadow ** pShadow = NULL) const;
+	void				mapXYToPosition(UT_sint32 xPos, UT_sint32 yPos, PT_DocPosition& pos, bool& bBOL, bool& bEOL, bool & isTOC,bool bUseHdrFtr = false, fl_HdrFtrShadow ** pShadow = NULL) const;
 	void				getScreenOffsets(fp_Container*, UT_sint32& xoff, UT_sint32& yoff) const;
 
 	void				draw(dg_DrawArgs*, bool bAlaysUseWhiteBackground=false);
@@ -99,7 +101,7 @@ public:
 	// Leader (e.g. column) functions.
 	void 				columnHeightChanged(fp_Column* pLeader);
 	bool                breakPage(void);
-	UT_uint32 			countColumnLeaders(void) const;
+	UT_sint32 			countColumnLeaders(void) const;
 	fp_Column*			getNthColumnLeader(UT_sint32 n) const;
 	bool				insertColumnLeader(fp_Column* pLeader, fp_Column* pAfter);
 	void				removeColumnLeader(fp_Column* pLeader);
@@ -114,23 +116,35 @@ public:
 
 	// Footnote functions.
 	void 				footnoteHeightChanged(void);
-	UT_uint32			countFootnoteContainers(void) const;
+	UT_sint32			countFootnoteContainers(void) const;
 	fp_FootnoteContainer* getNthFootnoteContainer(UT_sint32 n) const; 
 	bool				insertFootnoteContainer(fp_FootnoteContainer * pFC);
 	void				removeFootnoteContainer(fp_FootnoteContainer * pFC);
-	UT_sint32           findFootnoteContainer(fp_FootnoteContainer * pFC);
+	UT_sint32           findFootnoteContainer(fp_FootnoteContainer * pFC) const;
 	void                clearScreenFootnotes(void);
-	UT_sint32           getFootnoteHeight(void);
+	UT_sint32           getFootnoteHeight(void) const;
+
+
+	// Annotation functions.
+	void 				annotationHeightChanged(void);
+	UT_sint32			countAnnotationContainers(void) const;
+	fp_AnnotationContainer* getNthAnnotationContainer(UT_sint32 n) const; 
+	bool				insertAnnotationContainer(fp_AnnotationContainer * pFC);
+	void				removeAnnotationContainer(fp_AnnotationContainer * pFC);
+	UT_sint32           findAnnotationContainer(fp_AnnotationContainer * pFC) const;
+	void                clearScreenAnnotations(void);
+	UT_sint32           getAnnotationHeight(void) const;
+	UT_sint32           getAnnotationPos( UT_uint32 pid) const;
 
 	// Frame functions.
 	void 				frameHeightChanged(void);
-	UT_uint32			countAboveFrameContainers(void) const;
-	UT_uint32			countBelowFrameContainers(void) const;
+	UT_sint32			countAboveFrameContainers(void) const;
+	UT_sint32			countBelowFrameContainers(void) const;
 	fp_FrameContainer*  getNthAboveFrameContainer(UT_sint32 n) const; 
 	fp_FrameContainer*  getNthBelowFrameContainer(UT_sint32 n) const; 
 	bool				insertFrameContainer(fp_FrameContainer * pFC);
 	void				removeFrameContainer(fp_FrameContainer * pFC);
-	UT_sint32           findFrameContainer(fp_FrameContainer * pFC);
+	UT_sint32           findFrameContainer(fp_FrameContainer * pFC) const;
 	void                clearScreenFrames(void);
 	void                markDirtyOverlappingRuns(fp_FrameContainer * pFC);
     void                expandDamageRect(UT_sint32 x, UT_sint32 y, 
@@ -158,6 +172,7 @@ protected:
 	void				_reformat(void);
 	void				_reformatColumns(void);
 	void				_reformatFootnotes(void);
+	void				_reformatAnnotations(void);
 
 private:
 	// don't allow copying
@@ -182,6 +197,7 @@ private:
 	fp_ShadowContainer* m_pHeader;
 
 	UT_GenericVector<fp_FootnoteContainer *> m_vecFootnotes;
+	UT_GenericVector<fp_AnnotationContainer *> m_vecAnnotations;
 	fg_FillType         m_FillType;
 	UT_GenericVector<fp_FrameContainer *> m_vecAboveFrames;
 	UT_GenericVector<fp_FrameContainer *> m_vecBelowFrames;
