@@ -50,6 +50,7 @@ struct dg_DrawArgs;
 class fl_CharWidths;
 class fd_Field;
 class fp_HyperlinkRun;
+class fp_AnnotationRun;
 
 struct fp_RunSplitInfo
 {
@@ -90,6 +91,12 @@ enum FP_RUN_TYPE
 	FPRUN__LAST__					= 18
 };
 
+enum FP_HYPERLINK_TYPE
+{
+    HYPERLINK_NORMAL =1,
+    HYPERLINK_ANNOTATION = 2
+};
+
 // specifies how setX should handle screen clearing
 enum FPRUN_CLEAR_SCREEN
 {
@@ -114,6 +121,7 @@ enum FPRUN_CLEAR_SCREEN
 		fp_FieldEndRun
 		fp_BookmarkRun
 		fp_HyperlinkRun
+		fp_AnnotationRun
 		fp_DummyRun
 
 	As far as the formatter's concerned, each subclass behaves somewhat
@@ -147,7 +155,6 @@ public:
 	UT_uint32		        getDescent() const;
 	virtual UT_sint32       getDrawingWidth() const;
 	
-	
 	fp_Run* 		        getNextRun() const					{ return m_pNext; }
 	fp_Run*			        getPrevRun() const					{ return m_pPrev; }
 	bool                    isInSelectedTOC(void);
@@ -155,9 +162,9 @@ public:
 	virtual fp_ContainerObject * getPrev(void) const { return NULL;}
 	virtual fp_Container *       getNextContainerInSection(void) const { return NULL;}
 	virtual fp_Container *       getPrevContainerInSection(void) const { return NULL;}
-	virtual void                 setNext(fp_ContainerObject * pNull) {}
-	virtual void                 setPrev(fp_ContainerObject * pNull) {}
-	virtual void                 draw(GR_Graphics * pG) {}
+	virtual void                 setNext(fp_ContainerObject * /*pNull*/) {}
+	virtual void                 setPrev(fp_ContainerObject * /*pNull*/) {}
+	virtual void                 draw(GR_Graphics * /*pG*/) {}
 
 	UT_uint32		    getBlockOffset() const			{ return m_iOffsetFirst; }
 	UT_uint32		    getLength() const				{ return m_iLen; }
@@ -213,8 +220,8 @@ public:
 	virtual void		draw(dg_DrawArgs*);
 	virtual void        clearScreen(void);
 	void                Run_ClearScreen(bool bFullLineHeightRect = false);
-	virtual void        setWidth(UT_sint32 iW) {}
-	virtual void        setHeight(UT_sint32 iH) {}
+	virtual void        setWidth(UT_sint32 /*iW*/) {}
+	virtual void        setHeight(UT_sint32 /*iH*/) {}
 	virtual bool        isVBreakable(void) {return false;}
 	virtual bool        isHBreakable(void) {return false;}
 	virtual UT_sint32   wantVBreakAt(UT_sint32 i) {return i;}
@@ -243,7 +250,7 @@ public:
 	virtual bool		alwaysFits(void) const { return false; }
 	virtual bool		findMaxLeftFitSplitPoint(UT_sint32 iMaxLeftWidth, fp_RunSplitInfo& si,
 												 bool bForce=false);
-	
+	void                clearPrint(void);
 	virtual UT_sint32	findTrailingSpaceDistance(void) const { return 0; }
 	virtual bool		findFirstNonBlankSplitPoint(fp_RunSplitInfo& /*si*/) { return false; }
 	virtual void		mapXYToPosition(UT_sint32 xPos, UT_sint32 yPos,
@@ -285,12 +292,16 @@ public:
 	UT_uint32           getOffsetLog(UT_uint32 iVisOff);
 	fp_Run *			getNextVisual();
 	fp_Run *			getPrevVisual();
+	UT_sint32           getAuthorNum(void) const
+	{ return m_iAuthorColor;};
+	void                setAuthorNum(UT_sint32 i)
+	{ m_iAuthorColor=i;};
 
-	virtual UT_uint32   adjustCaretPosition(UT_uint32 iDocumentPosition, bool bForward)
+	virtual UT_uint32   adjustCaretPosition(UT_uint32 iDocumentPosition, bool /*bForward*/)
 	                           { return iDocumentPosition;}
 
-	virtual void        adjustDeletePosition(UT_uint32 &pos1, UT_uint32 &count)
-	                           {return;}
+	virtual void        adjustDeletePosition(UT_uint32 & /*pos1*/, 
+											 UT_uint32 & /*count*/) {}
 
 	bool                containsRevisions(){return (m_pRevisions != NULL);}
 	// would prefer to make the return value const, but the
@@ -329,12 +340,14 @@ public:
 	// the users backs.
 	bool        deleteFollowingIfAtInsPoint() const;
 
+	bool        displayAnnotations(void);
 	// Methods for selection drawing
 	void                 setSelectionMode(PT_DocPosition posLow, PT_DocPosition posHigh);
     void                 clearSelectionMode(void);
 	bool                 isSelectionDraw(void) const;
 	PT_DocPosition       posSelLow(void) const;
     PT_DocPosition       posSelHigh(void) const;
+	UT_RGBColor			_getColorFG(void) const { return m_pColorFG; }
 
 #ifdef FMT_TEST
 	virtual void		__dump(FILE * fp) const;
@@ -353,7 +366,6 @@ protected:
 	// By convention, _getFoo and _setFoo have no side effects.
 	// They can easily be inlined by a smart compiler.
 	UT_RGBColor			_getColorPG(void) const { return m_pColorPG; }
-	UT_RGBColor			_getColorFG(void) const { return m_pColorFG; }
 	UT_RGBColor			_getColorHL(void) const { return m_pColorHL; }
 	void				_setColorFG(UT_RGBColor c) { m_pColorFG = c; }
 	void				_setColorHL(UT_RGBColor c) { m_pColorHL = c; }
@@ -363,7 +375,7 @@ protected:
 	void				_setLine(fp_Line* pLine) { m_pLine = pLine; }
 	void				_setHeight(UT_sint32 iHeight)
 							{ m_iHeight = iHeight;}
-	void				_setWidth(UT_sint32 iWidth)
+	virtual void		_setWidth(UT_sint32 iWidth)
                         	{ m_iWidth = iWidth; }
 
 	// use these with great care -- most of the time we need to use
@@ -474,7 +486,6 @@ private:
 	bool                    m_bIsCleared;
 	fg_FillType             m_FillType;
 	bool                    m_bPrinting;
-	GR_Graphics *           m_pG;
 	UT_sint32               m_iTmpX;
 	UT_sint32               m_iTmpY;
 	UT_sint32               m_iTmpWidth;
@@ -485,6 +496,7 @@ private:
     PT_DocPosition          m_iSelLow;
     PT_DocPosition          m_iSelHigh;
 	bool                    m_bMustClearScreen;
+	UT_sint32               m_iAuthorColor;
 #ifdef DEBUG
 	UT_uint32               m_iFontAllocNo;
 #endif
@@ -723,7 +735,10 @@ class ABI_EXPORT fp_HyperlinkRun : public fp_Run
 {
 public:
 	fp_HyperlinkRun(fl_BlockLayout* pBL, UT_uint32 iOffsetFirst, UT_uint32 iLen);
-	~fp_HyperlinkRun();
+	virtual ~fp_HyperlinkRun();
+	virtual FP_HYPERLINK_TYPE    getHyperlinkType(void)
+	{ return HYPERLINK_NORMAL;}
+
 	bool 				isStartOfHyperlink() const {return m_bIsStart;};
 	const gchar * 	getTarget() const {return static_cast<const gchar *>(m_pTarget);};
 
@@ -751,7 +766,7 @@ public:
 	// for the purposes of linebreaking, just whitespace
 	virtual bool doesContainNonBlankData(void) const { return false; }
 	
-private:
+protected:
 	virtual void			_lookupProperties(const PP_AttrProp * pSpanAP,
 											  const PP_AttrProp * pBlockAP,
 											  const PP_AttrProp * pSectionAP,
@@ -768,6 +783,43 @@ private:
 };
 
 
+
+class ABI_EXPORT fp_AnnotationRun : public fp_HyperlinkRun
+{
+public:
+	fp_AnnotationRun(fl_BlockLayout* pBL, UT_uint32 iOffsetFirst, UT_uint32 iLen);
+	virtual ~fp_AnnotationRun();
+	virtual FP_HYPERLINK_TYPE    getHyperlinkType(void)
+	{
+		return HYPERLINK_ANNOTATION;
+	}
+	UT_uint32 getPID(void) { return m_iPID;}
+	const char * getValue(void);
+    void         recalcValue(void);
+	virtual bool canBreakAfter(void) const;
+	virtual bool canBreakBefore(void) const;
+	UT_sint32    getRealWidth(void) const {return m_iRealWidth;}
+    void         cleanDraw(dg_DrawArgs*);
+	UT_sint32    calcWidth(void);
+
+ protected:
+	virtual void			_draw(dg_DrawArgs*);
+	virtual void			_clearScreen(bool bFullLineHeightRect);
+	virtual bool			_recalcWidth(void);
+	bool                    _setValue(void);
+	virtual void            _setWidth(UT_sint32 iWidth);
+	virtual bool _letPointPass(void) const;
+	virtual bool _canContainPoint(void) const;
+    virtual void _lookupProperties(const PP_AttrProp * pSpanAP,
+									const PP_AttrProp * pBlockAP,
+									const PP_AttrProp * pSectionAP,
+								   GR_Graphics * pG);
+ private:
+	UT_uint32               m_iPID;
+	UT_UTF8String           m_sValue;
+	UT_sint32               m_iRealWidth;
+};
+
 class ABI_EXPORT fp_ImageRun : public fp_Run
 {
 public:
