@@ -66,7 +66,7 @@
 
 class fl_TableLayout;
 
-class fp_TableRowColumn
+class ABI_EXPORT fp_TableRowColumn
 {
 public:
 	fp_TableRowColumn(void);
@@ -134,7 +134,8 @@ public:
 	virtual UT_sint32   wantHBreakAt(UT_sint32) {return 0;}
 	virtual fp_ContainerObject * VBreakAt(UT_sint32);
 	virtual fp_ContainerObject * HBreakAt(UT_sint32) {return NULL;}
-	void                recalcMaxWidth(bool bDontClearIfNeeded = false) {}
+	void                recalcMaxWidth(bool bDontClearIfNeeded = false) 
+	{ UT_UNUSED(bDontClearIfNeeded); }
 	virtual void        setAssignedScreenHeight(UT_sint32) {}
 	virtual fp_Container * getNextContainerInSection(void) const;
 	virtual fp_Container * getPrevContainerInSection(void) const;
@@ -143,10 +144,12 @@ public:
 	UT_sint32           getCellY(fp_Line * pLine) const;
 	UT_sint32           getSpannedHeight(void);
 	void                setLineMarkers(void);
-	bool                containsFootnoteReference(void);
 	void                deleteBrokenTables(bool bClearFirst=true);
+	bool                containsFootnoteReference(void);
 	bool                getFootnoteContainers(UT_GenericVector<fp_FootnoteContainer*>* pvecFoots);
-	void                getLeftTopOffsets(UT_sint32 & xoff, UT_sint32 & yoff);
+	bool                containsAnnotations(void);
+	bool                getAnnotationContainers(UT_GenericVector<fp_AnnotationContainer*>* pvecAnns);
+	void                getLeftTopOffsets(UT_sint32 & xoff, UT_sint32 & yoff) const;
    UT_sint32           getLeftAttach(void) const
 		{ return m_iLeftAttach;}
     UT_sint32           getRightAttach(void) const
@@ -361,6 +364,8 @@ fp_Column *         getBrokenColumn(void);
 	void                drawLines();
 	bool                containsFootnoteReference(void);
 	bool                getFootnoteContainers(UT_GenericVector<fp_FootnoteContainer*>* pvecFoots);
+	bool                containsAnnotations(void);
+	bool                getAnnotationContainers(UT_GenericVector<fp_AnnotationContainer*>* pvecAnns);
 	UT_sint32           tweakBrokenTable(fp_TableContainer * pBroke);
     virtual void        clearScreen(void);
 	virtual bool        isVBreakable(void);
