@@ -41,6 +41,7 @@
 #include "fv_View.h"
 #include "fb_Alignment.h"
 #include "fl_TableLayout.h"
+#include "xap_App.h"
 #include "xap_Frame.h"
 #include "ut_debugmsg.h"
 #include "ut_assert.h"
@@ -66,6 +67,7 @@ fp_TextRun::fp_TextRun(fl_BlockLayout* pBL,
 					   UT_uint32 iLen,
 					   bool bLookupProperties)
 :	fp_Run(pBL,iOffsetFirst, iLen, FPRUN_TEXT),
+	m_TextTransform(GR_ShapingInfo::NONE),
 	m_fPosition(TEXT_POSITION_NORMAL),
 #ifdef ENABLE_SPELL
 	m_bSpellSquiggled(false),
@@ -311,14 +313,16 @@ void fp_TextRun::_lookupProperties(const PP_AttrProp * pSpanAP,
 		if the previous direction override was not strong, and the current one is, we have
 		to break this run's neighbours
 	*/
-	if(iNewOverride == UT_BIDI_UNSET && iOldOverride != UT_BIDI_UNSET)
+	if(iNewOverride ==  static_cast<UT_BidiCharType>(UT_BIDI_UNSET) && 
+	   iOldOverride !=  static_cast<UT_BidiCharType>(UT_BIDI_UNSET))
 	{
 		// we have to do this without applying the new override otherwise the
 		// LTR and RTL run counters of the present line will be messed up;
 		// breakMeAtDirBoundaries will take care of applying the new override
 		breakMeAtDirBoundaries(iNewOverride);
 	}
-	else if(iNewOverride != UT_BIDI_UNSET && iOldOverride == UT_BIDI_UNSET)
+	else if(iNewOverride !=  static_cast<UT_BidiCharType>(UT_BIDI_UNSET) && 
+			iOldOverride ==  static_cast<UT_BidiCharType>(UT_BIDI_UNSET))
 	{
 		// first we have to apply the new override
 		setDirection(UT_BIDI_UNSET, iNewOverride);
@@ -328,6 +332,22 @@ void fp_TextRun::_lookupProperties(const PP_AttrProp * pSpanAP,
 	else
 		setDirection(UT_BIDI_UNSET, iNewOverride);
 
+	const gchar *pszTextTransform = PP_evalProperty("text-transform",pSpanAP,pBlockAP,
+													pSectionAP, pDoc, true);
+
+	GR_ShapingInfo::TextTransform oldTextTransform = getTextTransform();
+	if(pszTextTransform && strcmp(pszTextTransform,"none") != 0)
+	{
+		if (strcmp(pszTextTransform,"capitalize") == 0)
+			setTextTransform(GR_ShapingInfo::CAPITALIZE);
+		else if (strcmp(pszTextTransform,"uppercase") == 0)
+			setTextTransform(GR_ShapingInfo::UPPERCASE);
+		else if (strcmp(pszTextTransform,"lowercase") == 0)
+			setTextTransform(GR_ShapingInfo::LOWERCASE);							 
+	}
+
+	bChanged |= (oldTextTransform != getTextTransform());
+
 	if(bChanged && !bDontClear)
 		clearScreen();
 	xxx_UT_DEBUGMSG(("fp_TextRun::lookupProperties: bChanged %d\n", static_cast<UT_uint32>(bChanged)));
@@ -337,7 +357,7 @@ void fp_TextRun::_lookupProperties(const PP_AttrProp * pSpanAP,
 * This method append the text in this run to the growbuf supplied in the
 * parameter.
 */
-void fp_TextRun::appendTextToBuf(UT_GrowBuf & buf)
+void fp_TextRun::appendTextToBuf(UT_GrowBuf & buf) const
 {
 	UT_GrowBuf myBuf;
 	getBlock()->getBlockBuf(&myBuf);
@@ -365,7 +385,7 @@ void fp_TextRun::printText(void)
 	while(text.getStatus() == UTIter_OK)
 	{
 		UT_UCS4Char c = text.getChar();
-		if(static_cast<char>(c)>=' ' && static_cast<char>(c) <128)
+		if(c >= ' ' && c <128)
 			sTmp +=  static_cast<char>(c);
 		++text;
 	}
@@ -470,8 +490,12 @@ bool fp_TextRun::alwaysFits(void) const
 	return true;
 }
 
-bool fp_TextRun::findFirstNonBlankSplitPoint(fp_RunSplitInfo& si)
+bool fp_TextRun::findFirstNonBlankSplitPoint(fp_RunSplitInfo& /*si*/ )
 {
+	//
+	// What is this code trying to achieve? 
+	// Why do we want to keep around for future reference?
+	//
 	return false;
 #if 0 // if turning this back on, replace the while loop with PD_StruxIterator
 	UT_GrowBuf * pgbCharWidths = getBlock()->getCharWidths()->getCharWidths();
@@ -584,9 +608,9 @@ bool	fp_TextRun::findMaxLeftFitSplitPoint(UT_sint32 iMaxLeftWidth, fp_RunSplitIn
 		// getTextWidth() takes LOGICAL offset
 		m_pRenderInfo->m_iOffset = i;
 		m_pRenderInfo->m_iLength = 1;
-		UT_sint32 iCW = getGraphics()->getTextWidth(*m_pRenderInfo);
-		iLeftWidth += iCW;
-		iRightWidth -= iCW;
+		UT_sint32 iCW2 = getGraphics()->getTextWidth(*m_pRenderInfo);
+		iLeftWidth += iCW2;
+		iRightWidth -= iCW2;
 
 		UT_UCS4Char c = text.getChar();
 		bool bCanBreak = false;
@@ -688,7 +712,7 @@ bool	fp_TextRun::findMaxLeftFitSplitPoint(UT_sint32 iMaxLeftWidth, fp_RunSplitIn
 
 void fp_TextRun::mapXYToPosition(UT_sint32 x, UT_sint32 y,
 								 PT_DocPosition& pos, 
-								 bool& bBOL, bool& bEOL, bool &isTOC)
+								 bool& bBOL, bool& bEOL, bool & /*isTOC*/)
 {
 	UT_BidiCharType iVisDirection = getVisDirection();
 	UT_BidiCharType iDomDirection = getBlock()->getDominantDirection();
@@ -766,9 +790,9 @@ void fp_TextRun::mapXYToPosition(UT_sint32 x, UT_sint32 y,
 		// catch the case of a click directly on the left half of the
 		// first character in the run
 		UT_uint32 k = iVisDirection == UT_BIDI_RTL ? getLength() - 1 : 0;
-		UT_sint32 iCW = RI.m_pWidths[k] > 0 ? RI.m_pWidths[k] : 0;
+		UT_sint32 iCW2 = RI.m_pWidths[k] > 0 ? RI.m_pWidths[k] : 0;
 
-		if (x < (iCW / 2))
+		if (x < (iCW2 / 2))
 		{
 			pos = getBlock()->getPosition() + getOffsetFirstVis();
 
@@ -818,7 +842,7 @@ void fp_TextRun::mapXYToPosition(UT_sint32 x, UT_sint32 y,
 	}
 	else
 	{
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 		// This is really for the benefit of the Pango graphics, which requires the raw
 		// text for almost anything; we do not need this on win32, and since this this
 		// called all the time, do not want it in here unless necessary
@@ -834,7 +858,7 @@ void fp_TextRun::mapXYToPosition(UT_sint32 x, UT_sint32 y,
 		pos = getGraphics()->XYToPosition(*m_pRenderInfo, x, y);
 		pos += getBlock()->getPosition() + getBlockOffset();
 
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 		// reset this, so we have no stale pointers there
 		m_pRenderInfo->m_pText = NULL;
 #endif
@@ -950,7 +974,7 @@ void fp_TextRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y,
 		m_pRenderInfo->m_iOffset = iOffset - getBlockOffset() - 1;
 		m_pRenderInfo->m_iLength = getLength();
 
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 		// This is really for the benefit of the Pango graphics, which requires the raw
 		// text for almost anything; we do not need this on win32, and since this this
 		// called all the time, do not want it in
@@ -964,7 +988,7 @@ void fp_TextRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y,
 		x += xoff;
 		x2 += xoff;
 		
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 		// reset this, so we have no stale pointers there
 		m_pRenderInfo->m_pText = NULL;
 #endif
@@ -1042,7 +1066,8 @@ bool fp_TextRun::canMergeWithNext(void)
 		    && !(*getRevisions() == *(pNext->getRevisions()))) //
 															   //non-null but different
 		|| (pNext->getVisibility() != getVisibility())
-
+		// Different authors
+		|| (pNext->getAuthorNum() != getAuthorNum())
 #if 0
 		// I do not think this should happen at all
 		|| ((pNext->m_bRenderInfo->isJustified() && m_bRenderInfo->isJustified())
@@ -1221,6 +1246,7 @@ bool fp_TextRun::split(UT_uint32 iSplitOffset)
 	pNew->_setColorHL(_getColorHL());
 	pNew->_setField(this->getField());
 	pNew->m_fPosition = this->m_fPosition;
+	pNew->setTextTransform(this->getTextTransform());
 
 	pNew->_setAscent(this->getAscent());
 	pNew->_setDescent(this->getDescent());
@@ -1235,7 +1261,7 @@ bool fp_TextRun::split(UT_uint32 iSplitOffset)
 	pNew->setVisDirection(iVisDirection);
 
 	pNew->_setHyperlink(this->getHyperlink());
-
+	pNew->setAuthorNum(this->getAuthorNum());
 	// when revisions are present, this gets bit trickier
 	if(getRevisions() != NULL)
 	{
@@ -1350,6 +1376,10 @@ bool fp_TextRun::split(UT_uint32 iSplitOffset)
 }
 
 
+UT_BidiCharType  fp_TextRun:: getDirection() const 
+{ return m_iDirOverride == static_cast<UT_BidiCharType>(UT_BIDI_UNSET) ? _getDirection() : m_iDirOverride;}
+
+
 UT_sint32 fp_TextRun::simpleRecalcWidth(UT_sint32 iLength)
 {
 
@@ -1447,7 +1477,7 @@ bool fp_TextRun::_addupCharWidths(void)
 	m_pRenderInfo->m_pFont = _getFont();
 	
 	iWidth = getGraphics()->getTextWidth(*m_pRenderInfo);
-	xxx_UT_DEBUGMSG(("Run %x Width %d \n",this,iWidth)); 
+	
 	if(iWidth != getWidth())
 	{
 		_setWidth(iWidth);
@@ -2025,7 +2055,7 @@ void fp_TextRun::_fillRect(UT_RGBColor& clr,
 						   UT_uint32 iPos1,
 						   UT_uint32 iLen,
 						   UT_Rect &r,
-						   GR_Graphics * pG)
+						   GR_Graphics * /*pG*/)
 {
 	/*
 	  Upon entry to this function, yoff is the TOP of the run,
@@ -2037,7 +2067,7 @@ void fp_TextRun::_fillRect(UT_RGBColor& clr,
 	if (getGraphics()->queryProperties(GR_Graphics::DGP_SCREEN))
 	{
 		//UT_Rect r;
-
+		xxx_UT_DEBUGMSG(("Doing _fillRect red %d blue %d green %d\n",clr.m_red,clr.m_blu,clr.m_grn));
 		_getPartRect(&r, xoff, yoff, iPos1, iLen);
 		r.height = getLine()->getHeight();
 		r.top = r.top + getAscent() - getLine()->getAscent();
@@ -2116,6 +2146,17 @@ void fp_TextRun::_getPartRect(UT_Rect* pRect,
 	}
 	UT_ASSERT(pRect->left < 10000000);
 	UT_ASSERT(pRect->width >= 0);
+	xxx_UT_DEBUGMSG(("part Rect left %d width %d \n",pRect->left,pRect->width));
+}
+
+static fp_Run* getPreviousInterestingRunForCapitalization(fp_Run* self) {
+	if (self == NULL)
+		return NULL;
+
+	if (self->getType() == FPRUN_FMTMARK)
+		return getPreviousInterestingRunForCapitalization(self->getPrevRun());
+
+	return self;
 }
 
 /*!
@@ -2152,9 +2193,29 @@ bool fp_TextRun::_refreshDrawBuffer()
 		PD_StruxIterator text(getBlock()->getStruxDocHandle(),
 							  getBlockOffset() + fl_BLOCK_STRUX_OFFSET);
 
+		bool lastWasSpace = false;
+		if (getTextTransform() == GR_ShapingInfo::CAPITALIZE) {
+			fp_Run* prevRun = getPreviousInterestingRunForCapitalization(this->getPrevRun());
+			if (prevRun == NULL) {
+				lastWasSpace = true;
+			}
+			else if (prevRun->getType() != FPRUN_TEXT) {
+				lastWasSpace = true;
+			} 
+			else if (prevRun->getType() == FPRUN_TEXT) {
+				UT_GrowBuf buf;
+				static_cast<fp_TextRun*>(prevRun)->appendTextToBuf(buf);
+				
+				if (buf.getLength() != 0) {
+					UT_GrowBufElement* elem = buf.getPointer(buf.getLength() - 1);
+					lastWasSpace = g_unichar_isspace(*elem);
+				}
+			}
+		}
+
 		GR_ShapingInfo si(text,iLen, m_pLanguage, iVisDir,
 						  m_pRenderInfo ? m_pRenderInfo->m_eShapingResult : GRSR_Unknown,
-						  _getFont(), m_pItem);
+						  _getFont(), m_pItem, getTextTransform(), lastWasSpace);		
 		getGraphics()->shape(si, m_pRenderInfo);
 		
 		UT_ASSERT(m_pRenderInfo && m_pRenderInfo->m_eShapingResult != GRSR_Error );
@@ -2191,7 +2252,7 @@ bool fp_TextRun::_refreshDrawBuffer()
 	this function expect to have m_pRenderInfo->m_x/yoff set
 	xoff is the right edge of this run !!!
 */
-void fp_TextRun::_drawLastChar(bool bSelection)
+void fp_TextRun::_drawLastChar(bool /*bSelection*/)
 {
 //
 // We appear to no longer need this code. Symptom would be if the last
@@ -2701,7 +2762,8 @@ void fp_TextRun::resetJustification(bool bPermanent)
 void fp_TextRun::justify(UT_sint32 iAmount, UT_uint32 iSpacesInRun)
 {
 	UT_return_if_fail(m_pRenderInfo);
-	
+	UT_uint32 len = getLength();
+
 	if(!iAmount)
 	{
 		// this can happend near the start of the line (the line is
@@ -2714,13 +2776,13 @@ void fp_TextRun::justify(UT_sint32 iAmount, UT_uint32 iSpacesInRun)
 		return;
 	}
 
-	if(iSpacesInRun && getLength() > 0)
+	if(iSpacesInRun && len > 0)
 	{
-		m_pRenderInfo->m_iLength = getLength();
+		m_pRenderInfo->m_iLength = len;
 	
 		_setWidth(getWidth() + iAmount);
 
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 		// Because Pango does not yet support justification, we need to iterate over the raw
 		// text counting spaces; this is not required by the default graphics class, nor
 		// Uniscribe and for performance reasons it is therefore only conditionally compiled
@@ -2728,9 +2790,10 @@ void fp_TextRun::justify(UT_sint32 iAmount, UT_uint32 iSpacesInRun)
 		// will be again removed
 		UT_uint32 iPosStart = getBlockOffset() + fl_BLOCK_STRUX_OFFSET;
 		PD_StruxIterator text(getBlock()->getStruxDocHandle(),iPosStart);
-		text.setUpperLimit(text.getPosition() + getLength() - 1);
+		text.setUpperLimit(text.getPosition() + len - 1);
 		m_pRenderInfo->m_pText = & text;
-		m_pRenderInfo->m_iLength = getLength();
+		UT_ASSERT(len == getLength());
+//		m_pRenderInfo->m_iLength = getLength();
 #else
 		m_pRenderInfo->m_pText = NULL;
 #endif
@@ -2739,7 +2802,7 @@ void fp_TextRun::justify(UT_sint32 iAmount, UT_uint32 iSpacesInRun)
 		m_pRenderInfo->m_iJustificationAmount = iAmount;
 		getGraphics()->justify(*m_pRenderInfo);
 
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 		// do not leave stale pointer behind
 		m_pRenderInfo->m_pText = NULL;
 #endif
@@ -2760,7 +2823,7 @@ UT_sint32 fp_TextRun::countJustificationPoints(bool bLast) const
 
 	UT_return_val_if_fail(m_pRenderInfo->m_iLength > 0, 0);
 
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 	// Because Pango does not yet support justification, we need to iterate over the raw
 	// text counting spaces; this is not required by the default graphics class, nor
 	// Uniscribe and for performance reasons it is therefore only conditionally compiled
@@ -2778,7 +2841,7 @@ UT_sint32 fp_TextRun::countJustificationPoints(bool bLast) const
 	m_pRenderInfo->m_bLastOnLine = bLast;
 	UT_sint32 iCount = getGraphics()->countJustificationPoints(*m_pRenderInfo);
 
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 	m_pRenderInfo->m_pText = NULL;
 #endif
 
@@ -2839,18 +2902,18 @@ UT_sint32 fp_TextRun::getStr(UT_UCSChar * pStr, UT_uint32 &iMax)
 void fp_TextRun::setDirection(UT_BidiCharType dir, UT_BidiCharType dirOverride)
 {
 	if( !getLength()
-	|| (   dir == UT_BIDI_UNSET
-		&& _getDirection() != UT_BIDI_UNSET
+		|| (   dir ==  static_cast<UT_BidiCharType>(UT_BIDI_UNSET)
+			   && _getDirection() !=  static_cast<UT_BidiCharType>(UT_BIDI_UNSET)
 		&& dirOverride == m_iDirOverride
 		)
 	  )
 		return; //ignore 0-length runs, let them be treated on basis of the app defaults
 
-	UT_BidiCharType prevDir = m_iDirOverride == UT_BIDI_UNSET ? _getDirection() : m_iDirOverride;
-	if(dir == UT_BIDI_UNSET)
+	UT_BidiCharType prevDir = m_iDirOverride ==  static_cast<UT_BidiCharType>(UT_BIDI_UNSET) ? _getDirection() : m_iDirOverride;
+	if(dir ==  static_cast<UT_BidiCharType>(UT_BIDI_UNSET))
 	{
 		// only do this once
-		if(_getDirection() == UT_BIDI_UNSET)
+		if(_getDirection() ==  static_cast<UT_BidiCharType>(UT_BIDI_UNSET))
 		{
 			// here we used to check the first character; we can no longer do that,
 			// because the latest versions of USP create items that are not homogenous and
@@ -2886,7 +2949,7 @@ void fp_TextRun::setDirection(UT_BidiCharType dir, UT_BidiCharType dirOverride)
 		_setDirection(dir);
 	}
 
-	if(dirOverride != UT_BIDI_IGNORE)
+	if(dirOverride !=  static_cast<UT_BidiCharType>(UT_BIDI_IGNORE))
 	{
 
 		m_iDirOverride = dirOverride;
@@ -2896,7 +2959,7 @@ void fp_TextRun::setDirection(UT_BidiCharType dir, UT_BidiCharType dirOverride)
 		// it to that direction, if it is weak, we have to make the line
 		// to calculate it
 
-		if(dirOverride != UT_BIDI_UNSET)
+		if(dirOverride !=  static_cast<UT_BidiCharType>(UT_BIDI_UNSET))
 			setVisDirection(dirOverride);
 	}
 
@@ -2910,9 +2973,9 @@ void fp_TextRun::setDirection(UT_BidiCharType dir, UT_BidiCharType dirOverride)
 		functions when the run is loaded from a document on the disk.)
 	*/
 
-	UT_BidiCharType curDir = m_iDirOverride == UT_BIDI_UNSET ? _getDirection() : m_iDirOverride;
+	UT_BidiCharType curDir = m_iDirOverride ==  static_cast<UT_BidiCharType>(UT_BIDI_UNSET) ? _getDirection() : m_iDirOverride;
 
-	UT_ASSERT(curDir != UT_BIDI_UNSET);
+	UT_ASSERT(curDir !=  static_cast<UT_BidiCharType>(UT_BIDI_UNSET));
 
 	if(curDir != prevDir)
 	{
@@ -2952,7 +3015,7 @@ void fp_TextRun::setDirection(UT_BidiCharType dir, UT_BidiCharType dirOverride)
 */
 void fp_TextRun::setDirOverride(UT_BidiCharType dir)
 {
-	if(dir == UT_BIDI_UNSET || dir == m_iDirOverride)
+	if(dir ==  static_cast<UT_BidiCharType>(UT_BIDI_UNSET) || dir ==  static_cast<UT_BidiCharType>(m_iDirOverride))
 		return;
 
 	const gchar * prop[] = {NULL, NULL, 0};
