@@ -36,25 +36,19 @@
 #include "xap_App.h"
 
 FV_FrameEdit::FV_FrameEdit (FV_View * pView)
-	: m_pView (pView), 
+	: FV_Base (pView), 
 	  m_iFrameEditMode(FV_FrameEdit_NOT_ACTIVE),
 	  m_pFrameLayout(NULL),
 	  m_pFrameContainer(NULL),
-	  m_iDraggingWhat( FV_FrameEdit_DragNothing),
 	  m_iLastX(0),
 	  m_iLastY(0),
-	  m_recCurFrame(0,0,0,0),
 	  m_iInitialDragX(0),
 	  m_iInitialDragY(0),
-	  m_bFirstDragDone(false),
 	  m_bInitialClick(false),
 	  m_pFrameImage(NULL),
 	  m_pAutoScrollTimer(NULL),
-	  m_xLastMouse(1),
-	  m_yLastMouse(1),
-	  m_iFirstEverX(0),
-	  m_iFirstEverY(0),
-	  m_iGlobCount(0)
+	  m_iInitialFrameX(0),
+	  m_iInitialFrameY(0)
 {
 	UT_ASSERT_HARMLESS(pView);
 }
@@ -86,24 +80,9 @@ bool FV_FrameEdit::isActive(void) const
 	return (FV_FrameEdit_NOT_ACTIVE != m_iFrameEditMode);
 }
 
-PD_Document * FV_FrameEdit::getDoc(void) const
-{
-	return m_pView->getDocument();
-}
-
-FL_DocLayout * FV_FrameEdit::getLayout(void) const
-{
-	return m_pView->getLayout();
-}
-
-GR_Graphics * FV_FrameEdit::getGraphics(void) const
-{
-	return m_pView->getGraphics();
-}
-
 void FV_FrameEdit::setMode(FV_FrameEditMode iEditMode)
 {
-        UT_DEBUGMSG(("Frame Edit mode set to %d \n",iEditMode));
+    UT_DEBUGMSG(("Frame Edit mode set to %d \n",iEditMode));
 	if(iEditMode == FV_FrameEdit_NOT_ACTIVE)
 	{
 		m_pFrameLayout = NULL;
@@ -111,27 +90,41 @@ void FV_FrameEdit::setMode(FV_FrameEditMode iEditMode)
 		DELETEP(m_pFrameImage);
 		m_recCurFrame.width = 0;
 		m_recCurFrame.height = 0;
-		m_iDraggingWhat = FV_FrameEdit_DragNothing;
+		setDragWhat( FV_DragNothing );
 		m_iLastX = 0;
 		m_iLastY = 0;
 	}
 	m_iFrameEditMode = iEditMode;
-	if(getGraphics() && getGraphics()->getCaret() && iEditMode !=  FV_FrameEdit_NOT_ACTIVE)
+	if(getGraphics()  && iEditMode !=  FV_FrameEdit_NOT_ACTIVE)
 	{
-	        getGraphics()->getCaret()->disable();
+	        getGraphics()->allCarets()->disable();
 		m_pView->m_countDisable++;
 	}
 }
 
 
-void FV_FrameEdit::_autoScroll(UT_Worker * pWorker)
-{
-	UT_return_if_fail(pWorker);
+static bool bScrollRunning = false;
+static UT_Worker * s_pScroll = NULL;
+static UT_sint32 iExtra = 0;
 
+void FV_FrameEdit::_actuallyScroll(UT_Worker * pWorker)
+{
 	// this is a static callback method and does not have a 'this' pointer.
 
 	FV_FrameEdit * pFE = static_cast<FV_FrameEdit *>(pWorker->getInstanceData());
 	UT_return_if_fail(pFE);
+	if(pFE->getFrameEditMode() != FV_FrameEdit_DRAG_EXISTING)
+	{
+		if(pFE->m_pAutoScrollTimer)
+			pFE->m_pAutoScrollTimer->stop();
+		DELETEP(pFE->m_pAutoScrollTimer);
+		iExtra = 0;
+		s_pScroll->stop();
+		delete s_pScroll;
+		s_pScroll = NULL;
+		bScrollRunning = false;
+		return;
+	}
 	FV_View * pView = pFE->m_pView;
 	UT_sint32 x = pFE->m_xLastMouse;
 	UT_sint32 y = pFE->m_yLastMouse;
@@ -139,13 +132,33 @@ void FV_FrameEdit::_autoScroll(UT_Worker * pWorker)
 	bool bScrollUp = false;
 	bool bScrollLeft = false;
 	bool bScrollRight = false;
+	bool bStop = false;
 	if(y<=0)
 	{
-		bScrollUp = true;
+	  if(pView->getYScrollOffset() <= 10)
+	  {
+	      pView->setYScrollOffset(0);
+	      pView->updateScreen(false);
+	      bStop = true;
+	  }
+	  else
+	  {
+	      bScrollUp = true;
+	  }
 	}
 	else if( y >= pView->getWindowHeight())
 	{
+	  if((pView->getYScrollOffset()+pView->getWindowHeight()+10) >= pView->getLayout()->getHeight())
+	  {
+	      pView->setYScrollOffset(pView->getLayout()->getHeight() - pView->getWindowHeight());
+	      pView->updateScreen(false);
+	      bStop = true;
+	      UT_DEBUGMSG(("!!!!!!!!!!!!PLLLLLLLLLEEEEAAAASSSEEEEE STOOPPP!!!! \n"));
+	  }
+	  else
+	  {
 		bScrollDown = true;
+	  }
 	}
 	if(x <= 0)
 	{
@@ -155,18 +168,25 @@ void FV_FrameEdit::_autoScroll(UT_Worker * pWorker)
 	{
 		bScrollRight = true;
 	}
-	if(bScrollDown || bScrollUp || bScrollLeft || bScrollRight)
+	if(!bStop && (bScrollDown || bScrollUp || bScrollLeft || bScrollRight))
 	{
 		pFE->getGraphics()->setClipRect(&pFE->m_recCurFrame);
 		pView->updateScreen(false);
 		pFE->getGraphics()->setClipRect(NULL);
+		UT_sint32 minScroll = pFE->getGraphics()->tlu(20);
 		if(bScrollUp)
 		{
-			pView->cmdScroll(AV_SCROLLCMD_LINEUP, static_cast<UT_uint32>( -y));
+		        UT_sint32 yscroll = abs(y);
+			if(yscroll < minScroll)
+			    yscroll = minScroll;
+			pView->cmdScroll(AV_SCROLLCMD_LINEUP, static_cast<UT_uint32>( yscroll +iExtra));
 		}
 		else if(bScrollDown)
 		{
-			pView->cmdScroll(AV_SCROLLCMD_LINEDOWN, static_cast<UT_uint32>(y - pView->getWindowHeight()));
+		        UT_sint32 yscroll = y - pView->getWindowHeight();
+			if(yscroll < minScroll)
+			    yscroll = minScroll;
+			pView->cmdScroll(AV_SCROLLCMD_LINEDOWN, static_cast<UT_uint32>(yscroll+iExtra));
 		}
 		if(bScrollLeft)
 		{
@@ -177,15 +197,54 @@ void FV_FrameEdit::_autoScroll(UT_Worker * pWorker)
 			pView->cmdScroll(AV_SCROLLCMD_LINERIGHT, static_cast<UT_uint32>(x -pView->getWindowWidth()));
 		}
 		pFE->drawFrame(true);
+		iExtra = 0;
 		return;
 	}
 	else
 	{
-		pFE->m_pAutoScrollTimer->stop();
+		if(pFE->m_pAutoScrollTimer)
+			pFE->m_pAutoScrollTimer->stop();
 		DELETEP(pFE->m_pAutoScrollTimer);
 
 	}
+	iExtra = 0;
+	s_pScroll->stop();
+	delete s_pScroll;
+	s_pScroll = NULL;
+	bScrollRunning = false;
+}
 
+void FV_FrameEdit::_autoScroll(UT_Worker * pWorker)
+{
+	UT_return_if_fail(pWorker);
+	// this is a static callback method and does not have a 'this' pointer.
+
+	FV_FrameEdit * pFE = static_cast<FV_FrameEdit *>(pWorker->getInstanceData());
+	UT_return_if_fail(pFE);
+	if(bScrollRunning)
+	{
+	    if(iExtra < pFE->getGraphics()->tlu(600))
+	      iExtra += pFE->getGraphics()->tlu(20);
+	    UT_DEBUGMSG(("Dropping FrameEditautoscroll !!!!!!! \n"));
+	    return;
+	}
+
+	UT_DEBUGMSG(("_autoscroll started!! \n"));
+	int inMode = UT_WorkerFactory::IDLE | UT_WorkerFactory::TIMER;
+	//	int inMode = UT_WorkerFactory::TIMER;
+	UT_WorkerFactory::ConstructMode outMode = UT_WorkerFactory::NONE;
+	s_pScroll = UT_WorkerFactory::static_constructor (_actuallyScroll,pFE, inMode, outMode);
+
+	// If the worker is working on a timer instead of in the idle
+	// time, set the frequency of the checks.
+	if ( UT_WorkerFactory::TIMER == outMode )
+	{
+		// this is really a timer, so it's safe to static_cast it
+		static_cast<UT_Timer*>(s_pScroll)->set(100);
+	}
+	bScrollRunning = true;
+	iExtra = 0;
+	s_pScroll->start();
 }
 
 /*!
@@ -202,310 +261,59 @@ UT_sint32 FV_FrameEdit::haveDragged(void) const
 	if((abs(m_xLastMouse - m_iFirstEverX) + abs(m_yLastMouse - m_iFirstEverY)) <
 		getGraphics()->tlu(3))
 	{
+	  UT_DEBUGMSG(("Not dragged enough - return 1 \n"));
 		return 1;
 	}
 	return 10;
 }
 
-UT_sint32 FV_FrameEdit::getGlobCount()
+void FV_FrameEdit::_mouseDrag(UT_sint32 x, UT_sint32 y)
 {
-	return m_iGlobCount;
-}
-
-void  FV_FrameEdit::_beginGlob(void)
-{
-	getDoc()->beginUserAtomicGlob();
-	m_iGlobCount++;
-	UT_DEBUGMSG(("Begin Glob count %d \n",m_iGlobCount));
-}
-
-
-void  FV_FrameEdit::_endGlob(void)
-{
-	getDoc()->endUserAtomicGlob();
-	m_iGlobCount--;
-	UT_DEBUGMSG(("End Glob count %d \n",m_iGlobCount));
-}
-
-
-void FV_FrameEdit::mouseDrag(UT_sint32 x, UT_sint32 y)
-{
-	if(!m_bFirstDragDone)
-	{
-		m_iFirstEverX = x;
-		m_iFirstEverY = y;
-	}
-	m_bFirstDragDone = true;
-	UT_sint32 diffx = 0;
-	UT_sint32 diffy = 0;
 	UT_sint32 dx = 0;
 	UT_sint32 dy = 0;
 	UT_Rect expX(0,m_recCurFrame.top,0,m_recCurFrame.height);
 	UT_Rect expY(m_recCurFrame.left,0,m_recCurFrame.width,0);
-	UT_sint32 iext = getGraphics()->tlu(3);
-	m_xLastMouse = x;
-	m_yLastMouse = y;
-	switch (m_iDraggingWhat)
-	{
-	case FV_FrameEdit_DragTopLeftCorner:
-		diffx = m_recCurFrame.left - x;
-		diffy = m_recCurFrame.top - y;
-		m_recCurFrame.left -= diffx;
-		m_recCurFrame.top -= diffy;
-		dx = -diffx;
-		dy = -diffy;
-		m_recCurFrame.width += diffx;
-		m_recCurFrame.height += diffy;
-		if(diffx < 0)
-		{
-			expX.left = m_recCurFrame.left + diffx -iext;
-			expX.width = -diffx + iext;
-			if(diffy > 0)
-			{
-				expX.top -=  diffy + iext;
-				expX.height += diffy + 2*iext;
-			}
-			else
-			{
-				expX.top -=  iext;
-				expX.height += (-diffy + 2*iext);
-			}
-		}
-		if(diffy < 0)
-		{
-			expY.top = m_recCurFrame.top + diffy - iext;
-			expY.height = -diffy + 2*iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_NE);
-			m_iDraggingWhat =  FV_FrameEdit_DragTopRightCorner;
-		}
-		if(m_recCurFrame.height < 0)
-		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_SW);
-			m_iDraggingWhat =  FV_FrameEdit_DragBotLeftCorner;
-		}
-		break;
-	case FV_FrameEdit_DragTopRightCorner:
-		diffx = m_recCurFrame.left + m_recCurFrame.width - x;
-		diffy = m_recCurFrame.top - y;
-		m_recCurFrame.top -= diffy;
-		dy = -diffy;
-		m_recCurFrame.width -= diffx;
-		m_recCurFrame.height += diffy;
-		if(diffx > 0)
-		{
-			expX.left = m_recCurFrame.left + m_recCurFrame.width;
-			expX.width = diffx + iext;
-			if(diffy > 0)
-			{
-				expX.top -=  iext;
-				expX.height += diffy + 2*iext;
-			}
-			else
-			{
-				expX.top -=  iext;
-				expX.height += (-diffy + 2*iext);
-			}
-		}
-		if(diffy < 0)
-		{
-			expY.top = m_recCurFrame.top + diffy - iext;
-			expY.height = -diffy + iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_NW);
-			m_iDraggingWhat =  FV_FrameEdit_DragTopLeftCorner;
-		}
-		if(m_recCurFrame.height < 0)
-		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_SE);
-			m_iDraggingWhat =  FV_FrameEdit_DragBotRightCorner;
-		}
-		break;
-	case FV_FrameEdit_DragBotLeftCorner:
-		diffx = m_recCurFrame.left - x;
-		diffy = m_recCurFrame.top + m_recCurFrame.height - y;
-		m_recCurFrame.left -= diffx;
-		dx = -diffx;
-		m_recCurFrame.width += diffx;
-		m_recCurFrame.height -= diffy;
-		if(diffx < 0)
-		{
-			expX.left = m_recCurFrame.left + diffx -iext;
-			expX.width = -diffx + iext;
-			if(diffy > 0)
-			{
-				expX.top -=  diffy + iext;
-				expX.height += diffy + 2*iext;
-			}
-			else
-			{
-				expX.top -=  iext;
-				expX.height += (-diffy + 2*iext);
-			}
-		}
-		if(diffy > 0)
-		{
-			expY.top = m_recCurFrame.top + m_recCurFrame.height - iext;
-			expY.height = diffy + 2*iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_SE);
-			m_iDraggingWhat =  FV_FrameEdit_DragBotRightCorner;
-
-		}
-		if(m_recCurFrame.height < 0)
-		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_NW);
-			m_iDraggingWhat =  FV_FrameEdit_DragTopLeftCorner;
-		}
-		break;
-	case FV_FrameEdit_DragBotRightCorner:
-		diffx = m_recCurFrame.left + m_recCurFrame.width - x;
-		diffy = m_recCurFrame.top + m_recCurFrame.height - y;
-		m_recCurFrame.width -= diffx;
-		m_recCurFrame.height -= diffy;
-		if(diffx > 0)
-		{
-			expX.left = m_recCurFrame.left + m_recCurFrame.width;
-			expX.width = diffx + iext;
-			if(diffy > 0)
-			{
-				expX.top -=  iext;
-				expX.height += diffy + 2*iext;
-			}
-			else
-			{
-				expX.top -=  iext;
-				expX.height += (-diffy + 2*iext);
-			}
-		}
-		if(diffy > 0)
-		{
-			expY.top = m_recCurFrame.top + m_recCurFrame.height;
-			expY.height = diffy + iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_SW);
-			m_iDraggingWhat =  FV_FrameEdit_DragBotLeftCorner;
-		}
-		if(m_recCurFrame.height < 0)
-		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_NE);
-			m_iDraggingWhat =  FV_FrameEdit_DragTopRightCorner;
-		}
-		break;
-	case FV_FrameEdit_DragLeftEdge:
-		diffx = m_recCurFrame.left - x;
-		m_recCurFrame.left -= diffx;
-		dx = -diffx;
-		m_recCurFrame.width += diffx;
-		if(diffx < 0)
-		{
-			expX.left = m_recCurFrame.left + diffx - iext;
-			expX.width = -diffx + iext;
-			expX.top -=  iext;
-			expX.height += 2*iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_W);
-			m_iDraggingWhat =  FV_FrameEdit_DragRightEdge;
-		}
-		break;
-	case FV_FrameEdit_DragRightEdge:
-		diffx = m_recCurFrame.left + m_recCurFrame.width - x;
-		m_recCurFrame.width -= diffx;
-		if(diffx > 0)
-		{
-			expX.left = m_recCurFrame.left + m_recCurFrame.width;
-			expX.width = diffx + iext;
-			expX.top -=  iext;
-			expX.height += 2*iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_E);
-			m_iDraggingWhat =  FV_FrameEdit_DragLeftEdge;
-		}
-		break;
-	case FV_FrameEdit_DragTopEdge:
-		diffy = m_recCurFrame.top - y;
-		m_recCurFrame.top -= diffy;
-		dy = -diffy;
-		m_recCurFrame.height += diffy;
-		if(diffy < 0)
-		{
-			expY.top = m_recCurFrame.top + diffy - iext;
-			expY.height = -diffy + iext;
-			expY.left -= iext;
-			expY.width += 2*iext;
-		}
-		if(m_recCurFrame.height < 0)
-		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_S);
-			m_iDraggingWhat =  FV_FrameEdit_DragBotEdge;
-		}
-		break;
-	case FV_FrameEdit_DragBotEdge:
-		diffy = m_recCurFrame.top + m_recCurFrame.height - y;
-		m_recCurFrame.height -= diffy;
-		if(diffy > 0)
-		{
-			expY.top = m_recCurFrame.top + m_recCurFrame.height;
-			expY.height = diffy + iext;
-			expY.left -= iext;
-			expY.width += 2*iext;
-			xxx_UT_DEBUGMSG(("expY.top %d expY.height %d \n",expY.top,expY.height));
-		}
-		if(m_recCurFrame.height < 0)
-		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_N);
-			m_iDraggingWhat =  FV_FrameEdit_DragTopEdge;
-		}
-		break;
-	case FV_FrameEdit_DragWholeFrame:
+	
+	FV_Base::_doMouseDrag( x, y, dx, dy, expX, expY );
+	if (getDragWhat()==FV_DragWhole) 
 	{
+		UT_sint32 diffx = 0;
+		UT_sint32 diffy = 0;
+		UT_sint32 iext = getGraphics()->tlu(3);
+		
 		bool bScrollDown = false;
 		bool bScrollUp = false;
 		bool bScrollLeft = false;
 		bool bScrollRight = false;
 		if(y<=0)
 		{
-			bScrollUp = true;
+		  if(m_pView->getYScrollOffset() <= 0)
+		    {
+		      m_pView->setYScrollOffset(0);
+		      m_pView->updateScreen(false);
+		      if(m_pAutoScrollTimer)
+			m_pAutoScrollTimer->stop();
+		      DELETEP(m_pAutoScrollTimer);
+		    }
+		  else
+		    {
+		      bScrollUp = true;
+		    }
 		}
 		else if( y >= m_pView->getWindowHeight())
 		{
-			bScrollDown = true;
+		  if(m_pView->getYScrollOffset() >= m_pView->getLayout()->getHeight())
+		  {
+		      m_pView->setYScrollOffset(m_pView->getLayout()->getHeight());
+		      m_pView->updateScreen(false);
+		      if(m_pAutoScrollTimer)
+			m_pAutoScrollTimer->stop();
+		      DELETEP(m_pAutoScrollTimer);
+		  }
+		  else
+		  {
+		      bScrollDown = true;
+		  }
 		}
 		if(x <= 0)
 		{
@@ -532,6 +340,7 @@ void FV_FrameEdit::mouseDrag(UT_sint32 x, UT_sint32 y)
 		dy = - diffy;
 		m_recCurFrame.left -= diffx;
 		m_recCurFrame.top -= diffy;
+		UT_DEBUGMSG(("Doing drag whole frame left %d top %d \n",m_recCurFrame.left, m_recCurFrame.top));
 		if(dx < 0)
 		{
 			expX.left = m_recCurFrame.left+m_recCurFrame.width -iext;
@@ -575,26 +384,7 @@ void FV_FrameEdit::mouseDrag(UT_sint32 x, UT_sint32 y)
 			expY.height = dy + 2*iext;
 		}
 	}
-	break;
-	default:
-		break;
-	}
-
-
-		// don't left widths and heights be too big
-
-	double dWidth = static_cast<double>(m_recCurFrame.width)/static_cast<double>(UT_LAYOUT_RESOLUTION);
-	double dHeight = static_cast<double>(m_recCurFrame.height)/static_cast<double>(UT_LAYOUT_RESOLUTION);
-	if(m_pView->getPageSize().Width(DIM_IN) < dWidth)
-	{
-	  dWidth = m_pView->getPageSize().Width(DIM_IN)*0.99;
-	  m_recCurFrame.width = static_cast<UT_sint32>(dWidth*UT_LAYOUT_RESOLUTION);
-	}
-	if(m_pView->getPageSize().Height(DIM_IN) < dHeight)
-	{
-	  dHeight = m_pView->getPageSize().Height(DIM_IN)*0.99;
-	  m_recCurFrame.height = static_cast<UT_sint32>(dHeight*UT_LAYOUT_RESOLUTION);
-	}
+	_checkDimensions();
 
 	if(FV_FrameEdit_RESIZE_INSERT == m_iFrameEditMode)
 	{
@@ -631,7 +421,7 @@ void FV_FrameEdit::mouseDrag(UT_sint32 x, UT_sint32 y)
 		getGraphics()->setClipRect(NULL);
 
 		drawFrame(true);
-		UT_DEBUGMSG(("Draw frame finished \n"));
+		xxx_UT_DEBUGMSG(("Draw frame finished \n"));
 	}
 	else if (FV_FrameEdit_DRAG_EXISTING == m_iFrameEditMode)
 	{
@@ -641,6 +431,7 @@ void FV_FrameEdit::mouseDrag(UT_sint32 x, UT_sint32 y)
 		newY += dy;
 		m_pFrameContainer->_setX(newX);
 		m_pFrameContainer->_setY(newY);
+		xxx_UT_DEBUGMSG(("Doing dragging existing frame left %d top %d \n",m_recCurFrame.left, m_recCurFrame.top));
 		if(expX.width > 0)
 		{
 			getGraphics()->setClipRect(&expX);
@@ -704,9 +495,9 @@ void FV_FrameEdit::setDragType(UT_sint32 x, UT_sint32 y, bool bDrawFrame)
 	if(!isActive() && (pFCon == NULL))
 	{
 		m_iFrameEditMode = 	FV_FrameEdit_EXISTING_SELECTED;
-		if(getGraphics() && getGraphics()->getCaret())
+		if(getGraphics())
 		{
-		      getGraphics()->getCaret()->disable();
+		      getGraphics()->allCarets()->disable();
 		      m_pView->m_countDisable++;
 		}
 		fl_ContainerLayout * pCL = pBL->myContainingLayout();
@@ -729,15 +520,15 @@ void FV_FrameEdit::setDragType(UT_sint32 x, UT_sint32 y, bool bDrawFrame)
 		}
 		m_iLastX = x;
 		m_iLastY = y;
-		m_iDraggingWhat = FV_FrameEdit_DragWholeFrame;
+		setDragWhat( FV_DragWhole );
 		return;	
 	}
 	if(!isActive())
 	{
 		m_iFrameEditMode = 	FV_FrameEdit_EXISTING_SELECTED;
-		if(getGraphics() && getGraphics()->getCaret())
+		if(getGraphics())
 		{
-		      getGraphics()->getCaret()->disable();
+		      getGraphics()->allCarets()->disable();
 		      m_pView->m_countDisable++;
 		}
 		m_pFrameLayout = pFL;
@@ -750,7 +541,7 @@ void FV_FrameEdit::setDragType(UT_sint32 x, UT_sint32 y, bool bDrawFrame)
 		}
 		m_iLastX = x;
 		m_iLastY = y;
-		m_iDraggingWhat = FV_FrameEdit_DragWholeFrame;
+		setDragWhat( FV_DragWhole );
 		return;
 	}
 	//
@@ -795,67 +586,67 @@ void FV_FrameEdit::setDragType(UT_sint32 x, UT_sint32 y, bool bDrawFrame)
 //
 	if((iLeft < x) && (x < iLeft + ires) && (iTop < y) && (y < iTop + ires))
 	{
-		m_iDraggingWhat = FV_FrameEdit_DragTopLeftCorner;
+		setDragWhat( FV_DragTopLeftCorner );
 	}
 //
 // top Right
 //
 	else if((iRight - ires < x) && (x < iRight) && (iTop < y) && (y < iTop + ires))
 	{
-		m_iDraggingWhat = FV_FrameEdit_DragTopRightCorner;
+		setDragWhat( FV_DragTopRightCorner );
 	}
 //
 // bot left
 //
 	else if((iLeft < x) && (x < iLeft + ires) && (iBot > y) && (y > iBot - ires))
 	{
-		m_iDraggingWhat = FV_FrameEdit_DragBotLeftCorner;
+		setDragWhat( FV_DragBotLeftCorner );
 	}
 //
 // Bot Right
 //
 	else if((iRight - ires < x) && (x < iRight) && (iBot > y) && (y > iBot - ires))
 	{
-		m_iDraggingWhat = FV_FrameEdit_DragBotRightCorner;
+		setDragWhat( FV_DragBotRightCorner );
 	}
 //
 // top Edge
 //
 	else if( bX && bTop)
 	{
-		m_iDraggingWhat = FV_FrameEdit_DragTopEdge;
+		setDragWhat( FV_DragTopEdge );
 	}
 //
 // left Edge
 //
 	else if(bLeft && bY)
 	{
-		m_iDraggingWhat = FV_FrameEdit_DragLeftEdge;
+		setDragWhat( FV_DragLeftEdge );
 	}
 //
 // right Edge
 //
 	else if(bRight && bY)
 	{
-		m_iDraggingWhat = FV_FrameEdit_DragRightEdge;
+		setDragWhat( FV_DragRightEdge );
 	}
 //
 // bot Edge
 //
 	else if(bBot && bX)
 	{
-		m_iDraggingWhat = FV_FrameEdit_DragBotEdge;
+		setDragWhat( FV_DragBotEdge );
 	}
 	else
 	{
 		if( bX && bY)
 		{
-			m_iDraggingWhat = FV_FrameEdit_DragWholeFrame;
+			setDragWhat( FV_DragWhole );
 			xxx_UT_DEBUGMSG(("Dragging Whole Frame \n"));
 		}
 		else
 		{
-			m_iDraggingWhat = FV_FrameEdit_DragNothing;
+			setDragWhat( FV_DragNothing );
 			return;
 		}
 	}
@@ -872,11 +663,11 @@ void FV_FrameEdit::setDragType(UT_sint32 x, UT_sint32 y, bool bDrawFrame)
 	m_iInitialDragX = iLeft;
 	m_iInitialDragY = iTop;
 	xxx_UT_DEBUGMSG(("Initial width %d \n",m_recCurFrame.width));
-	xxx_UT_DEBUGMSG((" Dragging What %d \n",m_iDraggingWhat));
+	xxx_UT_DEBUGMSG((" Dragging What %d \n",getDragWhat()));
 	m_pView->setCursorToContext();
-	if(getGraphics() && getGraphics()->getCaret())
+	if(getGraphics())
 	{
-	    getGraphics()->getCaret()->disable();
+	    getGraphics()->allCarets()->disable();
 	    m_pView->m_countDisable++;
 	}
 }
@@ -891,7 +682,7 @@ void FV_FrameEdit::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 	if(!isActive())
 	{
 		setDragType(x,y,true);
-		UT_DEBUGMSG(("Was not active now %d FrameLayout %x \n",getFrameEditMode(),getFrameLayout()));
+		UT_DEBUGMSG(("Was not active now %d FrameLayout %p \n",getFrameEditMode(),getFrameLayout()));
 		return;
 	}
 //
@@ -901,7 +692,7 @@ void FV_FrameEdit::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 	{
 		setDragType(x,y,true);
 		UT_DEBUGMSG(("Was Existing Selected now %d \n",getFrameEditMode()));
-		if(FV_FrameEdit_DragNothing == m_iDraggingWhat)
+		if(FV_DragNothing == getDragWhat())
 		{
 			m_bFirstDragDone = false;
 			m_iFrameEditMode = FV_FrameEdit_NOT_ACTIVE;
@@ -933,7 +724,7 @@ void FV_FrameEdit::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 			m_pView->setCursorToContext();
 			m_recCurFrame.width = 0;
 			m_recCurFrame.height = 0;
-			m_iDraggingWhat = FV_FrameEdit_DragNothing;
+			setDragWhat( FV_DragNothing );
 			m_iLastX = 0;
 			m_iLastY = 0;
 			while(m_iGlobCount > 0)
@@ -942,7 +733,7 @@ void FV_FrameEdit::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 		}
 		else
 		{
-			if( m_iDraggingWhat != FV_FrameEdit_DragWholeFrame)
+			if( getDragWhat() != FV_DragWhole)
 			{
 				m_iFrameEditMode = FV_FrameEdit_RESIZE_EXISTING;
 			}
@@ -951,10 +742,12 @@ void FV_FrameEdit::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 				m_iFrameEditMode = FV_FrameEdit_DRAG_EXISTING;
 				m_iInitialDragX = m_recCurFrame.left;
 				m_iInitialDragY = m_recCurFrame.top;
+				m_iInitialFrameX = m_pFrameContainer->getFullX();
+				m_iInitialFrameY = m_pFrameContainer->getFullY();
 			}
-			if(getGraphics() && getGraphics()->getCaret())
+			if(getGraphics())
 			{
-			        getGraphics()->getCaret()->disable();
+			        getGraphics()->allCarets()->disable();
 			        m_pView->m_countDisable++;
 			}
 		}
@@ -976,8 +769,8 @@ void FV_FrameEdit::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 		UT_sint32 origX = x + iCursorOff;
 		UT_sint32 origY = y + iCursorOff;
 		UT_sint32 iSize = getGraphics()->tlu(32);
-		m_recCurFrame.left = origX - iSize;;
-		m_recCurFrame.top = origY - iSize;;
+		m_recCurFrame.left = origX - iSize;
+		m_recCurFrame.top = origY - iSize;
 		m_recCurFrame.width = iSize;
 		m_recCurFrame.height = iSize;
 		m_iFrameEditMode = FV_FrameEdit_RESIZE_INSERT;
@@ -989,12 +782,12 @@ void FV_FrameEdit::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 		m_iLastY = y;
 		m_iInitialDragX = m_recCurFrame.left;
 		m_iInitialDragY = m_recCurFrame.top;
-		m_iDraggingWhat = FV_FrameEdit_DragBotRightCorner;
+		setDragWhat( FV_DragBotRightCorner );
 		m_bFirstDragDone = false;
 		m_bInitialClick = true;
-		if(getGraphics() && getGraphics()->getCaret())
+		if(getGraphics())
 		{
-		      getGraphics()->getCaret()->disable();
+		      getGraphics()->allCarets()->disable();
 		      m_pView->m_countDisable++;
 		}
 		getGraphics()->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_SE);
@@ -1023,7 +816,9 @@ bool FV_FrameEdit::getFrameStrings(UT_sint32 x, UT_sint32 y,
 				   UT_String & sColYpos,
 				   UT_String & sPageXpos,
 				   UT_String & sPageYpos,
-				   fl_BlockLayout ** pCloseBL)
+				   UT_String & sPrefPage,
+				   fl_BlockLayout ** pCloseBL,
+				   fp_Page ** ppPage)
 {
 //
 // Find the block that contains (x,y). We'll insert the frame after
@@ -1043,14 +838,22 @@ bool FV_FrameEdit::getFrameStrings(UT_sint32 x, UT_sint32 y,
 		bool bDir=false;
 		m_pView->_findPositionCoords(posAtXY,bEOL,x1,y1,x2,y2,height,bDir,&pBL,&pRun);
 		UT_DEBUGMSG((" Requested y %d frameEdit y1= %d y2= %d \n",y,y1,y2));
+		fp_Run * pRunOrig = pRun;
 		if((pBL == NULL) || (pRun == NULL))
 		{
 			return false;
 		}
 		fl_BlockLayout * pPrevBL = pBL;
-		while(pBL && (pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_ENDNOTE) || (pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_FOOTNOTE) || (pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_TOC)|| (pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_FRAME) )
+		while(pBL && pBL->myContainingLayout() &&
+		((pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_ENDNOTE) ||
+		(pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_FOOTNOTE) ||
+		(pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_TOC) ||
+		(pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_FRAME) ||
+		(pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_CELL) ||
+		(pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_SHADOW) ||
+		(pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_HDRFTR)))
 		{
-		  UT_DEBUGMSG(("Skipping Block %x \n",pBL));
+		  UT_DEBUGMSG(("Skipping Block %p \n",pBL));
 		     pPrevBL = pBL;
 		     pBL = pBL->getPrevBlockInDocument();
 		}
@@ -1063,12 +866,12 @@ bool FV_FrameEdit::getFrameStrings(UT_sint32 x, UT_sint32 y,
 		{
 			return false;
 		}
-		UT_ASSERT((pBL->myContainingLayout()->getContainerType() != FL_CONTAINER_HDRFTR) 
+		UT_ASSERT(pBL->myContainingLayout() && (pBL->myContainingLayout()->getContainerType() != FL_CONTAINER_HDRFTR) 
 			  && (pBL->myContainingLayout()->getContainerType() != FL_CONTAINER_SHADOW));
 		*pCloseBL = pBL;
 		posAtXY = pBL->getPosition();
 
-		// don't left widths and heights be too big
+		// don't let widths and heights be too big
 
 		double dWidth = static_cast<double>(m_recCurFrame.width)/static_cast<double>(UT_LAYOUT_RESOLUTION);
 		double dHeight = static_cast<double>(m_recCurFrame.height)/static_cast<double>(UT_LAYOUT_RESOLUTION);
@@ -1085,7 +888,10 @@ bool FV_FrameEdit::getFrameStrings(UT_sint32 x, UT_sint32 y,
 //
 // Need this for offset to column
 //
-		fp_Container * pCol = pLine->getColumn();
+		UT_return_val_if_fail(pBL->getFirstRun(),false);
+		UT_return_val_if_fail(pBL->getFirstRun()->getLine(),false);
+		UT_return_val_if_fail(pBL->getFirstRun()->getLine()->getColumn(),false);
+		fp_Container * pCol = pRunOrig->getLine()->getColumn();
 		UT_ASSERT(pCol->getContainerType() == FP_CONTAINER_COLUMN);
 //
 // Find the screen coords of pCol and substract then from x,y
@@ -1103,7 +909,8 @@ bool FV_FrameEdit::getFrameStrings(UT_sint32 x, UT_sint32 y,
 		}
 		else if(finalColx + iColx + m_recCurFrame.width - xp > pPage->getWidth())
 		{
-			x -= finalColx + iColx + m_recCurFrame.width -xp - pPage->getWidth();
+		  //			x -= finalColx + iColx + m_recCurFrame.width -xp - pPage->getWidth();
+		  x = pPage->getWidth() - m_recCurFrame.width;
 		}
 		finalColx = x - iColx;
 
@@ -1114,7 +921,8 @@ bool FV_FrameEdit::getFrameStrings(UT_sint32 x, UT_sint32 y,
 		}
 		else if (finalColy + iColy - yp+  m_recCurFrame.height  > pPage->getHeight())
 		{
-			y -= finalColy + iColy -yp + m_recCurFrame.height - pPage->getHeight();
+		  //			y -= finalColy + iColy -yp + m_recCurFrame.height - pPage->getHeight();
+		        y = pPage->getHeight() - m_recCurFrame.height;
 		}
 		finalColy = y - iColy;
 
@@ -1174,9 +982,25 @@ bool FV_FrameEdit::getFrameStrings(UT_sint32 x, UT_sint32 y,
 		sYpos = UT_formatDimensionedValue(yPos,"in", NULL);
 		sWidth = UT_formatDimensionedValue(dWidth,"in", NULL);
 		sHeight = UT_formatDimensionedValue(dHeight,"in", NULL);
+		*ppPage = pPage;
+		UT_sint32 iPage = getView()->getLayout()->findPage(pPage);
+		UT_String_sprintf(sPrefPage,"%d",iPage);
 		return true;
 }
 
+//
+// Abort the current drag
+//
+//
+void FV_FrameEdit::abortDrag(void)
+{
+  UT_DEBUGMSG(("Doing Abort Drag \n"));
+  m_xLastMouse = m_iFirstEverX;
+  m_yLastMouse = m_iFirstEverY;
+  mouseRelease(m_iInitialDragX,m_iInitialDragY);
+  getView()->updateScreen(false);
+}
+
 void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 {
 //
@@ -1214,14 +1038,16 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		UT_String sPageYpos("");
 		UT_String sWidth("");
 		UT_String sHeight("");
+		UT_String sPrefPage("");
 		fl_BlockLayout * pCloseBL = NULL;
-		getFrameStrings(m_recCurFrame.left,m_recCurFrame.top,sXpos,sYpos,sWidth,sHeight,sColXpos,sColYpos,sPageXpos,sPageYpos,&pCloseBL);
+		fp_Page * pPage = NULL;
+		getFrameStrings(m_recCurFrame.left,m_recCurFrame.top,sXpos,sYpos,sWidth,sHeight,sColXpos,sColYpos,sPageXpos,sPageYpos,sPrefPage,&pCloseBL,&pPage);
 		pf_Frag_Strux * pfFrame = NULL;
 		// WARNING: Will need to change this to accomodate variable styles without constantly resetting to solid.
 		//				 Recommend to do whatever is done for thickness, which must also have a default set but not
 		//				 reverted to on every change.
 		// TODO: if(pAP->getProperty("*-thickness", somePropHolder)) sLeftThickness = gchar_strdup(somePropHolder); else sLeftThickness = "1px";
-		const gchar * props[38] = {"frame-type","textbox",
+		const gchar * props[40] = {"frame-type","textbox",
 					      "wrap-mode","wrapped-both",
 					      "position-to","column-above-text",
 					      "xpos",sXpos.c_str(),
@@ -1232,6 +1058,7 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 					      "frame-col-ypos",sColYpos.c_str(),
 					      "frame-page-xpos",sPageXpos.c_str(),
 					      "frame-page-ypos",sPageYpos.c_str(),
+					   "pref-page",sPrefPage.c_str(),
 					      "background-color", "ffffff",
 						  "left-style","1",
 						  "right-style","1",
@@ -1278,9 +1105,9 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 
 //
 		m_iFrameEditMode = 	FV_FrameEdit_EXISTING_SELECTED;
-		if(getGraphics() && getGraphics()->getCaret())
+		if(getGraphics())
 		{
-		      getGraphics()->getCaret()->disable();
+		      getGraphics()->allCarets()->disable();
 		      m_pView->m_countDisable++;
 		}
 		fl_BlockLayout * pBL = m_pView->_findBlockAtPosition(posFrame+2);
@@ -1318,7 +1145,11 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		if(haveDragged() < 10)
 		{
 			m_iFrameEditMode = FV_FrameEdit_NOT_ACTIVE;
-			m_iDraggingWhat = FV_FrameEdit_DragNothing;
+			setDragWhat( FV_DragNothing );
+			m_pFrameContainer->_setX(m_iInitialFrameX);
+			m_pFrameContainer->_setY(m_iInitialFrameY);
+			m_iInitialFrameX = 0;
+			m_iInitialFrameY = 0;
 			drawFrame(false);
 			m_pFrameLayout = NULL;
 			m_pFrameContainer = NULL;
@@ -1342,6 +1173,7 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 			while(m_iGlobCount > 0)
 				_endGlob();
 			m_pView->warpInsPtToXY(x,y,true);
+			UT_DEBUGMSG(("Completed small drag \n"));
 			return;
 		}
 //
@@ -1422,7 +1254,13 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		{
 			sVal = pszWrapMode;
 		}
-		UT_String_setProperty(sFrameProps,sProp,sVal);		
+		UT_String_setProperty(sFrameProps,sProp,sVal);
+
+// Relocate     
+		sProp = "relocate";
+		sVal = "0";
+		UT_String_setProperty(sFrameProps,sProp,sVal);
+
 
 // Xpadding
 
@@ -1590,6 +1428,19 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 			sVal = pszBackgroundColor;
 			UT_String_setProperty(sFrameProps,sProp,sVal);		
 		}
+		const char * szTitle = NULL;
+		const char * szDescription = NULL;
+		bool bFound = pSectionAP->getAttribute("title",szTitle);
+		if(!bFound)
+		{
+		    szTitle = "";
+		}
+		bFound = pSectionAP->getAttribute("alt",szDescription);
+		if(!bFound)
+		{
+		    szDescription = "";
+		}
+
 
 		UT_String sXpos("");
 		UT_String sYpos("");
@@ -1599,8 +1450,11 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		UT_String sColYpos("");
 		UT_String sPageXpos("");
 		UT_String sPageYpos("");
+		UT_String sPrefPage("");
 		fl_BlockLayout * pCloseBL = NULL;
-		getFrameStrings(m_recCurFrame.left,m_recCurFrame.top,sXpos,sYpos,sWidth,sHeight,sColXpos,sColYpos,sPageXpos,sPageYpos,&pCloseBL);
+		fp_Page * pPage = NULL;
+		getFrameStrings(m_recCurFrame.left,m_recCurFrame.top,sXpos,sYpos,sWidth,sHeight,sColXpos,sColYpos,sPageXpos,sPageYpos,sPrefPage,&pCloseBL,&pPage);
+
 		sProp = "xpos";
 		sVal = sXpos;
 		UT_String_setProperty(sFrameProps,sProp,sVal);		
@@ -1629,6 +1483,18 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		sProp = "frame-height";
 		sVal = sHeight;
 		UT_String_setProperty(sFrameProps,sProp,sVal);		
+		sProp = "pref-page";
+		sVal = sPrefPage;
+		UT_String_setProperty(sFrameProps,sProp,sVal);		
+		//
+		// Get all the blocks around the frame
+		//
+		UT_GenericVector<fl_ContainerLayout *> AllLayouts;
+		AllLayouts.clear();
+		UT_sint32 i = 0;
+		m_pFrameContainer->getPage()->getAllLayouts(AllLayouts);
+
+		fl_DocSectionLayout * pDSL = getFrameLayout()->getDocSectionLayout();
 
 		PT_DocPosition oldPoint = m_pView->getPoint();
 		PT_DocPosition oldFramePoint = m_pFrameLayout->getPosition(true);
@@ -1680,6 +1546,22 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		PP_AttrProp * p_AttrProp_Before = NULL;
 
 		getDoc()->deleteSpan(posStart, posEnd, p_AttrProp_Before, iRealDeleteCount,true);
+		//
+		// Now collapse all the blocks around it.
+		//
+		for(i=0; i< AllLayouts.getItemCount();i++)
+		{
+		     fl_ContainerLayout * pCL = AllLayouts.getNthItem(i);
+		     pCL->collapse();
+		}
+		//
+		// Rebuild the page
+		//
+		pDSL->format();
+		//
+		// Frame has gone, now find out where to put it.
+		//
+		getFrameStrings(m_recCurFrame.left,m_recCurFrame.top,sXpos,sYpos,sWidth,sHeight,sColXpos,sColYpos,sPageXpos,sPageYpos,sPrefPage,&pCloseBL,&pPage);
 
 		m_pFrameLayout = NULL;
 		posAtXY = pCloseBL->getPosition();
@@ -1699,12 +1581,16 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		}
 		else
 		{
-			atts = new const gchar * [5];
+			atts = new const gchar * [9];
 			atts[0] = PT_STRUX_IMAGE_DATAID;
 			atts[1] = pszDataID;
 			atts[2] = "props";
 			atts[3] = sFrameProps.c_str();
-			atts[4] =  NULL;
+			atts[4]  = PT_IMAGE_TITLE;
+			atts[5] = szTitle;
+			atts[6] = PT_IMAGE_DESCRIPTION;
+			atts[7] = szDescription;
+			atts[8] =  NULL;
 		}
 		pf_Frag_Strux * pfFrame = NULL;
 		getDoc()->insertStrux(posAtXY,PTX_SectionFrame,atts,NULL,&pfFrame);
@@ -1782,7 +1668,7 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		}
 		else if((oldPoint >= oldFramePoint) && (oldPoint >= newFramePoint))
 		{
-			newPoint = oldPoint  - oldFrameLen + newFrameLen;;
+			newPoint = oldPoint  - oldFrameLen + newFrameLen;
 		}
 		else // oldPoint >= oldFramePoint && (oldPoint < newFramePoint)
 		{
@@ -1812,9 +1698,9 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		DELETEP(m_pFrameImage);
 		m_pView->updateScreen(false);
 		m_iFrameEditMode = FV_FrameEdit_EXISTING_SELECTED;
-		if(getGraphics() && getGraphics()->getCaret())
+		if(getGraphics())
 		{
-		      getGraphics()->getCaret()->disable();
+		      getGraphics()->allCarets()->disable();
 		      m_pView->m_countDisable++;
 		}
 		if(m_pFrameLayout)
@@ -1825,6 +1711,26 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 	m_bFirstDragDone = false;
 }
 
+/*
+ * Return the bytebuf of the image for this.
+ */
+const char * FV_FrameEdit::getPNGImage(const UT_ByteBuf ** ppByteBuf )
+{
+
+//  Frame Image
+      const PP_AttrProp* pSectionAP = NULL;
+      m_pFrameLayout->getAP(pSectionAP);
+      
+      const char * pszDataID = NULL;
+      pSectionAP->getAttribute(PT_STRUX_IMAGE_DATAID, (const gchar *&)pszDataID);
+      if(!pszDataID)
+      {
+	  *ppByteBuf = NULL;
+	  return NULL;
+      }
+      m_pView->getDocument()->getDataItemDataByName(pszDataID,ppByteBuf,NULL,NULL);
+      return pszDataID;
+}
 
 /*!
  * This method deletes the current selected frame
@@ -1895,7 +1801,7 @@ void FV_FrameEdit::deleteFrame(fl_FrameLayout * pFL)
 	DELETEP(m_pFrameImage);
 	m_recCurFrame.width = 0;
 	m_recCurFrame.height = 0;
-	m_iDraggingWhat = FV_FrameEdit_DragNothing;
+	setDragWhat( FV_DragNothing );
 	m_iLastX = 0;
 	m_iLastY = 0;
 
@@ -1907,9 +1813,9 @@ void FV_FrameEdit::deleteFrame(fl_FrameLayout * pFL)
 /*!
  * Method to deal with mouse coordinates.
  */
-FV_FrameEditDragWhat FV_FrameEdit::mouseMotion(UT_sint32 x, UT_sint32 y)
+FV_DragWhat FV_FrameEdit::mouseMotion(UT_sint32 /*x*/, UT_sint32 /*y*/)
 {
-	return getFrameEditDragWhat();
+	return getDragWhat();
 }
 
 void FV_FrameEdit::drawFrame(bool bWithHandles)
@@ -1927,7 +1833,7 @@ void FV_FrameEdit::drawFrame(bool bWithHandles)
 	m_pView->getPageScreenOffsets(pPage,xPage,yPage);
 	da.xoff = xPage + m_pFrameContainer->getX();
 	da.yoff = yPage + m_pFrameContainer->getY();
-	if((m_pFrameImage == NULL) || (m_iDraggingWhat != FV_FrameEdit_DragWholeFrame) )
+	if((m_pFrameImage == NULL) || (getDragWhat() != FV_DragWhole) )
 	{
 //		m_pFrameContainer->clearScreen();
 		m_pFrameContainer->draw(&da);
@@ -1935,7 +1841,7 @@ void FV_FrameEdit::drawFrame(bool bWithHandles)
 		{
 			m_pFrameContainer->drawHandles(&da);
 		}
-		if(m_iDraggingWhat == FV_FrameEdit_DragWholeFrame)
+		if(getDragWhat() == FV_DragWhole)
 		{
 			GR_Painter painter (getGraphics());
 			if(m_pFrameLayout->getFrameType() == FL_FRAME_TEXTBOX_TYPE)
