@@ -170,7 +170,6 @@ void FV_VisualInlineImage::_autoScroll(UT_Worker * pWorker)
 	if(bScrollRunning)
 	{
 	    UT_DEBUGMSG(("Dropping InlineImage autoscroll !!!!!!! \n"));
-	    FV_View * pView = pVis->m_pView;
 	    if(iExtra < pVis->getGraphics()->tlu(600))
 	      iExtra += pVis->getGraphics()->tlu(20);
 	    return;
@@ -671,9 +670,9 @@ void FV_VisualInlineImage::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 				m_iLastX = x;
 				m_iLastY = y;
 			}
-			if(getGraphics() && getGraphics()->getCaret())
+			if(getGraphics())
 			{
-			        getGraphics()->getCaret()->disable();
+			        getGraphics()->allCarets()->disable();
 			        m_pView->m_countDisable++;
 			}
 		}
