@@ -21,29 +21,30 @@
 #ifndef FV_VIEW_H
 #define FV_VIEW_H
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "xap_Features.h"
-
-//#include "ut_misc.h"
 #include "ut_types.h"
-
 #include "xav_View.h"
-
 #include "pt_Types.h"
-#include "ie_types.h"
-#include "ap_types.h"
 #include "fp_types.h"
+#ifdef ENABLE_SPELL
 #include "fl_Squiggles.h"
+#endif
 #include "ev_EditBits.h"
 
-
 // have to include these as they are instantiated in the FV_View
 // class definition
 #include "fv_FrameEdit.h"
 #include "fv_Selection.h"
 #include "fv_InlineImage.h"
 
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 #include "fv_UnixVisualDrag.h"
+#include "fv_UnixFrameEdit.h"
+#include "fv_UnixInlineImage.h"
 #else
 #include "fv_VisualDragText.h"
 #endif
@@ -94,6 +95,25 @@ class SpellChecker;
 
 class CellLine;
 
+typedef enum
+{
+	hori_left,
+	hori_mid,
+	hori_right,
+	vert_above,
+	vert_mid,
+	vert_below
+} AP_CellSplitType;
+
+
+typedef enum _AP_JumpTarget
+{
+	AP_JUMPTARGET_PAGE, 			// beginning of page
+	AP_JUMPTARGET_LINE,
+	AP_JUMPTARGET_BOOKMARK,
+	AP_JUMPTARGET_PICTURE // TODO
+} AP_JumpTarget;
+
 struct fv_ChangeState
 {
 	bool				bUndo;
@@ -147,22 +167,23 @@ class ABI_EXPORT fv_CaretProps
 public:
         fv_CaretProps(FV_View * pView, PT_DocPosition InsPoint);
 	virtual ~fv_CaretProps(void);
-	PT_DocPosition		        m_iInsPoint;
+	PT_DocPosition		m_iInsPoint;
 	UT_sint32			m_xPoint;
 	UT_sint32			m_yPoint;
 	UT_sint32			m_xPoint2;
 	UT_sint32			m_yPoint2;
-	bool			        m_bPointDirection;
+	bool				m_bPointDirection;
 	bool				m_bDefaultDirectionRtl;
 	bool				m_bUseHebrewContextGlyphs;
-	bool                            m_bPointEOL;
+	bool				m_bPointEOL;
 	UT_uint32			m_iPointHeight;
-	UT_RGBColor                     m_caretColor;
-	UT_UTF8String                   m_sDocUUID;
-	FV_Caret_Listener *             m_PropCaretListner;
-	GR_Caret *                      m_pCaret;
-	UT_uint32                       m_ListenerID;
-	FV_View *                       m_pView;
+	UT_RGBColor			m_caretColor;
+	FV_Caret_Listener*  m_PropCaretListner;
+	GR_Caret*			m_pCaret;
+	UT_uint32			m_ListenerID;
+	FV_View*			m_pView;
+	UT_sint32			m_iAuthorId;
+	std::string			m_sCaretID;
 };
 								
 class ABI_EXPORT FV_View : public AV_View
@@ -189,8 +210,8 @@ public:
 
 	virtual inline PT_DocPosition   getPoint(void) const { return m_iInsPoint; }
 	PT_DocPosition	getSelectionAnchor(void) const;
-	PT_DocPosition	getSelectionLeftAnchor(void);
-	PT_DocPosition	getSelectionRightAnchor(void) ;
+	PT_DocPosition	getSelectionLeftAnchor(void) const;
+	PT_DocPosition	getSelectionRightAnchor(void) const;
 	UT_uint32       getSelectionLength(void) const;
 
 	UT_sint32       getFrameMargin(void) const;
@@ -205,14 +226,14 @@ public:
 	virtual void	cmdHyperlinkJump(UT_sint32 xPos, UT_sint32 yPos);
 	void	        cmdHyperlinkJump(PT_DocPosition pos);
 	void			cmdHyperlinkCopyLocation(PT_DocPosition pos);
-
+	
 	virtual void	draw(const UT_Rect* pRect=static_cast<UT_Rect*>(NULL));
 	virtual void 	drawSelectionBox(UT_Rect & box, bool drawHandles);
 private: 
 	inline void 	_drawResizeHandle(UT_Rect & box);
 	
 public:
-	const PP_AttrProp * getAttrPropForPoint();
+	const PP_AttrProp * getAttrPropForPoint() const;
 
 	virtual bool	notifyListeners(const AV_ChangeMask hint);
 
@@ -270,32 +291,38 @@ public:
 	void                getMousePos(UT_sint32 * x, UT_sint32 * y);
 
 	virtual EV_EditMouseContext getMouseContext(UT_sint32 xPos, UT_sint32 yPos);
+	EV_EditMouseContext _getMouseContext(UT_sint32 xPos, UT_sint32 yPos);
 	virtual EV_EditMouseContext getInsertionPointContext(UT_sint32 * pxPos, UT_sint32 * pyPos);
+	void                setPrevMouseContext(EV_EditMouseContext  emc)
+	{m_prevMouseContext = emc;}
+
 	virtual void        updateLayout(void);
 	virtual void        rebuildLayout(void);
 	virtual void        remeasureCharsWithoutRebuild();
 	virtual void        fontMetricsChange();
 	virtual bool		isSelectionEmpty(void) const;
-	bool                isSelectAll(void)
+	bool                isSelectAll(void) const
 	{ return m_Selection.isSelectAll();}
 	virtual void		cmdUnselectSelection(void);
-	void				getDocumentRangeOfCurrentSelection(PD_DocumentRange * pdr);
+	void				getDocumentRangeOfCurrentSelection(PD_DocumentRange * pdr) const;
 	PT_DocPosition		mapDocPos( FV_DocPos dp );
 	PT_DocPosition		mapDocPosSimple( FV_DocPos dp );
 	PT_DocPosition saveSelectedImage (const char * toFile );
 	PT_DocPosition saveSelectedImage (const UT_ByteBuf ** outByteBuf);
-	PT_DocPosition getSelectedImage(const char **dataId);
-	fp_Run *getSelectedObject(void);
+	PT_DocPosition getSelectedImage(const char **dataId) const;
+	fp_Run *getSelectedObject(void) const;
 
-	void            getTextInCurrentBlock(UT_GrowBuf & buf);
-	void            getTextInCurrentSection(UT_GrowBuf & buf);
-	void            getTextInDocument(UT_GrowBuf & buf);
+	void            getTextInCurrentBlock(UT_GrowBuf & buf) const;
+	void            getTextInCurrentSection(UT_GrowBuf & buf) const;
+	void            getTextInDocument(UT_GrowBuf & buf) const;
+	bool            getLineBounds(PT_DocPosition pos, PT_DocPosition *start, PT_DocPosition *end);
+	UT_UCSChar getChar(PT_DocPosition pos, UT_sint32 *x = NULL, UT_sint32 *y = NULL, UT_uint32 *width = NULL, UT_uint32 *height = NULL);
 
 // ----------------------
 	FL_DocLayout*	getLayout() const;
 	UT_uint32		getCurrentPageNumForStatusBar(void) const;
 	fp_Page*		getCurrentPage(void) const;
-	fl_BlockLayout* getCurrentBlock(void);
+	fl_BlockLayout* getCurrentBlock(void) const;
 
 	void draw(int page, dg_DrawArgs* da);
 
@@ -308,29 +335,29 @@ public:
 	virtual UT_sint32 getPageViewSep(void) const;
 
 	bool	setSectionFormat(const gchar * properties[]);
-	bool	getSectionFormat(const gchar *** properties);
+	bool	getSectionFormat(const gchar *** properties) const;
 
 	bool	setBlockIndents(bool doLists, double indentChange, double page_size);
 	bool    setCollapsedRange(PT_DocPosition posLow,PT_DocPosition posHigh, const gchar * properties[]);
 	bool	setBlockFormat(const gchar * properties[]);
-	bool	getBlockFormat(const gchar *** properties,bool bExpandStyles=true);
+	bool	getBlockFormat(const gchar *** properties,bool bExpandStyles=true) const;
 	bool    removeStruxAttrProps(PT_DocPosition ipos1, PT_DocPosition ipos2, PTStruxType iStrux,const gchar * attributes[] ,const gchar * properties[]);
 	bool    isImageAtStrux(PT_DocPosition ipos1, PTStruxType iStrux);
 
 	bool	processPageNumber(HdrFtrType hfType, const gchar ** atts);
 
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 	bool	isTextMisspelled()const ;
 #endif
-	bool	isTabListBehindPoint(UT_sint32 & iNumToDelete);
-	bool	isTabListAheadPoint(void);
+	bool	isTabListBehindPoint(UT_sint32 & iNumToDelete) const;
+	bool	isTabListAheadPoint(void) const;
 	void	processSelectedBlocks(FL_ListType listType);
-	void	getBlocksInSelection(UT_GenericVector<fl_BlockLayout*> * vBlock);
-	UT_sint32 getNumColumnsInSelection(void);
-	UT_sint32 getNumRowsInSelection(void);
-	void	getAllBlocksInList(UT_GenericVector<fl_BlockLayout *> * vBlock);
-	bool	isPointBeforeListLabel(void);
-	bool	isCurrentListBlockEmpty(void);
+	void	getBlocksInSelection(UT_GenericVector<fl_BlockLayout*> * vBlock) const;
+	UT_sint32 getNumColumnsInSelection(void) const;
+	UT_sint32 getNumRowsInSelection(void) const;
+	void	getAllBlocksInList(UT_GenericVector<fl_BlockLayout *> * vBlock) const;
+	bool	isPointBeforeListLabel(void) const;
+	bool	isCurrentListBlockEmpty(void) const;
 	bool	cmdStartList(const gchar * style);
 	bool	cmdStopList(void);
 	void	changeListStyle(fl_AutoNum* pAuto,
@@ -345,7 +372,7 @@ public:
 	void	setDontChangeInsPoint(void);
 	void	allowChangeInsPoint(void);
 
-	bool    getAttributes(const PP_AttrProp ** ppSpanAP, const PP_AttrProp ** ppBlockAP = NULL, PT_DocPosition posStart = 0);
+	bool    getAttributes(const PP_AttrProp ** ppSpanAP, const PP_AttrProp ** ppBlockAP = NULL, PT_DocPosition posStart = 0) const;
 
 	/* Experimental, for the moment; use with caution. - fjf, 24th Oct. '04
 	 */
@@ -357,32 +384,32 @@ public:
 
 	bool	setCharFormat(const gchar * properties[], const gchar * attribs[] = NULL);
 	bool	resetCharFormat(bool bAll);
-	bool	getCharFormat(const gchar *** properties,bool bExpandStyles=true);
-	bool	getCharFormat(const gchar *** properties,bool bExpandStyles, PT_DocPosition posStart);
+	bool	getCharFormat(const gchar *** properties,bool bExpandStyles=true) const;
+	bool	getCharFormat(const gchar *** properties,bool bExpandStyles, PT_DocPosition posStart) const;
 	fl_BlockLayout * getBlockFromSDH(PL_StruxDocHandle sdh);
 	bool	setStyle(const gchar * style, bool bDontGeneralUpdate=false);
 	bool	setStyleAtPos(const gchar * style, PT_DocPosition posStart, PT_DocPosition posEnd, bool bDontGeneralUpdate=false);
-	bool    isNumberedHeadingHere(fl_BlockLayout * pBlock);
-	bool	getStyle(const gchar ** style);
+	bool    isNumberedHeadingHere(fl_BlockLayout * pBlock) const;
+	bool	getStyle(const gchar ** style) const;
 	bool appendStyle(const gchar ** style);
 
-	UT_uint32		getCurrentPageNumber(void);
+	UT_uint32		getCurrentPageNumber(void) const;
 
 	bool	getEditableBounds(bool bEnd, PT_DocPosition & docPos, bool bOverride=false)const;
 
-	bool    isParaBreakNeededAtPos(PT_DocPosition pos);
+	bool    isParaBreakNeededAtPos(PT_DocPosition pos) const;
 	bool    insertParaBreakIfNeededAtPos(PT_DocPosition pos);
 	void	insertParagraphBreak(void);
 	void	insertParagraphBreaknoListUpdate(void);
 	void	insertSectionBreak( BreakSectionType type);
 	void	insertSectionBreak(void);
-	void	insertSymbol(UT_UCSChar c, gchar * symfont);
+	void	insertSymbol(UT_UCSChar c, const gchar * symfont);
 
 	// ----------------------
-	bool			isLeftMargin(UT_sint32 xPos, UT_sint32 yPos);
+	bool			isLeftMargin(UT_sint32 xPos, UT_sint32 yPos) const;
 	void			cmdSelect(UT_sint32 xPos, UT_sint32 yPos, FV_DocPos dpBeg, FV_DocPos dpEnd);
 	void			cmdSelectTOC(UT_sint32 xPos, UT_sint32 yPos);
-	bool            isTOCSelected(void);
+	bool            isTOCSelected(void) const;
 	bool            setTOCProps(PT_DocPosition pos, const char * szProps);
 
 	void			cmdSelect(PT_DocPosition dpBeg, PT_DocPosition dpEnd);
@@ -390,9 +417,9 @@ public:
 	bool			cmdCharInsert(const UT_UCSChar * text, UT_uint32 count, bool bForce = false);
 	void			cmdCharDelete(bool bForward, UT_uint32 count);
 	void			delTo(FV_DocPos dp);
-	void            getSelectionText(UT_UCS4Char *& text);
+	void            getSelectionText(UT_UCS4Char *& text) const;
 
-	UT_UCSChar *	getTextBetweenPos(PT_DocPosition pos1, PT_DocPosition pos2);
+	UT_UCSChar *	getTextBetweenPos(PT_DocPosition pos1, PT_DocPosition pos2) const;
 	inline PT_DocPosition  getInsPoint () const { return m_iInsPoint; }
 	void			warpInsPtToXY(UT_sint32 xPos, UT_sint32 yPos, bool bClick);
 	void			moveInsPtTo(FV_DocPos dp, bool bClearSelection = true);
@@ -405,8 +432,8 @@ public:
 	void			extSelToXYword(UT_sint32 xPos, UT_sint32 yPos, bool bDrag);
 	void			extSelTo(FV_DocPos dp);
 
-#ifndef WITHOUT_SPELL
-	SpellChecker * getDictForSelection ();
+#ifdef ENABLE_SPELL
+	SpellChecker * getDictForSelection () const;
 #endif
 	void			extSelNextPrevLine(bool bNext);
 	void            extSelNextPrevPage(bool bNext);
@@ -420,7 +447,7 @@ public:
 
 	fl_BlockLayout* getBlockAtPosition(PT_DocPosition pos) const {return _findBlockAtPosition(pos);};
 	virtual void	updateScreen(bool bDirtyRunsOnly=true);
-	bool            isInDocSection(PT_DocPosition pos = 0);
+	bool            isInDocSection(PT_DocPosition pos = 0) const;
 
 //---------
 //Visual Drag stuff
@@ -430,9 +457,11 @@ public:
 	void            dragVisualText(UT_sint32 x, UT_sint32 y);
 	void            pasteVisualText(UT_sint32 x, UT_sint32 y);
 	void            btn0VisualDrag(UT_sint32 x, UT_sint32 y);
+	const FV_VisualDragText * getVisualText(void) const
+	  { return &m_VisualDragText;}
 	FV_VisualDragText * getVisualText(void)
 	  { return &m_VisualDragText;}
-
+	const UT_ByteBuf * getLocalBuf(void) const;
 
 //---------
 //Visual Inline Image Drag stuff
@@ -464,6 +493,8 @@ public:
 											const gchar ** attribs);
 
 	bool            convertPositionedToInLine(fl_FrameLayout * pFrame);
+	UT_Error        cmdInsertPositionedGraphic(FG_Graphic* pFG,UT_sint32 mouseX, UT_sint32 mouseY);
+	UT_Error        cmdInsertPositionedGraphic(FG_Graphic* pFG);
 
 // ----------------------
 
@@ -471,10 +502,10 @@ public:
 	bool			isXYSelected(UT_sint32 xPos, UT_sint32 yPos) const;
 	FV_SelectionMode getSelectionMode(void) const;
 	FV_SelectionMode getPrevSelectionMode(void) const;
-	PD_DocumentRange * getNthSelection(UT_sint32 i);
+	PD_DocumentRange * getNthSelection(UT_sint32 i) const;
 	UT_sint32          getNumSelections(void) const;
 	void            setSelectionMode(FV_SelectionMode selMode);
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 // ----------------------
 // Stuff for spellcheck context menu
 //
@@ -523,9 +554,39 @@ public:
 	bool                isInFootnote(void);
 	bool                isInEndnote(PT_DocPosition pos);
 	bool                isInEndnote(void);
+	bool                isInAnnotation(PT_DocPosition pos);
+	bool                isInAnnotation(void);
 	fl_FootnoteLayout * getClosestFootnote(PT_DocPosition pos);
 	fl_EndnoteLayout *  getClosestEndnote(PT_DocPosition pos);
+	fl_AnnotationLayout *  getClosestAnnotation(PT_DocPosition pos);
 	UT_sint32           getEmbedDepth(PT_DocPosition pos);
+	//
+	// ----------------------------------
+	// Stuff for Annotaions
+	//	
+	bool				insertAnnotation(UT_sint32 iAnnotation,
+										 UT_UTF8String * sDescr,
+										 UT_UTF8String * sAuthor,
+										 UT_UTF8String * sTitle,
+										 bool bReplace);
+	bool                getAnnotationText(UT_uint32 iAnnotaion, UT_UTF8String & sText);
+	bool                setAnnotationText(UT_uint32 iAnnotaion, UT_UTF8String & sText);
+	bool                setAnnotationText(UT_uint32 iAnnotaion, UT_UTF8String & sText,UT_UTF8String & sAuthor, UT_UTF8String & sTitle);
+	bool                getAnnotationRichText(UT_uint32 iAnnotation, UT_UTF8String & sRTF);
+    bool                setAnnotationRichText(UT_uint32 iAnnotaion, UT_UTF8String &sRTF);
+	// TODO getters and setters to implement/change/add as judged necessary
+	bool                getAnnotationTitle(UT_uint32 iAnnotaion, UT_UTF8String & sTitle);
+	bool                setAnnotationTitle(UT_uint32 iAnnotaion, UT_UTF8String & sTitle);
+	bool                getAnnotationAuthor(UT_uint32 iAnnotaion, UT_UTF8String & sAuthor);
+	bool                setAnnotationAuthor(UT_uint32 iAnnotaion, UT_UTF8String & sAuthor);
+
+	bool                isAnnotationPreviewActive(void) { return m_bAnnotationPreviewActive;}
+	void                setAnnotationPreviewActive(bool b) { m_bAnnotationPreviewActive = b;}
+	UT_uint32			getActivePreviewAnnotationID() { return m_iAnnPviewID;}
+	void				setActivePreviewAnnotationID(UT_uint32 iID) { m_iAnnPviewID = iID;}
+	void				killAnnotationPreview();
+	bool				cmdEditAnnotationWithDialog(UT_uint32 aID);
+	fl_AnnotationLayout * getAnnotationLayout(UT_uint32 iAnnotation);
 // ----------------------
 
 	bool		gotoTarget(AP_JumpTarget type, UT_UCSChar * data);
@@ -549,6 +610,7 @@ public:
 
 	bool			findAgain(void);
 
+	void			findSetStartAt(PT_DocPosition pos);
 	void			findSetStartAtInsPoint(void);
 
 	bool			findNext(bool& bDoneEntireDocument);
@@ -595,10 +657,11 @@ public:
 	bool				insertPageNum(const gchar ** props, HdrFtrType hfType);
 	virtual void        setPoint(UT_uint32 pt);
 	void                ensureInsertionPointOnScreen(void);
-        void                removeCaret(UT_UTF8String & sDocUUID);
-	void                addCaret(PT_DocPosition docPos,UT_UTF8String & sDocUUID);
+    void                removeCaret(const std::string& sCaretID);
+	void                addCaret(PT_DocPosition docPos,UT_sint32 iAuthorId);
 	void                setPointRemote(PT_DocPosition docPos);
-	void                updateCarets(PT_DocPosition docPos, UT_sint32 iLen);	void		    fixInsertionPointCoords(void);
+	void                updateCarets(PT_DocPosition docPos, UT_sint32 iLen);   
+	void		    fixInsertionPointCoords(void);
 
 // -----------------------
 	void                killBlink(void);
@@ -606,11 +669,11 @@ public:
 	inline bool 	getShowPara(void) const { return m_bShowPara; };
 
 	const fp_PageSize&	getPageSize(void) const;
-	UT_uint32			calculateZoomPercentForPageWidth();
-	UT_uint32			calculateZoomPercentForPageHeight();
-	UT_uint32			calculateZoomPercentForWholePage();
+	virtual UT_uint32	calculateZoomPercentForPageWidth() const;
+	virtual UT_uint32	calculateZoomPercentForPageHeight() const;
+	virtual UT_uint32	calculateZoomPercentForWholePage() const;
 	void 			    setViewMode (ViewMode vm);
-	inline ViewMode 		getViewMode (void) const  {return m_viewMode;}
+	ViewMode 			getViewMode (void) const  {return m_viewMode;}
 	bool				isPreview(void) const {return VIEW_PREVIEW == m_viewMode;}
 	void				setPreviewMode(PreViewMode pre) {m_previewMode = pre;}
 	PreViewMode 		getPreviewMode(void) { return m_previewMode;}
@@ -651,15 +714,15 @@ public:
 	/* Table related functions */
 	bool                isPointLegal(PT_DocPosition pos);
 	bool                isPointLegal(void);
-	bool				isInTable();
-	fl_TableLayout *    getTableAtPos(PT_DocPosition);
-	bool				isInTable(PT_DocPosition pos);
-	bool				isInTableForSure(PT_DocPosition pos);
+	bool				isInTable() const;
+	fl_TableLayout *    getTableAtPos(PT_DocPosition) const;
+	bool				isInTable(PT_DocPosition pos) const;
+	bool				isInTableForSure(PT_DocPosition pos) const;
 	bool                cmdAutoSizeCols(void);
 	bool                cmdTextToTable(bool bIgnoreSpaces);
 	bool                cmdAutoSizeRows(void);
 	bool                cmdAdvanceNextPrevCell(bool bGoNext);
-	fp_CellContainer *  getCellAtPos(PT_DocPosition pos);
+	fp_CellContainer *  getCellAtPos(PT_DocPosition pos) const;
 	PT_DocPosition      findCellPosAt(PT_DocPosition posTable, UT_sint32 row, UT_sint32 col);
 	bool                _deleteCellAt(PT_DocPosition posTable,UT_sint32 row, UT_sint32 col);
 	bool                _restoreCellParams(PT_DocPosition posTable, UT_sint32 iLineWidth);
@@ -679,14 +742,14 @@ public:
 
 	bool                _MergeCells( PT_DocPosition posDestination,PT_DocPosition posSource, bool bBefore);
 	bool                getCellParams(PT_DocPosition posCol, UT_sint32 *iLeft, 
-									  UT_sint32 *iRight,UT_sint32 *iTop, UT_sint32 *iBot);
+									  UT_sint32 *iRight,UT_sint32 *iTop, UT_sint32 *iBot) const;
 	bool				getCellLineStyle(PT_DocPosition posCell, UT_sint32 * pLeft, UT_sint32 * pRight,
-										 UT_sint32 * pTop, UT_sint32 * pBot);
+										 UT_sint32 * pTop, UT_sint32 * pBot) const;
 	bool				setCellFormat(const gchar * properties[], FormatTable applyTo, FG_Graphic * pFG, UT_String & sDataID);
-	bool				getCellProperty(gchar * szPropName, gchar * &szPropValue);
+	bool				getCellProperty(const gchar * szPropName, gchar * &szPropValue) const;
 	bool	            setTableFormat(const gchar * properties[]);
 	bool	            setTableFormat(PT_DocPosition pos,const gchar * properties[]);
-	bool                getCellFormat(PT_DocPosition pos, UT_String & sCellProps);
+	bool                getCellFormat(PT_DocPosition pos, UT_String & sCellProps) const;
 
 	UT_Error            cmdInsertTable(UT_sint32 numRows, UT_sint32 numCols, 
 									   const gchar * pPropsArray[]);
@@ -705,17 +768,20 @@ public:
 	void				_generalUpdate(void);
 	
 	UT_RGBColor			getColorShowPara(void) const { return m_colorShowPara; }
+#ifdef ENABLE_SPELL
 	UT_RGBColor			getColorSquiggle(FL_SQUIGGLE_TYPE iSquiggleType) const;
+#endif
 	UT_RGBColor			getColorMargin(void) const { return m_colorMargin; }
 	UT_RGBColor			getColorSelBackground(void);
-	UT_RGBColor			getColorSelForeground(void);
+	UT_RGBColor			getColorSelForeground(void) const;
 	UT_RGBColor			getColorFieldOffset(void) const { return m_colorFieldOffset; }
 	UT_RGBColor			getColorImage(void) const { return m_colorImage; }
 	UT_RGBColor			getColorImageResize(void) const { return m_colorImageResize; }
 	UT_RGBColor			getColorHyperLink(void) const { return m_colorHyperLink; }
+	UT_RGBColor			getColorAnnotation(const fp_Run * pRun) const; 
+	UT_RGBColor			getColorAnnotation(fp_Page * pPage,UT_uint32 pid) const; 
 	UT_RGBColor			getColorRevisions(int rev) const { 
-		if (rev < 0) rev = 9;
-		if (rev > 9) rev = 9;
+		if ((rev < 0) || (rev > 9)) rev = 9;
 		return m_colorRevisions[rev]; }
 	UT_RGBColor			getColorHdrFtr(void) const { return m_colorHdrFtr; }
 	UT_RGBColor			getColorColumnLine(void) const { return m_colorColumnLine; }
@@ -726,27 +792,9 @@ public:
 	//
 	// image selection && resizing && dragging functions
 	//
-	void				setImageSelRect(UT_Rect r);
-	UT_Rect				getImageSelRect();
-	UT_sint32			getImageSelInfo();
-	GR_Graphics::Cursor getImageSelCursor();
-	void				setCurImageSel(UT_Rect r);
-	UT_Rect				getCurImageSel();
-#if XAP_DONTUSE_XOR
-	void				setCurImageSelCache(GR_Image* cache);
-	GR_Image* 			getCurImageSelCache();
-#endif
-	bool				isOverImageResizeBox(GR_Graphics::Cursor &cur, UT_uint32 xPos, UT_uint32 yPos);
-	void				startImageResizing(UT_sint32 xPos, UT_sint32 yPos);
-	void				stopImageResizing();
-	bool				isResizingImage();
-	void				getResizeOrigin(UT_sint32 &xOrigin, UT_sint32 &yOrigin);
-	bool				isDraggingImage();
-	void				startImageDrag(fp_Run * pRun, UT_sint32 xPos, UT_sint32 yPos);
-	void				drawDraggedImage(UT_sint32 xPos, UT_sint32 yPos);
-	void				stopImageDrag(UT_sint32 xPos, UT_sint32 yPos);
-
-	bool                isImageSelected(void);
+	UT_sint32			getImageSelInfo() const;
+	GR_Graphics::Cursor getImageSelCursor() const;
+	bool                isImageSelected(void) const;
 
 //
 // Table resizing
@@ -765,20 +813,28 @@ public:
 		                { return m_pLeftRuler;}
 	
 
-	const gchar **   getViewPersistentProps();
+	const gchar **   getViewPersistentProps() const;
 	FV_BIDI_Order	    getBidiOrder()const {return m_eBidiOrder;}
 	void                setBidiOrder(FV_BIDI_Order o) {m_eBidiOrder = o;}
 
-	bool                isMathSelected(UT_sint32 x, UT_sint32 y, PT_DocPosition & pos);
+	bool                isMathSelected(UT_sint32 x, UT_sint32 y, PT_DocPosition & pos) const;
 	// -- plugins
-        bool                isMathLoaded(void);
-	bool                isGrammarLoaded(void);
+        bool                isMathLoaded(void) const;
+	bool                isGrammarLoaded(void) const;
 	// --
+	
+	UT_uint32			getNumHorizPages(void) const; //////////////////////////////////
+	void				calculateNumHorizPages(void);
+	UT_uint32			getMaxHeight(UT_uint32 iRow) const;
+	UT_uint32			getWidthPrevPagesInRow(UT_uint32 iPageNumber) const;
+	UT_uint32			getWidthPagesInRow(fp_Page *page) const;
+	UT_uint32			getHorizPageSpacing(void) const;
+	bool				rtlPages(void) const;
+	
 protected:
 	void				_saveAndNotifyPieceTableChange(void);
 	void				_restorePieceTableState(void);
 	
-
 	void				_draw(UT_sint32, UT_sint32, UT_sint32, UT_sint32, bool bDirtyRunsOnly, bool bClip=false);
 
 	void				_drawBetweenPositions(PT_DocPosition left, PT_DocPosition right);
@@ -788,13 +844,13 @@ protected:
 	void				_moveInsPtNextPrevPage(bool bNext);
 	void				_moveInsPtNextPrevScreen(bool bNext);
 	void				_moveInsPtNextPrevLine(bool bNext);
-	fp_Page *			_getCurrentPage(void);
-	void				_moveInsPtNthPage(UT_uint32 n);
+	fp_Page *			_getCurrentPage(void) const;
+	void				_moveInsPtNthPage(UT_sint32 n);
 	void				_moveInsPtToPage(fp_Page *page);
 	void				_insertSectionBreak(void);
 
-	PT_DocPosition		_getDocPosFromPoint(PT_DocPosition iPoint, FV_DocPos dp, bool bKeepLooking=true);
-	PT_DocPosition		_getDocPos(FV_DocPos dp, bool bKeepLooking=true);
+	PT_DocPosition		_getDocPosFromPoint(PT_DocPosition iPoint, FV_DocPos dp, bool bKeepLooking=true) const;
+	PT_DocPosition		_getDocPos(FV_DocPos dp, bool bKeepLooking=true) const;
 	void				_findPositionCoords(PT_DocPosition pos,
 											bool b,
 											UT_sint32& x,
@@ -827,32 +883,34 @@ protected:
 	bool				_insertFormatPair(const gchar * szName, const gchar * properties[]);
 	void				_updateInsertionPoint();
 	void				_fixInsertionPointCoords(bool bIgnoreAll = false);
-	void				_fixInsertionPointCoords(fv_CaretProps * pCP);
-	void				_fixAllInsertionPointCoords(void);
+	void				_fixInsertionPointCoords(fv_CaretProps * pCP) const;
+	void				_fixAllInsertionPointCoords(void) const;
 
 	void				_drawSelection();
 	void				_swapSelectionOrientation(void);
 	void				_extSel(UT_uint32 iOldPoint);
 	void				_extSelToPos(PT_DocPosition pos);
 	UT_Error			_insertGraphic(FG_Graphic*, const char*);
+	UT_Error			_insertGraphic(FG_Graphic*, const char*,PT_DocPosition pos);
 
-	UT_UCSChar *		_lookupSuggestion(fl_BlockLayout* pBL, fl_PartOfBlock* pPOB, UT_uint32 ndx);
+	UT_UCSChar *		_lookupSuggestion(fl_BlockLayout* pBL, fl_PartOfBlock* pPOB, UT_sint32 ndx);
 
 	static void 		_autoScroll(UT_Worker * pTimer);
+	static void 		_actuallyScroll(UT_Worker * pTimer);
 
 	// localize handling of insertion point logic
 	void				_setPoint(PT_DocPosition pt, bool bEOL = false);
-	void				_setPoint(fv_CaretProps * pCP, PT_DocPosition pt, UT_sint32 iLen);
-	UT_uint32			_getDataCount(UT_uint32 pt1, UT_uint32 pt2);
+	void				_setPoint(fv_CaretProps * pCP, PT_DocPosition pt, UT_sint32 iLen) const;
+	UT_uint32			_getDataCount(UT_uint32 pt1, UT_uint32 pt2) const;
 	bool				_charMotion(bool bForward,UT_uint32 countChars, bool bSkipCannotContainPoint = true);
 	void				_doPaste(bool bUseClipboard, bool bHonorFormatting = true);
 	void				_clearIfAtFmtMark(PT_DocPosition dpos);
 
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 	void				_checkPendingWordForSpell(void);
 #endif
 	
-	bool				_isSpaceBefore(PT_DocPosition pos);
+	bool				_isSpaceBefore(PT_DocPosition pos) const;
 	void				_removeThisHdrFtr(fl_HdrFtrSectionLayout * pHdrFtr);
 	void 				_cmdEditHdrFtr(HdrFtrType hfType);
 
@@ -865,6 +923,12 @@ protected:
 	void                _adjustDeletePosition(UT_uint32 &iDocPos, UT_uint32 &iCount);
 	
 private:
+
+	UT_uint32			m_iNumHorizPages; /////////////////////////////////////////////////
+	UT_uint32			m_getNumHorizPagesCachedWindowWidth;
+	bool				m_autoNumHorizPages;
+	UT_uint32			m_horizPageSpacing;
+
 	PT_DocPosition		m_iInsPoint;
 	UT_sint32			m_xPoint;
 	UT_sint32			m_yPoint;
@@ -907,12 +971,12 @@ private:
 	fl_HdrFtrShadow *	m_pEditShadow;
 	PT_DocPosition		m_iSavedPosition;
 	bool				m_bNeedSavedPosition;
-	PT_DocPosition		_BlockOffsetToPos(fl_BlockLayout * block, PT_DocPosition offset);
+	PT_DocPosition		_BlockOffsetToPos(fl_BlockLayout * block, PT_DocPosition offset) const;
 
-	fl_BlockLayout *	_findGetCurrentBlock(void);
-	PT_DocPosition		_findGetCurrentOffset(void);
+	fl_BlockLayout *	_findGetCurrentBlock(void) const;
+	PT_DocPosition		_findGetCurrentOffset(void) const;
 	UT_UCSChar *		_findGetNextBlockBuffer(fl_BlockLayout ** block, PT_DocPosition *offset);
-	UT_UCSChar *		_findGetPrevBlockBuffer(fl_BlockLayout ** block, PT_DocPosition *offset);
+	UT_UCSChar *		_findGetPrevBlockBuffer(fl_BlockLayout ** block, PT_DocPosition *offset, UT_sint32& endIndex);
 
 	bool				m_bReverseFind;
 	bool				m_bWholeWord;
@@ -923,7 +987,7 @@ private:
 	UT_sint32			_findBlockSearchRegexp(const UT_UCSChar * haystack, const UT_UCSChar * needle);
 
 	// prefs listener - to change cursor blink on/off (and possibly others)
-	static void _prefsListener( XAP_App *, XAP_Prefs *, UT_StringPtrMap *, void *);
+	static void _prefsListener( XAP_Prefs *, UT_StringPtrMap *, void *);
 
 	bool				 m_bShowPara;
 	ViewMode			 m_viewMode;
@@ -943,7 +1007,6 @@ private:
 
 	// properties for image selection
 	UT_Rect				m_selImageRect;
-	UT_uint32			m_iImageSelBoxSize;  // in device units!
 	GR_Graphics::Cursor	m_imageSelCursor;
 	UT_sint32			m_ixResizeOrigin;
 	UT_sint32			m_iyResizeOrigin;
@@ -971,6 +1034,7 @@ private:
 	UT_RGBColor         m_colorRevisions[10];
 	UT_RGBColor			m_colorHdrFtr;
 	UT_RGBColor			m_colorColumnLine;
+	UT_RGBColor         m_colorAnnotations[10];
 
 	UT_uint32 m_countDisable; // cursor disable count
 	bool                m_bDragTableLine;
@@ -983,12 +1047,16 @@ private:
 	bool m_bgColorInitted;
 	PT_DocPosition      m_iLowDrawPoint;
 	PT_DocPosition      m_iHighDrawPoint;
-	fv_PropCache        m_CharProps;
-	fv_PropCache        m_BlockProps;
-	fv_PropCache        m_SecProps;
+	mutable fv_PropCache        m_CharProps;
+	mutable fv_PropCache        m_BlockProps;
+	mutable fv_PropCache        m_SecProps;
 	AV_ListenerId       m_CaretListID;
+#ifdef TOOLKIT_GTK
+	FV_UnixFrameEdit    m_FrameEdit;
+#else
 	FV_FrameEdit        m_FrameEdit;
-#ifdef XP_UNIX_TARGET_GTK
+#endif
+#ifdef TOOLKIT_GTK
 	FV_UnixVisualDrag   m_VisualDragText;
 #else
 	FV_VisualDragText   m_VisualDragText;
@@ -1001,11 +1069,19 @@ private:
 	bool                m_bDontNotifyListeners;
 	UT_ByteBuf *        m_pLocalBuf;
 	UT_sint32           m_iGrabCell;
+#ifdef TOOLKIT_GTK
+	FV_UnixVisualInlineImage  m_InlineImage;
+#else
 	FV_VisualInlineImage  m_InlineImage;
+#endif
 	bool                m_bInsertAtTablePending;
 	PT_DocPosition      m_iPosAtTable;
 	UT_GenericVector<fv_CaretProps *> m_vecCarets;
 	UT_UTF8String       m_sDocUUID;
+	bool				m_bAnnotationPreviewActive;
+	UT_uint32			m_iAnnPviewID;
+	bool                m_bAllowSmartQuoteReplacement;  // Enable/disable replacing of quote with smart quote
+														// This allows temporarily disabling smart quotes to allow inserting ANSI quote.
 };
 
 #endif /* FV_VIEW_H */
