@@ -54,13 +54,15 @@ class pf_Frag_Strux;
 class PX_ChangeRecord;
 class PD_Style;
 class PD_DocIterator;
-class XAP_App;
 class fd_Field;
 class po_Bookmark;
 class fl_AutoNum;
 class fl_BlockLayout;
 class fp_Run;
 class UT_UTF8String;
+class pp_Author;
+class _ImagePage;
+class _TextboxPage;
 
 #ifdef PT_TEST
 #include "ut_test.h"
@@ -68,15 +70,15 @@ class UT_UTF8String;
 
 enum
 {
-	PD_SIGNAL_UPDATE_LAYOUT,
+	PD_SIGNAL_UPDATE_LAYOUT = 0,
 	PD_SIGNAL_REFORMAT_LAYOUT,
 	PD_SIGNAL_DOCPROPS_CHANGED_REBUILD,
 	PD_SIGNAL_DOCPROPS_CHANGED_NO_REBUILD,
 	PD_SIGNAL_REVISION_MODE_CHANGED,
 	PD_SIGNAL_DOCNAME_CHANGED,
 	PD_SIGNAL_DOCDIRTY_CHANGED,
-	PD_SIGNAL_DOCSAVED,
-	PD_SIGNAL_DOCCLOSED
+	PD_SIGNAL_SAVEDOC, 
+	PD_SIGNAL_DOCCLOSED // TODO: remove this
 };
 
 /////////////////////////////////////////////////////////
@@ -163,7 +165,7 @@ enum
 class ABI_EXPORT PD_Document : public AD_Document
 {
 public:
-	PD_Document(XAP_App *pApp);
+	PD_Document();
 
 	virtual AD_DOCUMENT_TYPE getType() const {return ADDOCUMENT_ABIWORD;}
 	
@@ -185,13 +187,42 @@ public:
 
 	virtual bool			isDirty(void) const;
 	virtual void            forceDirty();
-	bool                    isCACConnected(void);
+	bool                    isConnected(void);
 	virtual bool			canDo(bool bUndo) const;
 	virtual UT_uint32		undoCount(bool bUndo) const;
 	virtual bool			undoCmd(UT_uint32 repeatCount);
 	virtual bool			redoCmd(UT_uint32 repeatCount);
 	bool                    isDoingTheDo(void) const;
 
+	// Caret Methods
+
+	void                    removeCaret(const std::string& sCaretID);
+
+	// Author methods
+
+	void                    setShowAuthors(bool bAuthors);
+	bool                    isShowAuthors(void) const
+	{ return m_bShowAuthors;}
+	bool                    isExportAuthorAtts(void) const;
+	void                    setExportAuthorAtts(bool bExport);
+	UT_GenericVector<pp_Author*>& getAuthors()
+	{ return m_vecAuthors; }
+	UT_sint32               getMyAuthorInt(void) const;
+	void                    setMyAuthorInt(UT_sint32 iAuthor);
+	UT_sint32               getLastAuthorInt(void) const;
+	UT_sint32               getNumAuthors(void) const;
+	pp_Author *             getNthAuthor(UT_sint32 i) const;
+	pp_Author *             getAuthorByInt(UT_sint32 i) const;
+	pp_Author *             addAuthor(UT_sint32 iAuthor);
+	bool                    sendAddAuthorCR(pp_Author * pAuthor);
+	bool                    sendChangeAuthorCR(pp_Author * pAuthor);
+	UT_sint32               findFirstFreeAuthorInt(void) const;
+ private:
+	bool                    addAuthorAttributeIfBlank(const gchar ** szAttsIn, const gchar **& szAttsOut, UT_String & storage);
+	bool                    addAuthorAttributeIfBlank( PP_AttrProp *&p_AttrProp);
+	bool                    _buildAuthorProps(pp_Author * pAuthor, const gchar **& szProps);
+ public:
+	//
 	void					beginUserAtomicGlob(void);
 	void					endUserAtomicGlob(void);
 	void                    setMarginChangeOnly(bool b);
@@ -233,6 +264,8 @@ PT_AttrPropIndex            getAPIFromSOH(PL_ObjectHandle odh);
 										bool bRecordChange);
 
 	bool                    createAndSendCR(PT_DocPosition dpos,UT_sint32 iType,bool bsave,UT_Byte iGlob);
+	virtual bool            createAndSendDocPropCR( const gchar ** pAtts,const gchar ** pProps );
+	bool                    changeDocPropeties(const gchar ** szAtts, const gchar ** pProps);
 
 	bool					insertStrux(PT_DocPosition dpos,
 										PTStruxType pts,
@@ -285,7 +318,7 @@ PT_AttrPropIndex            getAPIFromSOH(PL_ObjectHandle odh);
 	bool                    appendLastStruxFmt(PTStruxType pts, const gchar ** attrs, const gchar * props,
 											   bool bSkipEmbededSections);
 	bool					appendFmt(const gchar ** attributes);
-	bool					appendFmt(const UT_GenericVector<gchar*> * pVecAttributes);
+	bool					appendFmt(const UT_GenericVector<const gchar*> * pVecAttributes);
 	bool					appendSpan(const UT_UCSChar * p, UT_uint32 length);
 	bool					appendObject(PTObjectType pto, const gchar ** attributes);
 	bool					appendFmtMark(void);
@@ -367,6 +400,7 @@ PT_AttrPropIndex            getAPIFromSOH(PL_ObjectHandle odh);
 	bool					setDataItemToken(void* pHandle, void* pToken);
 	bool					getDataItemData(void * pHandle,
 											const char ** pszName, const UT_ByteBuf ** ppByteBuf, const void** ppToken) const;
+	bool					getDataItemFileExtension(const char *szDataID, std::string &sExt, bool bDot = true) const;
 	bool					enumDataItems(UT_uint32 k,
 										  void ** ppHandle, const char ** pszName, const UT_ByteBuf ** ppByteBuf, const void** ppToken) const;
 
@@ -414,15 +448,15 @@ PT_AttrPropIndex            getAPIFromSOH(PL_ObjectHandle odh);
 	void					clearIfAtFmtMark(PT_DocPosition dpos);
 
 	virtual UT_uint32		getLastSavedAsType() const { return m_lastSavedAsType; }
-	UT_uint32				getLastOpenedType() { return m_lastOpenedType; }
-	XAP_App *				getApp() { return m_pApp; }
+	UT_uint32				getLastOpenedType() const { return m_lastOpenedType; }
 	bool					updateFields(void);
 	bool					getField(PL_StruxDocHandle sdh,
 									 UT_uint32 offset,
                                      fd_Field * &pField);
 	po_Bookmark * 			getBookmark(PL_StruxDocHandle sdh, UT_uint32 offset);
 	pf_Frag *               findBookmark(const char * pName, bool bEnd = false, pf_Frag * pfStart = NULL);
-	
+	bool                    hasMathorSVG(void);
+
 	void					setDontChangeInsPoint(void);
 	void					allowChangeInsPoint(void);
 	bool					getAllowChangeInsPoint(void) const;
@@ -485,9 +519,10 @@ PT_AttrPropIndex            getAPIFromSOH(PL_ObjectHandle odh);
 
 	// PageSize functions
 	bool                    convertPercentToInches(const char * szPercent, UT_UTF8String & sInches);
-	fp_PageSize				m_docPageSize;
 	bool					setPageSizeFromFile(const gchar ** attributes);
-
+	const	fp_PageSize	*	getPageSize(void) const	
+	{ return & m_docPageSize;}
+	fp_PageSize             m_docPageSize; // Move this to private later
 	bool					isBookmarkUnique(const gchar * pName) const;
 	bool					isBookmarkRelativeLink(const gchar * pName) const;
 	UT_uint32				getBookmarkCount()const {return m_vBookmarkNames.getItemCount();}
@@ -536,6 +571,10 @@ PT_AttrPropIndex            getAPIFromSOH(PL_ObjectHandle odh);
 	virtual void setMetaDataProp (const UT_String & key, const UT_UTF8String & value);
 	virtual bool getMetaDataProp (const UT_String & key, UT_UTF8String & outProp) const;
 
+	// RIVERA TODO not working and may not be needed
+	virtual void setAnnotationProp (const UT_String & key, const UT_UTF8String & value);
+	virtual bool getAnnotationProp (const UT_String & key, UT_UTF8String & outProp) const;
+
 	UT_GenericStringMap<UT_UTF8String*> & getMetaData () { return m_metaDataMap ; }
 
 	// document-level property handling functions
@@ -546,7 +585,7 @@ PT_AttrPropIndex            getAPIFromSOH(PL_ObjectHandle odh);
 	bool                    setProperties(const gchar ** ppProps);
 	void                     setDontImmediatelyLayout(bool b)
 		{ m_bDontImmediatelyLayout = b;}
-	bool                     isDontImmediateLayout(void)
+	bool                     isDontImmediateLayout(void) const
 		{ return m_bDontImmediatelyLayout;}
 
 	/* Okay, as far as I can tell this is a non-persistent document property since it is not
@@ -602,7 +641,7 @@ PT_AttrPropIndex            getAPIFromSOH(PL_ObjectHandle odh);
 
 	UT_uint32 getUID(UT_UniqueId::idType t) {return m_UID.getUID(t);}
 	bool      setMinUID(UT_UniqueId::idType t, UT_uint32 i) {return m_UID.setMinId(t,i);}
-	bool      isIdUnique(UT_UniqueId::idType t, UT_uint32 i) {return m_UID.isIdUnique(t,i);}
+	bool      isIdUnique(UT_UniqueId::idType t, UT_uint32 i) const {return m_UID.isIdUnique(t,i);}
 
 	virtual bool  areDocumentContentsEqual(const AD_Document &d, UT_uint32 &pos) const;
 	virtual bool  areDocumentFormatsEqual(const AD_Document &d, UT_uint32 &pos) const;
@@ -639,14 +678,21 @@ PT_AttrPropIndex            getAPIFromSOH(PL_ObjectHandle odh);
 
 	void                    setVDNDinProgress(bool b) {m_bVDND = b;}
 	bool                    isVDNDinProgress() const {return m_bVDND;}
+	UT_sint32               getCRNumber() const { return m_iCRCounter; }
+	void					setCRNumber(UT_sint32 iCRCounter) { m_iCRCounter = iCRCounter; }
 	UT_sint32               getNextCRNumber(void);
-    void                    getAllViews(UT_GenericVector<AV_View *> * vecViews);
+    void                    getAllViews(UT_GenericVector<AV_View *> * vecViews) const;
 	void                    ignoreSignals(void)
 	{ m_bIgnoreSignals = true;}
 	void                    dontIgnoreSignals(void)
 	{ m_bIgnoreSignals = false;}
 	void                    setClean(void)
 	{ _setClean();}
+
+	void					setCoalescingMask(bool mask)
+	{ m_bCoalescingMask = mask; }
+	bool					isCoalescingMasked() const
+	{ return m_bCoalescingMask; }
 protected:
 	virtual ~PD_Document();
 
@@ -673,7 +719,12 @@ protected:
 	UT_Error _importFile(GsfInput * input, int ieft,
 						 bool markClean, bool bImportStylesFirst,
 						 bool bIsImportFile, const char* impProps);
-	
+
+	bool     _removeRepeatedHdrFtr(pf_Frag_Strux * pfs ,UT_GenericVector<pf_Frag_Strux *> * vecHdrFtrs, UT_sint32 i);
+	bool     _pruneSectionAPI(pf_Frag_Strux * pfs,const char * szHType, UT_GenericVector<pf_Frag_Strux *> *vecHdrFtrs);
+	bool     _matchSection(pf_Frag_Strux * pfs, UT_GenericVector<pf_Frag_Strux *> *vecSections);
+	bool     _removeHdrFtr(pf_Frag_Strux * pfs);
+
 public:
 	// these functions allow us to retrieve visual direction at document
 	// position pos from an associated layout. They are intended to be
@@ -697,7 +748,6 @@ public:
 private:
 	IEFileType				m_lastOpenedType;
 	IEFileType				m_lastSavedAsType;
-	XAP_App *				m_pApp;
 	bool					m_bDoingPaste;
 	bool					m_bAllowInsertPointChange;
 	bool                    m_bRedrawHappenning;
@@ -729,6 +779,15 @@ private:
     UT_sint32               m_iCRCounter;
 	mutable UT_sint32       m_iUpdateCount;
 	bool                    m_bIgnoreSignals;
+
+	bool					m_bCoalescingMask;
+	bool                    m_bShowAuthors;
+	UT_GenericVector<pp_Author *>  m_vecAuthors;
+	bool                    m_bExportAuthorAtts;
+	UT_sint32               m_iMyAuthorInt;
+	UT_sint32               m_iLastAuthorInt;
+	UT_GenericVector<_ImagePage *> m_pPendingImagePage;
+	UT_GenericVector<_TextboxPage *> m_pPendingTextboxPage;
 };
 
 #endif /* PD_DOCUMENT_H */
