@@ -130,19 +130,19 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 				bRet = true;
 
 			// get attributes for this fragement
-			const PP_AttrProp * pAP;
+			const PP_AttrProp * pAP2;
 			pf_Frag::PFType eType = pf1->getType();
 			UT_uint32 iLen = 1;
-			PTStruxType eStruxType;
+			PTStruxType eStruxType = PTX_StruxDummy;
 
 			if(eType == pf_Frag::PFT_Text)
 			{
-				if(!getAttrProp(static_cast<pf_Frag_Text*>(pf1)->getIndexAP(),&pAP))
+				if(!getAttrProp(static_cast<pf_Frag_Text*>(pf1)->getIndexAP(),&pAP2))
 					return false;
 			}
 			else if(eType == pf_Frag::PFT_Strux)
 			{
-				if(!getAttrProp(static_cast<pf_Frag_Strux*>(pf1)->getIndexAP(),&pAP))
+				if(!getAttrProp(static_cast<pf_Frag_Strux*>(pf1)->getIndexAP(),&pAP2))
 					return false;
 
 				eStruxType = static_cast<pf_Frag_Strux*>(pf1)->getStruxType();
@@ -176,6 +176,7 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 						
 					case PTX_SectionEndnote:
 					case PTX_SectionFootnote:
+					case PTX_SectionAnnotation:
 					case PTX_SectionFrame:
 					case PTX_SectionTOC:
 						bHasEndStrux = true;
@@ -186,6 +187,7 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 					case PTX_Section:
 				    case PTX_EndFootnote:
 				    case PTX_EndEndnote:
+				    case PTX_EndAnnotation:
 				    case PTX_EndFrame:
 					case PTX_EndTOC:
 						iLen = pf_FRAG_STRUX_SECTION_LENGTH;
@@ -200,13 +202,13 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 			}
 			else if(eType == pf_Frag::PFT_Object)
 			{
-				if(!getAttrProp(static_cast<pf_Frag_Object*>(pf1)->getIndexAP(),&pAP))
+				if(!getAttrProp(static_cast<pf_Frag_Object*>(pf1)->getIndexAP(),&pAP2))
 					return false;
 			}
 			else if(eType == pf_Frag:: PFT_FmtMark)
 			{
 				iLen = 0;
-				if(!getAttrProp(static_cast<pf_Frag_Object*>(pf1)->getIndexAP(),&pAP))
+				if(!getAttrProp(static_cast<pf_Frag_Object*>(pf1)->getIndexAP(),&pAP2))
 					return false;
 				// something that does not carry AP
 			}
@@ -227,7 +229,7 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 				continue;
 			}
 			
-			if(!pAP->getAttribute(name, pRevision))
+			if(!pAP2->getAttribute(name, pRevision))
 				pRevision = NULL;
 
 			PP_RevisionAttr Revisions(pRevision);
@@ -290,6 +292,10 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 									if(eStrux2Type != PTX_EndFootnote)
 										continue;
 									break;
+								case PTX_SectionAnnotation:
+									if(eStrux2Type != PTX_EndAnnotation)
+										continue;
+									break;
 								case PTX_SectionFrame:
 									if(eStrux2Type != PTX_EndFrame)
 										continue;
@@ -447,10 +453,10 @@ bool pt_PieceTable::_fixHdrFtrReferences(const gchar * pszHdrType, const gchar *
 				continue;
 
 			// check for normal attribute
-			const gchar * pszMyHdrId = NULL;
-			if(pAP->getAttribute(pszHdrType, pszMyHdrId) && pszMyHdrId)
+			const gchar * pszMyHdrId2 = NULL;
+			if(pAP->getAttribute(pszHdrType, pszMyHdrId2) && pszMyHdrId2)
 			{
-				if(0 == strcmp(pszMyHdrId, pszHdrId))
+				if(0 == strcmp(pszMyHdrId2, pszHdrId))
 				{
 					const gchar* pAttrs [3];
 					pAttrs[0] = pszHdrType;
@@ -471,11 +477,11 @@ bool pt_PieceTable::_fixHdrFtrReferences(const gchar * pszHdrType, const gchar *
 
 				for(UT_uint32 i = 0; i < Revisions.getRevisionsCount(); ++i)
 				{
-					const PP_Revision * pRev = Revisions.getNthRevision(i);
-					UT_return_val_if_fail( pRev, false );
+					const PP_Revision * pRev2 = Revisions.getNthRevision(i);
+					UT_return_val_if_fail( pRev2, false );
 
 					const gchar * pszMyHdrId = NULL;
-					if(pRev->getAttribute(pszHdrType, pszMyHdrId) && pszMyHdrId)
+					if(pRev2->getAttribute(pszHdrType, pszMyHdrId) && pszMyHdrId)
 					{
 						if(0 != strcmp(pszHdrId, pszMyHdrId))
 							continue;
@@ -484,7 +490,7 @@ bool pt_PieceTable::_fixHdrFtrReferences(const gchar * pszHdrType, const gchar *
 						{
 							// NB: this is safe, since we own the PP_RevisionAttr object
 							// of local scope which in turn owns this revisions
-							const_cast<PP_Revision*>(pRev)->setAttribute(pszHdrType, "");
+							const_cast<PP_Revision*>(pRev2)->setAttribute(pszHdrType, "");
 						}
 						else
 						{
@@ -631,12 +637,13 @@ bool pt_PieceTable::_deleteSpanWithNotify(PT_DocPosition dpos,
 								   m_varset.getBufIndex(pft->getBufIndex(),fragOffset),
 								   length,blockOffset,pft->getField());
 	UT_return_val_if_fail (pcr, false);
-
+	pcr->setDocument(m_pDocument);
 	bool bResult = _deleteSpan(pft,fragOffset,pft->getBufIndex(),length,ppfEnd,pfragOffsetEnd);
 
-	if (!bAddChangeRec || _canCoalesceDeleteSpan(pcr))
+	bool canCoalesce = _canCoalesceDeleteSpan(pcr);
+	if (!bAddChangeRec || (canCoalesce && !m_pDocument->isCoalescingMasked()))
 	{
-		if (bAddChangeRec)
+		if (canCoalesce)
 			m_history.coalesceHistory(pcr);
 
 		m_pDocument->notifyListeners(pfs,pcr);
@@ -664,6 +671,8 @@ bool pt_PieceTable::_canCoalesceDeleteSpan(PX_ChangeRecord_Span * pcrSpan) const
 		return false;
 	if (pcrSpan->getIndexAP() != pcrUndo->getIndexAP())
 		return false;
+	if((pcrUndo->isFromThisDoc() != pcrSpan->isFromThisDoc()))
+	   return false;
 
 	PX_ChangeRecord_Span * pcrUndoSpan = static_cast<PX_ChangeRecord_Span *>(pcrUndo);
 	UT_uint32 lengthUndo = pcrUndoSpan->getLength();
@@ -804,6 +813,7 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 //
 		return true;
 	case PTX_SectionFootnote:
+	case PTX_SectionAnnotation:
 	case PTX_SectionEndnote:
 	{
 //
@@ -816,6 +826,7 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 	}
  	case PTX_EndFootnote:	
  	case PTX_EndEndnote:	
+ 	case PTX_EndAnnotation:	
  	{
 //
 // Get the actual block strux container for the endnote. 
@@ -950,8 +961,9 @@ bool pt_PieceTable::_tweakDeleteSpan(PT_DocPosition & dpos1,
 	
 	//
 // First we want to handle hyperlinks. If we delete all the text within
-// a hyperlink, then we should also delete the start and end point of the
-// hyperlink.
+// a hyperlink or annotation, then we should also delete the start 
+// and end point of the
+// hyperlink or annotation.
 //
 	pf_Frag * pf_First;
 	pf_Frag * pf_End;
@@ -994,7 +1006,7 @@ bool pt_PieceTable::_tweakDeleteSpan(PT_DocPosition & dpos1,
 		{
 //
 // OK these frags are entirely contained by dpos1  and dpos2
-// OK now look to see if there is a hyperlink just before and after these
+// OK now look to see if there is a hyperlink or annotation just before and after these
 //
 			if(pf_End->getType() != pf_Frag::PFT_Object)
 			{
@@ -1017,6 +1029,7 @@ bool pt_PieceTable::_tweakDeleteSpan(PT_DocPosition & dpos1,
 				pf_Frag_Object *pFO = static_cast<pf_Frag_Object *>(pf_First);
 				bool bFoundBook = false;
 				bool bFoundHype = false;
+				bool bFoundAnn = false;
 				if(pFO->getObjectType() == PTO_Bookmark)
 				{
 					bFoundBook = true;
@@ -1025,6 +1038,10 @@ bool pt_PieceTable::_tweakDeleteSpan(PT_DocPosition & dpos1,
 				{
 					bFoundHype = true;
 				}
+				if(pFO->getObjectType() == PTO_Annotation)
+				{
+					bFoundAnn = true;
+				}
 				if(pf_End && (pf_End->getType() == pf_Frag::PFT_Object) && (pf_End != pf_First))
 				{
 					pFO = static_cast<pf_Frag_Object *>(pf_End);
@@ -1040,6 +1057,14 @@ bool pt_PieceTable::_tweakDeleteSpan(PT_DocPosition & dpos1,
 					{
 //
 // Found a Hyperlink which will have all contents deleted so delte it too
+//
+						dpos1--;
+						dpos2++;
+					}
+					else if(pFO->getObjectType() == PTO_Annotation && bFoundAnn)
+					{
+//
+// Found a Annotation which will have all contents deleted so delte it too
 //
 						dpos1--;
 						dpos2++;
@@ -1084,7 +1109,7 @@ bool pt_PieceTable::_tweakDeleteSpan(PT_DocPosition & dpos1,
 	{
 		PT_DocPosition old_dpos1 = dpos1;
 		PT_DocPosition old_dpos2 = dpos2;
-		UT_uint32 old_iStackSize = pstDelayStruxDelete->getDepth();
+		UT_sint32 old_iStackSize = pstDelayStruxDelete->getDepth();
 
 		if(!_tweakDeleteSpanOnce(dpos1, dpos2, pstDelayStruxDelete))
 			return false;
@@ -1200,7 +1225,7 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 
 	bool bFound = getFragsFromPositions(dpos1,dpos2,&pf_First,&fragOffset_First,&pf_End,&fragOffset_End);
 	UT_return_val_if_fail (bFound, false);
-	UT_DEBUGMSG(("deleteComplex span dpos1 %d dpos2 %d pf_First %x pf_First pos %d \n",dpos1,dpos2,pf_First,pf_First->getPos()));
+	UT_DEBUGMSG(("deleteComplex span dpos1 %d dpos2 %d pf_First %p pf_First pos %d \n",dpos1,dpos2,pf_First,pf_First->getPos()));
 	pf_Frag_Strux * pfsContainer = NULL;
 	bool bFoundStrux = _getStruxFromPosition(dpos1,&pfsContainer);
 	UT_return_val_if_fail (bFoundStrux, false);
@@ -1239,8 +1264,10 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 	}
 	bool bPrevWasFootnote = false;
 	UT_sint32 iLoopCount = -1;
-	while (length > 0)
+	while (length >=0)
 	{
+	        if(length == 0 && iFootnoteCount <= 0)
+		  break;
 		iLoopCount++;
 		UT_uint32 lengthInFrag = pf_First->getLength() - fragOffset_First;
 		UT_uint32 lengthThisStep = UT_MIN(lengthInFrag, length);
@@ -1411,10 +1438,10 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 					{
 						m_fragments.cleanFrags();
 					}
-					PT_DocPosition myPos = pfs->getPos();
-					_deleteFormatting(myPos - pfs->getLength(), myPos);
+					PT_DocPosition myPos2 = pfs->getPos();
+					_deleteFormatting(myPos2 - pfs->getLength(), myPos2);
 					UT_DEBUGMSG(("DELeteing EndTable Strux, pos= %d \n",pfs->getPos()));
-					bResult = _deleteStruxWithNotify(myPos, pfs,
+					bResult = _deleteStruxWithNotify(myPos2, pfs,
 													  &pfNewEnd,
 													  &fragOffsetNewEnd);
 					while(bResult && myTable > 0)
@@ -1459,9 +1486,9 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 				{
 					m_fragments.cleanFrags();
 				}
-				PT_DocPosition myPos = pfs->getPos();
-				_deleteFormatting(myPos - pfs->getLength(), myPos);
-				bResult = _deleteStruxWithNotify(myPos, pfs,
+				PT_DocPosition myPos2 = pfs->getPos();
+				_deleteFormatting(myPos2 - pfs->getLength(), myPos2);
+				bResult = _deleteStruxWithNotify(myPos2, pfs,
 												  &pfNewEnd,
 												  &fragOffsetNewEnd);
 //
@@ -1524,10 +1551,10 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 				{
 					m_fragments.cleanFrags();
 				}
-				PT_DocPosition myPos = pfs->getPos();
-				_deleteFormatting(myPos - pfs->getLength(), myPos);
+				PT_DocPosition myPos2 = pfs->getPos();
+				_deleteFormatting(myPos2 - pfs->getLength(), myPos2);
 				bool isFrame =  pfs->getStruxType() == PTX_SectionFrame;
-				bResult = _deleteStruxWithNotify(myPos, pfs,
+				bResult = _deleteStruxWithNotify(myPos2, pfs,
 												  &pfNewEnd,
 												  &fragOffsetNewEnd);
 //
@@ -1586,7 +1613,7 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 			UT_return_val_if_fail (bResult, false);
 		}
 		break;
-		// the bookmark and hyperlink objects require specxial treatment; since
+		// the bookmark, hyperlink and annotation objects require specxial treatment; since
 		// they come in pairs, we have to ensure that both are deleted together
 		// so we have to find the other part of the pair, delete it, adjust the
 		// positional variables and the delete the one we were originally asked
@@ -1597,13 +1624,14 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 			{
 				_getStruxFromFragSkip(static_cast<pf_Frag *>(pfsContainer),&pfsContainer);
 			}
-			bool bResult, bResult2;
+			bool bResult = false;
+			bool bResult2;
 			pf_Frag_Object *pO = static_cast<pf_Frag_Object *>(pf_First);
 			switch(pO->getObjectType())
 			{
 				case PTO_Bookmark:
 				{
-					bool bFoundStrux2;
+					bool bFoundStrux3;
 					PT_DocPosition posComrade;
 					pf_Frag_Strux * pfsContainer2 = NULL;
 
@@ -1630,8 +1658,8 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 										origPos1--;
 									}
 									
-									bFoundStrux2 = _getStruxFromFragSkip(pOb,&pfsContainer2);
-									UT_return_val_if_fail (bFoundStrux2, false);
+									bFoundStrux3 = _getStruxFromFragSkip(pOb,&pfsContainer2);
+									UT_return_val_if_fail (bFoundStrux3, false);
 
 									bResult2 =
 											_deleteObjectWithNotify(posComrade,pOb,0,1,
@@ -1800,6 +1828,124 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 				}
 				break;
 
+				//
+				// When deleting either the start or end of annotation we have to deleted
+				// the content of the annotation as well.
+				// We have to always delete the start marker first; this is
+				// so that in the case of undo the endmarker would be restored
+				// first, because the mechanism that marks runs between them
+				// as a hyperlink depents on the end-marker being in place before
+				// the start marker
+				case PTO_Annotation:
+				{
+					bool bFoundStrux2;
+					PT_DocPosition posComrade;
+					pf_Frag_Strux * pfsContainer2 = NULL;
+
+					pf_Frag * pF;
+
+		    		const PP_AttrProp * pAP = NULL;
+				    pO->getPieceTable()->getAttrProp(pO->getIndexAP(),&pAP);
+				    UT_return_val_if_fail (pAP, false);
+				    const gchar* pszHref = NULL;
+				    const gchar* pszHname  = NULL;
+		            UT_uint32 k = 0;
+        		    bool bStart = false;
+				    while((pAP)->getNthAttribute(k++,pszHname, pszHref))
+				    {
+				      if((strcmp(pszHname, "annotation") == 0) ||(strcmp(pszHname, "Annotation") == 0) )
+				    	{
+				    		bStart = true;
+		    				break;
+				    	}
+				    }
+
+					if(!bStart)
+					{
+						// in this case we are looking for the start marker
+						// and so we delete it and then move on
+				    	pF = pO->getPrev();
+				    	while(pF)
+				    	{
+							if(pF->getType() == pf_Frag::PFT_Object)
+							{
+								pf_Frag_Object *pOb = static_cast<pf_Frag_Object*>(pF);
+								if(pOb->getObjectType() == PTO_Annotation)
+								{
+									posComrade = getFragPosition(pOb);
+									bFoundStrux2 = _getStruxFromFragSkip(pOb,&pfsContainer2);
+									UT_return_val_if_fail (bFoundStrux2, false);
+
+									bResult2 =
+											_deleteObjectWithNotify(posComrade,pOb,0,1,
+										  							pfsContainer2,0,0);
+
+									// now adjusting the positional variables
+									if(posComrade <= dpos1)
+										// delete before the start of the segement we are working on
+										dpos1--;
+									else
+									{
+										// we are inside that section
+										length--;
+									}
+									break;
+								}
+							}
+							pF = pF->getPrev();
+				    	}
+						bResult
+							= _deleteObjectWithNotify(dpos1,pO,fragOffset_First,lengthThisStep,
+										  pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
+
+					}
+					else
+					{
+						// in this case we are looking for the end marker,
+						// so we have to be carefult the get rid of the start
+						// marker first
+				    	pF = pO->getNext();
+				    	while(pF)
+				    	{
+							if(pF->getType() == pf_Frag::PFT_Object)
+							{
+								pf_Frag_Object *pOb = static_cast<pf_Frag_Object*>(pF);
+								if(pOb->getObjectType() == PTO_Annotation)
+								{
+									posComrade = getFragPosition(pOb);
+									bFoundStrux2 = _getStruxFromFragSkip(pOb,&pfsContainer2);
+									UT_return_val_if_fail (bFoundStrux2, false);
+					                // delete the original start marker
+									bResult
+										= _deleteObjectWithNotify(dpos1,pO,fragOffset_First,lengthThisStep,
+													  pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
+
+									// now adjusting the positional variables
+									posComrade--;
+
+									bResult2 =
+											_deleteObjectWithNotify(posComrade,pOb,0,1,
+									  							pfsContainer2,0,0);
+
+									// FIXME!! Need to work out how to delete the content of the annotation
+									// at this point
+
+									if(posComrade >= dpos1 && posComrade <= dpos1 + length - 2)
+									{
+										// the endmarker was inside of the segment we are working
+										// so we have to adjust the length
+										length--;
+									}
+
+									break;
+								}
+							}
+							pF = pF->getNext();
+				    	}
+					}
+				}
+				break;
+
 				case PTO_Field:
 				{
 				}
@@ -2014,7 +2160,7 @@ bool pt_PieceTable::_realDeleteSpan(PT_DocPosition dpos1,
 		//  first, and then the actual spans.
 		_changePointWithNotify(old_dpos2);
 
-		UT_uint32 oldDepth = stDelayStruxDelete.getDepth();
+		UT_sint32 oldDepth = stDelayStruxDelete.getDepth();
 		bSuccess = _deleteFormatting(dpos1, dpos2);
 		if (bSuccess)
 		{
