@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998-2000 AbiSource, Inc.
  * 
@@ -41,7 +43,7 @@
 #include <io.h>
 #include <fcntl.h>
 
-#if !defined(__WINE__) && (!defined(_MSC_VER) || _MSC_VER < 1310)
+#if !defined(__WINE__) && (!defined(_MSC_VER) || _MSC_VER < 1310) && !defined(__MINGW32__)
 #include <iostream.h>
 #elif _MSC_VER >= 1310
 #include <iostream>
@@ -69,6 +71,7 @@
 #include "xap_ModuleManager.h"
 #include "ev_EditMethod.h"
 #include "xap_Module.h"
+#include "abi-builtin-plugins.h"
 
 #include "ap_Win32Resources.rc2"
 #include "ap_Clipboard.h"
@@ -105,8 +108,8 @@
 extern XAP_Dialog_MessageBox::tAnswer s_CouldNotLoadFileMessage(XAP_Frame * pFrame, const char * pNewFile, UT_Error errorCode);
 /*****************************************************************/
 
-AP_Win32App::AP_Win32App(HINSTANCE hInstance, XAP_Args * pArgs, const char * szAppName)
-	: AP_App(hInstance, pArgs,szAppName)
+AP_Win32App::AP_Win32App(HINSTANCE hInstance, const char * szAppName)
+	: AP_App(hInstance, szAppName)
 {
 	m_pStringSet = NULL;
 	m_pClipboard = NULL;
@@ -147,11 +150,12 @@ bool AP_Win32App::initialize(void)
 	bool bSuccess = true;
 	const char * szUserPrivateDirectory = getUserPrivateDirectory();
 	bool bVerified = s_createDirectoryIfNecessary(szUserPrivateDirectory);
+
 	UT_return_val_if_fail (bVerified, false);
 
 	// load the preferences.
 	
-	m_prefs = new AP_Win32Prefs(this);
+	m_prefs = new AP_Win32Prefs();
 	UT_return_val_if_fail (m_prefs, false);
 	
 	m_prefs->fullInit();
@@ -249,7 +253,9 @@ bool AP_Win32App::initialize(void)
 	//////////////////////////////////////////////////////////////////
 	
 	{
+#if ENABLE_SPELL
 		SpellManager::instance();
+#endif
 	}
 	
 	
@@ -336,6 +342,10 @@ bool AP_Win32App::initialize(void)
 	// load the all Plugins from the correct directory
 	//////////////////////////////////////////////////////////////////
 
+#ifndef DISABLE_BUILTIN_PLUGINS
+	abi_register_builtin_plugins();
+#endif
+
 	bool bLoadPlugins = true;
 	bool bFound = getPrefsValueBool(XAP_PREF_KEY_AutoLoadPlugins,&bLoadPlugins);
 
@@ -344,7 +354,11 @@ bool AP_Win32App::initialize(void)
 		char szPath[PATH_MAX];
 		char szPlugin[PATH_MAX];
 		_getExeDir( szPath, PATH_MAX);
-		strcat(szPath, "..\\Plugins\\*.dll");
+#ifdef _MSC_VER
+		strcat(szPath, "..\\plugins\\*.dll");
+#else
+		strcat(szPath, "..\\lib\\" PACKAGE "-" ABIWORD_SERIES "\\plugins\\*.dll");
+#endif
 
 	    struct _finddata_t cfile;
 		long findtag = _findfirst( szPath, &cfile );
@@ -353,7 +367,11 @@ bool AP_Win32App::initialize(void)
 			do
 			{	
 				_getExeDir( szPlugin, PATH_MAX );
-				strcat( szPlugin, "..\\Plugins\\" );
+#ifdef _MSC_VER
+				strcat( szPlugin, "..\\plugins\\" );
+#else
+				strcat( szPlugin, "..\\lib\\" PACKAGE "-" ABIWORD_SERIES "\\plugins\\" );
+#endif
 				strcat( szPlugin, cfile.name );
 				XAP_ModuleManager::instance().loadModule( szPlugin );
 			} while( _findnext( findtag, &cfile ) == 0 );
@@ -431,13 +449,7 @@ bool AP_Win32App::getPrefsValueDirectory(bool bAppSpecific,
 
 const char * AP_Win32App::getAbiSuiteAppDir(void) const
 {
-	// we return a static string, use it quickly.
-	
-	static gchar buf[1024];
-	UT_return_val_if_fail ((strlen(getAbiSuiteLibDir()) + strlen(PACKAGE_NAME) + 2) < sizeof(buf), NULL);
-
-	sprintf(buf,"%s\\%s",getAbiSuiteLibDir(),PACKAGE_NAME);
-	return buf;
+    return getAbiSuiteLibDir();
 }
 
 HICON AP_Win32App::getIcon(void)
@@ -509,7 +521,7 @@ bool AP_Win32App::_cacheClipboardDoc(PD_DocumentRange *pDocRange)
 	}
 
 	// now create a subdocument ...
-	PD_Document * pDoc = new PD_Document(XAP_App::getApp());
+	PD_Document * pDoc = new PD_Document();
 
 	if(!pDoc)
 		return false;
@@ -823,7 +835,7 @@ PBITMAPINFO CreateBitmapInfoStruct(HBITMAP hBmp)
 //
 //
 //
-void CreateBMP(HWND hwnd, UT_ByteBuf & pBB, PBITMAPINFO pbi, 
+void CreateBMP(HWND /*hwnd*/, UT_ByteBuf & pBB, PBITMAPINFO pbi, 
                   HBITMAP hBMP, HDC hDC) 
 { 
 	BITMAPFILEHEADER hdr;       // bitmap file-header 
@@ -870,7 +882,7 @@ void CreateBMP(HWND hwnd, UT_ByteBuf & pBB, PBITMAPINFO pbi,
 
 
 
-void AP_Win32App::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipboard, bool bHonorFormatting)
+void AP_Win32App::pasteFromClipboard(PD_DocumentRange * pDocRange, bool /*bUseClipboard*/, bool bHonorFormatting)
 {
 	// paste from the system clipboard using the best-for-us format
 	// that is present.
@@ -935,7 +947,7 @@ bool AP_Win32App::_pasteFormatFromClipboard(PD_DocumentRange * pDocRange, const
  		AP_FrameData* 			pFrameData;		
  		FL_DocLayout*			pDocLy;	
  		FV_View* 				pView;						
-		UT_ByteBuf				bBufBMP;
+		UT_ByteBuf*				bBufBMP = new UT_ByteBuf;
  		
  		hBitmap = (HBITMAP)hData;					
  		hWnd =  GetDesktopWindow();
@@ -943,21 +955,24 @@ bool AP_Win32App::_pasteFormatFromClipboard(PD_DocumentRange * pDocRange, const
  		
  		// Create a BMP file from a BITMAP
  		bi =  CreateBitmapInfoStruct(hBitmap);						
- 		CreateBMP(hWnd, bBufBMP, bi, hBitmap,hdc);                  										
+ 		CreateBMP(hWnd, *bBufBMP, bi, hBitmap,hdc);                  										
  		
  		// Since we are providing the file type, there is not need to pass the bytebuff filled up
- 		errorCode = IE_ImpGraphic::constructImporter(&bBufBMP, iegft, &pIEG);				 				
+ 		errorCode = IE_ImpGraphic::constructImporter(*bBufBMP, iegft, &pIEG);				 				
 		 				
  		if(errorCode != UT_OK)		
 			return false;				  	
 		 				 			
- 		errorCode = pIEG->importGraphic(&bBufBMP, &pFG); 		
+ 		errorCode = pIEG->importGraphic(bBufBMP, &pFG); 		
  		
  		if(errorCode != UT_OK || !pFG)
 		{
+			DELETEP(bBufBMP);
 			DELETEP(pIEG);
  			return false;
 		}
+		// sunk in importGraphic
+		bBufBMP = NULL;
  		 
  		// Insert graphic in the view
  		pFrame = getLastFocussedFrame(); 						
@@ -1086,17 +1101,17 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 {	
 	// Load the command line into an XAP_Args class
 #ifdef _MSC_VER	// when using MSVC use already split arguments
-	XAP_Args XArgs = XAP_Args(__argc, (const char**)__argv);
+	XAP_Args XArgs = XAP_Args(__argc, __argv);
 #else			// but for other compiles may not be available so use szCmdLine
 	UT_String szNewCmdLine = UT_String_sprintf ( "%s %s", "AbiWord.exe", szCmdLine ) ;
 	XAP_Args XArgs = XAP_Args(szNewCmdLine.c_str());
 #endif
 
 	// Step 1: Initialize our application.
-	pMyWin32App = new AP_Win32App(hInstance, &XArgs, szAppName);
+	pMyWin32App = new AP_Win32App(hInstance, szAppName);
 	AP_Args Args = AP_Args(&XArgs, szAppName, pMyWin32App);
 
-	Args.parsePoptOpts();
+	Args.parseOptions();
 	pMyWin32App->initialize();
   
 	// Step 2: Handle all non-window args.
@@ -1144,7 +1159,7 @@ try
 	if (bShowApp)
 	{
 		// display the windows
-		for(UT_uint32 i = 0;i<pMyWin32App->m_vecFrames.getItemCount();i++)
+		for(UT_sint32 i = 0; i < pMyWin32App->m_vecFrames.getItemCount(); i++)
 		{
 			AP_Win32Frame * curFrame = (AP_Win32Frame*)pMyWin32App->m_vecFrames[i];
 			UT_continue_if_fail(curFrame);
@@ -1155,7 +1170,7 @@ try
 		}	
 
 		// do dispatch loop
-		while(UT_GetMessage(&msg, NULL, 0, 0))
+		while(UT_GetMessage(&msg, NULL, 0, 0, m_bForceAnsi))
 	    {
    	      	// TranslateMessage is not called because AbiWord
 	      	// has its own way of decoding keyboard accelerators
@@ -1223,10 +1238,8 @@ catch (...)
 		// do nothing
 	}
 	
-	UT_uint32 i = 0;
-	
 	IEFileType abiType = IE_Imp::fileTypeForSuffix(".abw");
-	for(;i<pApp->m_vecFrames.getItemCount();i++)
+	for (UT_sint32 i = 0; i < pApp->m_vecFrames.getItemCount(); i++)
 	{
 		AP_Win32Frame * curFrame = (AP_Win32Frame*)pApp->m_vecFrames[i];
 		UT_continue_if_fail(curFrame);
@@ -1310,8 +1323,18 @@ bool AP_Win32App::handleModelessDialogMessage( MSG * msg )
 		{
 			hWnd = (HWND) m_IdTable[ iCounter ].pDialog->pGetWindowHandle();
 
-			if( hWnd && IsDialogMessage( hWnd, msg ) )
-				return true;
+			if(!m_bForceAnsi && UT_IsWinNT())
+			{
+				// Since the message was fetched with GetMessageW,
+				// it must be processed with Unicode functions
+				if( hWnd && IsDialogMessageW( hWnd, msg ) )
+					return true;
+			}
+			else
+			{
+				if( hWnd && IsDialogMessage( hWnd, msg ) )
+					return true;
+			}
 		}
 		else
 			break;
@@ -1321,15 +1344,11 @@ bool AP_Win32App::handleModelessDialogMessage( MSG * msg )
 }
 
 // cmdline processing call back I reckon
-void AP_Win32App::errorMsgBadArg(AP_Args * Args, int nextopt)
+void AP_Win32App::errorMsgBadArg(const char *msg)
 {
-	char *pszMessage = (char*)g_try_malloc( 500 );
-	UT_return_if_fail(pszMessage);
-	strcpy( pszMessage, "Error on option " );
-	strcat( pszMessage, poptBadOption (Args->poptcon, 0) );
-	strcat( pszMessage, ": " );
-	strcat( pszMessage, poptStrerror (nextopt) );
-	strcat( pszMessage, "\nRun with --help' to see a full list of available command line options.\n" );
+	char *pszMessage;
+
+	pszMessage = g_strdup_printf ("%s\nRun with --help' to see a full list of available command line options.\n", msg);
 	MessageBox(NULL, pszMessage, "Command Line Option Error", MB_OK|MB_ICONERROR);
 	g_free( pszMessage );
 }
@@ -1372,9 +1391,9 @@ bool AP_Win32App::doWindowlessArgs(const AP_Args *Args, bool & bSuccess)
 	else
 	if (Args->m_sPrintTo) 
 	{
-		if ((Args->m_sFile = poptGetArg (Args->poptcon)) != NULL)
+		if (Args->m_sFiles[0])
 		{
-			UT_DEBUGMSG(("DOM: Printing file %s\n", Args->m_sFile));
+			UT_DEBUGMSG(("DOM: Printing file %s\n", Args->m_sFiles[0]));
 			AP_Convert conv ;
 
 			if (Args->m_sMerge)
@@ -1386,7 +1405,7 @@ bool AP_Win32App::doWindowlessArgs(const AP_Args *Args, bool & bSuccess)
 				conv.setExpProps (Args->m_expProps);
 			
 			UT_String s = "AbiWord: ";
-			s+= Args->m_sFile;
+			s+= Args->m_sFiles[0];
 			
 			GR_Graphics * pG = GR_Win32Graphics::getPrinterGraphics(Args->m_sPrintTo, s.c_str());
 			if(!pG)
@@ -1397,43 +1416,47 @@ bool AP_Win32App::doWindowlessArgs(const AP_Args *Args, bool & bSuccess)
 			}
 			
 			conv.setVerbose(Args->m_iVerbose);
-			conv.print (Args->m_sFile, pG, Args->m_sFileExtension);
+			conv.print (Args->m_sFiles[0], pG, Args->m_sFileExtension);
 	      
 			delete pG;
 		}
 		else
 		{
 			// couldn't load document
-			printf("Error: no file to print!\n");
+			UT_DEBUGMSG(("Error: no file to print!\n"));
 			bSuccess = false;
 		}
 
 		return false;
 	}
 
-	if(Args->m_sPlugin)
+	if(Args->m_sPluginArgs)
 	{
 	//
 	// Start a plugin rather than the main abiword application.
 	//
-		const char * szName = NULL;
+	    const char * szName = NULL;
 		XAP_Module * pModule = NULL;
-		Args->m_sPlugin = poptGetArg(Args->poptcon);
-		bool bFound = false;
-		if(Args->m_sPlugin != NULL)
+		bool bFound = false;	
+		if(Args->m_sPluginArgs[0])
 		{
-			const UT_GenericVector<class XAP_Module *> *pVec = XAP_ModuleManager::instance().enumModules ();
-			for (UT_uint32 i = 0; (i < pVec->size()) && !bFound; i++)
+			const char * szRequest = Args->m_sPluginArgs[0];
+			const UT_GenericVector<XAP_Module*> * pVec = XAP_ModuleManager::instance().enumModules ();
+			UT_DEBUGMSG((" %d plugins loaded \n",pVec->getItemCount()));
+			for (UT_sint32 i = 0; (i < pVec->size()) && !bFound; i++)
 			{
-				pModule = (XAP_Module *)pVec->getNthItem (i);
+				pModule = pVec->getNthItem (i);
 				szName = pModule->getModuleInfo()->name;
-				if(strcmp(szName,Args->m_sPlugin) == 0)
+				UT_DEBUGMSG(("%s\n", szName));
+				if(strcmp(szName,szRequest) == 0)
+				{
 					bFound = true;
+				}
 			}
 		}
 		if(!bFound)
 		{
-			printf("Plugin %s not found or loaded \n",Args->m_sPlugin);
+			UT_DEBUGMSG(("Plugin %s not found or loaded \n",Args->m_sPluginArgs[0]));
 			bSuccess = false;
 			return false;
 		}
@@ -1447,8 +1470,8 @@ bool AP_Win32App::doWindowlessArgs(const AP_Args *Args, bool & bSuccess)
 		const EV_EditMethod * pInvoke = pEMC->findEditMethodByName(evExecute);
 		if(!pInvoke)
 		{
-			printf("Plugin %s invoke method %s not found \n",
-				   Args->m_sPlugin,evExecute);
+			UT_DEBUGMSG(("Plugin %s invoke method %s not found \n",
+				   Args->m_sPluginArgs[0],evExecute));
 			bSuccess = false;
 			return false;
 		}
