@@ -33,6 +33,7 @@
 #	include "xap_Win32Dlg_MessageBox.h"
 #	include "xap_Win32Dlg_FileOpenSaveAs.h"
 #	include "xap_Win32Dlg_Print.h"
+#       include "xap_Win32Dlg_PrintPreview.h"
 #	include "xap_Win32Dlg_FontChooser.h"
 #	include "xap_Win32Dlg_WindowMore.h"
 #	include "xap_Win32Dlg_About.h"
@@ -52,7 +53,9 @@
 #	include "ap_Win32Dialog_Break.h"
 #	include "ap_Win32Dialog_Options.h"
 #	include "ap_Win32Dialog_Paragraph.h"
+#ifdef ENABLE_SPELL
 #	include "ap_Win32Dialog_Spell.h"
+#endif
 #	include "ap_Win32Dialog_Styles.h"
 #	include "ap_Win32Dialog_Insert_DateTime.h"
 #	include "ap_Win32Dialog_WordCount.h"
@@ -86,6 +89,7 @@
 #endif
 	// ... add new dialogs here ...
 #	include "ap_Win32Dialog_Stylist.h"
+#	include "ap_Win32Dialog_Annotation.h"
 
 #else
 
@@ -96,6 +100,7 @@
 	DeclareDialog(XAP_DIALOG_ID_FILE_EXPORT,	XAP_Win32Dialog_FileOpenSaveAs)
 	DeclareDialog(XAP_DIALOG_ID_INSERT_FILE,	XAP_Win32Dialog_FileOpenSaveAs)
 	DeclareDialog(XAP_DIALOG_ID_PRINT,			XAP_Win32Dialog_Print)
+	DeclareDialog(XAP_DIALOG_ID_PRINTPREVIEW,		XAP_Win32Dialog_PrintPreview)
 	DeclareDialog(XAP_DIALOG_ID_PRINTTOFILE,	XAP_Win32Dialog_FileOpenSaveAs)
 	DeclareDialog(XAP_DIALOG_ID_RECORDTOFILE,	XAP_Win32Dialog_FileOpenSaveAs)
 	DeclareDialog(XAP_DIALOG_ID_REPLAYFROMFILE,	XAP_Win32Dialog_FileOpenSaveAs)
@@ -122,7 +127,9 @@
 	DeclareDialog(AP_DIALOG_ID_REPLACE,			AP_Win32Dialog_Replace)
 	DeclareDialog(AP_DIALOG_ID_FIND,			AP_Win32Dialog_Replace)
 	DeclareDialog(AP_DIALOG_ID_BREAK,			AP_Win32Dialog_Break)
+#ifdef ENABLE_SPELL
 	DeclareDialog(AP_DIALOG_ID_SPELL,			AP_Win32Dialog_Spell)
+#endif
 	DeclareDialog(AP_DIALOG_ID_STYLES,			AP_Win32Dialog_Styles)
 	DeclareDialog(AP_DIALOG_ID_OPTIONS,			AP_Win32Dialog_Options)
 	DeclareDialog(AP_DIALOG_ID_PARAGRAPH,	   	AP_Win32Dialog_Paragraph)
@@ -156,6 +163,7 @@
 	DeclareDialog(AP_DIALOG_ID_DOWNLOAD_FILE,	AP_Win32Dialog_Download_File)
 #endif
  	// ... also add new dialogs here ...
+ 	DeclareDialog(AP_DIALOG_ID_ANNOTATION, 	AP_Win32Dialog_Annotation)
 	DeclareDialog(AP_DIALOG_ID_STYLIST,		AP_Win32Dialog_Stylist)
 	DeclareDialog(AP_DIALOG_ID_LATEX,	    AP_Win32Dialog_Latex)
 	// Remember to place the dialog box with the higher ID
