@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
  *			 (c) 2002-2006 Jordi Mas i Hernndez jmas@softcatala.org
@@ -36,6 +38,7 @@
 #include "xap_Win32App.h"
 #include "xap_Win32FrameImpl.h"
 #include "xap_Prefs.h"
+#include "xap_EncodingManager.h"
 
 #include "ap_Dialog_Id.h"
 #include "ap_Prefs_SchemeIds.h"
@@ -105,8 +108,6 @@ void AP_Win32Dialog_Options::_initEnableControlsPlatformSpecific()
 {
 	_controlEnable( id_CHECK_LANG_WITH_KEYBOARD, true); 
 	_controlEnable( id_CHECK_DIR_MARKER_AFTER_CLOSING_PARENTHESIS,_gatherLanguageWithKeyboard());  
-	//TODO: remove the following line when Windows has support for custom toolbars (Bug 1717)
-	EnableWindow(GetDlgItem((HWND)getPage(PG_GENERAL),AP_RID_DIALOG_OPTIONS_CHK_AllowCustomToolbars),false);
 }
 
 
@@ -132,6 +133,10 @@ void AP_Win32Dialog_Options::runModal(XAP_Frame * pFrame)
 	m_spelling.createPage(pWin32App, AP_RID_DIALOG_OPT_SPELL, AP_STRING_ID_DLG_Options_TabLabel_Spelling);	
 	sheet.addPage(&m_spelling);	
 
+	m_smartquotes.setContainer(this);
+	m_smartquotes.createPage(pWin32App, AP_RID_DIALOG_OPT_SMARTQUOTES, AP_STRING_ID_DLG_Options_TabLabel_SmartQuotes);
+	sheet.addPage(&m_smartquotes);
+
 	sheet.setApplyButton(true);       
 	sheet.setParent(this);	       
 	if (sheet.runModal(pWin32App, pFrame, AP_STRING_ID_DLG_Options_OptionsTitle)==IDOK)	
@@ -184,7 +189,11 @@ HWND	AP_Win32Dialog_Options::getPage(PSH_PAGES page)
 		case PG_DOCUMENT:
 			hWnd = m_document.getHandle();
 			break;				
-			
+
+		case PG_SMARTQUOTES:
+			hWnd = m_smartquotes.getHandle();
+			break;
+
 		default:
 			break;
 	}
@@ -207,16 +216,10 @@ void AP_Win32Dialog_Options::_controlEnable( tControl id, bool value )
 			EnableWindow(GetDlgItem((HWND)getPage(PG_GENERAL),AP_RID_DIALOG_OPTIONS_CHK_LanguageWithKeyboard),value);
 			return;
 
-		case id_CHECK_ALLOW_CUSTOM_TOOLBARS:
-			//TODO: enable this when Windows has support for custom toolbars (Bug 1717)
-			//EnableWindow(GetDlgItem((HWND)getPage(PG_GENERAL),AP_RID_DIALOG_OPTIONS_CHK_AllowCustomToolbars),value);
-			return;			
-		
 		case id_CHECK_AUTO_LOAD_PLUGINS:
 			EnableWindow(GetDlgItem((HWND)getPage(PG_GENERAL),AP_RID_DIALOG_OPTIONS_CHK_AutoLoadPlugins),value);
 			return;
 
-
 		case id_CHECK_AUTO_SAVE_FILE:
 			EnableWindow(GetDlgItem((HWND)getPage(PG_DOCUMENT),AP_RID_DIALOG_OPTIONS_CHK_AutoSaveFile),value);
 			return;			
@@ -224,8 +227,6 @@ void AP_Win32Dialog_Options::_controlEnable( tControl id, bool value )
 		case id_CHECK_OTHER_DEFAULT_DIRECTION_RTL:
 			EnableWindow(GetDlgItem((HWND)getPage(PG_DOCUMENT),AP_RID_DIALOG_OPTIONS_CHK_OtherDirectionRtl),value);
 			return;			
-			
-
 				
 		case id_CHECK_SPELL_CHECK_AS_TYPE:
 			EnableWindow(GetDlgItem((HWND)getPage(PG_SPELL),AP_RID_DIALOG_OPTIONS_CHK_SpellCheckAsType),value);
@@ -254,6 +255,15 @@ void AP_Win32Dialog_Options::_controlEnable( tControl id, bool value )
 		case id_CHECK_GRAMMAR_CHECK:
 			EnableWindow(GetDlgItem((HWND)getPage(PG_SPELL),AP_RID_DIALOG_OPTIONS_CHK_GrammarCheck),value);
 			return;
+
+		case id_CHECK_SMART_QUOTES_ENABLE:
+			EnableWindow(GetDlgItem((HWND)getPage(PG_SMARTQUOTES),AP_RID_DIALOG_OPTIONS_CHK_CustomSmartQuotes),value);
+			return;
+
+		case id_CHECK_CUSTOM_SMART_QUOTES:
+			EnableWindow(GetDlgItem((HWND)getPage(PG_SMARTQUOTES),AP_RID_DIALOG_OPTIONS_COMBO_OUTERQUOTE),value);
+			EnableWindow(GetDlgItem((HWND)getPage(PG_SMARTQUOTES),AP_RID_DIALOG_OPTIONS_COMBO_INNERQUOTE),value);
+			return;
 				
 		default:
 			//UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
@@ -278,17 +288,15 @@ void AP_Win32Dialog_Options::_set##button(const bool b) { \
 }
 
 // dummy implementations
-#define DEFINE_GET_SET_BOOL_DUMMY(Bool)	\
-bool	AP_Win32Dialog_Options::_gather##Bool(void) { \
-		return m_bool##Bool;					\
-} \
-void	AP_Win32Dialog_Options::_set##Bool(bool b) { \
-	m_bool##Bool = b;					\
+#define DEFINE_GET_SET_BOOL_DUMMY(Bool)						\
+bool	AP_Win32Dialog_Options::_gather##Bool(void) { 		\
+		return m_bool##Bool;								\
+} 															\
+void	AP_Win32Dialog_Options::_set##Bool(bool b) { 		\
+	m_bool##Bool = b;										\
 }
 
-
 DEFINE_GET_SET_BOOL(PG_GENERAL,LanguageWithKeyboard)
-DEFINE_GET_SET_BOOL(PG_GENERAL,AllowCustomToolbars)
 DEFINE_GET_SET_BOOL(PG_GENERAL,AutoLoadPlugins)
 
 DEFINE_GET_SET_BOOL(PG_DOCUMENT,OtherDirectionRtl)
@@ -301,7 +309,8 @@ DEFINE_GET_SET_BOOL(PG_SPELL,SpellUppercase)
 DEFINE_GET_SET_BOOL(PG_SPELL,SpellNumbers)
 DEFINE_GET_SET_BOOL(PG_SPELL,GrammarCheck)
 
-
+DEFINE_GET_SET_BOOL(PG_SMARTQUOTES,SmartQuotes)
+DEFINE_GET_SET_BOOL(PG_SMARTQUOTES,CustomSmartQuotes)
 
 /* Not used */
 DEFINE_GET_SET_BOOL_DUMMY (ViewCursorBlink)
@@ -313,7 +322,6 @@ DEFINE_GET_SET_BOOL_DUMMY (ViewShowRuler)
 DEFINE_GET_SET_BOOL_DUMMY (ViewShowStatusBar)
 DEFINE_GET_SET_BOOL_DUMMY (ViewUnprintable)
 
-
 #undef DEFINE_GET_SET_BOOL
 
 
@@ -393,7 +401,7 @@ int AP_Win32Dialog_Options::_gatherNotebookPageNum(void)
 	return 0;
 }
 
-void AP_Win32Dialog_Options::_setNotebookPageNum(const int pn)
+void AP_Win32Dialog_Options::_setNotebookPageNum(const int /*pn*/)
 {
 
 }
@@ -436,6 +444,50 @@ void AP_Win32Dialog_Options::_setUILanguage(const UT_String &stExt)
 		m_curLang = stExt;
 }
 
+void AP_Win32Dialog_Options::_setOuterQuoteStyle(const gint index)
+{
+	HWND hCombo = GetDlgItem((HWND)getPage(PG_SMARTQUOTES), AP_RID_DIALOG_OPTIONS_COMBO_OUTERQUOTE);
+	UT_return_if_fail(hCombo);
+
+	int nCount = SendMessage(hCombo, CB_GETCOUNT, 0, 0);
+	UT_return_if_fail(index >= 0 && index < nCount);
+
+	SendMessage(hCombo, CB_SETCURSEL, index, 0);
+}
+
+void AP_Win32Dialog_Options::_setInnerQuoteStyle(const gint index)
+{
+	HWND hCombo = GetDlgItem((HWND)getPage(PG_SMARTQUOTES), AP_RID_DIALOG_OPTIONS_COMBO_INNERQUOTE);
+	UT_return_if_fail(hCombo);
+
+	int nCount = SendMessage(hCombo, CB_GETCOUNT, 0, 0);
+	UT_return_if_fail(index >= 0 && index < nCount);
+
+	SendMessage(hCombo, CB_SETCURSEL, index, 0);
+}
+
+gint AP_Win32Dialog_Options::_gatherOuterQuoteStyle()
+{
+	HWND hCombo = GetDlgItem((HWND)getPage(PG_SMARTQUOTES), AP_RID_DIALOG_OPTIONS_COMBO_OUTERQUOTE);
+	UT_return_val_if_fail(hCombo, 0);
+
+	int nIndex = SendMessage(hCombo,  CB_GETCURSEL , 0,0);
+	UT_return_val_if_fail(nIndex != CB_ERR, 0);
+
+	return nIndex;
+}
+
+gint AP_Win32Dialog_Options::_gatherInnerQuoteStyle()
+{
+	HWND hCombo = GetDlgItem((HWND)getPage(PG_SMARTQUOTES), AP_RID_DIALOG_OPTIONS_COMBO_INNERQUOTE);
+	UT_return_val_if_fail(hCombo, 0);
+
+	int nIndex = SendMessage(hCombo,  CB_GETCURSEL , 0,0);
+	UT_return_val_if_fail(nIndex != CB_ERR, 0);
+
+	return nIndex;	
+}
+
 
 /*
 
@@ -454,7 +506,7 @@ XAP_Win32PropertySheet()
 /*
 	Sheet window procedure
 */
-int AP_Win32Dialog_Options_Sheet::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
+int AP_Win32Dialog_Options_Sheet::_onCommand(HWND hWnd, WPARAM wParam, LPARAM /*lParam*/)
 {
 	WORD wID = LOWORD(wParam); 
 	
@@ -481,7 +533,7 @@ int AP_Win32Dialog_Options_Sheet::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lP
 	return 1;	// The application did not process the message
 }
 
-int CALLBACK AP_Win32Dialog_Options_Sheet::s_sheetInit(HWND hwnd,  UINT uMsg,  LPARAM lParam)
+int CALLBACK AP_Win32Dialog_Options_Sheet::s_sheetInit(HWND hwnd,  UINT uMsg,  LPARAM /*lParam*/)
 {	
 	if (uMsg==PSCB_INITIALIZED)
 	{	
@@ -489,6 +541,7 @@ int CALLBACK AP_Win32Dialog_Options_Sheet::s_sheetInit(HWND hwnd,  UINT uMsg,  L
 		PropSheet_SetCurSel(hwnd, 0,0);
 		PropSheet_SetCurSel(hwnd, 0,1);
 		PropSheet_SetCurSel(hwnd, 0,2);		
+		PropSheet_SetCurSel(hwnd, 0,3);
 	}			
 	return 	0;
 }
@@ -532,7 +585,7 @@ AP_Win32Dialog_Options_Spelling::~AP_Win32Dialog_Options_Spelling()
 	
 }
 
-void AP_Win32Dialog_Options_Spelling::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
+void AP_Win32Dialog_Options_Spelling::_onCommand(HWND /*hWnd*/, WPARAM /*wParam*/, LPARAM /*lParam*/)
 {
 
 }
@@ -598,7 +651,7 @@ AP_Win32Dialog_Options_General::~AP_Win32Dialog_Options_General()
 {
 	if (m_pVecUILangs)
 	{		
-		for (UT_uint32 i=0; i < m_pVecUILangs->getItemCount(); i++)
+		for (UT_sint32 i=0; i < m_pVecUILangs->getItemCount(); i++)
 			g_free ((void *)m_pVecUILangs->getNthItem(i));
 			
 		delete m_pVecUILangs;		
@@ -608,7 +661,7 @@ AP_Win32Dialog_Options_General::~AP_Win32Dialog_Options_General()
 /*
 	
 */	
-void AP_Win32Dialog_Options_General::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
+void AP_Win32Dialog_Options_General::_onCommand(HWND /*hWnd*/, WPARAM wParam, LPARAM /*lParam*/)
 {
 	WORD wId = LOWORD(wParam);
 	AP_Win32Dialog_Options*	 pParent=  (AP_Win32Dialog_Options*)getContainer();	
@@ -647,7 +700,7 @@ void AP_Win32Dialog_Options_General::_onInitDialog()
 	const gchar *pLangCode;
 	const gchar *pLang;
 	UT_Language	lang;	
-	int	i, nIndex;;
+	int nIndex;
 	const XAP_StringSet * pSS = getApp()->getStringSet();	
 	AP_Win32App * pApp = static_cast<AP_Win32App*>(XAP_App::getApp());
 	HWND hCtrlUILang	= GetDlgItem(getHandle(), AP_RID_DIALOG_OPTIONS_COMBO_UILANG);
@@ -655,7 +708,6 @@ void AP_Win32Dialog_Options_General::_onInitDialog()
 	// localize controls	
 	_DS2(OPTIONS_LBL_UNITS,					DLG_Options_Label_ViewUnits);	
 	_DS2(OPTIONS_BTN_BGColor,				DLG_Options_Label_ChooseForTransparent);
-	_DS2(OPTIONS_CHK_AllowCustomToolbars,	DLG_Options_Label_CheckAllowCustomToolbars);
 	_DS2(OPTIONS_CHK_AutoLoadPlugins,		DLG_Options_Label_CheckAutoLoadPlugins);
 	_DS2(OPTIONS_STATIC_UI,					DLG_Options_Label_UI);
 	_DS2(OPTIONS_STATIC_APPSTARTUP,			DLG_Options_Label_AppStartup);
@@ -665,7 +717,7 @@ void AP_Win32Dialog_Options_General::_onInitDialog()
 	
 	// Populate values in the _COMBO_UNITS
 	HWND hwndAlign = GetDlgItem(getHandle(), AP_RID_DIALOG_OPTIONS_COMBO_UNITS);
-	for(i = 0; i < SIZE_aAlignUnit; i++ )
+	for (UT_uint32 i = 0; i < SIZE_aAlignUnit; i++)
 	{
 		SendMessage(hwndAlign, CB_ADDSTRING, 0, (LPARAM)pSS->getValue(s_aAlignUnit[i].id));
 	}	
@@ -674,7 +726,7 @@ void AP_Win32Dialog_Options_General::_onInitDialog()
 	UINT boldFields[]={AP_RID_DIALOG_OPTIONS_STATIC_UI, AP_RID_DIALOG_OPTIONS_STATIC_APPSTARTUP, 
 		AP_RID_DIALOG_OPTIONS_STATIC_LANGUAGE, 0};
 
-	for(i=0; boldFields[i]; i++) 
+	for(UT_uint32 i = 0; boldFields[i]; i++) 
 	{
 		SendMessage(GetDlgItem(getHandle(), boldFields[i]), WM_SETFONT, 
 			(WPARAM)(AP_Win32Dialog_Options*)getContainer()->getBoldFontHandle(), MAKELPARAM(FALSE /* Redraw */, 0));
@@ -685,7 +737,7 @@ void AP_Win32Dialog_Options_General::_onInitDialog()
 	if (m_pVecUILangs->getItemCount())		
 	{	
 		/* Fill all up languages names for UI*/
-		for (i=0; i < m_pVecUILangs->getItemCount(); i++)
+		for (UT_sint32 i = 0; i < m_pVecUILangs->getItemCount(); i++)
 		{
 			pLangCode = (const char *) m_pVecUILangs->getNthItem(i);
 			
@@ -745,7 +797,7 @@ AP_Win32Dialog_Options_Document::~AP_Win32Dialog_Options_Document()
 /*
 	
 */	
-void AP_Win32Dialog_Options_Document::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
+void AP_Win32Dialog_Options_Document::_onCommand(HWND hWnd, WPARAM wParam, LPARAM /*lParam*/)
 {
 	WORD wId = LOWORD(wParam);	
 	AP_Win32Dialog_Options*	 pParent=  (AP_Win32Dialog_Options*)getContainer();	
@@ -829,4 +881,88 @@ bool AP_Win32Dialog_Options_Document::isAutoSaveInRange()
 }
 
 
+/*
 
+	Smart Quotes page
+	
+*/
+AP_Win32Dialog_Options_SmartQuotes::AP_Win32Dialog_Options_SmartQuotes()
+{
+	
+}
+
+
+AP_Win32Dialog_Options_SmartQuotes::~AP_Win32Dialog_Options_SmartQuotes()
+{
+	
+}
+
+
+/*
+	
+*/	
+void AP_Win32Dialog_Options_SmartQuotes::_onInitDialog()
+{
+	const XAP_StringSet * pSS = getApp()->getStringSet();
+	UT_return_if_fail(pSS);
+
+	// localize controls
+	_DS2(OPTIONS_CHK_SmartQuotes,			DLG_Options_Label_SmartQuotes);
+	_DS2(OPTIONS_CHK_CustomSmartQuotes,		DLG_Options_Label_CustomSmartQuotes);
+	_DS2(OPTIONS_LBL_OuterQuoteStyle,		DLG_Options_Label_OuterQuoteStyle);
+	_DS2(OPTIONS_LBL_InnerQuoteStyle,		DLG_Options_Label_InnerQuoteStyle);
+
+	HWND hComboOuter = GetDlgItem(getHandle(), AP_RID_DIALOG_OPTIONS_COMBO_OUTERQUOTE);
+	UT_return_if_fail(hComboOuter);
+	HWND hComboInner = GetDlgItem(getHandle(), AP_RID_DIALOG_OPTIONS_COMBO_INNERQUOTE);
+	UT_return_if_fail(hComboInner);
+
+	UT_UCSChar buf[4];
+
+	for(size_t i = 0; XAP_EncodingManager::smartQuoteStyles[i].leftQuote != (UT_UCSChar)0; i++)
+	{
+		buf[0] = XAP_EncodingManager::smartQuoteStyles[i].leftQuote;
+		buf[1] = 'O';
+		buf[2] = XAP_EncodingManager::smartQuoteStyles[i].rightQuote;
+		buf[3] = NULL;
+
+		gchar *szDisplayString = g_ucs4_to_utf8(buf, -1, NULL, NULL, NULL);
+		if(szDisplayString)
+		{
+			UT_String tmp = AP_Win32App::s_fromUTF8ToWinLocale(szDisplayString);
+			SendMessage(hComboOuter, CB_ADDSTRING, 0, (LPARAM)tmp.c_str());
+			SendMessage(hComboInner, CB_ADDSTRING, 0, (LPARAM)tmp.c_str());
+			FREEP(szDisplayString);
+		}
+	}
+}
+
+
+/*
+	
+*/	
+void AP_Win32Dialog_Options_SmartQuotes::_onCommand(HWND hWnd, WPARAM wParam, LPARAM /*lParam*/)
+{
+	WORD wId = LOWORD(wParam);
+	
+	switch (wId)
+	{					
+		case AP_RID_DIALOG_OPTIONS_CHK_SmartQuotes:
+		{
+			bool bChecked = (IsDlgButtonChecked( hWnd, AP_RID_DIALOG_OPTIONS_CHK_SmartQuotes ) == BST_CHECKED);
+			EnableWindow( GetDlgItem( hWnd, AP_RID_DIALOG_OPTIONS_CHK_CustomSmartQuotes), bChecked );
+			return;
+		}
+
+		case AP_RID_DIALOG_OPTIONS_CHK_CustomSmartQuotes:
+		{
+			bool bChecked = (IsDlgButtonChecked( hWnd, AP_RID_DIALOG_OPTIONS_CHK_CustomSmartQuotes ) == BST_CHECKED);
+			EnableWindow( GetDlgItem( hWnd, AP_RID_DIALOG_OPTIONS_COMBO_OUTERQUOTE), bChecked );
+			EnableWindow( GetDlgItem( hWnd, AP_RID_DIALOG_OPTIONS_COMBO_INNERQUOTE), bChecked );
+			return;
+		}
+
+		default:
+			break;
+	}
+}
