@@ -48,6 +48,7 @@
 #include "fg_GraphicRaster.h"
 #include "fg_GraphicVector.h"
 #include "ap_Dialog_FormatFrame.h"
+#include "ap_Strings.h"
 #include "ut_png.h"
 #include "gr_Painter.h"
 
@@ -112,7 +113,7 @@ AP_Dialog_FormatFrame::tAnswer AP_Dialog_FormatFrame::getAnswer(void) const
 	return m_answer;
 }
 
-void AP_Dialog_FormatFrame::setActiveFrame(XAP_Frame *pFrame)
+void AP_Dialog_FormatFrame::setActiveFrame(XAP_Frame * /*pFrame*/)
 {
 	notifyActiveFrame(getActiveFrame());
 }
@@ -255,11 +256,19 @@ void AP_Dialog_FormatFrame::askForGraphicPathName(void)
 	DELETEP(m_pImage);
 	m_pGraphic = pFG->clone();
 	GR_Graphics * pG = m_pFormatFramePreview->getGraphics();
+
+	FV_View * pView = static_cast<FV_View *>(pFrame->getCurrentView());
+	UT_return_if_fail(pView && pView->getDocument());
+
+	UT_uint32 uid = pView->getDocument()->getUID(UT_UniqueId::Image); //see Bug 10851
+	m_sImagePath.clear();
+	UT_String_sprintf(m_sImagePath,"%d",uid);
+
 	if(m_pGraphic->getType() == FGT_Raster)
 	{
 		UT_sint32 iImageWidth;
 		UT_sint32 iImageHeight;
-		UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
+		const UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
 		UT_PNG_getDimensions(pBB, iImageWidth, iImageHeight);
 		m_pImage = static_cast<GR_Image *>(
 			pG->createNewImage( m_sImagePath.c_str(),
@@ -503,7 +512,7 @@ void AP_Dialog_FormatFrame::setCurFrameProps(void)
 					{
 						UT_sint32 iImageWidth;
 						UT_sint32 iImageHeight;
-						UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
+						const UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
 						UT_PNG_getDimensions(pBB, iImageWidth, iImageHeight);
 						m_pImage = static_cast<GR_Image *>(
 							pG->createNewImage( m_sImagePath.c_str(),
@@ -881,11 +890,11 @@ static UT_UTF8String s_canonical_thickness (const UT_UTF8String & sThickness, fl
 	UT_UTF8String sThick;
 
 	if (thickness < 0.01) {
-		thickness = 0.01;
+		thickness = 0.01f;
 		sThick = "0.01pt";
 	}
 	else if (thickness > 99.99) {
-		thickness = 99.99;
+		thickness = 99.99f;
 		sThick = "99.99pt";
 	}
 	else {
@@ -1122,7 +1131,7 @@ void AP_FormatFrame_preview::draw(void)
 		const char * szName = pFG->getDataId();
 		if(pFG->getType() == FGT_Raster)
 		{
-			UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
+			const UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
 			pImg = static_cast<GR_Image *>(
 				m_gc->createNewImage( szName,
 									pBB,
