@@ -48,6 +48,7 @@
 #include "ut_png.h"
 #include "gr_Painter.h"
 #include "ut_units.h"
+#include "ap_Strings.h"
 
 AP_Dialog_FormatTable::AP_Dialog_FormatTable(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id id)
 	: XAP_Dialog_Modeless(pDlgFactory,id, "interface/dialogformattable"),
@@ -92,14 +93,14 @@ AP_Dialog_FormatTable::tAnswer AP_Dialog_FormatTable::getAnswer(void) const
 	return m_answer;
 }
 
-void AP_Dialog_FormatTable::setActiveFrame(XAP_Frame *pFrame)
+void AP_Dialog_FormatTable::setActiveFrame(XAP_Frame * /*pFrame*/)
 {
 	notifyActiveFrame(getActiveFrame());
 }
 
 void AP_Dialog_FormatTable::ConstructWindowName(void)
 {
-	const XAP_StringSet * pSS = m_pApp->getStringSet();
+	const XAP_StringSet * pSS = XAP_App::getApp()->getStringSet();
 	gchar * tmp = NULL;
 	UT_uint32 title_width = 26;
 	UT_XML_cloneNoAmpersands(tmp, pSS->getValue(AP_STRING_ID_DLG_FormatTableTitle));
@@ -152,8 +153,7 @@ void AP_Dialog_FormatTable::autoUpdateMC(UT_Worker * pTimer)
  */
 void AP_Dialog_FormatTable::askForGraphicPathName(void)
 {
-	UT_return_if_fail(m_pApp);
-	XAP_Frame * pFrame = m_pApp->getLastFocussedFrame();
+	XAP_Frame * pFrame = XAP_App::getApp()->getLastFocussedFrame();
 
 	UT_return_if_fail(pFrame);
 	XAP_DialogFactory * pDialogFactory
@@ -236,11 +236,19 @@ void AP_Dialog_FormatTable::askForGraphicPathName(void)
 	DELETEP(m_pImage);
 	m_pGraphic = pFG->clone();
 	GR_Graphics * pG = m_pFormatTablePreview->getGraphics();
+
+	FV_View * pView = static_cast<FV_View *>(pFrame->getCurrentView());
+	UT_return_if_fail(pView && pView->getDocument());
+
+	UT_uint32 uid = pView->getDocument()->getUID(UT_UniqueId::Image); //see Bug 10852
+	m_sImagePath.clear();
+	UT_String_sprintf(m_sImagePath,"%d",uid);
+
 	if(m_pGraphic->getType() == FGT_Raster)
 	{
 		UT_sint32 iImageWidth;
 		UT_sint32 iImageHeight;
-		UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
+		const UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
 		UT_PNG_getDimensions(pBB, iImageWidth, iImageHeight);
 		m_pImage = static_cast<GR_Image *>(
 			pG->createNewImage( m_sImagePath.c_str(),
@@ -268,7 +276,7 @@ void AP_Dialog_FormatTable::askForGraphicPathName(void)
 void AP_Dialog_FormatTable::ShowErrorBox(UT_String & sFile, UT_Error errorCode)
 {
 	XAP_String_Id String_id;
-	XAP_Frame * pFrame = m_pApp->getLastFocussedFrame();
+	XAP_Frame * pFrame = XAP_App::getApp()->getLastFocussedFrame();
 	switch (errorCode)
 	  {
 	  case -301:
@@ -321,7 +329,7 @@ void AP_Dialog_FormatTable::ShowErrorBox(UT_String & sFile, UT_Error errorCode)
  */
 void AP_Dialog_FormatTable::setAllSensitivities(void)
 {
-	XAP_Frame *frame = m_pApp->getLastFocussedFrame();
+	XAP_Frame *frame = XAP_App::getApp()->getLastFocussedFrame();
 	if (frame) {
 		FV_View * pView = static_cast<FV_View *>(frame->getCurrentView());
 		setSensitivity(pView->isInTable());
@@ -333,7 +341,7 @@ void AP_Dialog_FormatTable::setAllSensitivities(void)
 
 void AP_Dialog_FormatTable::setCurCellProps(void)
 {
-	XAP_Frame *frame = m_pApp->getLastFocussedFrame();
+	XAP_Frame *frame = XAP_App::getApp()->getLastFocussedFrame();
 	if (frame) {
 		FV_View * pView = static_cast<FV_View *>(frame->getCurrentView());
 
@@ -416,7 +424,7 @@ void AP_Dialog_FormatTable::setCurCellProps(void)
 						{
 							UT_sint32 iImageWidth;
 							UT_sint32 iImageHeight;
-							UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
+							const UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
 							UT_PNG_getDimensions(pBB, iImageWidth, iImageHeight);
 							m_pImage = static_cast<GR_Image *>(
 								pG->createNewImage( m_sImagePath.c_str(),
@@ -471,7 +479,7 @@ void AP_Dialog_FormatTable::applyChanges()
 	if (m_vecProps.getItemCount() == 0)
 		return;
 
-    FV_View * pView = static_cast<FV_View *>(m_pApp->getLastFocussedFrame()->getCurrentView());
+    FV_View * pView = static_cast<FV_View *>(XAP_App::getApp()->getLastFocussedFrame()->getCurrentView());
 	const gchar ** propsArray  = new const gchar * [m_vecProps.getItemCount()+1];
 	propsArray[m_vecProps.getItemCount()] = NULL;
 	
@@ -690,7 +698,7 @@ void AP_FormatTable_preview::draw(void)
 		const char * szName = pFG->getDataId();
 		if(pFG->getType() == FGT_Raster)
 		{
-			UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
+			const UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
 			pImg = static_cast<GR_Image *>(
 				m_gc->createNewImage( szName,
 									pBB,
