@@ -64,7 +64,7 @@ AP_Dialog_Stylist::~AP_Dialog_Stylist(void)
 	DELETEP(m_pStyleTree);
 }
 
-void AP_Dialog_Stylist::setActiveFrame(XAP_Frame *pFrame)
+void AP_Dialog_Stylist::setActiveFrame(XAP_Frame * /*pFrame*/)
 {
 	updateDialog();
 	notifyActiveFrame(getActiveFrame());
@@ -219,7 +219,7 @@ Stylist_tree::Stylist_tree(PD_Document *pDoc)
 
 Stylist_tree::~Stylist_tree(void)
 {
-	UT_DEBUGMSG(("Deleteing Stylist_tree %x \n",this));
+	UT_DEBUGMSG(("Deleteing Stylist_tree %p \n",this));
 	UT_VECTOR_PURGEALL(Stylist_row *, m_vecStyleRows);
 }
 
@@ -436,7 +436,7 @@ bool Stylist_tree::isUser(const PD_Style * pStyle) const
  */
 UT_sint32 Stylist_tree::getNumRows(void)
 {
-	return static_cast<UT_sint32>(m_vecStyleRows.getItemCount());
+	return m_vecStyleRows.getItemCount();
 }
 
 
@@ -499,7 +499,7 @@ UT_sint32 Stylist_tree::getNumCols(UT_sint32 row)
  */
 UT_sint32 Stylist_tree::getNumStyles(void) const
 {
-	return static_cast<UT_sint32>(m_vecAllStyles.getItemCount());
+	return m_vecAllStyles.getItemCount();
 }
 
 /*!
@@ -524,12 +524,12 @@ Stylist_row::Stylist_row(void):
 	m_sRowName("")
 {
 	m_vecStyles.clear();
-	UT_DEBUGMSG(("Creating Stylist_row %x \n",this));
+	UT_DEBUGMSG(("Creating Stylist_row %p \n",this));
 }
 
 Stylist_row::~Stylist_row(void)
 {
-	UT_DEBUGMSG(("Deleteing Stylist_row %x num styles %d\n",this,m_vecStyles.getItemCount()));
+	UT_DEBUGMSG(("Deleteing Stylist_row %p num styles %d\n",this,m_vecStyles.getItemCount()));
 	UT_VECTOR_PURGEALL(UT_UTF8String *, m_vecStyles);
 }
 
@@ -551,7 +551,7 @@ void Stylist_row::getRowName(UT_UTF8String & sRowName)
 
 UT_sint32 Stylist_row::getNumCols(void)
 {
-	return static_cast<UT_sint32>(m_vecStyles.getItemCount());
+	return m_vecStyles.getItemCount();
 }
 
 bool Stylist_row::findStyle(UT_UTF8String & sStyleName, UT_sint32 & col)
