@@ -21,10 +21,12 @@
 
 #include "ut_types.h"
 #include "ap_Frame.h"
+#include "xap_App.h"
 
 #include "ap_Features.h"
 
 #include "ap_FrameData.h"
+#include "ap_Strings.h"
 #include "fv_View.h"
 #include "xav_View.h"
 #include "xad_Document.h"
@@ -64,6 +66,38 @@ void AP_Frame::quickZoom(UT_uint32 iZoom)
 		GR_Graphics * pOldGraphics = pView->getGraphics();
 		pOldGraphics->setZoomPercentage(iZoom);
 		pOldGraphics->clearFont();
+
+		if(pView->getViewMode() == VIEW_WEB)
+		{
+			UT_sint32 iAdjustZoom = pView->calculateZoomPercentForPageWidth();
+			UT_Dimension orig_ut = DIM_IN;
+			orig_ut = pDocLayout->m_docViewPageSize.getDims();
+			double orig_width = pDocLayout->getDocument()->m_docPageSize.Width(orig_ut);
+			double orig_height = pDocLayout->getDocument()->m_docPageSize.Height(orig_ut);
+			double rat = static_cast<double>(iAdjustZoom)/static_cast<double>(iZoom) ;
+			double new_width = orig_width*rat;
+			UT_DEBUGMSG(("AP_Frame zoom VIEW_WEB old width %f new width %f old height %f \n",orig_width,new_width,orig_height));
+			bool p = pDocLayout->m_docViewPageSize.isPortrait();
+			pDocLayout->m_docViewPageSize.Set(new_width,orig_height,orig_ut);
+			pDocLayout->m_docViewPageSize.Set(fp_PageSize::psCustom,orig_ut);
+			if(p)
+			{
+				pDocLayout->m_docViewPageSize.setPortrait();
+			}
+			else
+			{
+				pDocLayout->m_docViewPageSize.setLandscape();
+			}
+			fl_SectionLayout* pSL = pDocLayout->getFirstSection();
+			while (pSL)
+			{
+				pSL->lookupMarginProperties();
+				pSL = static_cast<fl_SectionLayout *>(pSL->getNext());
+			}
+			pView->rebuildLayout();
+			pDocLayout->formatAll();
+
+		}
 		AP_TopRuler * pTop = pView->getTopRuler();
 		if(pTop)
 		{
@@ -74,14 +108,9 @@ void AP_Frame::quickZoom(UT_uint32 iZoom)
 		{
 			pLeft->setZoom(iZoom);
 		}
+		pView->calculateNumHorizPages();
 		setYScrollRange();
 		setXScrollRange();
-//
-// Redraw the entire screen
-//
-		pView->updateScreen(false);
-		pView->setPoint(pView->getPoint()); // place the cursor correctly
-		                                    // on the screen.
 		if(pTop && !pTop->isHidden())
 		{
 			pTop->draw(NULL);
@@ -90,6 +119,12 @@ void AP_Frame::quickZoom(UT_uint32 iZoom)
 		{
 			pLeft->draw(NULL);
 		}
+//
+// Redraw the entire screen
+//
+		pView->setPoint(pView->getPoint()); // place the cursor correctly
+		pView->ensureInsertionPointOnScreen(); // on the screen
+		pView->updateScreen(false);
 	}
 	else
 	{
@@ -162,18 +197,18 @@ UT_Error AP_Frame::_loadDocument(const char * szFilename, IEFileType ieft,
 	{
 		XAP_App::getApp()->rememberFrame(this);
 	}
-	AD_Document * pNewDoc = new PD_Document(XAP_App::getApp());
+	AD_Document * pNewDoc = new PD_Document();
 	UT_return_val_if_fail (pNewDoc, UT_ERROR);
-	
+	UT_Error errorCode = UT_OK;
+
 	if (!szFilename || !*szFilename)
 	{
 		pNewDoc->newDocument();
 		m_iUntitled = _getNextUntitledNumber();
 		goto ReplaceDocument;
 	}
-	UT_Error errorCode;
 	errorCode = pNewDoc->readFromFile(szFilename, ieft);
-	if (!errorCode)
+	if (UT_IS_IE_SUCCESS(errorCode))
 		goto ReplaceDocument;
 
 	if (createNew)
@@ -200,7 +235,7 @@ UT_Error AP_Frame::_loadDocument(const char * szFilename, IEFileType ieft,
 			{
 				errorCode = 0;
 			}
-			UT_DEBUGMSG(("errocode after save is \n",errorCode));
+			UT_DEBUGMSG(("errocode after save is %d\n",errorCode));
 		}
 	  }
 	if (!errorCode)
@@ -216,7 +251,7 @@ ReplaceDocument:
 	
 	// NOTE: prior document is discarded in _showDocument()
 	m_pDoc = pNewDoc;
-	return UT_OK;
+	return errorCode;
 }
 
 UT_Error AP_Frame::_loadDocument(GsfInput * input, IEFileType ieft)
@@ -236,7 +271,7 @@ UT_Error AP_Frame::_loadDocument(GsfInput * input, IEFileType ieft)
 	{
 		XAP_App::getApp()->rememberFrame(this);
 	}
-	AD_Document * pNewDoc = new PD_Document(XAP_App::getApp());
+	AD_Document * pNewDoc = new PD_Document();
 	UT_return_val_if_fail (pNewDoc, UT_ERROR);
 	
 	UT_Error errorCode;
@@ -271,7 +306,7 @@ UT_Error AP_Frame::_importDocument(const char * szFilename, int ieft,
 	// load a document into the current frame.
 	// if no filename, create a new document.
 
-	AD_Document * pNewDoc = new PD_Document(XAP_App::getApp());
+	AD_Document * pNewDoc = new PD_Document();
 	UT_return_val_if_fail (pNewDoc, UT_ERROR);
 
 	if (!szFilename || !*szFilename)
@@ -303,7 +338,7 @@ XAP_Frame * AP_Frame::buildFrame(XAP_Frame * pF)
 {
 	UT_Error error = UT_OK;
 	AP_Frame * pClone = static_cast<AP_Frame *>(pF);
-	XAP_Frame::tZoomType iZoomType = pF->getZoomType();;
+	XAP_Frame::tZoomType iZoomType = pF->getZoomType();
 	setZoomType(iZoomType);
 	UT_uint32 iZoom = XAP_Frame::getZoomPercentage();
 	ENSUREP_C(pClone);
@@ -334,7 +369,7 @@ UT_Error AP_Frame::loadDocument(AD_Document* pDoc) {
 	bool bUpdateClones;
 	UT_GenericVector<XAP_Frame*> vClones;
 	XAP_App * pApp = XAP_App::getApp();
-	UT_uint32 j = 0;
+	UT_sint32 j = 0;
 	if(pApp->findFrame(this) < 0)
 	{
 			pApp->rememberFrame(this);
@@ -361,7 +396,7 @@ UT_Error AP_Frame::loadDocument(GsfInput * input, int ieft)
 	bool bUpdateClones;
 	UT_GenericVector<XAP_Frame*> vClones;
 	XAP_App * pApp = XAP_App::getApp();
-	UT_uint32 j = 0;
+	UT_sint32 j = 0;
 	if(pApp->findFrame(this) < 0)
 	{
 			pApp->rememberFrame(this);
@@ -381,7 +416,7 @@ UT_Error AP_Frame::loadDocument(GsfInput * input, int ieft)
 	}
 	UT_Error errorCode;
 	errorCode =  _loadDocument(input, static_cast<IEFileType>(ieft));
-	if (errorCode)
+	if (!UT_IS_IE_SUCCESS(errorCode))
 	{
 		// we could not load the document.
 		// we cannot complain to the user here, we don't know
@@ -398,7 +433,7 @@ UT_Error AP_Frame::loadDocument(GsfInput * input, int ieft)
 	}
 	if (bUpdateClones)
 	{
-		for (UT_uint32 i = 0; i < vClones.getItemCount(); i++)
+		for (UT_sint32 i = 0; i < vClones.getItemCount(); i++)
 		{
 			AP_Frame * pFrame = static_cast<AP_Frame*>(vClones.getNthItem(i));
 			if(pFrame != this)
@@ -416,7 +451,7 @@ UT_Error AP_Frame::loadDocument(const char * szFilename, int ieft, bool createNe
 	bool bUpdateClones;
 	UT_GenericVector<XAP_Frame*> vClones;
 	XAP_App * pApp = XAP_App::getApp();
-	UT_uint32 j = 0;
+	UT_sint32 j = 0;
 	if(pApp->findFrame(this) < 0)
 	{
 			pApp->rememberFrame(this);
@@ -436,7 +471,7 @@ UT_Error AP_Frame::loadDocument(const char * szFilename, int ieft, bool createNe
 	}
 	UT_Error errorCode;
 	errorCode =  _loadDocument(szFilename, static_cast<IEFileType>(ieft), createNew);
-	if (errorCode)
+	if (!UT_IS_IE_SUCCESS(errorCode))
 	{
 		// we could not load the document.
 		// we cannot complain to the user here, we don't know
@@ -453,7 +488,7 @@ UT_Error AP_Frame::loadDocument(const char * szFilename, int ieft, bool createNe
 	}
 	if (bUpdateClones)
 	{
-		for (UT_uint32 i = 0; i < vClones.getItemCount(); i++)
+		for (UT_sint32 i = 0; i < vClones.getItemCount(); i++)
 		{
 			AP_Frame * pFrame = static_cast<AP_Frame*>(vClones.getNthItem(i));
 			if(pFrame != this)
@@ -463,7 +498,12 @@ UT_Error AP_Frame::loadDocument(const char * szFilename, int ieft, bool createNe
 		}
 	}
 
-	return _showDocument(iZoom);
+	UT_Error errorCode2 =  _showDocument(iZoom);
+    if((errorCode2 == UT_OK) && (errorCode == UT_IE_TRY_RECOVER)) 
+    {
+        return errorCode;
+    }
+    return errorCode2;
 }
 
 UT_Error AP_Frame::loadDocument(const char * szFilename, int ieft)
@@ -484,14 +524,14 @@ UT_Error AP_Frame::importDocument(const char * szFilename, int ieft, bool markCl
 	}
 	UT_Error errorCode;
 	errorCode =  _importDocument(szFilename, static_cast<IEFileType>(ieft), markClean);
-	if (errorCode)
+	if (!UT_IS_IE_SUCCESS(errorCode))
 	{
 		return errorCode;
 	}
 
 	if (bUpdateClones)
 	{
-		for (UT_uint32 i = 0; i < vClones.getItemCount(); i++)
+		for (UT_sint32 i = 0; i < vClones.getItemCount(); i++)
 		{
 			AP_Frame * pFrame = static_cast<AP_Frame *>(vClones.getNthItem(i));
 			if(pFrame != this)
@@ -503,7 +543,12 @@ UT_Error AP_Frame::importDocument(const char * szFilename, int ieft, bool markCl
 	XAP_Frame::tZoomType iZoomType;
 	UT_uint32 iZoom = getNewZoom(&iZoomType);
 	setZoomType(iZoomType);
-	return _showDocument(iZoom);
+	UT_Error errorCode2 =  _showDocument(iZoom);
+    if((errorCode2 == UT_OK) && (errorCode == UT_IE_TRY_RECOVER)) 
+    {
+        return errorCode;
+    }
+    return errorCode2;
 }
 
 /*!
@@ -547,11 +592,11 @@ UT_uint32 AP_Frame::getNewZoom(XAP_Frame::tZoomType * tZoom)
 			return iZoom;
 		}
 		XAP_App::getApp()->getClones(&vecClones,this);
-		UT_uint32 i =0;
+		UT_sint32 i =0;
 		bool bMatch = false;
 		for (i=0; !bMatch && (i< vecClones.getItemCount()); i++)
 		{
-			XAP_Frame *pF = static_cast<XAP_Frame *>(vecClones.getNthItem(i));
+			pF = static_cast<XAP_Frame *>(vecClones.getNthItem(i));
 			bMatch = (pF == pLastFrame);
 		}
 		if(bMatch)
