@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998-2000 AbiSource, Inc.
  *
@@ -191,7 +193,7 @@ ap_bs_Mouse MouseTable[] =
 //	Button-2, Text-context
 //  { context	{ click				doubleclick		drag,		dbldrag,		release,	doublerelease	}},
 
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 // special key binding for X11-style middle mouse selection (not to be confused with the clipboard)
 	{_CT _B2,	{ "pasteSelection",	"",				"",			"",				"",			""				}},
 	{_CH _B2,	{ "pasteSelection",	"",				"",			"",				"",			""				}},
@@ -495,7 +497,7 @@ ap_bs_Char CharTable[] =
 	{0x3a, /* :      */ { "insertData",			"",					"",				""					}},
 	{0x3b, /* ;      */ { "insertData",			"",					"",				""					}},
 	{0x3c, /* <      */ { "insertData",			"fontSizeDecrease",	"",				"insertRLM"			}},
-	{0x3d, /* =      */ { "insertData",			"toggleSuper",		"",				""					}},
+	{0x3d, /* =      */ { "insertData",			"toggleSuper",		"toggleSub",	""					}},
 	{0x3e, /* >      */ { "insertData",			"fontSizeIncrease",	"",				"insertLRM"			}},
 	{0x3f, /* ?      */ { "insertData",			"",					"",				""					}},
 	{0x40, /* @      */ { "insertData",			"",					"",				""					}},
@@ -510,7 +512,7 @@ ap_bs_Char CharTable[] =
 	{0x49, /* I      */ { "insertData",			"toggleItalic",		DO_NOT_USE,		""					}},
 	{0x4a, /* J      */ { "insertData",			"alignJustify",		DO_NOT_USE,		""					}},
 	{0x4b, /* K      */ { "insertData",			"toggleStrike",		DO_NOT_USE,		""					}},
-	{0x4c, /* L      */ { "insertData",			"alignLeft",		DO_NOT_USE,		""					}},
+	{0x4c, /* L      */ { "insertData",			"doBullets",		DO_NOT_USE,		""					}},
 	{0x4d, /* M      */ { "insertData",			"insSymbol",		DO_NOT_USE,		""					}},
 	{0x4e, /* N      */ { "insertData",			"fileNewUsingTemplate",	DO_NOT_USE,		""					}},
 	{0x4f, /* O      */ { "insertData",			"fileOpen",			DO_NOT_USE,		""					}},
