@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
  *
@@ -17,6 +19,10 @@
  * 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <glib.h>
 #include <string.h>
 #include <stdlib.h>
@@ -44,7 +50,7 @@
 #include "pd_Document.h"
 #include "ut_Script.h"
 
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 #include "spell_manager.h"
 #endif
 
@@ -52,7 +58,7 @@
 #include "fp_TableContainer.h"
 #include "fl_BlockLayout.h"
 
-#ifdef _WIN32
+#ifdef TOOLKIT_WIN
 #include "ap_Win32App.h" 
 #endif
 
@@ -69,7 +75,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Toolbar)
 	UT_ASSERT_HARMLESS(id >= AP_MENU_ID_VIEW_TB_1);
 	UT_ASSERT_HARMLESS(id <= AP_MENU_ID_VIEW_TB_4);
 
-	UT_uint32 ndx = (id - AP_MENU_ID_VIEW_TB_1);
+	UT_sint32 ndx = (id - AP_MENU_ID_VIEW_TB_1);
 	const UT_Vector & vec = pApp->getToolbarFactory()->getToolbarNames();
 
 
@@ -99,7 +105,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Recent)
 
 	UT_ASSERT_HARMLESS(id >= AP_MENU_ID_FILE_RECENT_1);
 
-	UT_uint32 ndx = (id - AP_MENU_ID_FILE_RECENT_1 + 1);
+	UT_sint32 ndx = (id - AP_MENU_ID_FILE_RECENT_1 + 1);
 
 	XAP_Prefs * pPrefs = pApp->getPrefs();
 	UT_return_val_if_fail (pPrefs, NULL);
@@ -111,11 +117,13 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Recent)
 
 		const char * szURI = pPrefs->getRecent(ndx);
 		char *szRecent = g_filename_from_uri(szURI, NULL, NULL);
-		char *szFile = g_path_get_basename (szRecent);
+		char *szFile = NULL;
+		UT_UTF8String sFile(szRecent ? szFile = g_path_get_basename (szRecent) : "");
 
-		snprintf(buf, PATH_MAX, szFormat, szFile);
+		snprintf(buf, PATH_MAX, szFormat, sFile.utf8_str());
 		g_free (szRecent);
-		g_free (szFile);
+		if (szFile)
+		    g_free(szFile);
 		return buf;
 	}
 
@@ -251,7 +259,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Window)
 	UT_ASSERT_HARMLESS(id >= AP_MENU_ID_WINDOW_1);
 	UT_ASSERT_HARMLESS(id <= AP_MENU_ID_WINDOW_9);
 
-	UT_uint32 ndx = (id - AP_MENU_ID_WINDOW_1);
+	UT_sint32 ndx = (id - AP_MENU_ID_WINDOW_1);
 
 	// use the applications window list and compute a menu label
 	// for the window with the computed index.  use the static
@@ -265,9 +273,9 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Window)
 		XAP_Frame * pFrame = pApp->getFrame(ndx);
 		UT_return_val_if_fail (pFrame, NULL);
 
-		const char * szTitle = pFrame->getTitle(128 - strlen(szFormat));
-
-		sprintf(buf,szFormat,szTitle);
+		memset(buf, 0, sizeof(buf));
+		snprintf(buf, sizeof(buf), szFormat, pFrame->getTitle().utf8_str());
+		buf[sizeof(buf) - 1] = '\0';
 		return buf;
 	}
 
@@ -298,9 +306,11 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_WindowMore)
 /*****************************************************************/
 /*****************************************************************/
 
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Spelling)
 {
+  UT_UNUSED(pAV_View);
+  UT_UNUSED(id);
   EV_Menu_ItemState s = EV_MIS_ZERO ;
 
   XAP_Prefs *pPrefs = XAP_App::getApp()->getPrefs();
@@ -322,6 +332,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Spelling)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_ColumnsActive)
 {
+  UT_UNUSED(id);
   ABIWORD_VIEW ;
   UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -335,6 +346,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_ColumnsActive)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_BookmarkOK)
 {
+	UT_UNUSED(id);
   ABIWORD_VIEW ;
   UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -366,6 +378,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_BookmarkOK)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_TOCOK)
 {
+  UT_UNUSED(id);
   ABIWORD_VIEW ;
   UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -404,10 +417,18 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_TOCOK)
   {
     s = EV_MIS_Gray;
   }
+  else if(pView->isInAnnotation())
+  {
+    s = EV_MIS_Gray;
+  }
   else if(pView->isInFootnote(pView->getSelectionAnchor()))
   {
     s = EV_MIS_Gray;
   }
+  else if(pView->isInAnnotation(pView->getSelectionAnchor()))
+  {
+    s = EV_MIS_Gray;
+  }
   else if(pView->isInEndnote())
   {
     s = EV_MIS_Gray;
@@ -420,6 +441,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_TOCOK)
   {
     s = EV_MIS_Gray;
   }
+  else if(pView->isInTable() && (pView->getPoint() > 3) && pView->isInAnnotation(pView->getPoint()-2))
+  {
+    s = EV_MIS_Gray;
+  }
   else if(pView->isInTable()  && (pView->getPoint() > 3) && pView->isInEndnote(pView->getPoint()-2))
   {
     s = EV_MIS_Gray;
@@ -437,6 +462,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_TOCOK)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_SomethingSelected)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW ;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -453,6 +479,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_SomethingSelected)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_TextToTableOK)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW ;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -469,6 +496,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_TextToTableOK)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_HyperlinkOK)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW ;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -524,7 +552,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_HyperlinkOK)
 	return s ;
 }
 
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Suggest)
 {
 	ABIWORD_VIEW;
@@ -691,6 +719,8 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Changes)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_ScriptsActive)
 {
+  UT_UNUSED(pAV_View);
+  UT_UNUSED(id);
   EV_Menu_ItemState s = EV_MIS_ZERO;
 
   UT_ScriptLibrary * instance = UT_ScriptLibrary::instance ();
@@ -719,6 +749,8 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Selection)
 	case AP_MENU_ID_EDIT_CUT:
 	case AP_MENU_ID_EDIT_LATEXEQUATION:
 	case AP_MENU_ID_EDIT_COPY:
+	// RIVERA
+	case AP_MENU_ID_TOOLS_ANNOTATIONS_INSERT_FROMSEL:
 		if (pView->isSelectionEmpty())
 			s = EV_MIS_Gray;
 		break;
@@ -784,6 +816,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Prefs)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_FmtHdrFtr)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 	EV_Menu_ItemState s = EV_MIS_ZERO;
@@ -1150,14 +1183,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_LOCK_TB_LAYOUT:
-		if ( !pApp->areToolbarsCustomizable() )
-			s = EV_MIS_Toggled;
-		else
-			s = EV_MIS_ZERO;
+		s = EV_MIS_Toggled;
 		break;
 	case AP_MENU_ID_VIEW_DEFAULT_TB_LAYOUT:
-		if ( !pApp->areToolbarsCustomizable() || !pApp->areToolbarsCustomized() )
-			s = EV_MIS_Gray;
+		s = EV_MIS_Gray;
 		break;
 	case AP_MENU_ID_VIEW_STATUSBAR:
               if ( pFrameData->m_bShowStatusBar &&
@@ -1196,7 +1225,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 	  else
 	    s = EV_MIS_ZERO;
 	  break;
-
+	
 	default:
 		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
@@ -1207,11 +1236,12 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InsTextBox)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
-        if(pView->getViewMode() == VIEW_NORMAL)
-		{
+        if((pView->getViewMode() == VIEW_NORMAL) || (pView->getViewMode() == VIEW_WEB))
+	{
             return EV_MIS_Gray;
         }
 
@@ -1220,6 +1250,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InsTextBox)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_StylesLocked)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1232,6 +1263,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_StylesLocked)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_History)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 	PD_Document * pDoc = pView->getDocument();
@@ -1246,6 +1278,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_History)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisionsCheck)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1253,6 +1286,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisionsCheck)
 	{
 		return EV_MIS_Gray;
 	}
+	if(pView->getDocument()->isConnected())
+	{
+		return EV_MIS_Gray;
+	}
 
 	if(pView->isMarkRevisions())
 	{
@@ -1264,6 +1301,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisionsCheck)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisions)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1271,7 +1309,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisions)
 	{
 		return EV_MIS_Gray;
 	}
-
+	if(pView->getDocument()->isConnected())
+	{
+		return EV_MIS_Gray;
+	}
 	if(pView->isMarkRevisions())
 	{
 		return EV_MIS_ZERO;
@@ -1282,6 +1323,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisions)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionsSelectLevel)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1295,6 +1337,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionsSelectLevel)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_HasRevisions)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1306,8 +1349,13 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_HasRevisions)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_AutoRevision)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
+	if(pView->getDocument()->isConnected())
+	{
+		return EV_MIS_Gray;
+	}
 
 	if(pView->getDocument()->isAutoRevisioning())
 	{
@@ -1319,6 +1367,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_AutoRevision)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisions)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1326,6 +1375,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisions)
 	{
 		return EV_MIS_Gray;
 	}
+	if(pView->getDocument()->isConnected())
+	{
+		return EV_MIS_Gray;
+	}
 	   
 	if(pView->getDocument()->getHighestRevisionId() == 0)
 		return EV_MIS_Gray;
@@ -1340,6 +1393,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisions)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsAfter)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1347,6 +1401,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsAfter)
 	{
 		return EV_MIS_Gray;
 	}
+	if(pView->getDocument()->isConnected())
+	{
+		return EV_MIS_Gray;
+	}
 
 	if(pView->getDocument()->getHighestRevisionId() == 0)
 		return EV_MIS_Gray;
@@ -1368,6 +1426,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsAfter)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsAfterPrev)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1394,6 +1453,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsAfterPrev)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsBefore)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1421,6 +1481,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsBefore)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresent)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1434,6 +1495,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresent)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresentContext)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1445,6 +1507,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresentContext)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InTable)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1456,6 +1519,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InTable)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_PointInTable)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1467,6 +1531,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_PointInTable)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_PointOrAnchorInTable)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 	bool bP = pView->isInTable(pView->getPoint());
@@ -1480,6 +1545,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_PointOrAnchorInTable)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InTableIsRepeat)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1498,6 +1564,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InTableIsRepeat)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_TableOK)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1513,6 +1580,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_TableOK)
 	{
 		return EV_MIS_Gray;
 	}
+	else if(pView->isInAnnotation())
+	{
+		return EV_MIS_Gray;
+	}
 	else if(pView->isInEndnote())
 	{
 		return EV_MIS_Gray;
@@ -1535,6 +1606,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_TableOK)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InTOC)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 	if(pView->isTOCSelected())
@@ -1546,6 +1618,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InTOC)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InTableMerged)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
@@ -1559,6 +1632,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InTableMerged)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InFootnote)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 	if(pView->getHyperLinkRun(pView->getPoint()) != NULL)
@@ -1573,7 +1647,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InFootnote)
 	{
 	        return EV_MIS_Gray;
 	}
-	if(!pView->isInFootnote() && !pView->isHdrFtrEdit() && !pView->isInHdrFtr(pView->getPoint()) && !pView->isInFrame(pView->getPoint()) 
+	if(!pView->isInFootnote() && !pView->isInAnnotation() && !pView->isHdrFtrEdit() && !pView->isInHdrFtr(pView->getPoint()) && !pView->isInFrame(pView->getPoint()) 
 		&& !pView->isTOCSelected())
 	{
 		return EV_MIS_ZERO;
@@ -1582,8 +1656,59 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InFootnote)
 
 }
 
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InAnnotation)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+	if((id==AP_MENU_ID_TOOLS_ANNOTATIONS_INSERT_FROMSEL) &&
+	   (pView->isSelectionEmpty()))
+	{
+		return EV_MIS_Gray;
+	}
+	PT_DocPosition point = pView->getPoint();
+	PT_DocPosition anchor = pView->getSelectionAnchor();
+	if((pView->getHyperLinkRun(point) != NULL) || (pView->getHyperLinkRun(anchor) != NULL))
+	{
+		return EV_MIS_Gray;
+	}
+	else if((pView->getEmbedDepth(point) > 0) || (pView->getEmbedDepth(anchor) > 0))
+	{
+		return EV_MIS_Gray;
+	}
+	else if(pView->getFrameEdit()->isActive())
+	{
+	        return EV_MIS_Gray;
+	}
+	if(!pView->isInFootnote() &&!pView->isInAnnotation() && !pView->isHdrFtrEdit() && !pView->isInHdrFtr(point) && !pView->isInFrame(point)  && !pView->isInFrame(anchor) 
+		&& !pView->isTOCSelected())
+	{
+		return EV_MIS_ZERO;
+	}
+	return EV_MIS_Gray;
+
+}
+
+// RIVERA
+Defun_EV_GetMenuItemState_Fn(ap_GetState_ToggleAnnotations)
+{
+	UT_UNUSED(id);
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+	XAP_App *pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp, EV_MIS_Gray);
+	XAP_Prefs * pPrefs = pApp->getPrefs();
+	UT_return_val_if_fail (pPrefs, EV_MIS_Gray);
+	XAP_PrefsScheme * pScheme = pPrefs->getCurrentScheme(true);
+	UT_return_val_if_fail(pScheme, EV_MIS_Gray);
+	bool b = false;
+	pScheme->getValueBool(static_cast<const gchar *>(AP_PREF_KEY_DisplayAnnotations), &b );
+	return (b ? EV_MIS_Toggled : EV_MIS_ZERO);
+}
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InImage)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 	if(pView->isImageSelected())
@@ -1604,6 +1729,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InImage)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_SetPosImage)
 {
+	UT_UNUSED(id);
         ABIWORD_VIEW;
   	UT_return_val_if_fail (pView, EV_MIS_Gray);
 	bool bCont= false;
@@ -1611,7 +1737,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_SetPosImage)
 	{
 		bCont = true;
 	}
-	if(!bCont & pView->getFrameEdit()->isActive())
+	if(!bCont && pView->getFrameEdit()->isActive())
 	{
 	        fl_FrameLayout * pFL = pView->getFrameLayout();
 		if(pFL && pFL->getFrameType() == FL_FRAME_TEXTBOX_TYPE)
@@ -1633,6 +1759,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_SetPosImage)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InFrame)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 	if(pView->isInFrame(pView->getPoint()))
@@ -1645,6 +1772,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InFrame)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_BreakOK)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 	
@@ -1656,6 +1784,14 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_BreakOK)
 	{
 		return EV_MIS_Gray;
 	}
+	if(pView->isInAnnotation())
+	{
+		return EV_MIS_Gray;
+	}
+	if(pView->isInAnnotation(pView->getSelectionAnchor()))
+	{
+		return EV_MIS_Gray;
+	}
 	if(pView->isInEndnote())
 	{
 		return EV_MIS_Gray;
@@ -1705,11 +1841,15 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_BreakOK)
 // HACK TO ALWAYS DISABLE A MENU ITEM... DELETE ME
 Defun_EV_GetMenuItemState_Fn(ap_GetState_AlwaysDisabled)
 {
+	UT_UNUSED(pAV_View);
+	UT_UNUSED(id);
     return EV_MIS_Gray;
 }
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Recent)
 {
+	UT_UNUSED(pAV_View);
+	UT_UNUSED(id);
   // ABIWORD_VIEW;
   // UT_return_val_if_fail(pView, EV_MIS_ZERO);
 
@@ -1769,6 +1909,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Zoom)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Lists)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail(pView, EV_MIS_ZERO);
 
@@ -1782,6 +1923,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Lists)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_MailMerge)
 {
+	UT_UNUSED(id);
 	ABIWORD_VIEW;
 	UT_return_val_if_fail(pView, EV_MIS_ZERO);
 	
