@@ -19,6 +19,10 @@
  * 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "ap_Features.h"
 #ifdef APF_MENU_LAYOUTS_MAIN_MENU
 #  include APF_MENU_LAYOUTS_MAIN_MENU
@@ -38,7 +42,7 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_FILE_NEW_USING_TEMPLATE)
 		MenuItem(AP_MENU_ID_FILE_OPEN)
 	
-#if !XP_SIMPLE_MENU	
+#if !XAP_SIMPLE_MENU	
 		MenuItem(AP_MENU_ID_FILE_IMPORTSTYLES)
 #endif	
 		Separator()
@@ -48,17 +52,19 @@ BeginLayout(Main,0)
 //		MenuItem(AP_MENU_ID_FILE_IMPORT)
 		MenuItem(AP_MENU_ID_FILE_EXPORT)
 		MenuItem(AP_MENU_ID_FILE_REVERT)
-#if !XP_SIMPLE_MENU
+#ifdef ENABLE_PRINT
 		Separator()
 		MenuItem(AP_MENU_ID_FILE_PAGESETUP)
-#ifdef XP_UNIX_TARGET_GTK
+#if defined(TOOLKIT_GTK) || defined (TOOLKIT_WIN)
 		MenuItem(AP_MENU_ID_FILE_PRINT_PREVIEW)
 #endif
 		MenuItem(AP_MENU_ID_FILE_PRINT)
+#endif
+#if !XAP_SIMPLE_MENU
 		Separator()
 		MenuItem(AP_MENU_ID_FILE_PROPERTIES)
 		Separator()
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 		// GNOME HIG style recent files 
 		MenuItem(AP_MENU_ID_FILE_RECENT_1)
 		MenuItem(AP_MENU_ID_FILE_RECENT_2)
@@ -79,10 +85,10 @@ BeginLayout(Main,0)
 #endif
 		Separator()
 #endif
-#if  !XP_SIMPLE_MENU
+#if  !XAP_SIMPLE_MENU
 		MenuItem(AP_MENU_ID_FILE_CLOSE)
-#endif
 		MenuItem(AP_MENU_ID_FILE_EXIT)
+#endif
 	EndSubMenu()
 
 	BeginSubMenu(AP_MENU_ID_EDIT)
@@ -92,23 +98,23 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_EDIT_CUT)
 		MenuItem(AP_MENU_ID_EDIT_COPY)
 		MenuItem(AP_MENU_ID_EDIT_PASTE)
-#if !XP_SIMPLE_MENU	
+#if !XAP_SIMPLE_MENU	
 		MenuItem(AP_MENU_ID_EDIT_PASTE_SPECIAL)
 #endif	
 		Separator()
-#if !XP_SIMPLE_MENU	
+#if !XAP_SIMPLE_MENU	
 		MenuItem(AP_MENU_ID_EDIT_CLEAR)
 #endif	
 		MenuItem(AP_MENU_ID_EDIT_SELECTALL)
 		Separator()
-#if !XP_SIMPLE_MENU	
+#if !XAP_SIMPLE_MENU	
 		MenuItem(AP_MENU_ID_EDIT_REMOVEHEADER)
 		MenuItem(AP_MENU_ID_EDIT_REMOVEFOOTER)
 		Separator()
 #endif	
 		MenuItem(AP_MENU_ID_EDIT_FIND)
 		MenuItem(AP_MENU_ID_EDIT_REPLACE)
-#if !XP_SIMPLE_MENU	
+#if !XAP_SIMPLE_MENU	
 		MenuItem(AP_MENU_ID_EDIT_GOTO)
 #endif	
 #if !(XAP_PREFSMENU_UNDER_TOOLS)
@@ -120,10 +126,10 @@ BeginLayout(Main,0)
 	BeginSubMenu(AP_MENU_ID_VIEW)
 		MenuItem(AP_MENU_ID_VIEW_NORMAL)
 		MenuItem(AP_MENU_ID_VIEW_WEB)
-	  MenuItem(AP_MENU_ID_VIEW_PRINT)
-#if !XP_SIMPLE_MENU        
+		MenuItem(AP_MENU_ID_VIEW_PRINT)
+#if !XAP_SIMPLE_MENU        
 		MenuItem(AP_MENU_ID_WEB_WEBPREVIEW)
-	  Separator()
+		Separator()
 		BeginSubMenu(AP_MENU_ID_VIEW_TOOLBARS)
 			MenuItem(AP_MENU_ID_VIEW_TB_1)
 			MenuItem(AP_MENU_ID_VIEW_TB_2)
@@ -137,10 +143,10 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_VIEW_LOCKSTYLES)
 #endif
       
-    MenuItem(AP_MENU_ID_VIEW_SHOWPARA)
-	  Separator()
+		MenuItem(AP_MENU_ID_VIEW_SHOWPARA)
+		Separator()
 
-#if !XP_SIMPLE_MENU
+#if !XAP_SIMPLE_MENU
 		MenuItem(AP_MENU_ID_VIEW_FULLSCREEN)
 #endif  
 		BeginSubMenu(AP_MENU_ID_VIEW_ZOOM_MENU)
@@ -160,9 +166,11 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_INSERT_DATETIME)
 		MenuItem(AP_MENU_ID_INSERT_FIELD)
 		MenuItem(AP_MENU_ID_INSERT_TEXTBOX)
+#if !XAP_SIMPLE_MENU
 		MenuItem(AP_MENU_ID_INSERT_MAILMERGE)
+#endif
 		MenuItem(AP_MENU_ID_INSERT_SYMBOL)
-
+		
 		Separator()
 
 		MenuItem(AP_MENU_ID_INSERT_HEADER)
@@ -170,21 +178,23 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_INSERT_FILE)
 		MenuItem(AP_MENU_ID_INSERT_BOOKMARK)
 		MenuItem(AP_MENU_ID_INSERT_HYPERLINK)
-		MenuItem(AP_MENU_ID_INSERT_TABLEOFCONTENTS)
-		MenuItem(AP_MENU_ID_INSERT_FOOTNOTE)
+		MenuItem(AP_MENU_ID_INSERT_TABLEOFCONTENTS)	
+        MenuItem(AP_MENU_ID_INSERT_FOOTNOTE)
 		MenuItem(AP_MENU_ID_INSERT_ENDNOTE)
 
-#ifdef XP_UNIX_TARGET_GTK
+#if defined(TOOLKIT_GTK) && !XAP_SIMPLE_MENU
 		MenuItem(AP_MENU_ID_INSERT_CLIPART)
 #endif
 		MenuItem(AP_MENU_ID_INSERT_GRAPHIC)
 
+#if !XAP_SIMPLE_MENU
 		Separator()
 
 		BeginSubMenu(AP_MENU_ID_INSERT_DIRECTIONMARKER)
  	        MenuItem(AP_MENU_ID_INSERT_DIRECTIONMARKER_LRM)
 	        MenuItem(AP_MENU_ID_INSERT_DIRECTIONMARKER_RLM)
 	    EndSubMenu()
+#endif
 
 	EndSubMenu()
 
@@ -193,13 +203,13 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_FMT_PARAGRAPH)
 		MenuItem(AP_MENU_ID_FMT_BULLETS)
 		MenuItem(AP_MENU_ID_FMT_FRAME)
-    MenuItem(AP_MENU_ID_FMT_IMAGE)
+		MenuItem(AP_MENU_ID_FMT_IMAGE)
 #if 0 // someone code and turn this back on
 		MenuItem(AP_MENU_ID_FMT_BORDERS)
 #endif
 		Separator()
 		MenuItem(AP_MENU_ID_FMT_COLUMNS)
-#if !XP_SIMPLE_MENU	
+#if !XAP_SIMPLE_MENU	
 		MenuItem(AP_MENU_ID_FMT_TABS)
 #endif	
 		MenuItem(AP_MENU_ID_FMT_HDRFTR)
@@ -228,20 +238,22 @@ BeginLayout(Main,0)
 			MenuItem(AP_MENU_ID_FMT_SUBSCRIPT)
 		EndSubMenu()
 
+#if !XAP_SIMPLE_MENU	
 		BeginSubMenu(AP_MENU_ID_FMT_BACKGROUND)
 			MenuItem(AP_MENU_ID_FMT_BACKGROUND_PAGE_IMAGE)
 			MenuItem(AP_MENU_ID_FMT_BACKGROUND_PAGE_COLOR)
 		EndSubMenu()
+#endif
 
-    BeginSubMenu(AP_MENU_ID_FMT_DIRECTION)
-      MenuItem(AP_MENU_ID_FMT_DIRECTION_DOCD_RTL)
-      MenuItem(AP_MENU_ID_FMT_DIRECTION_SD_RTL)
-      MenuItem(AP_MENU_ID_FMT_DIRECTION_DD_RTL)
-      MenuItem(AP_MENU_ID_FMT_DIRECTION_DO_LTR)
-      MenuItem(AP_MENU_ID_FMT_DIRECTION_DO_RTL)
-    EndSubMenu()
+		BeginSubMenu(AP_MENU_ID_FMT_DIRECTION)
+			MenuItem(AP_MENU_ID_FMT_DIRECTION_DOCD_RTL)
+			MenuItem(AP_MENU_ID_FMT_DIRECTION_SD_RTL)
+			MenuItem(AP_MENU_ID_FMT_DIRECTION_DD_RTL)
+			MenuItem(AP_MENU_ID_FMT_DIRECTION_DO_LTR)
+			MenuItem(AP_MENU_ID_FMT_DIRECTION_DO_RTL)
+		EndSubMenu()
         
-#if !XP_SIMPLE_MENU        
+#if !XAP_SIMPLE_MENU        
 		Separator()
 		MenuItem(AP_MENU_ID_FMT_STYLE_DEFINE)
 #endif
@@ -250,12 +262,12 @@ BeginLayout(Main,0)
 
 	BeginSubMenu(AP_MENU_ID_TOOLS)
 
-#ifndef WITHOUT_SPELL
+#ifdef ENABLE_SPELL
 		MenuItem(AP_MENU_ID_TOOLS_SPELL)
 #endif
 		MenuItem(AP_MENU_ID_FMT_LANGUAGE)
 		MenuItem(AP_MENU_ID_TOOLS_WORDCOUNT)
-#if !XP_SIMPLE_MENU	
+#if !XAP_SIMPLE_MENU	
         MenuItem(AP_MENU_ID_FMT_STYLIST)
 #endif	
 
@@ -265,14 +277,14 @@ BeginLayout(Main,0)
 	        MenuItem(AP_MENU_ID_TOOLS_HISTORY_SHOW)
 	        MenuItem(AP_MENU_ID_TOOLS_REVISIONS_COMPARE_DOCUMENTS)
 		    MenuItem(AP_MENU_ID_TOOLS_REVISIONS_AUTO)
-#if !XP_SIMPLE_MENU	
+#if !XAP_SIMPLE_MENU	
 		    MenuItem(AP_MENU_ID_TOOLS_HISTORY_PURGE)
 #endif	
 		EndSubMenu()
 	
 		BeginSubMenu(AP_MENU_ID_TOOLS_REVISIONS)
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_MARK)
-#if !XP_SIMPLE_MENU	
+#if !XAP_SIMPLE_MENU	
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_NEW_REVISION)
 #endif	
 		    Separator()
@@ -287,20 +299,24 @@ BeginLayout(Main,0)
 	        Separator()
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_ACCEPT_REVISION)
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_REJECT_REVISION)
-#if !XP_SIMPLE_MENU	
+#if !XAP_SIMPLE_MENU	
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_PURGE)
 #endif	
 		EndSubMenu()
-	
+
+		BeginSubMenu(AP_MENU_ID_TOOLS_ANNOTATIONS)
+			MenuItem(AP_MENU_ID_TOOLS_ANNOTATIONS_INSERT)
+			MenuItem(AP_MENU_ID_TOOLS_ANNOTATIONS_INSERT_FROMSEL)
+			Separator()
+			MenuItem(AP_MENU_ID_TOOLS_ANNOTATIONS_TOGGLE_DISPLAY)
+		EndSubMenu()
+
 	    Separator()
 
-#if !XP_SIMPLE_MENU
+#if !XAP_SIMPLE_MENU
 		MenuItem(AP_MENU_ID_TOOLS_PLUGINS)
 		MenuItem(AP_MENU_ID_TOOLS_SCRIPTS)
 		MenuItem(AP_MENU_ID_TOOLS_MAILMERGE)
-#ifndef XP_MAC_TARGET_MACOSX
-		// On MacOS X don't put a separator as the "Option" menu item is moved away at run time
-#endif
 #if XAP_PREFSMENU_UNDER_TOOLS
 		Separator()
 		MenuItem(AP_MENU_ID_TOOLS_OPTIONS)
@@ -310,7 +326,7 @@ BeginLayout(Main,0)
 
 	BeginSubMenu(AP_MENU_ID_TABLE)
 
-    		BeginSubMenu(AP_MENU_ID_TABLE_INSERT)
+		BeginSubMenu(AP_MENU_ID_TABLE_INSERT)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_TABLE)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_COLUMNS_BEFORE)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_COLUMNS_AFTER)
@@ -349,19 +365,19 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_TABLE_SPLIT_TABLE)
 #endif
 		MenuItem(AP_MENU_ID_TABLE_FORMAT)
-                BeginSubMenu(AP_MENU_ID_TABLE_TEXTTOTABLE)
-		   MenuItem(AP_MENU_ID_TABLE_TEXTTOTABLE_ALL)
-		   MenuItem(AP_MENU_ID_TABLE_TEXTTOTABLE_NOSPACES)
-                EndSubMenu()
+		BeginSubMenu(AP_MENU_ID_TABLE_TEXTTOTABLE)
+			MenuItem(AP_MENU_ID_TABLE_TEXTTOTABLE_ALL)
+			MenuItem(AP_MENU_ID_TABLE_TEXTTOTABLE_NOSPACES)
+		EndSubMenu()
 #if DEBUG
 	    BeginSubMenu(AP_MENU_ID_TABLE_SORT)
-	       MenuItem(AP_MENU_ID_TABLE_SORTROWSASCEND)
-	       MenuItem(AP_MENU_ID_TABLE_SORTROWSDESCEND)
-	       MenuItem(AP_MENU_ID_TABLE_SORTCOLSASCEND)
-	       MenuItem(AP_MENU_ID_TABLE_SORTCOLSDESCEND)
+			MenuItem(AP_MENU_ID_TABLE_SORTROWSASCEND)
+			MenuItem(AP_MENU_ID_TABLE_SORTROWSDESCEND)
+			MenuItem(AP_MENU_ID_TABLE_SORTCOLSASCEND)
+			MenuItem(AP_MENU_ID_TABLE_SORTCOLSDESCEND)
 	    EndSubMenu()
 #endif
-#if !XP_SIMPLE_MENU
+#if !XAP_SIMPLE_MENU
 	    BeginSubMenu(AP_MENU_ID_TABLE_TABLETOTEXT)
 	       MenuItem(AP_MENU_ID_TABLE_TABLETOTEXTCOMMAS)
 	       MenuItem(AP_MENU_ID_TABLE_TABLETOTEXTTABS)
@@ -377,7 +393,7 @@ BeginLayout(Main,0)
 #endif
 	EndSubMenu()
 
-#if !XP_SIMPLE_MENU 
+#if !XAP_SIMPLE_MENU 
 	BeginSubMenu(AP_MENU_ID_WINDOW)
 		MenuItem(AP_MENU_ID_WINDOW_NEW)
 		Separator()
@@ -399,10 +415,10 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_HELP_CHECKVER)
 		MenuItem(AP_MENU_ID_HELP_REPORT_BUG)
 		Separator()
-#ifdef XP_UNIX_TARGET_GTK
+#ifdef TOOLKIT_GTK
 		MenuItem(AP_MENU_ID_HELP_ABOUT_GNOMEOFFICE)
 #endif
-#ifndef XP_UNIX_TARGET_GTK
+#ifndef TOOLKIT_GTK
 		MenuItem(AP_MENU_ID_HELP_CREDITS)
 #endif
 		MenuItem(AP_MENU_ID_HELP_ABOUT)
