@@ -33,6 +33,7 @@ dcl(MSG_SaveFailedWrite,		"Writing error when attempting to save %s")
 dcl(MSG_SaveFailedExport,		"Error while attempting to save %s: could not construct exporter")
 dcl(MSG_SaveFailedName, 		"Error while attempting to save %s: invalid name")
 dcl(MSG_OpenFailed, "Could not open file %s.")
+dcl(MSG_OpenRecovered,          "Some errors were encountered during the import.")
 dcl(MSG_DlgNotImp,				"%s not implemented yet.\n\nIf you are a programmer, feel free to add code in %s, line %d\nand mail patches to:\n\n\tabiword-dev@abisource.com\n\nOtherwise, please be patient.")
 dcl(MSG_RevertBuffer,			"Revert to saved copy of %s?")
 dcl(MSG_QueryExit,				"Close all windows and exit?")
@@ -357,6 +358,7 @@ dcl(DLG_Options_TabLabel_Other, 		"Other")
 dcl(DLG_Options_TabLabel_Preferences,	"Preference Schemes")
 dcl(DLG_Options_TabLabel_View,			"View")
 dcl(DLG_Options_TabLabel_Misc,	   "Misc.")
+dcl(DLG_Options_TabLabel_SmartQuotes,   "Smart Quotes")
 
 dcl(DLG_Options_Btn_Save,				"Sa&ve")
 dcl(DLG_Options_Btn_Apply,				"Apply")
@@ -371,6 +373,11 @@ dcl(DLG_Options_Label_SpellHighlightMisspelledWords, "&Highlight misspelled word
      // Grammar Check
 dcl(DLG_Options_Label_Grammar, "Automatic Grammar Checking")
 dcl(DLG_Options_Label_GrammarCheck, "Check &grammar as you type")
+
+dcl(DLG_Options_Label_SmartQuotes, "Enable smart (curly) quotes")
+dcl(DLG_Options_Label_CustomSmartQuotes, "Use custom smart quote style")
+dcl(DLG_Options_Label_OuterQuoteStyle, "Outer (double) quote style")
+dcl(DLG_Options_Label_InnerQuoteStyle, "Inner (single) quote style")
 										      // should be merged one day
 dcl(DLG_Options_Label_SpellSuggest, 	"A&lways suggest corrections")
 dcl(DLG_Options_Label_SpellMainOnly,	"Suggest from &main dictionary only")
@@ -427,7 +434,6 @@ dcl(DLG_Options_Label_ViewViewFrame,	"View...")
 dcl(DLG_Options_Label_ViewAll,			"&All")
 dcl(DLG_Options_Label_ViewHiddenText,	"&Hidden Text")
 dcl(DLG_Options_Label_ViewUnprintable,	"Invisible &Layout Marks")
-dcl(DLG_Options_Label_CheckAllowCustomToolbars, "Allow Custom Toolbars")
 dcl(DLG_Options_Label_CheckEnableSmoothScrolling, "Enable Smooth Scrolling")
 
 dcl(DLG_Options_Label_CheckWhiteForTransparent, "Allow screen colors other than white")
@@ -696,12 +702,12 @@ dcl(DLG_Lists_FoldingLevel3,				 "Fold below level 3")
 dcl(DLG_Lists_FoldingLevel4,				 "Fold below level 4")
 
 	 /* page numbers dialog */
-dcl(DLG_PageNumbers_Title,			 "Page Numbers")
-dcl(DLG_PageNumbers_Left,			 "Left")
-dcl(DLG_PageNumbers_Right,			 "Right")
+dcl(DLG_PageNumbers_Title,		 "Page Numbers")
+dcl(DLG_PageNumbers_Left,		 "Left")
+dcl(DLG_PageNumbers_Right,		 "Right")
 dcl(DLG_PageNumbers_Center, 		 "Center")
-dcl(DLG_PageNumbers_Header, 		 "Header")
-dcl(DLG_PageNumbers_Footer, 		 "Footer")
+dcl(DLG_PageNumbers_Header,		 "Top")
+dcl(DLG_PageNumbers_Footer,		 "Bottom")
 dcl(DLG_PageNumbers_Preview,		 "Preview")
 dcl(DLG_PageNumbers_Alignment,		 "Alignment:")
 dcl(DLG_PageNumbers_Alignment_No_Colon,	 "Alignment")
@@ -808,6 +814,14 @@ dcl(DLG_ListRevisions_Column3Label, "Comment")
 dcl(DLG_ListRevisions_Label1, "Existing revisions:")
 dcl(DLG_ListRevisions_LevelZero, "(All revisions visible)")
 
+// RIVERA
+dcl(DLG_Annotation_Title, "Edit Annotation")
+dcl(DLG_Annotation_Title_LBL, "Title:")
+dcl(DLG_Annotation_Author_LBL, "Author:")
+dcl(DLG_Annotation_Description_LBL, "Description:")
+dcl(DLG_Annotation_Replace_LBL, "Replace")
+dcl(DLG_Annotation_OK_tooltip, "Update the annotation")
+dcl(DLG_Annotation_Replace_tooltip, "Replace selection with annotation")
 
 dcl(WORD_PassRequired, "Password required, this is an encrypted document")
 dcl(WORD_PassInvalid, "Incorrect Password")
@@ -920,7 +934,6 @@ dcl(MENU_LABEL_INSERT_FILE,"F&ile")
 dcl(MENU_LABEL_INSERT_SYMBOL,"Sy&mbol")
 dcl(MENU_LABEL_INSERT_ENDNOTE,"&Endnote")
 dcl(MENU_LABEL_INSERT_FOOTNOTE,"Foot&note")
-dcl(MENU_LABEL_INSERT_ANNOTATION,"Annotation")
 dcl(MENU_LABEL_INSERT_TABLEOFCONTENTS,"Table of Contents")
 dcl(MENU_LABEL_INSERT_CLIPART,"&Clip Art")
 dcl(MENU_LABEL_INSERT_GRAPHIC,"&Picture")
@@ -1002,6 +1015,14 @@ dcl(MENU_LABEL_TOOLS_REVISIONS_FIND_PREV,"Find &previous revision")
 dcl(MENU_LABEL_TOOLS_REVISIONS_COMPARE_DOCUMENTS,"&Compare documents")
 dcl(MENU_LABEL_TOOLS_REVISIONS_NEW_REVISION,"Start ne&w revision")
 dcl(MENU_LABEL_TOOLS_REVISIONS_PURGE,"P&urge revisions")
+// RIVERA
+dcl(MENU_LABEL_TOOLS_ANNOTATIONS,"&Annotations")
+dcl(MENU_LABEL_TOOLS_ANNOTATIONS_INSERT,"New &annotation")
+dcl(MENU_LABEL_TOOLS_ANNOTATIONS_INSERT_FROMSEL,"Create from &selection")
+dcl(MENU_LABEL_TOOLS_ANNOTATIONS_TOGGLE_DISPLAY,"&Show annotations")
+dcl(MENU_LABEL_GOTO_ANNOTATION,"&Jump to annotation")
+dcl(MENU_LABEL_EDIT_ANNOTATION,"&Edit annotation")
+dcl(MENU_LABEL_DELETE_ANNOTATION,"&Delete annotation")
 dcl(MENU_LABEL_TABLE, "T&able")
 dcl(MENU_LABEL_TABLE_INSERT, "&Insert")
 dcl(MENU_LABEL_TABLE_INSERT_TABLE, "&Table")
@@ -1190,7 +1211,6 @@ dcl(MENU_STATUSLINE_INSERT_MAILMERGE, "Insert a mail merge field")
 dcl(MENU_STATUSLINE_INSERT_FILE,"Insert the contents of another file")
 dcl(MENU_STATUSLINE_INSERT_SYMBOL,"Insert a symbol or other special character")
 dcl(MENU_STATUSLINE_INSERT_FOOTNOTE,"Insert a footnote")
-dcl(MENU_STATUSLINE_INSERT_ANNOTATION,"Insert an Annotation")
 dcl(MENU_STATUSLINE_INSERT_TABLEOFCONTENTS,"Insert a Table of Contents based on Headings")
 dcl(MENU_STATUSLINE_INSERT_ENDNOTE,"Insert an endnote")
 dcl(MENU_STATUSLINE_INSERT_CLIPART,"Insert clipart")
@@ -1258,7 +1278,7 @@ dcl(MENU_STATUSLINE_TOOLS_SCRIPTS,"Execute helper scripts")
 dcl(MENU_STATUSLINE_TOOLS_HISTORY,"See history of the current document")
 dcl(MENU_STATUSLINE_TOOLS_HISTORY_SHOW,"View document history")
 dcl(MENU_STATUSLINE_TOOLS_HISTORY_PURGE,"Remove full document history from the document")
-dcl(MENU_STATUSLINE_TOOLS_REVISIONS,"Manage changes in document")
+dcl(MENU_STATUSLINE_TOOLS_REVISIONS,"")
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_SET_VIEW_LEVEL,"Choose which revision you wish to view")
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_MARK,"Mark changes as you type")
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_SHOW,"Show revisions that are present in document")
@@ -1274,6 +1294,14 @@ dcl(MENU_STATUSLINE_TOOLS_REVISIONS_MERGE_DOCUMENTS,"Merge another document into
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_NEW_REVISION,"Start revision distinct from the current one")
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_PURGE,"Remove all revision information from the document")
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_AUTO,"Retain all document changes")
+// RIVERA
+dcl(MENU_STATUSLINE_TOOLS_ANNOTATIONS,"Manage annotations in document")
+dcl(MENU_STATUSLINE_TOOLS_ANNOTATIONS_INSERT,"Insert an annotation")
+dcl(MENU_STATUSLINE_TOOLS_ANNOTATIONS_INSERT_FROMSEL,"Insert selected text into a new annotation")
+dcl(MENU_STATUSLINE_TOOLS_ANNOTATIONS_TOGGLE_DISPLAY,"Show/hide annotations")
+dcl(MENU_STATUSLINE_GOTO_ANNOTATION,"Jump to annotation")
+dcl(MENU_STATUSLINE_EDIT_ANNOTATION,"Edit annotation")
+dcl(MENU_STATUSLINE_DELETE_ANNOTATION,"Delete annotation")
 dcl(MENU_STATUSLINE_TABLE, "Table")
 dcl(MENU_STATUSLINE_TABLE_INSERT, "Insert")
 dcl(MENU_STATUSLINE_TABLE_INSERT_TABLE, "Insert Table")
@@ -1593,3 +1621,5 @@ dcl(TOOLBAR_TOOLTIP_MERGERIGHT, "Merge with right cell")
 dcl(TOOLBAR_TOOLTIP_MERGEABOVE, "Merge with cell above")
 dcl(TOOLBAR_TOOLTIP_MERGEBELOW, "Merge with cell below")
 dcl(TOOLBAR_TOOLTIP_MENU, "Menu")
+
+dcl(COMMAND_LINE_PRINTING_DEPRECATED, "The '--print' option has been removed as of AbiWord 2.8. You can achieve the desired behavior using AbiWord's command-line conversion tools and then piping the output to a print spooler. Eg: 'abiword --to=ps --to-name=fd://1 myfile.txt | lpr'")
