@@ -38,6 +38,7 @@
 #include "ie_exp.h"
 #include "ie_exp_AbiWord_1.h"
 
+#include "fl_DocLayout.h"
 #include "pd_Document.h"
 
 #include <gsf/gsf-output-impl.h>
@@ -49,12 +50,9 @@ static UT_GenericVector<IE_ExpSniffer *> m_sniffers(20);
 
 #include "fv_View.h"
 #include "xap_App.h"
+#include "gr_Graphics.h"
 
-#if defined(XP_UNIX_TARGET_GTK)
-#include "gr_UnixNullGraphics.h"
-#endif
-
-class IE_FieldUpdater
+class ABI_EXPORT IE_FieldUpdater
 {
 public:
 	
@@ -67,13 +65,8 @@ public:
 	{
 		if (updatedFields_)
 			return;
-		// todo: support other platforms when possible
-		GR_Graphics * graphics = NULL;
 
-#if defined(XP_UNIX_TARGET_GTK)
-		GR_UnixNullGraphicsAllocInfo ai;
-		graphics = XAP_App::getApp()->newGraphics(GRID_UNIX_NULL, (GR_AllocInfo&)ai);
-#endif
+		GR_Graphics * graphics = GR_Graphics::newNullGraphics();
 
 		if (graphics)
 			{
@@ -100,8 +93,8 @@ private:
 /*****************************************************************/
 /*****************************************************************/
 
-IE_ExpSniffer::IE_ExpSniffer (const char * name, bool canCopy)
-	: m_name(name),
+IE_ExpSniffer::IE_ExpSniffer (const char * _name, bool canCopy)
+	: m_name(_name),
 	  m_type(IEFT_Bogus),
 	  m_bCanCopy(canCopy)
 {
@@ -136,11 +129,10 @@ UT_UTF8String IE_ExpSniffer::getPreferredSuffix()
 
 void IE_Exp::registerExporter (IE_ExpSniffer * s)
 {
-	UT_uint32 ndx = 0;
+	UT_sint32 ndx = 0;
 	UT_Error err = m_sniffers.addItem (s, &ndx);
 
 	UT_return_if_fail(err == UT_OK);
-	UT_return_if_fail(ndx >= 0);
 
 	s->setFileType(ndx+1);
 }
@@ -151,8 +143,6 @@ void IE_Exp::unregisterExporter (IE_ExpSniffer * s)
 
 	ndx = s->getFileType(); // 1:1 mapping
 
-	UT_return_if_fail(ndx >= 0);
-
 	m_sniffers.deleteNthItem (ndx-1);
 
 	// Refactor the indexes
@@ -175,8 +165,7 @@ void IE_Exp::unregisterAllExporters ()
 	for (UT_uint32 i = 0; i < size; i++)
 	{
 		pSniffer = m_sniffers.getNthItem(i);
-		if (pSniffer)
-			delete pSniffer;
+		DELETEP(pSniffer);
 	}
 
 	m_sniffers.clear();
@@ -286,14 +275,14 @@ bool IE_Exp::_closeFile(void)
 void IE_Exp::_abortFile(void)
 {
 	// abort the write
-	UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
+    UT_DEBUGMSG(("aborting file"));
 	if (m_fp)
-		{
-			_closeFile();
-			
-			// then remove the unwanted file
-			(void)UT_go_file_remove (m_szFileName, NULL);
-		}
+    {
+        _closeFile();
+		
+        // then remove the unwanted file
+        (void)UT_go_file_remove (m_szFileName, NULL);
+    }
 }
 
 //////////////////////////////////////////////////////////////////
@@ -361,6 +350,11 @@ UT_Error IE_Exp::copyToBuffer(PD_DocumentRange * pDocRange, UT_ByteBuf * pBuf)
   	// write trailing zero to byte buffer (not required for file)
   	write("",1);
 
+#if (DEBUG && 0)
+	printf("Text from copy is... \n");
+	printf("%s",reinterpret_cast<const char *>(m_pByteBuf->getPointer(0)));
+	printf("\n");
+#endif
 	return err;
 }
 
