@@ -43,6 +43,7 @@
 #include "pd_Style.h"
 
 #include "pp_AttrProp.h"
+#include "pp_Author.h"
 
 #include "px_ChangeRecord.h"
 #include "px_CR_Object.h"
@@ -218,6 +219,7 @@ protected:
 	void				_closeSpan(void);
 	void				_closeField(void);
 	void				_closeHyperlink(void);
+	void				_closeAnnotation(void);
 	void				_closeTag(void);
 	void				_openSpan(PT_AttrPropIndex apiSpan);
 	void				_openTag(const char * szPrefix, const char * szSuffix,
@@ -233,6 +235,7 @@ protected:
     void                _handleMetaData(void);
 	void                _handleRevisions(void);
 	void                _handleHistory(void);
+	void                _handleAuthors(void);
 
 	PD_Document *		m_pDocument;
 	IE_Exp_AbiWord_1 *	m_pie;
@@ -248,6 +251,7 @@ protected:
     fd_Field *          m_pCurrentField;
 	bool                m_bOpenChar;
 	UT_GenericVector<UT_UTF8String *> m_vecSnapNames;
+	bool				m_bInAnnotation;
 
 
 private:
@@ -335,6 +339,18 @@ void s_AbiWord_1_Listener::_closeHyperlink(void)
 	return;
 }
 
+
+void s_AbiWord_1_Listener::_closeAnnotation(void)
+{
+	if (!m_bInAnnotation)
+		return;
+	UT_DEBUGMSG(("Doing close annotation object method \n"));
+    _closeSpan();
+	m_pie->write("</ann>");
+    m_bInAnnotation = false;
+	return;
+}
+
 void s_AbiWord_1_Listener::_openSpan(PT_AttrPropIndex apiSpan)
 {
 	if (m_bInSpan)
@@ -377,6 +393,13 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 		UT_uint32 k = 0;
 		while (pAP->getNthAttribute (k++, szName, szValue))
 		{
+#if 0
+			//
+			// Strip out Author attributes for now.
+			//
+			if( !m_pDocument->isExportAuthorAtts() && strcmp(szName,PT_AUTHOR_NAME) == 0)
+				continue;
+#endif
 			tag += " ";
 			tag += szName;
 			tag += "=\"";
@@ -479,6 +502,11 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 			// TODO consider scanning the value to see if it has one
 			// TODO in it and escaping it or using single-quotes.
 			// Let's also escape ampersands and other goodies.
+			//
+			// Strip out Author attributes for now.
+			//
+			if( !m_pDocument->isExportAuthorAtts() && strcmp(szName,PT_AUTHOR_NAME) == 0)
+				continue;
 
 			m_pie->write(" ");
 			m_pie->write(static_cast<const char*>(szName));
@@ -666,6 +694,7 @@ void s_AbiWord_1_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length
 	const UT_UCSChar * pData;
 
 	UT_return_if_fail(sizeof(UT_Byte) == sizeof(char));
+	sBuf.reserve(length);
 
 	for (pData=data; (pData<data+length); /**/)
 	{
@@ -738,6 +767,7 @@ s_AbiWord_1_Listener::s_AbiWord_1_Listener(PD_Document * pDocument,
 	m_pCurrentField = 0;
 	m_iInTable = 0;
 	m_iInCell = 0;
+	m_bInAnnotation = false;
 
 	m_pie->write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
 	m_pie->write ("<!DOCTYPE abiword PUBLIC \"-//ABISOURCE//DTD AWML 1.0 Strict//EN\" \"http://www.abisource.com/awml.dtd\">\n");
@@ -783,6 +813,8 @@ s_AbiWord_1_Listener::s_AbiWord_1_Listener(PD_Document * pDocument,
 	_handleStyles();
 	_handleLists();
 	_handlePageSize();
+	if(m_pDocument->isExportAuthorAtts())
+		_handleAuthors();
 }
 
 s_AbiWord_1_Listener::~s_AbiWord_1_Listener()
@@ -790,6 +822,7 @@ s_AbiWord_1_Listener::~s_AbiWord_1_Listener()
 	_closeSpan();
 	_closeField();
 	_closeHyperlink();
+	_closeAnnotation();
 	_closeBlock();
 	_closeSection();
 	_handleDataItems();
@@ -944,6 +977,40 @@ bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
    					}
 
 
+   					return true;
+
+   				}
+
+   			case PTO_Annotation:
+   				{
+   					_closeSpan();
+   					_closeField();
+					const PP_AttrProp * pAP = NULL;
+					m_pDocument->getAttrProp(api,&pAP);
+					const gchar * pName;
+					const gchar * pValue;
+					bool bFound = false;
+					UT_uint32 k = 0;
+
+					while(!bFound && pAP->getNthAttribute(k++, pName, pValue))
+					{
+						bFound = (0 == g_ascii_strncasecmp(pName,"Annotation",10));
+					}
+
+					if(bFound)
+					{
+						//this is the start of the Annotation
+   						_openTag("ann", "",false, api,pcr->getXID(),true);
+						UT_DEBUGMSG(("Doing open annotation object \n"));
+   						m_bInAnnotation = true;
+   					}
+   					else
+   					{
+						UT_DEBUGMSG(("Doing close annotation object \n"));
+   						_closeAnnotation();
+   					}
+
+					
    					return true;
 
    				}
@@ -994,7 +1061,8 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 		{
 			_closeSpan();
             _closeField();
-            _closeHyperlink();
+            _closeHyperlink();	
+			_closeAnnotation();
 			_closeBlock();
 			_closeSection();
 			_openTag("section","",true,pcr->getIndexAP(),pcr->getXID());
@@ -1006,6 +1074,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			_openTag("table","",true,pcr->getIndexAP(),pcr->getXID());
 			m_iInTable++;
@@ -1016,6 +1085,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			_openTag("cell","",true,pcr->getIndexAP(),pcr->getXID());
 			m_iInCell++;
@@ -1026,15 +1096,29 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			m_bInBlock = false;
 			_openTag("foot","",true,pcr->getIndexAP(),pcr->getXID());
 			return true;
 		}
+	case PTX_SectionAnnotation:
+		{
+			// spans and field have been closed by the annotation
+			//			_closeSpan();
+            // _closeField();
+			// We may have to close hyperlinks but I hope not.
+			//            _closeHyperlink();
+			UT_DEBUGMSG(("Found start annotation strux \n"));
+			m_bInBlock = false;
+			_openTag("annotate","",true,pcr->getIndexAP(),pcr->getXID());
+			return true;
+		}
 	case PTX_SectionEndnote:
 		{
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			m_bInBlock = false;
 			_openTag("endnote","",true,pcr->getIndexAP(),pcr->getXID());
 			return true;
@@ -1044,6 +1128,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			m_bInBlock = false;
 			_openTag("toc","",true,pcr->getIndexAP(),pcr->getXID());
@@ -1054,6 +1139,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			_openTag("margin","",true,pcr->getIndexAP(),pcr->getXID());
 			return true;
@@ -1063,6 +1149,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			_openTag("frame","",true,pcr->getIndexAP(),pcr->getXID());
 			return true;
@@ -1072,6 +1159,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			_closeTable();
 			return true;
@@ -1081,6 +1169,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			_closeCell();
 			return true;
@@ -1090,16 +1179,30 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			m_pie->write("</foot>");
 			m_bInBlock = true;
 			return true;
 		}
+	case PTX_EndAnnotation:
+		{
+			UT_DEBUGMSG(("End of annotation strux \n"));
+			_closeSpan();
+            _closeField();
+			// Lets not close out hyperlinks to start with
+			//            _closeHyperlink();
+			_closeBlock();
+			m_pie->write("</annotate>");
+			m_bInBlock = true;
+			return true;
+		}
 	case PTX_EndEndnote:
 		{
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			m_pie->write("</endnote>");
 			m_bInBlock = true;
@@ -1110,6 +1213,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			m_pie->write("</toc>");
 			return true;
@@ -1119,6 +1223,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			return true;
 		}
@@ -1127,6 +1232,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_closeSpan();
             _closeField();
             _closeHyperlink();
+			_closeAnnotation();
 			_closeBlock();
 			m_pie->write("</frame>");
 			return true;
@@ -1270,7 +1376,7 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 	const PD_Style * pStyle=NULL;
 	UT_GenericVector<PD_Style *> vecStyles;
 	m_pDocument->getAllUsedStyles(&vecStyles);
-	UT_uint32 k = 0;
+	UT_sint32 k = 0;
 	for (k=0; k < vecStyles.getItemCount(); k++)
 	{
 		pStyle = vecStyles.getNthItem(k);
@@ -1287,7 +1393,7 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 	UT_GenericVector<PD_Style*> * pStyles = NULL;
 	m_pDocument->enumStyles(pStyles);
 	UT_return_if_fail( pStyles );
-	UT_uint32 iStyleCount = m_pDocument->getStyleCount();
+	UT_sint32 iStyleCount = m_pDocument->getStyleCount();
 	
 	for (k=0; k < iStyleCount; k++)
 	{
@@ -1462,13 +1568,15 @@ void s_AbiWord_1_Listener::_handleDataItems(void)
 
 	const char * szName;
    	const char * szMimeType;
+   	const char ** pszMimeType = &szMimeType;
 	const UT_ByteBuf * pByteBuf;
 
 	UT_ByteBuf bbEncoded(1024);
 	string_set::iterator end(m_pUsedImages.end());
 	UT_DEBUGMSG(("Used images are... \n"));
 	for (UT_uint32 k=0;
-		 (m_pDocument->enumDataItems(k,NULL,&szName,&pByteBuf,reinterpret_cast<const void**>(&szMimeType)));
+		 
+		 (m_pDocument->enumDataItems(k,NULL,&szName,&pByteBuf,reinterpret_cast<const void**>(pszMimeType)));
 		 k++)
 	{
 		string_set::iterator it(m_pUsedImages.find(szName));
@@ -1573,7 +1681,7 @@ void s_AbiWord_1_Listener::_handleRevisions(void)
 
 	const UT_GenericVector<AD_Revision*> & vRevisions = m_pDocument->getRevisions();
 
-	UT_uint32 k = 0;
+	UT_sint32 k = 0;
 	for (k=0; k < vRevisions.getItemCount(); k++)
 	{
 		pRev = vRevisions.getNthItem(k);
@@ -1593,9 +1701,9 @@ void s_AbiWord_1_Listener::_handleRevisions(void)
 			bWroteOpenRevisionsSection = true;
 		}
 
-		UT_String_sprintf(s, "<r id=\"%d\" time-started=\"%d\" version=\"%d\">",
+		UT_String_sprintf(s, "<r id=\"%d\" time-started=\"%lld\" version=\"%d\">",
 						  pRev->getId(),
-						  pRev->getStartTime(),
+						  (UT_uint64)pRev->getStartTime(),
 						  pRev->getVersion());
 		
 		m_pie->write(s.c_str());
@@ -1636,11 +1744,10 @@ void s_AbiWord_1_Listener::_handleHistory(void)
 		
 		if (!bWroteOpenSection)
 		{
-			UT_UTF8String_sprintf(s, "<history version=\"%d\" edit-time=\"%d\" last-saved=\"%d\" "
-							     "uid=\"%s\">\n",
+			UT_UTF8String_sprintf(s, "<history version=\"%d\" edit-time=\"%d\" last-saved=\"%d\" uid=\"%s\">\n",
 							  m_pDocument->getDocVersion(),
-							  m_pDocument->getEditTime(),
-							  m_pDocument->getLastSavedTime(),
+							  static_cast<UT_sint32>(m_pDocument->getEditTime()),
+							  static_cast<UT_sint32>(m_pDocument->getLastSavedTime()),
 							  m_pDocument->getDocUUIDString());
 			
 			m_pie->write(s.utf8_str());
@@ -1648,7 +1755,7 @@ void s_AbiWord_1_Listener::_handleHistory(void)
 		}
 
 		UT_UTF8String_sprintf(s, "<version id=\"%d\" started=\"%d\" uid=\"%s\" auto=\"%d\" top-xid=\"%d\"/>\n",
-						  iVersion, tStarted, hUid.utf8_str(),bAuto, iXID);
+						  iVersion, static_cast<UT_sint32>(tStarted), hUid.utf8_str(),bAuto, iXID);
 		
 		m_pie->write(s.utf8_str());
 	}
@@ -1658,3 +1765,44 @@ void s_AbiWord_1_Listener::_handleHistory(void)
 
 	return;
 }
+
+void s_AbiWord_1_Listener::_handleAuthors(void)
+{
+	UT_sint32 nAuthors = m_pDocument-> getNumAuthors();
+	if(nAuthors <= 0)
+		return;
+	m_pie->write("<authors>\n");
+	UT_sint32 i = 0;
+	UT_String sVal;
+	for(i=0;i<nAuthors;i++)
+	{
+		pp_Author * pAuthor = m_pDocument->getNthAuthor(i);
+		m_pie->write("<author id=\"");
+		UT_String_sprintf(sVal,"%d",pAuthor->getAuthorInt());
+		m_pie->write(sVal.c_str());
+		m_pie->write("\" ");
+		PP_AttrProp * pAP = pAuthor->getAttrProp();
+		if(pAP->getPropertyCount()>0)
+		{
+			m_pie->write(static_cast<const char*>(PT_PROPS_ATTRIBUTE_NAME));
+			m_pie->write("=\"");
+			const gchar * szName = NULL;
+			const gchar * szValue = NULL;
+			UT_uint32 j = 0;
+			while (pAP->getNthProperty(j++,szName,szValue))
+			{
+				if (szName && *szName && szValue && *szValue)
+				{
+					if(j>1)
+						m_pie->write("; ");
+					m_pie->write(static_cast<const char*>(szName));
+					m_pie->write(":");
+					_outputXMLChar(szValue, strlen(szValue));
+				}
+			}
+		m_pie->write("\"");
+		}
+		m_pie->write("/>\n");
+	}
+	m_pie->write("</authors>\n");
+}
