@@ -72,7 +72,7 @@ IE_Imp::~IE_Imp()
 void IE_Imp::setProps (const char * props)
 {
 	m_props_map.clear ();
-	m_props_map.parse_properties (props);
+	UT_parse_properties (props, m_props_map);
 }
 
 PT_DocPosition IE_Imp::getDocPos() const
@@ -169,7 +169,7 @@ bool IE_Imp::appendFmt(const gchar ** attributes)
 	return bRes;
 }
 
-bool IE_Imp::appendFmt(const UT_GenericVector<gchar*> * pVecAttributes)
+bool IE_Imp::appendFmt(const UT_GenericVector<const gchar*> * pVecAttributes)
 {
 	bool bRes;
 
@@ -195,10 +195,10 @@ bool IE_Imp::appendFmt(const UT_GenericVector<gchar*> * pVecAttributes)
 
 // default impl, meant to abort
 
-bool	  IE_Imp::pasteFromBuffer(PD_DocumentRange * pDocRange,
-				  const unsigned char * pData, 
-				  UT_uint32 lenData, 
-				  const char * szEncoding)
+bool	  IE_Imp::pasteFromBuffer(PD_DocumentRange * /*pDocRange*/,
+								  const unsigned char * /*pData*/, 
+								  UT_uint32 /*lenData*/, 
+								  const char * /*szEncoding*/)
 {
   UT_ASSERT_NOT_REACHED();
   return false;
@@ -212,8 +212,8 @@ PD_Document * IE_Imp::getDoc () const
 /*****************************************************************/
 /*****************************************************************/
 
-IE_ImpSniffer::IE_ImpSniffer(const char * name, bool canPaste)
-	: m_name(name),
+IE_ImpSniffer::IE_ImpSniffer(const char * _name, bool canPaste)
+	: m_name(_name),
 	  m_type(IEFT_Bogus),
 	  m_bCanPaste(canPaste)
 {
@@ -233,8 +233,8 @@ UT_Confidence_t IE_ImpSniffer::recognizeContents (GsfInput * input)
 	return recognizeContents(szBuf, iNumbytes);
 }
 
-UT_Confidence_t IE_ImpSniffer::recognizeContents (const char * szBuf, 
-												  UT_uint32 iNumbytes)
+UT_Confidence_t IE_ImpSniffer::recognizeContents (const char * /*szBuf*/, 
+												  UT_uint32 /*iNumbytes*/)
 {
 	// should be explicitly overriden, or not return anything
 	UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
@@ -247,11 +247,10 @@ UT_Confidence_t IE_ImpSniffer::recognizeContents (const char * szBuf,
 
 void IE_Imp::registerImporter (IE_ImpSniffer * s)
 {
-	UT_uint32 ndx = 0;
+	UT_sint32 ndx = 0;
 	UT_Error err = IE_IMP_Sniffers.addItem (s, &ndx);
 
 	UT_return_if_fail(err == UT_OK);
-	UT_return_if_fail(ndx >= 0);
 
 	s->setFileType(ndx+1);
 }
@@ -260,8 +259,6 @@ void IE_Imp::unregisterImporter (IE_ImpSniffer * s)
 {
 	UT_uint32 ndx = s->getFileType(); // 1:1 mapping
 
-	UT_return_if_fail(ndx >= 0);
-
 	IE_IMP_Sniffers.deleteNthItem (ndx-1);
 
 	// Refactor the indexes
@@ -288,9 +285,10 @@ void IE_Imp::unregisterAllImporters ()
 	for (UT_uint32 i = 0; i < size; i++)
 	{
 		pSniffer = IE_IMP_Sniffers.getNthItem(i);
-		if (pSniffer)
-			pSniffer->unref();
+		DELETEP(pSniffer);
 	}
+
+	IE_IMP_Sniffers.clear();
 }
 
 /*!
@@ -303,7 +301,7 @@ std::vector<std::string> & IE_Imp::getSupportedMimeTypes ()
 	}
 
 	const IE_MimeConfidence *mc;
-	for (guint i = 0; i < IE_IMP_Sniffers.size(); i++) {
+	for (UT_sint32 i = 0; i < IE_IMP_Sniffers.size(); i++) {
 		mc = IE_IMP_Sniffers.getNthItem(i)->getMimeConfidence();
 		while (mc && mc->match) {
 			if (mc->match == IE_MIME_MATCH_FULL) {
@@ -327,7 +325,7 @@ std::vector<std::string> & IE_Imp::getSupportedMimeClasses ()
 	}
 
 	const IE_MimeConfidence *mc;
-	for (guint i = 0; i < IE_IMP_Sniffers.size(); i++) {
+	for (UT_sint32 i = 0; i < IE_IMP_Sniffers.size(); i++) {
 		mc = IE_IMP_Sniffers.getNthItem(i)->getMimeConfidence();
 		while (mc && mc->match) {
 			if (mc->match == IE_MIME_MATCH_CLASS) {
@@ -351,7 +349,7 @@ std::vector<std::string> & IE_Imp::getSupportedSuffixes()
 	}
 
 	const IE_SuffixConfidence *sc;
-	for (guint i = 0; i < IE_IMP_Sniffers.size(); i++) {
+	for (UT_sint32 i = 0; i < IE_IMP_Sniffers.size(); i++) {
 		sc = IE_IMP_Sniffers.getNthItem(i)->getSuffixConfidence();
 		while (sc && !sc->suffix.empty()) {
 			IE_IMP_Suffixes.push_back(sc->suffix);
@@ -373,10 +371,10 @@ const char * IE_Imp::getMimeTypeForSuffix(const char * suffix)
 	}
 
 	const IE_SuffixConfidence *sc;
-	for (guint i = 0; i < IE_IMP_Sniffers.size(); i++) {
+	for (UT_sint32 i = 0; i < IE_IMP_Sniffers.size(); i++) {
 		IE_ImpSniffer *sniffer = IE_IMP_Sniffers.getNthItem(i);
 		sc = sniffer->getSuffixConfidence();
-		while (sc && sc->suffix.empty()) {
+		while (sc && !sc->suffix.empty()) {
 			if (0 == g_ascii_strcasecmp(suffix, sc->suffix.c_str())) {
 				const IE_MimeConfidence *mc = sniffer->getMimeConfidence();
 				if (mc) {
@@ -826,6 +824,11 @@ UT_Error IE_Imp::constructImporter(PD_Document * pDocument,
 					best_sniffer = s;
 					best_confidence = confidence;
 					ieft = (IEFileType) (k+1);
+
+					// short-circuit when we have perfect confidence for both
+					if (suffix_confidence == UT_CONFIDENCE_PERFECT &&
+					    content_confidence == UT_CONFIDENCE_PERFECT)
+						break;
 				}
 		  }
 		if (best_sniffer)
