@@ -45,11 +45,10 @@ static std::vector<std::string> 		IE_IMP_GraphicSuffixes;
 
 void IE_ImpGraphic::registerImporter (IE_ImpGraphicSniffer * s)
 {
-	UT_uint32 ndx = 0;
+	UT_sint32 ndx = 0;
 	UT_Error err = IE_IMP_GraphicSniffers.addItem (s, &ndx);
 
 	UT_return_if_fail(err == UT_OK);
-	UT_return_if_fail(ndx >= 0);
 
 	s->setType(ndx+1);
 }
@@ -58,8 +57,6 @@ void IE_ImpGraphic::unregisterImporter (IE_ImpGraphicSniffer * s)
 {
 	UT_uint32 ndx = s->getType(); // 1:1 mapping
 
-	UT_return_if_fail(ndx >= 0);
-
 	IE_IMP_GraphicSniffers.deleteNthItem (ndx-1);
 
 	// Refactor the indexes
@@ -86,9 +83,10 @@ void IE_ImpGraphic::unregisterAllImporters ()
 	for (UT_uint32 i = 0; i < size; i++)
 	{
 		pSniffer = IE_IMP_GraphicSniffers.getNthItem(i);
-		if (pSniffer)
-			pSniffer->unref();
+		DELETEP(pSniffer);
 	}
+
+	IE_IMP_GraphicSniffers.clear();
 }
 
 /*!
@@ -101,7 +99,7 @@ std::vector<std::string> & IE_ImpGraphic::getSupportedMimeTypes ()
 	}
 
 	const IE_MimeConfidence *mc;
-	for (guint i = 0; i < IE_IMP_GraphicSniffers.size(); i++) {
+	for (UT_sint32 i = 0; i < IE_IMP_GraphicSniffers.size(); i++) {
 		mc = IE_IMP_GraphicSniffers.getNthItem(i)->getMimeConfidence();
 		while (mc && mc->match) {
 			if (mc->match == IE_MIME_MATCH_FULL) {
@@ -125,7 +123,7 @@ std::vector<std::string> & IE_ImpGraphic::getSupportedMimeClasses ()
 	}
 
 	const IE_MimeConfidence *mc;
-	for (guint i = 0; i < IE_IMP_GraphicSniffers.size(); i++) {
+	for (UT_sint32 i = 0; i < IE_IMP_GraphicSniffers.size(); i++) {
 		mc = IE_IMP_GraphicSniffers.getNthItem(i)->getMimeConfidence();
 		while (mc && mc->match) {
 			if (mc->match == IE_MIME_MATCH_CLASS) {
@@ -149,7 +147,7 @@ std::vector<std::string> & IE_ImpGraphic::getSupportedSuffixes()
 	}
 
 	const IE_SuffixConfidence *sc;
-	for (guint i = 0; i < IE_IMP_GraphicSniffers.size(); i++) {
+	for (UT_sint32 i = 0; i < IE_IMP_GraphicSniffers.size(); i++) {
 		sc = IE_IMP_GraphicSniffers.getNthItem(i)->getSuffixConfidence();
 		while (sc && !sc->suffix.empty()) {
 			IE_IMP_GraphicSuffixes.push_back(sc->suffix);
@@ -171,10 +169,10 @@ const char * IE_ImpGraphic::getMimeTypeForSuffix(const char * suffix)
 	}
 
 	const IE_SuffixConfidence *sc;
-	for (guint i = 0; i < IE_IMP_GraphicSniffers.size(); i++) {
+	for (UT_sint32 i = 0; i < IE_IMP_GraphicSniffers.size(); i++) {
 		IE_ImpGraphicSniffer *sniffer = IE_IMP_GraphicSniffers.getNthItem(i);
 		sc = sniffer->getSuffixConfidence();
-		while (sc && sc->suffix.empty()) {
+		while (sc && !sc->suffix.empty()) {
 			if (0 == g_ascii_strcasecmp(suffix, sc->suffix.c_str())) {
 				const IE_MimeConfidence *mc = sniffer->getMimeConfidence();
 				if (mc) {
@@ -383,7 +381,7 @@ UT_Error IE_ImpGraphic::constructImporterWithDescription(const char * szDesc, IE
 	return err;
 }
 
-UT_Error IE_ImpGraphic:: constructImporter(const UT_ByteBuf * bytes,
+UT_Error IE_ImpGraphic:: constructImporter(const UT_ByteBuf & bytes,
 					   IEGraphicFileType ft,
 					   IE_ImpGraphic **ppieg)
 {
@@ -391,19 +389,18 @@ UT_Error IE_ImpGraphic:: constructImporter(const UT_ByteBuf * bytes,
 	// caller is responsible for deleting the importer object
 	// when finished with it.
 	UT_return_val_if_fail(ppieg, UT_ERROR);
-	UT_return_val_if_fail(bytes, UT_ERROR);
 
 	// no filter will support IEGFT_Unknown, so we detect from the
 	// suffix of the filename and the contents of the file, the real 
         // importer to use and assign that back to ieft.
 	if (ft == IEGFT_Unknown)
 	{
-	  ft = IE_ImpGraphic::fileTypeForContents( reinterpret_cast<const char *>(bytes->getPointer(0)),
-						   bytes->getLength() );
+	  ft = IE_ImpGraphic::fileTypeForContents( reinterpret_cast<const char *>(bytes.getPointer(0)),
+						   bytes.getLength() );
 	}
 
 	// use the importer for the specified file type
-	for (UT_uint32 k=0; (k < IE_IMP_GraphicSniffers.size()); k++)
+	for (UT_sint32 k=0; (k < IE_IMP_GraphicSniffers.size()); k++)
 	{
 		IE_ImpGraphicSniffer * s = IE_IMP_GraphicSniffers[k];
 		if (s->supportsType(ft))
@@ -580,7 +577,7 @@ UT_Error IE_ImpGraphic::convertGraphic(UT_ByteBuf* pBB,
 	FG_Graphic * graphic = NULL;
 	UT_Error result;
 
-	result = IE_ImpGraphic::loadGraphic(pBB, IEGFT_Unknown, &graphic);
+	result = IE_ImpGraphic::loadGraphic(*pBB, IEGFT_Unknown, &graphic);
 
 	// method assumes that we take ownership of pBB
 	DELETEP(pBB);
@@ -588,7 +585,7 @@ UT_Error IE_ImpGraphic::convertGraphic(UT_ByteBuf* pBB,
 	if (result != UT_OK)
 		return result;
 
-	UT_ByteBuf * graphic_bytebuf;
+	const UT_ByteBuf * graphic_bytebuf;
 
 	if (graphic->getType() == FGT_Raster) {
 		graphic_bytebuf = static_cast<FG_GraphicRaster *>(graphic)->getRaster_PNG();
@@ -656,13 +653,13 @@ UT_Error IE_ImpGraphic::loadGraphic(GsfInput * input,
 	return result;
 }
 
-UT_Error IE_ImpGraphic::loadGraphic(UT_ByteBuf *pBB,
+UT_Error IE_ImpGraphic::loadGraphic(const UT_ByteBuf &pBB,
 									IEGraphicFileType iegft,
 									FG_Graphic ** ppfg)
 {
 	GsfInput * input;
 
-	input = gsf_input_memory_new (pBB->getPointer (0), pBB->getLength(), FALSE);
+	input = gsf_input_memory_new (pBB.getPointer (0), pBB.getLength(), FALSE);
 	if (!input)
 		return UT_IE_NOMEMORY;
 
@@ -683,8 +680,8 @@ UT_Confidence_t IE_ImpGraphicSniffer::recognizeContents (GsfInput * input)
 	return recognizeContents(szBuf, iNumbytes);
 }
 
-UT_Confidence_t IE_ImpGraphicSniffer::recognizeContents (const char * szBuf, 
-														 UT_uint32 iNumbytes)
+UT_Confidence_t IE_ImpGraphicSniffer::recognizeContents (const char * /*szBuf*/, 
+														 UT_uint32 /*iNumbytes*/)
 {
 	// should be explicitly overriden, or not return anything
 	UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
