@@ -19,7 +19,6 @@
  * 02111-1307, USA.
  */
 
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,6 +43,8 @@
 #include "ie_impexp_AbiWord_1.h"
 #include "ie_imp_AbiWord_1.h"
 #include "ie_types.h"
+#include "pp_Author.h"
+#include "pp_AttrProp.h"
 
 /*****************************************************************/
 /*****************************************************************/
@@ -117,7 +118,7 @@ UT_Confidence_t IE_Imp_AbiWord_1_Sniffer::recognizeContents (const char * szBuf,
 	UT_uint32 iLinesToRead = 6 ;  // Only examine the first few lines of the file
 	UT_uint32 iBytesScanned = 0 ;
 	const char *p ;
-	char *magic ;
+	const char *magic ;
 	p = szBuf ;
 	while( iLinesToRead-- )
 	{
@@ -245,7 +246,11 @@ IE_Imp_AbiWord_1::IE_Imp_AbiWord_1(PD_Document * pDocument)
 #define TT_VERSION         33 //<version>
 #define TT_TOC             34 //<toc> (Table of Contents
 #define TT_MATH            35 //<math> Math Run
-#define TT_EMBED            36 //<embed> Generic Embeded Run
+#define TT_EMBED           36 //<embed> Generic Embeded Run
+#define TT_AUTHORSECTION   37 //<authors>
+#define TT_AUTHOR          38 //<author>
+#define TT_ANN             39 //<ann> Annotate region
+#define TT_ANNOTATE        40 //<annotate> Annotation content
 
 
 /*
@@ -268,6 +273,10 @@ static struct xmlToIdMapping s_Tokens[] =
 {
 	{	"a",			TT_HYPERLINK	},
 	{	"abiword",		TT_DOCUMENT		},
+	{	"ann",		    TT_ANN  		},
+	{	"annotate",		TT_ANNOTATE		},
+	{   "author",       TT_AUTHOR      },
+	{   "authors",      TT_AUTHORSECTION},
 	{	"awml",			TT_DOCUMENT		},
 	{	"bookmark",		TT_BOOKMARK		},
 	{	"br",			TT_BREAK		},
@@ -403,6 +412,28 @@ void IE_Imp_AbiWord_1::startElement(const gchar *name,
 		xxx_UT_DEBUGMSG(("FInished Append footnote strux \n"));
 		goto cleanup;
 	}
+	case TT_ANNOTATE:
+	{
+		// Annotations are contained inside a Block
+		X_VerifyParseState(_PS_Block);
+		m_parseState = _PS_Sec;
+		m_bWroteSection = true;
+
+		// Don't check for id on the footnote. It should match an id on
+        // a footnote reference field.
+
+		// Do we Need to set the min Unique id now???
+		const gchar * pszId = static_cast<const gchar*>(_getXMLPropValue("annotation-id", atts));
+		bool bOK = true;
+		if(pszId)
+		{
+			UT_uint32 id = atoi(pszId);
+			bOK = getDoc()->setMinUID(UT_UniqueId::Annotation, id+1);
+		}
+		X_CheckError(appendStrux(PTX_SectionAnnotation,atts));
+		UT_DEBUGMSG(("FInished Append Annotation strux \n"));
+		goto cleanup;
+	}
 	case TT_ENDNOTE:
 	{
 		// Endnotes are contained inside a Block
@@ -547,6 +578,12 @@ void IE_Imp_AbiWord_1::startElement(const gchar *name,
 		X_CheckError(appendObject(PTO_Hyperlink,atts));
 		goto cleanup;
 
+	case TT_ANN:
+		X_VerifyParseState(_PS_Block);
+		UT_DEBUGMSG(("Annotation object found \n"));
+		X_CheckError(appendObject(PTO_Annotation,atts));
+		goto cleanup;
+
 	case TT_FIELD:
 	{
 		X_VerifyParseState(_PS_Block);
@@ -748,6 +785,35 @@ void IE_Imp_AbiWord_1::startElement(const gchar *name,
 		goto cleanup;
 	}
 
+	case TT_AUTHORSECTION:
+	{
+				
+		X_VerifyParseState(_PS_Doc);
+		m_parseState = _PS_AuthorSec;
+		goto cleanup;
+	}
+
+	case TT_AUTHOR:
+	{
+		X_VerifyParseState(_PS_AuthorSec);
+		m_parseState = _PS_Author;
+
+		const gchar * szInt = UT_getAttribute("id",atts);
+		UT_sint32 iAuthorInt = atoi(szInt);
+		pp_Author * pA = getDoc()->addAuthor(iAuthorInt);
+		PP_AttrProp * pPA = pA->getAttrProp();
+		const gchar * szProps = UT_getAttribute(PT_PROPS_ATTRIBUTE_NAME,atts);
+		if(szProps)
+		{
+			const gchar * szExtraAtts[3] = {PT_PROPS_ATTRIBUTE_NAME,
+											szProps,
+											NULL};
+			pPA->setAttributes(szExtraAtts);
+		}
+		goto cleanup;
+	}
+
+
 	case TT_HISTORYSECTION:
 	{
 				
@@ -814,8 +880,11 @@ void IE_Imp_AbiWord_1::startElement(const gchar *name,
 			
 			szS = UT_getAttribute("uid",atts);
 
-			AD_VersionData v(iId, szS, tStarted, bAuto, iXID);
-			getDoc()->addRecordToHistory(v);
+			if(szS)
+			{
+				AD_VersionData v(iId, szS, tStarted, bAuto, iXID);
+				getDoc()->addRecordToHistory(v);
+			}
 		}
 
 		goto cleanup;
@@ -941,6 +1010,13 @@ void IE_Imp_AbiWord_1::endElement(const gchar *name)
 		m_parseState = _PS_Block;
 		return;
 
+	case TT_ANNOTATE:
+		X_VerifyParseState(_PS_Sec);
+		X_CheckError(appendStrux(PTX_EndAnnotation,NULL));
+		UT_DEBUGMSG(("FInished Append End annotation strux \n"));
+		m_parseState = _PS_Block;
+		return;
+
 	case TT_ENDNOTE:
 		X_VerifyParseState(_PS_Sec);
 		X_CheckError(appendStrux(PTX_EndEndnote,NULL));
@@ -1029,6 +1105,14 @@ void IE_Imp_AbiWord_1::endElement(const gchar *name)
 		X_CheckError(appendObject(PTO_Hyperlink,NULL));
 		return;
 
+	case TT_ANN:						// not a container, so we don't pop stack
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
+		X_VerifyParseState(_PS_Block);
+		UT_DEBUGMSG(("End of annotation region \n"));
+		// we append another Hyperlink Object, but with no attributes
+		X_CheckError(appendObject(PTO_Annotation,NULL));
+		return;
+
 		return;
 
 	case TT_FIELD:						// not a container, so we don't pop stack
@@ -1132,13 +1216,25 @@ void IE_Imp_AbiWord_1::endElement(const gchar *name)
 			// added to the doc by the xml paraser
 			X_CheckError(getDoc()->addRevision(m_currentRevisionId, NULL, 0,
 											   m_currentRevisionTime,
-											   m_currentRevisionVersion));
+											   m_currentRevisionVersion, true));
 			m_currentRevisionId = 0;
 		}
 		
 		m_parseState = _PS_RevisionSec;
 		return;
 
+
+	case TT_AUTHORSECTION:
+		X_VerifyParseState(_PS_AuthorSec);
+		m_parseState = _PS_Doc;
+		return;
+
+	case TT_AUTHOR:
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
+		X_VerifyParseState(_PS_Author);
+		m_parseState = _PS_AuthorSec;
+		return;
+
 	case TT_LISTSECTION:
 		X_VerifyParseState(_PS_ListSec);
 		if (m_bDocHasLists)
@@ -1329,6 +1425,7 @@ bool IE_Imp_AbiWord_1::_handleImage(const gchar ** atts)
 
 	return success;
 #else
+	UT_UNUSED(atts);
 	return false;
 #endif
 }
@@ -1485,6 +1582,8 @@ bool IE_Imp_AbiWord_1::_handleResource (const gchar ** atts, bool isResource)
 			return add_resource;
 		}
 #else
+	UT_UNUSED(atts);
+	UT_UNUSED(isResource);
 	return false;
 #endif
 }
