@@ -86,7 +86,7 @@ bool IE_Imp_RTF::LoadPictData(PictFormat format, const char * image_name,
 	const UT_uint16 BITS_PER_BYTE = 8;
 	const UT_uint16 bits_per_char = BITS_PER_BYTE / chars_per_byte;
 
-	UT_ByteBuf *pictData = new UT_ByteBuf();
+	UT_ByteBuf pictData;
 	UT_uint16 chLeft = chars_per_byte;
 	UT_Byte pic_byte = 0;
 	FG_Graphic* pFG = NULL;
@@ -113,7 +113,7 @@ bool IE_Imp_RTF::LoadPictData(PictFormat format, const char * image_name,
 			// if we have a complete byte, we put it in the buffer
 			if (--chLeft == 0)
 			{
-				pictData->append(&pic_byte, 1);
+				pictData.append(&pic_byte, 1);
 				chLeft = chars_per_byte;
 				pic_byte = 0;
 			}
@@ -125,29 +125,23 @@ bool IE_Imp_RTF::LoadPictData(PictFormat format, const char * image_name,
 		}
 	} else {
 		UT_ASSERT_HARMLESS(binaryLen);
-		UT_DEBUGMSG(("Loading binary data image of %d bytes\n", binaryLen));
+		UT_DEBUGMSG(("Loading binary data image of %ld bytes\n", binaryLen));
 		for (long i = 0; i < binaryLen; i++) {
 			if (!ReadCharFromFileWithCRLF(&ch)) {
 				retval = false;
 				goto cleanup;
 			}
-			pictData->append(&ch, 1);
+			pictData.append(&ch, 1);
 		}
 	}
 
 	// We let the caller handle this
 	SkipBackChar(ch);
 
-	// TODO: investigate whether pictData is leaking memory or not
-
 	error = IE_ImpGraphic::loadGraphic(pictData, iegftForRTF(format), &pFG);
 
 	if ((error == UT_OK) && pFG)
 	{
-		// TODO: according with IE_ImpGraphic header, we shouldn't g_free
-		// TODO: the buffer. Confirm that.
-		pictData = NULL;
-
 		const UT_ByteBuf * buf = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
 		imgProps.width = static_cast<UT_uint32>(pFG->getWidth ());
 		imgProps.height = static_cast<UT_uint32>(pFG->getHeight ());
@@ -174,7 +168,6 @@ bool IE_Imp_RTF::LoadPictData(PictFormat format, const char * image_name,
 	}
 
  cleanup:
-	DELETEP(pictData);
 	return retval;
 }
 
@@ -197,7 +190,7 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 	double cropb = 0.0f;
 	double cropl = 0.0f;
 	double cropr = 0.0f;
-	bool resize = false;
+	bool resize1 = false;
 	if (!bUseInsertNotAppend())
 	{
 		// non-null file, we're importing a doc
@@ -210,13 +203,13 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 		{
 		case RTFProps_ImageProps::ipstGoal:
 			UT_DEBUGMSG (("Goal\n"));
-			resize = true;
+			resize1 = true;
 			wInch = static_cast<double>(imgProps.wGoal) / 1440.0f;
 			hInch = static_cast<double>(imgProps.hGoal) / 1440.0f;
 			break;
 		case RTFProps_ImageProps::ipstScale:
 			UT_DEBUGMSG (("Scale: x=%d, y=%d, w=%d, h=%d\n", imgProps.scaleX, imgProps.scaleY, imgProps.width, imgProps.height));
-			resize = true;
+			resize1 = true;
 			if ((imgProps.wGoal != 0) && (imgProps.hGoal != 0)) {
 				// want image scaled against the w&h specified, not the image's natural size
 				wInch = ((static_cast<double>(imgProps.scaleX) / 100.0f) * (imgProps.wGoal/ 1440.0f));
@@ -227,7 +220,7 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 			}
 			break;
 		default:
-			resize = false;
+			resize1 = false;
 			break;
 		}
 
@@ -237,10 +230,10 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 			cropb = imgProps.cropb / 1440.0f;
 			cropl = imgProps.cropl / 1440.0f;
 			cropr = imgProps.cropr / 1440.0f;
-			resize = true;
+			resize1 = true;
 		}
 	  
-		if (resize) 
+		if (resize1) 
 		{
 			UT_LocaleTransactor t(LC_NUMERIC, "C");
 			UT_DEBUGMSG (("resizing...\n"));
@@ -252,7 +245,7 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 		const gchar* propsArray[5];
 		propsArray[0] = static_cast<const gchar *>("dataid");
 		propsArray[1] = static_cast<const gchar *>(image_name);
-		if (resize)
+		if (resize1)
 		{
 			propsArray[2] = static_cast<const gchar *>("props");
 			propsArray[3] = propBuffer.c_str();
@@ -409,7 +402,7 @@ bool IE_Imp_RTF::HandlePicture()
 	// this method loads a picture from the file
 	// and insert it in the document.
 
-	unsigned char ch;
+	unsigned char ch1;
 	bool bPictProcessed = false;
 	PictFormat format = picNone;
 
@@ -423,10 +416,10 @@ bool IE_Imp_RTF::HandlePicture()
 	RTF_KEYWORD_ID keywordID;
 
 	do {
-		if (!ReadCharFromFile(&ch))
+		if (!ReadCharFromFile(&ch1))
 			return false;
 
-		switch (ch)
+		switch (ch1)
 		{
 		case '\\':
 			UT_return_val_if_fail(!bPictProcessed, false);
@@ -558,7 +551,7 @@ bool IE_Imp_RTF::HandlePicture()
 				UT_UTF8String_sprintf(image_name,"%d",getDoc()->getUID(UT_UniqueId::Image));
 
 				// the first char belongs to the picture too
-				SkipBackChar(ch);
+				SkipBackChar(ch1);
 
 				if (!LoadPictData(format, image_name.utf8_str(), imageProps, isBinary, binaryLen))
 					if (!SkipCurrentGroup(false))
@@ -567,11 +560,11 @@ bool IE_Imp_RTF::HandlePicture()
 				bPictProcessed = true;
 			}
 		}
-	} while (ch != '}');
+	} while (ch1 != '}');
 
 	// The last brace is put back on the stream, so that the states stack
 	// doesn't get corrupted
-	SkipBackChar(ch);
+	SkipBackChar(ch1);
 
 	return true;
 }
@@ -651,43 +644,55 @@ void RTFProps_FrameProps::clear(void)
  */
 void RTFProps_FrameProps::_setProperty(const PropertyPair *pair)
 {
+	UT_return_if_fail(pair);
+
 	const UT_UTF8String *propName = pair->first;
 	const UT_UTF8String *propValue = pair->second;
 
+	if(!propName)
+		return;
+
 	UT_sint32 ival = 0;
 	if(strcmp(propName->utf8_str(),"dxTextLeft")== 0)
 	{
-		ival = atoi(propValue->utf8_str());
+		if(propValue)
+			ival = atoi(propValue->utf8_str());
 		m_iLeftPad = ival;
 	}
 	else if(strcmp(propName->utf8_str(),"dxTextRight")== 0)
 	{
-		ival = atoi(propValue->utf8_str());
+		if(propValue)
+			ival = atoi(propValue->utf8_str());
 		m_iRightPad = ival;
 	}
 	else if(strcmp(propName->utf8_str(),"dxTextTop")== 0)
 	{
-		ival = atoi(propValue->utf8_str());
+		if(propValue)
+			ival = atoi(propValue->utf8_str());
 		m_iTopPad = ival;
 	}
 	else if(strcmp(propName->utf8_str(),"dxTextBottom")== 0)
 	{
-		ival = atoi(propValue->utf8_str());
+		if(propValue)
+			ival = atoi(propValue->utf8_str());
 		m_iBotPad = ival;
 	}
 	else if(strcmp(propName->utf8_str(),"fillColor")== 0)
 	{
-		ival = atoi(propValue->utf8_str());
+		if(propValue)
+			ival = atoi(propValue->utf8_str());
 		m_iBackgroundColor = ival;
 	}
 	else if(strcmp(propName->utf8_str(),"fillType")== 0)
 	{
-		ival = atoi(propValue->utf8_str());
+		if(propValue)
+			ival = atoi(propValue->utf8_str());
 		m_iFillType = ival;
 	}
 	else if(strcmp(propName->utf8_str(),"shapeType")== 0)
 	{
-		ival = atoi(propValue->utf8_str());
+		if(propValue)
+			ival = atoi(propValue->utf8_str());
 		m_iFrameType = 0; // no others implemented
 		if(ival == 202)
 		{
@@ -705,7 +710,7 @@ void RTFProps_FrameProps::_setProperty(const PropertyPair *pair)
 //
 		UT_DEBUGMSG(("Found positioned Image \n"));
 	}
-	else if( propName && propValue)
+	else if(propValue)
 	{
 		UT_DEBUGMSG(("unknown property %s with value %s\n", propName->utf8_str(),
 					 propValue->utf8_str() ));
@@ -719,7 +724,7 @@ void RTFProps_FrameProps::_setProperty(const PropertyPair *pair)
   Handle the \\sp keyword inside shapes
 */
 
-class IE_Imp_ShpPropParser
+class ABI_EXPORT IE_Imp_ShpPropParser
 	: public IE_Imp_RTFGroupParser
 {
 public:
@@ -768,7 +773,7 @@ private:
 
 
 bool IE_Imp_ShpPropParser::tokenKeyword(IE_Imp_RTF * ie, RTF_KEYWORD_ID kwID, 
-										UT_sint32 param, bool paramUsed)
+										UT_sint32 /*param*/, bool /*paramUsed*/)
 {
 	switch(kwID) {
 	case RTF_KW_sn:
@@ -781,6 +786,7 @@ bool IE_Imp_ShpPropParser::tokenKeyword(IE_Imp_RTF * ie, RTF_KEYWORD_ID kwID,
 		ie->setStruxImage(true);
 		ie->clearImageName();
 		ie->HandlePicture();
+		break;
 	default:
 		break;
 	}
@@ -820,7 +826,7 @@ bool IE_Imp_ShpPropParser::tokenCloseBrace(IE_Imp_RTF * ie)
 	return IE_Imp_RTFGroupParser::tokenCloseBrace(ie);
 }
 
-bool IE_Imp_ShpPropParser::tokenData(IE_Imp_RTF * ie, UT_UTF8String & data)
+bool IE_Imp_ShpPropParser::tokenData(IE_Imp_RTF * /*ie*/, UT_UTF8String & data)
 {
 	DELETEP(m_lastData);
 	m_lastData = new UT_UTF8String(data);
@@ -842,7 +848,7 @@ bool IE_Imp_ShpPropParser::finalizeParse(void)
 // FIXME: move away from this file. Should belong to the main RTF import
 // or to ie_imp_RTFText.cpp
 
-class IE_Imp_TextParaPropParser
+class ABI_EXPORT IE_Imp_TextParaPropParser
 	: public IE_Imp_RTFGroupParser
 {
 public:
@@ -906,7 +912,7 @@ bool IE_Imp_TextParaPropParser::finalizeParse(void)
 
 //////////////////////////////////////////////////////////////////////////////////////////
 
-class IE_Imp_ShpGroupParser
+class ABI_EXPORT IE_Imp_ShpGroupParser
 	: public IE_Imp_RTFGroupParser
 {
 public:
@@ -964,7 +970,7 @@ IE_Imp_ShpGroupParser::~IE_Imp_ShpGroupParser()
 
 bool
 IE_Imp_ShpGroupParser::tokenKeyword(IE_Imp_RTF * ie, RTF_KEYWORD_ID kwID, 
-									UT_sint32 param, bool paramUsed)
+									UT_sint32 param, bool /*paramUsed*/)
 {
 	UT_DEBUGMSG(("IE_Imp_ShpGroupParser::tokenKeyword %d\n", kwID));
 	switch(kwID) {
@@ -1330,6 +1336,11 @@ bool IE_Imp_RTF::HandleObject()
 				// handle a paragraph.
 				beginResult = nested;
 				break;
+			case RTF_KW_pict:
+				if (beginResult <= nested) {
+					HandlePicture();
+				}
+				break;
 			case RTF_KW_shppict:
 				if (beginResult <= nested) {
 					HandleShapePict();
