@@ -27,7 +27,6 @@
 
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
-#include "ut_exception.h"
 #include "ut_types.h"
 #include "ut_base64.h"
 #include "ut_bytebuf.h"
@@ -684,7 +683,7 @@ bool IE_Imp_XHTML::pasteFromBuffer(PD_DocumentRange * pDocRange,
 	UT_return_val_if_fail(getDoc() == pDocRange->m_pDoc,false);
 	UT_return_val_if_fail(pDocRange->m_pos1 == pDocRange->m_pos2,false);
 	
-	PD_Document * newDoc = new PD_Document(getDoc()->getApp());
+	PD_Document * newDoc = new PD_Document();
 	newDoc->createRawDocument();
 	UT_XML * newXML;
 
@@ -766,7 +765,6 @@ void IE_Imp_XHTML::startElement(const gchar *name,
 	const gchar ** atts =
 		(const gchar **)UT_cloneAndDecodeAttributes (attributes);
 	
-	int i = 0;
 	int failLine;
 	failLine = 0;
 	UT_DEBUGMSG(("startElement: %s, parseState: %u, listType: %u\n", name, m_parseState, m_listType));
@@ -775,10 +773,10 @@ void IE_Imp_XHTML::startElement(const gchar *name,
 	                                                // this just avoids all the processing if there is an error
 #define NEW_ATTR_SZ 3
  	const gchar *new_atts[NEW_ATTR_SZ];
-	gchar * sz;
-	sz = NULL;
+	gchar * sz1;
+	sz1 = NULL;
 
-	for(i = 0; i < NEW_ATTR_SZ; i++)
+	for(int i = 0; i < NEW_ATTR_SZ; i++)
 	  new_atts[i] = NULL;
 #undef NEW_ATTR_SZ
 	UT_uint16 *parentID;
@@ -1165,11 +1163,11 @@ void IE_Imp_XHTML::startElement(const gchar *name,
 		if( p_val )
 		{
 			X_CheckError(requireBlock ());
-		    sz = g_strdup("xlink:href");
-		    new_atts[0] = sz;
-	    	sz = NULL;
-		    sz = g_strdup(p_val);
-		    new_atts[1] = sz;
+		    sz1 = g_strdup("xlink:href");
+		    new_atts[0] = sz1;
+	    	sz1 = NULL;
+		    sz1 = g_strdup(p_val);
+		    new_atts[1] = sz1;
 			X_CheckError(appendObject(PTO_Hyperlink,new_atts));
 		}
 		else
@@ -1183,19 +1181,15 @@ void IE_Imp_XHTML::startElement(const gchar *name,
 				UT_sint32 i;
  				const gchar *bm_new_atts[5];
 				for( i = 0; i < 5; i++) bm_new_atts[i] = NULL;
-	    		sz = NULL;
-			    sz = g_strdup("type");
-				bm_new_atts[0] = sz; 
-	    		sz = NULL;
-			    sz = g_strdup("start");
-				bm_new_atts[1] = sz;
-	    		sz = NULL;
-			    sz = g_strdup("name");
-			    bm_new_atts[2] = sz;
-	    		sz = NULL;
-		    	sz = g_strdup(p_val);
+			    sz1 = g_strdup("type");
+				bm_new_atts[0] = sz1; 
+			    sz1 = g_strdup("start");
+				bm_new_atts[1] = sz1;
+			    sz1 = g_strdup("name");
+			    bm_new_atts[2] = sz1;
+		    	sz1 = g_strdup(p_val);
 				m_szBookMarkName = g_strdup(p_val);
-			    bm_new_atts[3] = sz;
+			    bm_new_atts[3] = sz1;
 				if (m_szBookMarkName)
 				{
 					X_CheckError(appendObject(PTO_Bookmark,bm_new_atts));
@@ -1204,18 +1198,16 @@ void IE_Imp_XHTML::startElement(const gchar *name,
 
 				if (m_szBookMarkName && (m_parseState == _PS_Sec))
 				{
-					for(i = 0; i < 5; i++) bm_new_atts[i] = NULL;
-					sz = g_strdup("type");
-					bm_new_atts[0] = sz; 
-					sz = NULL;
-					sz = g_strdup("end");
-					bm_new_atts[1] = sz;
-					sz = NULL;
-					sz = g_strdup("name");
-					bm_new_atts[2] = sz;
-					sz = NULL;
-					sz = g_strdup(m_szBookMarkName);
-					bm_new_atts[3] = sz;
+					for(i = 0; i < 5; i++) 
+						bm_new_atts[i] = NULL;
+					sz1 = g_strdup("type");
+					bm_new_atts[0] = sz1; 
+					sz1 = g_strdup("end");
+					bm_new_atts[1] = sz1;
+					sz1 = g_strdup("name");
+					bm_new_atts[2] = sz1;
+					sz1 = g_strdup(m_szBookMarkName);
+					bm_new_atts[3] = sz1;
 					X_CheckError(appendObject(PTO_Bookmark,bm_new_atts));
 
 					FREEP(m_szBookMarkName);
@@ -1351,11 +1343,11 @@ void IE_Imp_XHTML::startElement(const gchar *name,
 				width  = static_cast<float>(100);
 				height = static_cast<float>(100);
 			}
-			width = width/96.0;
-			height = height/96.0;
+			width = width/96.0f;
+			height = height/96.0f;
 			if(height > 8.0)
 				{
-					float rat = 8.0/height;
+					float rat = 8.0f/height;
 					width = width * rat;
 					height = 8.0;
 				}
@@ -1375,40 +1367,33 @@ void IE_Imp_XHTML::startElement(const gchar *name,
 		UT_String dataid;
 		UT_String_sprintf (dataid, "image%u", static_cast<unsigned int>(m_iNewImage++));
 
-		sz = NULL;
-		sz = g_strdup(PT_PROPS_ATTRIBUTE_NAME);
-		X_CheckError(sz);
-		api_atts[0] = sz;
-		sz = NULL;
-		sz = g_strdup(utf8val.utf8_str ());
-		X_CheckError(sz);
-		api_atts[1] = sz;
-		sz = NULL;
-		sz = g_strdup("dataid");
-		X_CheckError(sz);
-		api_atts[2] = sz;
-		sz = NULL;
-		sz = g_strdup(dataid.c_str ());
-		X_CheckError(sz);
-		api_atts[3] = sz;
+		sz1 = g_strdup(PT_PROPS_ATTRIBUTE_NAME);
+		X_CheckError(sz1);
+		api_atts[0] = sz1;
+		sz1 = g_strdup(utf8val.utf8_str ());
+		X_CheckError(sz1);
+		api_atts[1] = sz1;
+		sz1 = g_strdup("dataid");
+		X_CheckError(sz1);
+		api_atts[2] = sz1;
+		sz1 = g_strdup(dataid.c_str ());
+		X_CheckError(sz1);
+		api_atts[3] = sz1;
 
-		sz = g_strdup("title");
-		X_CheckError(sz);
-		api_atts[4] = sz;
-		sz = NULL;
-		sz = g_strdup(szTitle == NULL ? "" : szTitle);
-		X_CheckError(sz);
-		api_atts[5] = sz;
-		sz = NULL;
+		sz1 = g_strdup("title");
+		X_CheckError(sz1);
+		api_atts[4] = sz1;
+		sz1 = g_strdup(szTitle == NULL ? "" : szTitle);
+		X_CheckError(sz1);
+		api_atts[5] = sz1;
 
-		sz = g_strdup("alt");
-		X_CheckError(sz);
-		api_atts[6] = sz;
-		sz = NULL;
-		sz = g_strdup(szAlt == NULL ? "" : szAlt);
-		X_CheckError(sz);
-		api_atts[7] = sz;
-		sz = NULL;
+		sz1 = g_strdup("alt");
+		X_CheckError(sz1);
+		api_atts[6] = sz1;
+		sz1 = g_strdup(szAlt == NULL ? "" : szAlt);
+		X_CheckError(sz1);
+		api_atts[7] = sz1;
+		sz1 = NULL;
 
 		api_atts[8] = NULL;
 
@@ -1972,14 +1957,9 @@ FG_Graphic * IE_Imp_XHTML::importDataURLImage (const gchar * szData)
 		}
 	binlength = binmaxlen - binlength;
 
-	UT_ByteBuf * pBB = new UT_ByteBuf;
-	if (pBB == 0)
-		{
-			UT_DEBUGMSG(("importDataURLImage: out of memory\n"));
-			FREEP(binbuffer);
-			return 0;
-		}
-	pBB->ins (0, reinterpret_cast<const UT_Byte *>(binbuffer), binlength);
+	UT_ByteBuf pBB;
+
+	pBB.ins (0, reinterpret_cast<const UT_Byte *>(binbuffer), binlength);
 	FREEP(binbuffer);
 
 	FG_Graphic * pfg = 0;
@@ -2168,7 +2148,7 @@ bool IE_Imp_XHTML::requireSection ()
 
 bool IE_Imp_XHTML::appendStrux(PTStruxType pts, const gchar ** attributes)
 {
-	UT_DEBUGMSG(("XHTML Import - appendStruxStrux type %d document %x \n",pts,getDoc()));
+	UT_DEBUGMSG(("XHTML Import - appendStruxStrux type %d document %p \n",pts,getDoc()));
 	if(pts == PTX_Section)
 	{
 		m_bFirstBlock = false;
@@ -2243,7 +2223,7 @@ bool IE_Imp_XHTML::appendFmt(const UT_GenericVector<const gchar*>* pVecAttribute
 		{
 			const gchar * attributes[3] = {"props",NULL,NULL};
 			UT_String sPropString("");
-			UT_uint32 i = 0;
+			UT_sint32 i = 0;
 			for(i=0; i< pVecAttributes->getItemCount(); i +=2)
 				{
 					UT_String sProp = pVecAttributes->getNthItem(i);
@@ -2470,7 +2450,7 @@ static unsigned char s_rgb_number (float f, bool bIsPercent)
 {
 	if (f < 0) return 0;
 
-	if (bIsPercent) f *= 2.55;
+	if (bIsPercent) f *= 2.55f;
 
 	if (f > 254.5) return 0xff;
 
@@ -2697,6 +2677,16 @@ static void s_props_append (UT_UTF8String & props, UT_uint32 css_mask,
 							verbatim = color.lookupNamedColor (value);
 						}
 				}
+			else if (strcmp (name, "text-transform") == 0)
+				{
+					if ((strcmp (value, "none") == 0) ||
+					    (strcmp (value, "capitalize") == 0) ||
+					    (strcmp (value, "uppercase") == 0) ||
+					    (strcmp (value, "lowercase") == 0))
+					{
+						verbatim = value;
+					}
+				}
 		}
 	if (css_mask & CSS_MASK_BLOCK)
 		{
@@ -2704,6 +2694,8 @@ static void s_props_append (UT_UTF8String & props, UT_uint32 css_mask,
 			 */
 			if ((strcmp (name, "margin-left")   == 0) ||
 				(strcmp (name, "margin-right")  == 0) ||
+				(strcmp (name, "margin-top")  == 0) ||
+				(strcmp (name, "margin-bottom")  == 0) ||
 				(strcmp (name, "text-align")    == 0) ||
 				(strcmp (name, "text-indent")   == 0) ||
 				(strcmp (name, "orphans")       == 0) ||
