@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 2003 Dom Lachowicz
  * 
@@ -307,9 +309,12 @@ UT_Error IE_MailMerge::constructMerger(const char * szFilename,
 		GsfInput *f = NULL;
 		
 		// we must open in binary mode for UCS-2 compatibility
-		if ( ( f = UT_go_file_open( szFilename, NULL ) ) != static_cast<GsfInput *>(0) )
+		if ( ( f = UT_go_file_open( szFilename, NULL ) ) != NULL )
 		{
-		  iNumbytes = UT_MIN(sizeof(szBuf) - 1, gsf_input_size(f));
+		  gsf_off_t stream_size = gsf_input_size(f);
+		  if (stream_size == -1)
+				return UT_ERROR;
+		  iNumbytes = UT_MIN(sizeof(szBuf) - 1, static_cast<UT_uint64>(stream_size));
 		  gsf_input_read(f, iNumbytes, (guint8*)szBuf);
 		  g_object_unref(G_OBJECT(f));
 		  szBuf[iNumbytes] = '\0';
@@ -389,11 +394,10 @@ UT_uint32 IE_MailMerge::getMergerCount(void)
 
 void IE_MailMerge::registerMerger (IE_MergeSniffer * s)
 {
-	UT_uint32 ndx = 0;
+	UT_sint32 ndx = 0;
 	UT_Error err = m_sniffers.addItem (s, &ndx);
 	
 	UT_return_if_fail(err == UT_OK);
-	UT_return_if_fail(ndx >= 0);
 	
 	s->setFileType(ndx+1);
 }
@@ -402,7 +406,7 @@ void IE_MailMerge::unregisterMerger (IE_MergeSniffer * s)
 {
 	UT_uint32 ndx = s->getFileType(); // 1:1 mapping
 	
-	UT_return_if_fail(ndx >= 0);
+	UT_return_if_fail(ndx > 0);
 	
 	m_sniffers.deleteNthItem (ndx-1);
 	
@@ -426,9 +430,10 @@ void IE_MailMerge::unregisterAllMergers ()
 	for (UT_uint32 i = 0; i < size; i++)
     {
 		pSniffer = m_sniffers.getNthItem(i);
-		if (pSniffer)
-			pSniffer->unref();
-    }
+		DELETEP(pSniffer);
+	}
+
+	m_sniffers.clear();
 }
 
 /************************************************************************/
@@ -496,7 +501,10 @@ public:
 			UT_XML default_xml;
 
 			default_xml.setListener (this);
-			return default_xml.parse (szFilename);
+
+			std::string sFile;
+			convertURI(sFile, szFilename);
+			return default_xml.parse (sFile.c_str());
 		}
 	
 	virtual UT_Error getHeaders (const char * szFilename, UT_Vector & out_vec) {
@@ -505,7 +513,10 @@ public:
 		m_vecHeaders = &out_vec;
 		
 		default_xml.setListener (this);
-		return default_xml.parse (szFilename);
+
+		std::string sFile;
+		convertURI(sFile, szFilename);
+		return default_xml.parse (sFile.c_str());
 	}
 
 private:
@@ -524,6 +535,22 @@ private:
 		m_vecHeaders->addItem (new UT_UTF8String (str));
 	}
 
+	void convertURI(std::string &sFile, const char *szURI)
+	{
+		bool bURI = UT_go_path_is_uri(szURI);
+		if(bURI)
+		{
+			const gchar *filename = UT_go_filename_from_uri(szURI);
+			sFile = filename;
+			FREEP(filename);
+		}
+		else
+		{
+			sFile = szURI;
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
+		}
+	}
+
 	UT_UTF8String mKey;
 	UT_UTF8String mCharData;
 	bool mAcceptingText;
@@ -541,7 +568,7 @@ public:
 	virtual ~IE_XMLMerge_Sniffer (){}
 	
 	virtual UT_Confidence_t recognizeContents (const char * szBuf, 
-											   UT_uint32 iNumbytes){
+											   UT_uint32 /*iNumbytes*/){
 		if (strstr(szBuf, "http://www.abisource.com/mailmerge/1.0") != 0 &&
 			strstr(szBuf, "merge-set") != 0)
 			return UT_CONFIDENCE_PERFECT;
@@ -678,7 +705,7 @@ public:
 	  UT_Error err = mergeFile(szFilename, true);
 
 	  if (err == UT_OK) {
-		for (UT_uint32 i = 0; i < m_headers.size(); i++) {
+		for (UT_sint32 i = 0; i < m_headers.size(); i++) {
 			UT_UTF8String * clone = new UT_UTF8String (*(UT_UTF8String *)m_headers[i]);
 			out_vec.addItem (clone);
 		}
@@ -705,7 +732,7 @@ private:
 				return false;
 			}
 
-			for (UT_uint32 i = 0; i < m_headers.size (); i++) {
+			for (UT_sint32 i = 0; i < m_headers.size (); i++) {
 				UT_UTF8String * key, * val;
 
 				key = m_headers.getNthItem(i);
@@ -741,7 +768,7 @@ public:
 	virtual ~IE_Delimiter_Sniffer (){}
 	
 	virtual UT_Confidence_t recognizeContents (const char * szBuf, 
-											   UT_uint32 iNumbytes){
+											   UT_uint32 /*iNumbytes*/){
 		char delim[2];
 		delim[0] = m_delim;
 		delim[1] = '\0';
